@@ -17,6 +17,7 @@
 
 */
 
+#include <cstring>
 #include <vector>
 #include <string>
 #include <list>
@@ -44,6 +45,7 @@ using namespace PBD;
 using namespace ARDOUR;
 
 vector<RefPtr<Gtk::Action> > ActionManager::session_sensitive_actions;
+vector<RefPtr<Gtk::Action> > ActionManager::write_sensitive_actions;
 vector<RefPtr<Gtk::Action> > ActionManager::region_list_selection_sensitive_actions;
 vector<RefPtr<Gtk::Action> > ActionManager::plugin_selection_sensitive_actions;
 vector<RefPtr<Gtk::Action> > ActionManager::region_selection_sensitive_actions;
@@ -79,7 +81,7 @@ ActionManager::init ()
 		error << string_compose (_("badly formatted UI definition file: %1"), err.what()) << endmsg;
 		cerr << string_compose (_("badly formatted UI definition file: %1"), err.what()) << endl;
 	} catch (...) {
-		error << _("Ardour menu definition file not found") << endmsg;
+		error << string_compose (_("%1 menu definition file not found"), PROGRAM_NAME) << endmsg;
 	}
 
 	if (!loaded) {
@@ -156,6 +158,50 @@ struct SortActionsByLabel {
     }
 };
 
+void
+ActionManager::get_all_actions (vector<string>& groups, vector<string>& names, vector<AccelKey>& bindings)
+{
+	/* the C++ API for functions used here appears to be broken in
+	   gtkmm2.6, so we fall back to the C level.
+	*/
+
+	GList* list = gtk_ui_manager_get_action_groups (ui_manager->gobj());
+	GList* node;
+	GList* acts;
+
+	for (node = list; node; node = g_list_next (node)) {
+		
+		GtkActionGroup* group = (GtkActionGroup*) node->data;
+		
+		/* first pass: collect them all */
+		
+		typedef std::list<Glib::RefPtr<Gtk::Action> > action_list;
+		action_list the_acts;
+
+		for (acts = gtk_action_group_list_actions (group); acts; acts = g_list_next (acts)) {
+			GtkAction* action = (GtkAction*) acts->data;
+			the_acts.push_back (Glib::wrap (action, true));
+		}
+		
+		/* now sort by label */
+		
+		SortActionsByLabel cmp;
+		the_acts.sort (cmp);
+
+		for (action_list::iterator a = the_acts.begin(); a != the_acts.end(); ++a) {
+
+			string accel_path = (*a)->get_accel_path ();
+
+			groups.push_back (gtk_action_group_get_name(group));
+			names.push_back (accel_path.substr (accel_path.find_last_of ('/') + 1));
+			
+			AccelKey key;
+			lookup_entry (accel_path, key);
+			bindings.push_back (AccelKey (key.get_key(), Gdk::ModifierType (key.get_mod())));
+		}
+	}
+}
+
 void
 ActionManager::get_all_actions (vector<string>& names, vector<string>& paths, vector<string>& keys, vector<AccelKey>& bindings)
 {
@@ -221,6 +267,19 @@ ActionManager::get_widget (const char * name)
 	return ui_manager->get_widget (name);
 }
 
+RefPtr<Action>
+ActionManager::get_action (const char* path)
+{
+	GtkAction* _act;
+	RefPtr<Action> act;
+
+	if ((_act = gtk_ui_manager_get_action (ui_manager->gobj(), path)) != 0) {
+		return Glib::wrap (_act, true);
+	}
+
+	return act;
+}
+
 RefPtr<Action>
 ActionManager::get_action (const char* group_name, const char* action_name)
 {
@@ -261,7 +320,7 @@ ActionManager::set_sensitive (vector<RefPtr<Action> >& actions, bool state)
 void
 ActionManager::uncheck_toggleaction (const char * name)
 {
-	char *last_slash = strrchr (name, '/');
+	const char *last_slash = strrchr (name, '/');
 
 	if (last_slash == 0) {
 		fatal << string_compose (_("programmer error: %1 %2"), X_("illegal toggle action name"), name) << endmsg;
@@ -276,7 +335,7 @@ ActionManager::uncheck_toggleaction (const char * name)
 	memcpy (group_name, name + 10, len);
 	group_name[len] = '\0';
 
-	char* action_name = last_slash + 1;
+	const char* action_name = last_slash + 1;
 
         RefPtr<Action> act = get_action (group_name, action_name);
 	if (act) {
