@@ -20,6 +20,8 @@
 #ifndef __ardour_gui_h__
 #define __ardour_gui_h__
 
+#include <time.h>
+
 /* need _BSD_SOURCE to get timersub macros */
 
 #ifdef _BSD_SOURCE
@@ -31,7 +33,6 @@
 #endif
 
 #include <list>
-
 #include <cmath>
 
 #include <libgnomecanvasmm/canvas.h>
@@ -58,6 +59,7 @@
 #include <gtkmm2ext/bindable_button.h>
 #include <ardour/ardour.h>
 #include <ardour/session.h>
+#include <ardour/types.h>
 
 #include "audio_clock.h"
 #include "ardour_dialog.h"
@@ -112,10 +114,16 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void show_splash ();
 	void hide_splash ();
 
+	void launch_chat ();
+	void launch_manual ();
+	void launch_reference ();
+
 	void show_about ();
 	void hide_about ();
 	
 	void idle_load (const Glib::ustring& path);
+	void finish();
+
 	int load_session (const Glib::ustring& path, const Glib::ustring& snapshot, Glib::ustring mix_template = Glib::ustring());
 	bool session_loaded;
 	int build_session (const Glib::ustring& path, const Glib::ustring& snapshot, 
@@ -146,9 +154,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	int  unload_session (bool hide_stuff = false);
 	void close_session(); 
 
-	int  save_state_canfail (string state_name = "");
-	void save_state (const string & state_name = "");
-	void restore_state (string state_name = "");
+	int  save_state_canfail (string state_name = "", bool switch_to_it = false);
+	void save_state (const string & state_name = "", bool switch_to_it = false);
 
 	static double gain_to_slider_position (ARDOUR::gain_t g);
         static ARDOUR::gain_t slider_position_to_gain (double pos);
@@ -176,16 +183,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	static sigc::signal<void>      SuperRapidScreenUpdate;
 	static sigc::signal<void,nframes_t, bool, nframes_t> Clock;
 
-	/* this is a helper function to centralize the (complex) logic for
-	   blinking rec-enable buttons.
-	*/
-
-	void rec_enable_button_blink (bool onoff, ARDOUR::AudioDiskstream *, Gtk::Widget *w);
-
 	void name_io_setup (ARDOUR::AudioEngine&, string&, ARDOUR::IO& io, bool in);
 
-	static gint hide_and_quit (GdkEventAny *ev, ArdourDialog *);
-
 	XMLNode* editor_settings() const;
 	XMLNode* mixer_settings () const;
 	XMLNode* keyboard_settings () const;
@@ -245,6 +244,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	bool ignore_dual_punch;
 	void toggle_punch_in ();
 	void toggle_punch_out ();
+	void show_loop_punch_ruler_and_disallow_hide ();
+	void reenable_hide_loop_punch_ruler_if_appropriate ();
 	void toggle_auto_return ();
 	void toggle_click ();
 
@@ -282,12 +283,10 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	Gtk::Tooltips          _tooltips;
 
-	void                     goto_editor_window ();
-	void                     goto_mixer_window ();
-	
-	Gtk::Table               adjuster_table;
-	Gtk::Frame               adjuster_frame;
-	Gtk::Fixed               adjuster_base;
+	void                goto_editor_window ();
+	void                goto_mixer_window ();
+	void                toggle_editor_mixer_on_top ();
+	bool                _mixer_on_top;
 
 	GlobalClickBox     *online_control_button;
 	vector<string>      online_control_strings;
@@ -298,9 +297,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	Gtk::ToggleButton   preroll_button;
 	Gtk::ToggleButton   postroll_button;
 
-	Gtk::Table          transport_table;
-	Gtk::Table          option_table;
-
 	int  setup_windows ();
 	void setup_transport ();
 	void setup_clock ();
@@ -311,9 +307,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void startup ();
 	void shutdown ();
 
-	void finish();
 	int  ask_about_saving_session (const string & why);
-	int  save_the_session;
 
 	/* periodic safety backup, to be precise */
 	gint autosave_session();
@@ -324,7 +318,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void map_transport_state ();
 	int32_t do_engine_start ();
 	
-	void engine_halted ();
+	void engine_halted (const char* reason, bool free_reason);
 	void engine_stopped ();
 	void engine_running ();
 
@@ -351,9 +345,11 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void manage_window (Gtk::Window&);
 	
 	AudioClock   big_clock;
-	Gtk::Frame   big_clock_frame;
 	Gtk::Window* big_clock_window;
 
+	void float_big_clock (Gtk::Window* parent);
+	bool main_window_state_event_handler (GdkEventWindowState*, bool window_was_editor);
+
 	void update_transport_clocks (nframes_t pos);
 	void record_state_changed ();
 
@@ -576,6 +572,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void transport_goto_zero ();
 	void transport_goto_start ();
 	void transport_goto_end ();
+	void transport_goto_wallclock ();
 	void transport_stop ();
 	void transport_stop_and_forget_capture ();
 	void transport_record (bool roll);
@@ -584,11 +581,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void transport_forward (int option);
 	void transport_rewind (int option);
 	void transport_loop ();
-
-	void transport_rolling ();
-	void transport_rewinding ();
-	void transport_forwarding ();
-	void transport_stopped ();
+	void toggle_roll (bool with_abort, bool roll_out_of_bounded_mode);
 
 	bool _session_is_new;
 	void connect_to_session (ARDOUR::Session *);
@@ -599,7 +592,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	
 	guint32  last_key_press_time;
 
-	void snapshot_session ();
+	void snapshot_session (bool switch_to_it);
 
 	Mixer_UI   *mixer;
 	int         create_mixer ();
@@ -654,21 +647,24 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	Gtk::MenuItem *cleanup_item;
 
-	void display_cleanup_results (ARDOUR::Session::cleanup_report& rep, const gchar* list_title, const string & msg);
+	void display_cleanup_results (ARDOUR::Session::cleanup_report& rep, const gchar* list_title, 
+				      const string& plural_msg, const string& singular_msg);
 	void cleanup ();
 	void flush_trash ();
 
 	bool have_configure_timeout;
-	struct timeval last_configure_time;
+	ARDOUR::microseconds_t last_configure_time;
 	gint configure_timeout ();
 
-	struct timeval last_peak_grab;
-	struct timeval last_shuttle_request;
+	ARDOUR::microseconds_t last_peak_grab;
+	ARDOUR::microseconds_t last_shuttle_request;
 
 	bool have_disk_speed_dialog_displayed;
 	void disk_speed_dialog_gone (int ignored_response, Gtk::MessageDialog*);
 	void disk_overrun_handler ();
 	void disk_underrun_handler ();
+	
+	bool preset_file_exists_handler ();
 
 	void session_dialog (std::string);
 	int pending_state_dialog ();
@@ -682,9 +678,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	Gtk::MenuItem* jack_reconnect_item;
 	Gtk::Menu*     jack_bufsize_menu;
 
-	int make_session_clean ();
-	bool filter_ardour_session_dirs (const Gtk::FileFilter::Info&);
-
 	Glib::RefPtr<Gtk::ActionGroup> common_actions;
 
 	void editor_realized ();
@@ -707,7 +700,9 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void set_remote_model (ARDOUR::RemoteModel);
 	void set_denormal_model (ARDOUR::DenormalModel);
 
+	void toggle_seamless_loop ();
 	void toggle_sync_order_keys ();
+	void toggle_new_plugins_active();
 	void toggle_StopPluginsWithTransport();
 	void toggle_DoNotRunPluginsWhileRecording();
 	void toggle_VerifyRemoveLastCapture();
@@ -718,6 +713,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_GainReduceFastTransport();
 	void toggle_LatchedSolo();
 	void toggle_ShowSoloMutes();
+	void toggle_SoloMuteOverride();
 	void toggle_LatchedRecordEnable ();
 	void toggle_RegionEquivalentsOverlap ();
 	void toggle_PrimaryClockDeltaEditCursor ();
@@ -725,6 +721,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_only_copy_imported_files ();
 	void toggle_ShowTrackMeters ();
 	void toggle_use_narrow_ms();
+	void toggle_NameNewMarkers ();
 	void toggle_rubberbanding_snaps_to_grid ();
 	void toggle_auto_analyse_audio ();
 	void toggle_TapeMachineMode();
@@ -761,6 +758,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void display_message (const char *prefix, gint prefix_len, 
 			      Glib::RefPtr<Gtk::TextBuffer::Tag> ptag, Glib::RefPtr<Gtk::TextBuffer::Tag> mtag, const char *msg);
 	Gtk::Label status_bar_label;
+	bool status_bar_button_press (GdkEventButton*);
 	Gtk::ToggleButton error_log_button;
 	
 	void loading_message (const std::string& msg);
@@ -769,6 +767,12 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void platform_specific ();
 	void platform_setup ();
 	void fontconfig_dialog ();
+	void toggle_translations ();
+
+	/* these are used only in response to a platform-specific "ShouldQuit" signal
+	 */
+	bool idle_finish ();
+	void queue_finish ();
 };
 
 #endif /* __ardour_gui_h__ */
