@@ -80,12 +80,21 @@ ARDOUR_UI::setup_windows ()
 
 #ifdef TOP_MENUBAR
 	HBox* status_bar_packer = manage (new HBox);
-	
+	EventBox* status_bar_event_box = manage (new EventBox);
+
+	status_bar_event_box->add (status_bar_label);
+	status_bar_event_box->add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK);
 	status_bar_label.set_size_request (300, -1);
-	status_bar_packer->pack_start (status_bar_label, true, true, 6);
+	status_bar_packer->pack_start (*status_bar_event_box, true, true, 6);
 	status_bar_packer->pack_start (error_log_button, false, false);
-	
+
+	status_bar_label.show ();
+	status_bar_event_box->show ();
+	status_bar_packer->show ();
+	error_log_button.show ();
+
 	error_log_button.signal_clicked().connect (mem_fun (*this, &UI::toggle_errors));
+	status_bar_event_box->signal_button_press_event().connect (mem_fun (*this, &ARDOUR_UI::status_bar_button_press));
 
 	editor->get_status_bar_packer().pack_start (*status_bar_packer, true, true);
 	editor->get_status_bar_packer().pack_start (menu_bar_base, false, false, 6);
@@ -99,6 +108,23 @@ ARDOUR_UI::setup_windows ()
 	return 0;
 }
 
+bool
+ARDOUR_UI::status_bar_button_press (GdkEventButton* ev)
+{
+	bool handled = false;
+
+	switch (ev->button) {
+	case 1:
+		status_bar_label.set_text ("");
+		handled = true;
+		break;
+	default:
+		break;
+	}
+
+	return handled;
+}
+
 void
 ARDOUR_UI::display_message (const char *prefix, gint prefix_len, RefPtr<TextBuffer::Tag> ptag, RefPtr<TextBuffer::Tag> mtag, const char *msg)
 {
@@ -125,71 +151,16 @@ ARDOUR_UI::display_message (const char *prefix, gint prefix_len, RefPtr<TextBuff
 #endif
 }
 
-void
-ARDOUR_UI::transport_stopped ()
-{
-	stop_button.set_visual_state (1);
-	
-	roll_button.set_visual_state (0);
-	play_selection_button.set_visual_state (0);
-	auto_loop_button.set_visual_state (0);
-
-	shuttle_fract = 0;
-	shuttle_box.queue_draw ();
-
-	update_disk_space ();
-}
-
-void
-ARDOUR_UI::transport_rolling ()
-{
-	stop_button.set_visual_state (0);
-	if (session->get_play_range()) {
-		play_selection_button.set_visual_state (1);
-		roll_button.set_visual_state (0);
-		auto_loop_button.set_visual_state (0);
-
-	} else if (session->get_play_loop ()) {
-		auto_loop_button.set_visual_state (1);
-		play_selection_button.set_visual_state (0);
-		roll_button.set_visual_state (0);
-
-	} else {
-
-		roll_button.set_visual_state (1);
-		play_selection_button.set_visual_state (0);
-		auto_loop_button.set_visual_state (0);
-	}
-
-	/* reset shuttle controller */
-
-	shuttle_fract = SHUTTLE_FRACT_SPEED1;  /* speed = 1.0, believe it or not */
-	shuttle_box.queue_draw ();
-}
-
-void
-ARDOUR_UI::transport_rewinding ()
-{
-	stop_button.set_visual_state (0);
-	roll_button.set_visual_state (1);
-	play_selection_button.set_visual_state (0);
-	auto_loop_button.set_visual_state (0);
-}
-
-void
-ARDOUR_UI::transport_forwarding ()
-{
-	stop_button.set_visual_state (0);
-	roll_button.set_visual_state (1);
-	play_selection_button.set_visual_state (0);
-	auto_loop_button.set_visual_state (0);
-}
-
 void
 ARDOUR_UI::setup_transport ()
 {
 	transport_tearoff = manage (new TearOff (transport_tearoff_hbox));
 	transport_tearoff->set_name ("TransportBase");
+	transport_tearoff->tearoff_window().signal_key_press_event().connect (bind (sigc::ptr_fun (relay_key_press), &transport_tearoff->tearoff_window()));
+
+	if (Profile->get_sae()) {
+		transport_tearoff->set_can_be_torn_off (false);
+	}
 
 	transport_hbox.pack_start (*transport_tearoff, true, false);
 
@@ -225,6 +196,7 @@ ARDOUR_UI::setup_transport ()
 	punch_out_button.set_name ("TransportButton");
 	click_button.set_name ("TransportButton");
 	time_master_button.set_name ("TransportButton");
+	sync_option_combo.set_name ("TransportButton");
 
 	stop_button.set_size_request(29, -1);
 	roll_button.set_size_request(29, -1);
@@ -293,7 +265,7 @@ ARDOUR_UI::setup_transport ()
 	ARDOUR_UI::instance()->tooltips().set_tip (punch_out_button, _("Stop recording at auto-punch end"));
 	ARDOUR_UI::instance()->tooltips().set_tip (click_button, _("Enable/Disable audio click"));
 	ARDOUR_UI::instance()->tooltips().set_tip (sync_option_combo, _("Positional sync source"));
-	ARDOUR_UI::instance()->tooltips().set_tip (time_master_button, _("Does Ardour control the time?"));
+	ARDOUR_UI::instance()->tooltips().set_tip (time_master_button, string_compose (_("Does %1 control the time?"), PROGRAM_NAME));
 	ARDOUR_UI::instance()->tooltips().set_tip (shuttle_box, _("Shuttle speed control"));
 	ARDOUR_UI::instance()->tooltips().set_tip (shuttle_units_button, _("Select semitones or %%-age for speed display"));
 	ARDOUR_UI::instance()->tooltips().set_tip (speed_display_box, _("Current transport speed"));
@@ -366,16 +338,17 @@ ARDOUR_UI::setup_transport ()
 
 	speed_display_box.add (speed_display_label);
 	speed_display_box.set_name (X_("ShuttleDisplay"));
+	set_size_request_to_display_given_text (speed_display_label, X_("> 24.0"), 2, 2);
 
 	shuttle_units_button.set_name (X_("ShuttleButton"));
 	shuttle_units_button.signal_clicked().connect (mem_fun(*this, &ARDOUR_UI::shuttle_unit_clicked));
 	
-	shuttle_style_button.set_name (X_("ShuttleButton"));
+	shuttle_style_button.set_name (X_("ShuttleStyleButton"));
 
 	vector<string> shuttle_strings;
 	shuttle_strings.push_back (_("sprung"));
 	shuttle_strings.push_back (_("wheel"));
-	set_popdown_strings (shuttle_style_button, shuttle_strings);
+	set_popdown_strings (shuttle_style_button, shuttle_strings, true);
 	shuttle_style_button.signal_changed().connect (mem_fun (*this, &ARDOUR_UI::shuttle_style_changed));
 
 	Frame* sdframe = manage (new Frame);
@@ -385,8 +358,9 @@ ARDOUR_UI::setup_transport ()
 
 	mtc_port_changed ();
 	sync_option_combo.signal_changed().connect (mem_fun (*this, &ARDOUR_UI::sync_option_changed));
-	const guint32 FUDGE = 25; // Combo's are stupid - they steal space from the entry for the button
-	set_size_request_to_display_given_text (sync_option_combo, X_("Igternal"), 2+FUDGE, 10);
+	// XXX HOW TO USE set_popdown_strings() and combo_fudge with this when we don't know
+	// the real strings till later?
+	set_size_request_to_display_given_text (sync_option_combo, X_("Igternal"), 4+COMBO_FUDGE, 15);
 
 	shbox->pack_start (*sdframe, false, false);
 	shbox->pack_start (shuttle_units_button, true, true);
@@ -664,7 +638,7 @@ ARDOUR_UI::shuttle_box_button_release (GdkEventButton* ev)
 				roll_button.set_visual_state (1);
 			} else {
 				shuttle_fract = 0;
-				session->request_transport_speed (0.0);
+				session->request_stop ();
 			}
 			shuttle_box.queue_draw ();
 		}
@@ -755,31 +729,45 @@ ARDOUR_UI::set_shuttle_fract (double f)
 void
 ARDOUR_UI::use_shuttle_fract (bool force)
 {
-	struct timeval now;
-	struct timeval diff;
+	microseconds_t now = get_microseconds();
 	
 	/* do not attempt to submit a motion-driven transport speed request
 	   more than once per process cycle.
 	 */
 
-	gettimeofday (&now, 0);
-	timersub (&now, &last_shuttle_request, &diff);
-
-	if (!force && (diff.tv_usec + (diff.tv_sec * 1000000)) < engine->usecs_per_cycle()) {
+	if (!force && (last_shuttle_request - now) < (microseconds_t) engine->usecs_per_cycle()) {
 		return;
 	}
 	
 	last_shuttle_request = now;
 
-	bool neg = (shuttle_fract < 0.0);
+	if (Config->get_shuttle_units() == Semitones) {
 
-	double fract = 1 - sqrt (1 - (shuttle_fract * shuttle_fract)); // Formula A1
+		const double step = 1.0 / 24.0; // range is 24 semitones up & down
+		double semitones;
+		double speed;
 
-	if (neg) {
-		fract = -fract;
+		semitones = round (shuttle_fract / step);
+		speed = pow (2.0, (semitones / 12.0));
+
+		session->request_transport_speed (speed);
+
+	} else {
+
+		bool neg;
+		double fract;
+		
+		neg = (shuttle_fract < 0.0);
+
+		fract = 1 - sqrt (1 - (shuttle_fract * shuttle_fract)); // Formula A1
+
+		if (neg) {
+			fract = -fract;
+		}
+
+		session->request_transport_speed (shuttle_max_speed * fract);
 	}
 
-	session->request_transport_speed (shuttle_max_speed * fract); // Formula A2
 	shuttle_box.queue_draw ();
 }
 
@@ -850,10 +838,11 @@ ARDOUR_UI::update_speed_display ()
 			if (Config->get_shuttle_units() == Percentage) {
 				snprintf (buf, sizeof (buf), "%.2f", x);
 			} else {
+
 				if (x < 0) {
-					snprintf (buf, sizeof (buf), "< %.1f", 12.0 * fast_log2 (-x));
+					snprintf (buf, sizeof (buf), "< %d", (int) round (12.0 * fast_log2 (-x)));
 				} else {
-					snprintf (buf, sizeof (buf), "> %.1f", 12.0 * fast_log2 (x));
+					snprintf (buf, sizeof (buf), "> %d", (int) round (12.0 * fast_log2 (x)));
 				}
 			}
 			speed_display_label.set_text (buf);
@@ -878,8 +867,6 @@ ARDOUR_UI::editor_realized ()
 	Config->map_parameters (mem_fun (*this, &ARDOUR_UI::parameter_changed));
 
 	set_size_request_to_display_given_text (speed_display_box, _("-0.55"), 2, 2);
-	const guint32 FUDGE = 25; // Combo's are stupid - they steal space from the entry for the button
-	set_size_request_to_display_given_text (shuttle_style_button, _("sprung"), 2+FUDGE, 10);
 	reset_dpi();
 }
 
