@@ -31,6 +31,7 @@
 #include <ardour/playlist_templates.h>
 #include <ardour/source.h>
 #include <ardour/region_factory.h>
+#include <ardour/profile.h>
 
 #include "audio_streamview.h"
 #include "audio_region_view.h"
@@ -59,14 +60,8 @@ AudioStreamView::AudioStreamView (AudioTimeAxisView& tv)
 	crossfades_visible = true;
 	_waveform_scale = LinearWaveform;
 	_waveform_shape = Traditional;
-	
-	if (tv.is_audio_track())
-		stream_base_color = ARDOUR_UI::config()->canvasvar_AudioTrackBase.get();
-	else
-		stream_base_color = ARDOUR_UI::config()->canvasvar_AudioBusBase.get();
-	
-	canvas_rect->property_fill_color_rgba() = stream_base_color;
-	canvas_rect->property_outline_color_rgba() = RGBA_BLACK;
+
+	color_handler ();
 
 	_amplitude_above_axis = 1.0;
 
@@ -129,7 +124,7 @@ AudioStreamView::set_amplitude_above_axis (gdouble app)
 }
 
 void
-AudioStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wait_for_waves)
+AudioStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wait_for_waves, bool recording)
 {
 	AudioRegionView *region_view = 0;
 
@@ -161,8 +156,13 @@ AudioStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wai
 
 	switch (_trackview.audio_track()->mode()) {
 	case Normal:
-		region_view = new AudioRegionView (canvas_group, _trackview, region, 
-						   _samples_per_unit, region_color);
+		if (recording) {
+			region_view = new AudioRegionView (canvas_group, _trackview, region, 
+						   _samples_per_unit, region_color, recording, TimeAxisViewItem::Visibility(TimeAxisViewItem::ShowFrame | TimeAxisViewItem::HideFrameRight));
+		} else {
+			region_view = new AudioRegionView (canvas_group, _trackview, region, 
+							   _samples_per_unit, region_color);
+		}
 		break;
 	case Destructive:
 		region_view = new TapeAudioRegionView (canvas_group, _trackview, region, 
@@ -178,7 +178,6 @@ AudioStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wai
 	region_view->set_amplitude_above_axis(_amplitude_above_axis);
 	region_views.push_front (region_view);
 
-	
 	/* if its the special single-sample length that we use for rec-regions, make it 
 	   insensitive to events 
 	*/
@@ -465,7 +464,7 @@ AudioStreamView::set_waveform_scale (WaveformScale scale)
 void
 AudioStreamView::setup_rec_box ()
 {
-	// cerr << _trackview.name() << " streamview SRB\n";
+	//cerr << _trackview.name() << " streamview SRB region_views.size() = " << region_views.size() << endl;
 
 	if (_trackview.session().transport_rolling()) {
 
@@ -474,7 +473,6 @@ AudioStreamView::setup_rec_box ()
 		if (!rec_active && 
 		    _trackview.session().record_status() == Session::Recording && 
 		    _trackview.get_diskstream()->record_enabled()) {
-
 			if (_trackview.audio_track()->mode() == Normal && use_rec_regions && rec_regions.size() == rec_rects.size()) {
 
 				/* add a new region, but don't bother if they set use_rec_regions mid-record */
@@ -541,8 +539,9 @@ AudioStreamView::setup_rec_box ()
 			rec_rect->property_x1() = xstart;
 			rec_rect->property_y1() = 1.0;
 			rec_rect->property_x2() = xend;
-			rec_rect->property_y2() = (double) _trackview.height - 1;
-			rec_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_RecordingRect.get();
+			rec_rect->property_y2() = (double) _trackview.current_height() - 1;
+			rec_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_TimeAxisFrame.get();
+			rec_rect->property_outline_what() = 0x1 | 0x2 | 0x4 | 0x8;
 			rec_rect->property_fill_color_rgba() = fill_color;
 			
 			RecBoxInfo recbox;
@@ -560,11 +559,9 @@ AudioStreamView::setup_rec_box ()
 		} else if (rec_active &&
 			   (_trackview.session().record_status() != Session::Recording ||
 			    !_trackview.get_diskstream()->record_enabled())) {
-
 			screen_update_connection.disconnect();
 			rec_active = false;
 			rec_updating = false;
-
 		}
 		
 	} else {
@@ -651,7 +648,6 @@ void
 AudioStreamView::update_rec_regions ()
 {
 	if (use_rec_regions) {
-
 		uint32_t n = 0;
 
 		for (list<boost::shared_ptr<Region> >::iterator iter = rec_regions.begin(); iter != rec_regions.end(); n++) {
@@ -689,7 +685,7 @@ AudioStreamView::update_rec_regions ()
 
 						if (origlen == 1) {
 							/* our special initial length */
-							add_region_view_internal (region, false);
+							add_region_view_internal (region, false, true);
 						}
 
 						/* also update rect */
@@ -714,7 +710,7 @@ AudioStreamView::update_rec_regions ()
 						
 						if (origlen == 1) {
 							/* our special initial length */
-							add_region_view_internal (region, false);
+							add_region_view_internal (region, false, true);
 						}
 						
 						/* also hide rect */
@@ -730,6 +726,28 @@ AudioStreamView::update_rec_regions ()
 	}
 }
 
+void
+AudioStreamView::show_all_fades ()
+{
+	for (list<RegionView*>::iterator i = region_views.begin(); i != region_views.end(); ++i) {
+		AudioRegionView* const arv = dynamic_cast<AudioRegionView*>(*i);
+		if (arv) {
+			arv->set_fade_visibility (true);
+		}
+	}
+}
+
+void
+AudioStreamView::hide_all_fades ()
+{
+	for (list<RegionView*>::iterator i = region_views.begin(); i != region_views.end(); ++i) {
+		AudioRegionView* const arv = dynamic_cast<AudioRegionView*>(*i);
+		if (arv) {
+			arv->set_fade_visibility (false);
+		}
+	}
+}
+
 void
 AudioStreamView::show_all_xfades ()
 {
@@ -772,8 +790,11 @@ AudioStreamView::color_handler ()
 	}
 
 	if (!_trackview.is_track()) {
-		canvas_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_AudioBusBase.get();
+		if (Profile->get_sae() && _trackview.route()->master()) {
+			canvas_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_AudioMasterBusBase.get();
+		} else {
+			canvas_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_AudioBusBase.get();
+		}
 	}
- 
 }
 
