@@ -229,7 +229,7 @@ AutomationLine::AutomationLine (const string & name, TimeAxisView& tv, ArdourCan
 	points_visible = false;
 	update_pending = false;
 	_vc_uses_gain_mapping = false;
-	no_draw = false;
+	auto_is_boolean = false;
 	_visible = true;
 	terminal_points_can_slide = true;
 	_height = 0;
@@ -496,7 +496,7 @@ AutomationLine::model_representation (ControlPoint& cp, ModelRepresentation& mr)
 	mr.xval = (nframes_t) floor (cp.get_x());
 	mr.yval = 1.0 - (cp.get_y() / _height);
 
-        /* if xval has not changed, set it directly from the model to avoid rounding errors */
+	/* if xval has not changed, set it directly from the model to avoid rounding errors */
 
 	if (mr.xval == trackview.editor.frame_to_unit((*cp.model)->when)) {
 		mr.xval = (nframes_t) (*cp.model)->when;
@@ -605,6 +605,12 @@ AutomationLine::determine_visible_control_points (ALPoints& points)
 
 		double tx = points[pi].x;
 		double ty = points[pi].y;
+		
+		if (isnan (tx) || isnan (ty)) {
+			warning << string_compose (_("Ignoring illegal points on AutomationLine \"%1\""),
+						   _name) << endmsg;
+			continue;
+		}
 
 		/* now ensure that the control_points vector reflects the current curve
 		   state, but don't plot control points too close together. also, don't
@@ -759,7 +765,7 @@ AutomationLine::get_verbose_cursor_string (float fraction)
 		if (fraction == 0.0) {
 			snprintf (buf, sizeof (buf), "-inf dB");
 		} else {
-			snprintf (buf, sizeof (buf), "%.1fdB", coefficient_to_dB (slider_position_to_gain (fraction)));
+			snprintf (buf, sizeof (buf), "%.1fdB", accurate_coefficient_to_dB (slider_position_to_gain (fraction)));
 		}
 	} else {
 		snprintf (buf, sizeof (buf), "%.2f", fraction);
@@ -840,13 +846,29 @@ AutomationLine::line_drag (uint32_t i1, uint32_t i2, float fraction, bool with_p
 	line_drag_cp1 = i1;
 	line_drag_cp2 = i2;
 	
+	//check if one of the control points on the line is in a selected range
+	bool range_found = false;
 	ControlPoint *cp;
-
 	for (uint32_t i = i1 ; i <= i2; i++) {
 		cp = nth (i);
-		modify_view_point (*cp, trackview.editor.unit_to_frame (cp->get_x()), ((_height - cp->get_y()) /_height) + ydelta, with_push);
+		if ( cp->selected )
+			range_found = true;
 	}
-
+	
+	if (range_found) {
+		for (vector<ControlPoint*>::iterator i = control_points.begin(); i != control_points.end(); ++i) {
+			if ( (*i)->selected ) {
+				modify_view_point (*(*i), trackview.editor.unit_to_frame ((*i)->get_x()), ((_height - (*i)->get_y()) /_height) + ydelta, with_push);
+			}
+		}
+	} else {
+		ControlPoint *cp;
+		for (uint32_t i = i1 ; i <= i2; i++) {
+			cp = nth (i);
+			modify_view_point (*cp, trackview.editor.unit_to_frame (cp->get_x()), ((_height - cp->get_y()) /_height) + ydelta, with_push);
+		}
+	}
+	
 	if (line_points.size() > 1) {
 		line->property_points() = line_points;
 	}
@@ -1183,7 +1205,7 @@ AutomationLine::list_changed ()
 void
 AutomationLine::reset_callback (const AutomationList& events)
 {
-        ALPoints tmp_points;
+	ALPoints tmp_points;
 	uint32_t npoints = events.size();
 
 	if (npoints == 0) {
@@ -1198,26 +1220,28 @@ AutomationLine::reset_callback (const AutomationList& events)
 	AutomationList::const_iterator ai;
 
 	for (ai = events.const_begin(); ai != events.const_end(); ++ai) {
-
+		
 		double translated_y = (*ai)->value;
 		model_to_view_y (translated_y);
 
-		tmp_points.push_back (ALPoint (trackview.editor.frame_to_unit ((*ai)->when),
-					       _height - (translated_y * _height)));
+		add_model_point (tmp_points, (*ai)->when, translated_y);
 	}
 	
 	determine_visible_control_points (tmp_points);
 }
 
+void
+AutomationLine::add_model_point (ALPoints& tmp_points, double frame, double yfract)
+{
+	tmp_points.push_back (ALPoint (trackview.editor.frame_to_unit (frame),
+				       _height - (yfract * _height)));
+}
+
 void
 AutomationLine::reset ()
 {
 	update_pending = false;
 
-	if (no_draw) {
-		return;
-	}
-
 	alist.apply_to_points (*this, &AutomationLine::reset_callback);
 }
 
@@ -1225,9 +1249,9 @@ void
 AutomationLine::clear ()
 {
 	/* parent must create command */
-        XMLNode &before = get_state();
+        XMLNode &before = alist.get_state();
 	alist.clear();
-	trackview.editor.current_session()->add_command (new MementoCommand<AutomationLine>(*this, &before, &get_state()));
+	trackview.editor.current_session()->add_command (new MementoCommand<AutomationList>(alist, &before, &get_state()));
 	trackview.editor.current_session()->commit_reversible_command ();
 	trackview.editor.current_session()->set_dirty ();
 }
@@ -1246,6 +1270,10 @@ AutomationLine::change_model_range (AutomationList::iterator start, AutomationLi
 void
 AutomationLine::show_all_control_points ()
 {
+	if (auto_is_boolean) {  //show the automation line but don't allow any control points
+		return;
+	}
+
 	points_visible = true;
 
 	for (vector<ControlPoint*>::iterator i = control_points.begin(); i != control_points.end(); ++i) {
