@@ -31,7 +31,6 @@
 #include <pbd/shortpath.h>
 
 #include <gtkmm2ext/choice.h>
-#include <gtkmm2ext/window_title.h>
 
 #include <ardour/session.h>
 #include <ardour/audioplaylist.h>
@@ -52,6 +51,7 @@
 #include "editing.h"
 #include "audio_time_axis.h"
 #include "utils.h"
+#include "gui_thread.h"
 
 #include "i18n.h"
 
@@ -70,7 +70,7 @@ void
 Editor::add_external_audio_action (ImportMode mode_hint)
 {
 	if (session == 0) {
-		MessageDialog msg (0, _("You can't import or embed an audiofile until you have a session loaded."));
+		MessageDialog msg (_("You can't import or embed an audiofile until you have a session loaded."));
 		msg.run ();
 		return;
 	}
@@ -91,7 +91,7 @@ Editor::external_audio_dialog ()
 	uint32_t track_cnt;
 
 	if (session == 0) {
-		MessageDialog msg (0, _("You can't import or embed an audiofile until you have a session loaded."));
+		MessageDialog msg (_("You can't import or embed an audiofile until you have a session loaded."));
 		msg.run ();
 		return;
 	}
@@ -203,6 +203,7 @@ Editor::check_whether_and_how_to_import(string path, bool all_or_nothing)
 	bool wave_name_exists = false;
 
 	for (AudioSourceList::iterator i = all_sources.begin(); i != all_sources.end(); ++i) {
+
 		boost::shared_ptr<AudioFileSource> afs = boost::dynamic_pointer_cast<AudioFileSource>(i->second);
 
 		string tmp (Glib::path_get_basename (afs->path()));
@@ -215,7 +216,6 @@ Editor::check_whether_and_how_to_import(string path, bool all_or_nothing)
 
 	int function = 1;
 
-
 	if (wave_name_exists) {
 		string message;
 		if (all_or_nothing) {
@@ -238,7 +238,6 @@ Editor::check_whether_and_how_to_import(string path, bool all_or_nothing)
 			dialog.add_button("Cancel", 2);
 		}
 		
-		
 		//dialog.add_button("Skip all", 4); // All or rest?
 
 		dialog.show();
@@ -281,29 +280,14 @@ Editor::get_nth_selected_audio_track (int nth) const
 	return atv->audio_track();
 }	
 
-bool
-Editor::idle_do_import (vector<ustring> paths, ImportDisposition chns, ImportMode mode, SrcQuality quality, nframes64_t& pos)
-{
-	_do_import (paths, chns, mode, quality, pos);
-	return false;
-}
-
 void
 Editor::do_import (vector<ustring> paths, ImportDisposition chns, ImportMode mode, SrcQuality quality, nframes64_t& pos)
 {
-#ifdef GTKOSX
-	Glib::signal_idle().connect (bind (mem_fun (*this, &Editor::idle_do_import), paths, chns, mode, quality, pos));
-#else
-	_do_import (paths, chns, mode, quality, pos);
-#endif
-}
+	//this is where an import "starts" so initialize the count here ... the rest gets initialized in import_sndfiles()
+	import_status.count = 0;
 
-void
-Editor::_do_import (vector<ustring> paths, ImportDisposition chns, ImportMode mode, SrcQuality quality, nframes64_t& pos)
-{
 	boost::shared_ptr<AudioTrack> track;
 	vector<ustring> to_import;
-	bool ok = true;
 	int nth = 0;
 
 	if (interthread_progress_window == 0) {
@@ -311,11 +295,13 @@ Editor::_do_import (vector<ustring> paths, ImportDisposition chns, ImportMode mo
 	}
 
 	if (chns == Editing::ImportMergeFiles) {
+
 		/* create 1 region from all paths, add to 1 track,
 		   ignore "track"
 		*/
+
 		bool cancel = false;
-		for (vector<ustring>::iterator a = paths.begin(); a != paths.end() && ok; ++a) {
+		for (vector<ustring>::iterator a = paths.begin(); a != paths.end(); ++a) {
 			int check = check_whether_and_how_to_import(*a, false);
 			if (check == 2) {
 				cancel = true;
@@ -324,106 +310,77 @@ Editor::_do_import (vector<ustring> paths, ImportDisposition chns, ImportMode mo
 		}
 
 		if (!cancel) {
-			if (import_sndfiles (paths, mode, quality, pos, 1, 1, track, false)) {
-				ok = false;
-			}
+			import_sndfiles (paths, mode, quality, pos, 1, 1, track, false, paths.size());
 		}
 
 	} else {
-		bool replace;
+
+		bool replace = false;
+		bool ok = true;
+		vector<ustring>::size_type total = paths.size();
 
 		for (vector<ustring>::iterator a = paths.begin(); a != paths.end() && ok; ++a) {
 
-			int check = check_whether_and_how_to_import(*a, true);
+			int check = check_whether_and_how_to_import (*a, true);
 
-			if (check == 2 ) { 
-				// skip
+			switch (check) {
+			case 2:
+				// user said skip
 				continue;
+			case 0:
+				fatal << "Updating existing sources should be disabled!" << endmsg;
+				/* NOTREACHED*/
+				break;
+			case 1:
+				replace = false;
+				break;
+			default:
+				fatal << "Illegal return " << check <<  " from check_whether_and_how_to_import()!" << endmsg;
+				/* NOTREACHED*/
 			}
 
-			if (check == 0) {
-				fatal << "Updating existing sources should be disabled!" << endl;
-				replace = true;
-			} else if (check == 1) {
-				replace = false;
-			}
-			
 
 			switch (chns) {
-				case Editing::ImportDistinctFiles:
+			case Editing::ImportDistinctFiles:
+				
+				to_import.clear ();
+				to_import.push_back (*a);
+				
+				if (mode == Editing::ImportToTrack) {
+					track = get_nth_selected_audio_track (nth++);
+				}
+				
+				ok = (import_sndfiles (to_import, mode, quality, pos, 1, -1, track, replace, total) == 0);
+				break;
+				
+			case Editing::ImportDistinctChannels:
+				
+				to_import.clear ();
+				to_import.push_back (*a);
+				
+				ok = (import_sndfiles (to_import, mode, quality, pos, -1, -1, track, replace, total) == 0);
+				break;
+				
+			case Editing::ImportSerializeFiles:
+				
+				to_import.clear ();
+				to_import.push_back (*a);
 
-					to_import.clear ();
-					to_import.push_back (*a);
+				ok = (import_sndfiles (to_import, mode, quality, pos, 1, 1, track, replace, total) == 0);
+				break;
 
-					if (mode == Editing::ImportToTrack) {
-						track = get_nth_selected_audio_track (nth++);
-					}
-
-					if (import_sndfiles (to_import, mode, quality, pos, 1, -1, track, replace)) {
-						ok = false;
-					}
-
-					break;
-
-				case Editing::ImportDistinctChannels:
-
-					to_import.clear ();
-					to_import.push_back (*a);
-
-					if (import_sndfiles (to_import, mode, quality, pos, -1, -1, track, replace)) {
-						ok = false;
-					}
-
-					break;
-
-				case Editing::ImportSerializeFiles:
-
-					to_import.clear ();
-					to_import.push_back (*a);
-
-					/* create 1 region from this path, add to 1 track,
-					   reuse "track" across paths
-					   */
-
-					if (import_sndfiles (to_import, mode, quality, pos, 1, 1, track, replace)) {
-						ok = false;
-					}
-
-					break;
-
-				case Editing::ImportMergeFiles:
-					// Not entered
-					break;
+			case Editing::ImportMergeFiles:
+				// Not entered, handled in earlier if() branch
+				break;
 			}
 		}
-
-		if (ok) {
-			session->save_state ("");
-		}
-
-		interthread_progress_window->hide_all ();
 	}
-}
 
-bool
-Editor::idle_do_embed (vector<ustring> paths, ImportDisposition chns, ImportMode mode, nframes64_t& pos)
-{
-	_do_embed (paths, chns, mode, pos);
-	return false;
+	interthread_progress_window->hide_all ();
 }
 
 void
 Editor::do_embed (vector<ustring> paths, ImportDisposition chns, ImportMode mode, nframes64_t& pos)
-{
-#ifdef GTKOSX
-	Glib::signal_idle().connect (bind (mem_fun (*this, &Editor::idle_do_embed), paths, chns, mode, pos));
-#else
-	_do_embed (paths, chns, mode, pos);
-#endif
-}
-
-void
-Editor::_do_embed (vector<ustring> paths, ImportDisposition chns, ImportMode mode, nframes64_t& pos)
 {
 	boost::shared_ptr<AudioTrack> track;
 	bool check_sample_rate = true;
@@ -473,10 +430,10 @@ Editor::_do_embed (vector<ustring> paths, ImportDisposition chns, ImportMode mod
 			to_embed.clear ();
 			to_embed.push_back (*a);
 
-			if (embed_sndfiles (to_embed, multi, check_sample_rate, mode, pos, 1, 1, track) < -1) {
-				goto out;
-			}
-		}
+                        if (embed_sndfiles (to_embed, multi, check_sample_rate, mode, pos, 1, 1, track) < -1) {
+                                goto out;
+                        }
+                }
 		break;
 	}
 
@@ -490,15 +447,14 @@ Editor::_do_embed (vector<ustring> paths, ImportDisposition chns, ImportMode mod
 
 int
 Editor::import_sndfiles (vector<ustring> paths, ImportMode mode, SrcQuality quality, nframes64_t& pos, 
-			 int target_regions, int target_tracks, boost::shared_ptr<AudioTrack>& track, bool replace)
+			 int target_regions, int target_tracks, boost::shared_ptr<AudioTrack>& track, bool replace,
+			 uint32_t total)
 {
-	WindowTitle title = string_compose (_("importing %1"), paths.front());
-
-	interthread_progress_window->set_title (title.get_string());
+	interthread_progress_window->set_title (string_compose (_("Importing %1"), paths.front()));
 	interthread_progress_window->set_position (Gtk::WIN_POS_MOUSE);
 	interthread_progress_bar.set_fraction (0.0f);
 	interthread_cancel_label.set_text (_("Cancel Import"));
-	current_interthread_info = &import_status;
+ 	current_interthread_info = &import_status;
 
 	import_status.paths = paths;
 	import_status.done = false;
@@ -507,42 +463,52 @@ Editor::import_sndfiles (vector<ustring> paths, ImportMode mode, SrcQuality qual
 	import_status.done = 0.0;
 	import_status.quality = quality;
 	import_status.replace_existing_source = replace;
+	import_status.total = total;
 
+	import_status.mode = mode;
+	import_status.pos = pos;
+	import_status.target_tracks = target_tracks;
+	import_status.target_regions = target_regions;
+	import_status.track = track;
+	import_status.replace = replace;
 	interthread_progress_connection = Glib::signal_timeout().connect 
 		(bind (mem_fun(*this, &Editor::import_progress_timeout), (gpointer) 0), 500);
 	
 	track_canvas->get_window()->set_cursor (Gdk::Cursor (Gdk::WATCH));
-	ARDOUR_UI::instance()->flush_pending ();
+	gdk_flush ();
 
 	/* start import thread for this spec. this will ultimately call Session::import_audiofile()
-	   and if successful will add the file(s) as a region to the session region list.
+	   which, if successful, will add the files as regions to the region list. its up to us
+	   (the GUI) to direct additional steps after that.
 	*/
-	
+
 	pthread_create_and_store ("import", &import_status.thread, 0, _import_thread, this);
 	pthread_detach (import_status.thread);
 	
-	while (!(import_status.done || import_status.cancel)) {
+	while (!import_status.done && !import_status.cancel) {
 		gtk_main_iteration ();
 	}
 
 	interthread_progress_window->hide ();
-	
 	import_status.done = true;
 	interthread_progress_connection.disconnect ();
 	
-	/* import thread finished - see if we should build a new track */
-
-	boost::shared_ptr<AudioRegion> r;
-	
-	if (import_status.cancel || import_status.sources.empty()) {
-		goto out;
+	if (!import_status.cancel && !import_status.sources.empty()) {
+		if (add_sources (import_status.paths, 
+				 import_status.sources, 
+				 import_status.pos, 
+				 import_status.mode, 
+				 import_status.target_regions, 
+				 import_status.target_tracks, 
+				 track, false) == 0) {
+			session->save_state ("");
+		}
+		
+		/* update position from results */
+		
+		pos = import_status.pos;
 	}
 
-	if (add_sources (paths, import_status.sources, pos, mode, target_regions, target_tracks, track, false) == 0) {
-		session->save_state ("");
-	}
-
-  out:
 	track_canvas->get_window()->set_cursor (*current_canvas_cursor);
 	return 0;
 }
@@ -557,9 +523,10 @@ Editor::embed_sndfiles (vector<Glib::ustring> paths, bool multifile,
 	string linked_path;
 	SoundFileInfo finfo;
 	int ret = 0;
+	Glib::ustring path_to_use;
 
 	track_canvas->get_window()->set_cursor (Gdk::Cursor (Gdk::WATCH));
-	ARDOUR_UI::instance()->flush_pending ();
+	gdk_flush ();
 
 	for (vector<Glib::ustring>::iterator p = paths.begin(); p != paths.end(); ++p) {
 
@@ -567,9 +534,8 @@ Editor::embed_sndfiles (vector<Glib::ustring> paths, bool multifile,
 
 		/* lets see if we can link it into the session */
 		
-		linked_path = session->sound_dir();
-		linked_path += '/';
-		linked_path += Glib::path_get_basename (path);
+                linked_path = Glib::build_filename (session->sound_dir(), Glib::path_get_basename (path));
+		path_to_use = path;
 		
 		if (link (path.c_str(), linked_path.c_str()) == 0) {
 
@@ -580,6 +546,7 @@ Editor::embed_sndfiles (vector<Glib::ustring> paths, bool multifile,
 			*/
 			
 			path = linked_path;
+			path_to_use = Glib::path_get_basename (linked_path);
 
 		} else {
 
@@ -591,6 +558,7 @@ Editor::embed_sndfiles (vector<Glib::ustring> paths, bool multifile,
 				if (stat (linked_path.c_str(), &sb) == 0) {
 					if (sb.st_nlink > 1) { // its a hard link, assume its the one we want
 						path = linked_path;
+						path_to_use = Glib::path_get_basename (linked_path);
 					}
 				}
 			}
@@ -670,10 +638,8 @@ Editor::embed_sndfiles (vector<Glib::ustring> paths, bool multifile,
 
 				if ((s = session->source_by_path_and_channel (path, n)) == 0) {
 
-					cerr << "add embed/import source with defer_peaks = true\n";
-
 					source = boost::dynamic_pointer_cast<AudioFileSource> (SourceFactory::createReadable 
-											       (*session, path,  n,
+											       (*session, path_to_use,  n,
 												(mode == ImportAsTapeTrack ? 
 												 AudioFileSource::Destructive : 
 												 AudioFileSource::Flag (0)),
@@ -697,7 +663,6 @@ Editor::embed_sndfiles (vector<Glib::ustring> paths, bool multifile,
 	if (sources.empty()) {
 		goto out;
 	}
-
 	ret = add_sources (paths, sources, pos, mode, target_regions, target_tracks, track, true);
 
   out:
@@ -713,8 +678,11 @@ Editor::add_sources (vector<Glib::ustring> paths, SourceList& sources, nframes64
 	ustring region_name;
 	uint32_t input_chan = 0;
 	uint32_t output_chan = 0;
+	bool use_timestamp;
+	
+	use_timestamp = (pos == -1);
 
-	if (pos == -1) { // "use timestamp"
+	if (use_timestamp) {
 		if (sources[0]->natural_position() != 0) {
 			pos = sources[0]->natural_position();
 		} else {
@@ -727,12 +695,17 @@ Editor::add_sources (vector<Glib::ustring> paths, SourceList& sources, nframes64
 		/* take all the sources we have and package them up as a region */
 
 		region_name = region_name_from_path (paths.front(), (sources.size() > 1), false);
-		
-		regions.push_back (boost::dynamic_pointer_cast<AudioRegion> 
+		boost::shared_ptr<AudioRegion> ar = boost::dynamic_pointer_cast<AudioRegion> 
 				   (RegionFactory::create (sources, 0, sources[0]->length(), region_name, 0,
-							   Region::Flag (Region::DefaultFlags|Region::WholeFile|Region::External))));
+							   Region::Flag (Region::DefaultFlags|Region::WholeFile|Region::External)));
+							   
+        if (use_timestamp) {
+		    ar->special_set_position(sources[0]->natural_position());
+		}
 		
-	} else if (target_regions == -1) {
+		regions.push_back (ar);
+		
+	} else if (target_regions == -1 || target_regions > 1) {
 
 		/* take each source and create a region for each one */
 
@@ -748,11 +721,15 @@ Editor::add_sources (vector<Glib::ustring> paths, SourceList& sources, nframes64
 			boost::shared_ptr<AudioFileSource> afs = boost::dynamic_pointer_cast<AudioFileSource> (*x);
 
 			region_name = region_name_from_path (afs->path(), false, true, sources.size(), n);
-
-			regions.push_back (boost::dynamic_pointer_cast<AudioRegion> 
+			boost::shared_ptr<AudioRegion> ar = boost::dynamic_pointer_cast<AudioRegion> 
 					   (RegionFactory::create (just_one, 0, (*x)->length(), region_name, 0,
-								   Region::Flag (Region::DefaultFlags|Region::WholeFile|Region::External))));
+								   Region::Flag (Region::DefaultFlags|Region::WholeFile|Region::External)));
 
+                        if (use_timestamp) {
+                                ar->special_set_position((*x)->natural_position());
+                        }
+                        
+                        regions.push_back (ar);
 		}
 	}
 
@@ -774,12 +751,12 @@ Editor::add_sources (vector<Glib::ustring> paths, SourceList& sources, nframes64
 
 	int n = 0;
 
-	for (vector<boost::shared_ptr<AudioRegion> >::iterator r = regions.begin(); r != regions.end(); ++r, ++n) {
+        for (vector<boost::shared_ptr<AudioRegion> >::iterator r = regions.begin(); r != regions.end(); ++r, ++n) {
+                
+                finish_bringing_in_audio (*r, input_chan, output_chan, pos, mode, track);
 
-		finish_bringing_in_audio (*r, input_chan, output_chan, pos, mode, track);
-
-		if (target_tracks != 1) {
-			track.reset ();
+                if (target_tracks != 1) {
+                        track.reset ();
 		} else {
 			pos += (*r)->length();
 		} 
@@ -861,7 +838,7 @@ Editor::finish_bringing_in_audio (boost::shared_ptr<AudioRegion> region, uint32_
 void *
 Editor::_import_thread (void *arg)
 {
-	PBD::ThreadCreated (pthread_self(), X_("Import"));
+	PBD::notify_gui_about_thread_creation (pthread_self(), X_("Import"));
 
 	Editor *ed = (Editor *) arg;
 	return ed->import_thread ();
@@ -885,7 +862,7 @@ Editor::import_progress_timeout (void *arg)
 		interthread_progress_window->show_all ();
 		reset = true;
 	}
-
+	
 	interthread_progress_label.set_text (import_status.doing_what);
 
 	if (import_status.freeze) {
