@@ -63,38 +63,95 @@ Editor::initialize_rulers ()
 	_ruler_separator->set_size_request(-1, 2);
 	_ruler_separator->show();
 
+	_minsec_ruler = gtk_custom_hruler_new ();
+	minsec_ruler = Glib::wrap (_minsec_ruler);
+	minsec_ruler->set_name ("MinSecRuler");
+	minsec_ruler->set_size_request (-1, (int)timebar_height);
+	gtk_custom_ruler_set_metric (GTK_CUSTOM_RULER(_minsec_ruler), &ruler_metrics[ruler_metric_minsec]);
+	minsec_ruler->hide ();
+	minsec_ruler->set_no_show_all();
+
 	_smpte_ruler = gtk_custom_hruler_new ();
 	smpte_ruler = Glib::wrap (_smpte_ruler);
 	smpte_ruler->set_name ("SMPTERuler");
 	smpte_ruler->set_size_request (-1, (int)timebar_height);
 	gtk_custom_ruler_set_metric (GTK_CUSTOM_RULER(_smpte_ruler), &ruler_metrics[ruler_metric_smpte]);
-	
-	_bbt_ruler = gtk_custom_hruler_new ();
-	bbt_ruler = Glib::wrap (_bbt_ruler);
-	bbt_ruler->set_name ("BBTRuler");
-	bbt_ruler->set_size_request (-1, (int)timebar_height);
-	gtk_custom_ruler_set_metric (GTK_CUSTOM_RULER(_bbt_ruler), &ruler_metrics[ruler_metric_bbt]);
+	smpte_ruler->hide ();
+	smpte_ruler->set_no_show_all();
 
 	_frames_ruler = gtk_custom_hruler_new ();
 	frames_ruler = Glib::wrap (_frames_ruler);
 	frames_ruler->set_name ("FramesRuler");
 	frames_ruler->set_size_request (-1, (int)timebar_height);
 	gtk_custom_ruler_set_metric (GTK_CUSTOM_RULER(_frames_ruler), &ruler_metrics[ruler_metric_frames]);
+	frames_ruler->hide ();
+	frames_ruler->set_no_show_all();
 
-	_minsec_ruler = gtk_custom_hruler_new ();
-	minsec_ruler = Glib::wrap (_minsec_ruler);
-	minsec_ruler->set_name ("MinSecRuler");
-	minsec_ruler->set_size_request (-1, (int)timebar_height);
-	gtk_custom_ruler_set_metric (GTK_CUSTOM_RULER(_minsec_ruler), &ruler_metrics[ruler_metric_minsec]);
+	_bbt_ruler = gtk_custom_hruler_new ();
+	bbt_ruler = Glib::wrap (_bbt_ruler);
+	bbt_ruler->set_name ("BBTRuler");
+	bbt_ruler->set_size_request (-1, (int)timebar_height);
+	gtk_custom_ruler_set_metric (GTK_CUSTOM_RULER(_bbt_ruler), &ruler_metrics[ruler_metric_bbt]);
+	bbt_ruler->hide ();
+	bbt_ruler->set_no_show_all();
 
-	visible_timebars = 7; /* 4 here, 3 in time_canvas */
+	using namespace Box_Helpers;
+	BoxList & ruler_lab_children =  ruler_label_vbox.children();
+	BoxList & ruler_children =  time_canvas_vbox.children();
+	BoxList & lab_children =  time_button_vbox.children();
+
+	BoxList::iterator canvaspos = ruler_children.begin();
+
+	lab_children.push_back (Element(meter_label, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(tempo_label, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(range_mark_label, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(transport_mark_label, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(cd_mark_label, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(mark_label, PACK_SHRINK, PACK_START));
+
+	ruler_lab_children.push_back (Element(minsec_label, PACK_SHRINK, PACK_START));
+	ruler_children.insert (canvaspos, Element(*minsec_ruler, PACK_SHRINK, PACK_START));
+	ruler_lab_children.push_back (Element(smpte_label, PACK_SHRINK, PACK_START));
+	ruler_children.insert (canvaspos, Element(*smpte_ruler, PACK_SHRINK, PACK_START));
+	ruler_lab_children.push_back (Element(frame_label, PACK_SHRINK, PACK_START));
+	ruler_children.insert (canvaspos, Element(*frames_ruler, PACK_SHRINK, PACK_START));
+	ruler_lab_children.push_back (Element(bbt_label, PACK_SHRINK, PACK_START));
+	ruler_children.insert (canvaspos, Element(*bbt_ruler, PACK_SHRINK, PACK_START));
+
+	smpte_ruler->add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK|Gdk::SCROLL_MASK);
+	bbt_ruler->add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK|Gdk::SCROLL_MASK);
+	frames_ruler->add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK|Gdk::SCROLL_MASK);
+	minsec_ruler->add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK|Gdk::SCROLL_MASK);
+
+	smpte_ruler->signal_button_release_event().connect (mem_fun(*this, &Editor::ruler_button_release));
+	bbt_ruler->signal_button_release_event().connect (mem_fun(*this, &Editor::ruler_button_release));
+	frames_ruler->signal_button_release_event().connect (mem_fun(*this, &Editor::ruler_button_release));
+	minsec_ruler->signal_button_release_event().connect (mem_fun(*this, &Editor::ruler_button_release));
+
+	smpte_ruler->signal_button_press_event().connect (mem_fun(*this, &Editor::ruler_button_press));
+	bbt_ruler->signal_button_press_event().connect (mem_fun(*this, &Editor::ruler_button_press));
+	frames_ruler->signal_button_press_event().connect (mem_fun(*this, &Editor::ruler_button_press));
+	minsec_ruler->signal_button_press_event().connect (mem_fun(*this, &Editor::ruler_button_press));
+	
+	smpte_ruler->signal_motion_notify_event().connect (mem_fun(*this, &Editor::ruler_mouse_motion));
+	bbt_ruler->signal_motion_notify_event().connect (mem_fun(*this, &Editor::ruler_mouse_motion));
+	frames_ruler->signal_motion_notify_event().connect (mem_fun(*this, &Editor::ruler_mouse_motion));
+	minsec_ruler->signal_motion_notify_event().connect (mem_fun(*this, &Editor::ruler_mouse_motion));
+
+	smpte_ruler->signal_scroll_event().connect (mem_fun(*this, &Editor::ruler_scroll));
+	bbt_ruler->signal_scroll_event().connect (mem_fun(*this, &Editor::ruler_scroll));
+	frames_ruler->signal_scroll_event().connect (mem_fun(*this, &Editor::ruler_scroll));
+	minsec_ruler->signal_scroll_event().connect (mem_fun(*this, &Editor::ruler_scroll));
+
+	visible_timebars = 0; /*this will be changed below */
 	ruler_pressed_button = 0;
+	canvas_timebars_vsize = 0;
 }
 
 bool
 Editor::ruler_scroll (GdkEventScroll* event)
 {
-	nframes_t xdelta;
+	nframes64_t xdelta;
 	int direction = event->direction;
 	bool handled = false;
 
@@ -167,7 +224,7 @@ Editor::ruler_button_press (GdkEventButton* ev)
 	/* need to use the correct x,y, the event lies */
 	time_canvas_event_box.get_window()->get_pointer (x, y, state);
 
-	nframes_t where = leftmost_frame + pixel_to_frame (x);
+	nframes64_t where = leftmost_frame + pixel_to_frame (x);
 
 	switch (ev->button) {
 	case 1:
@@ -211,7 +268,7 @@ Editor::ruler_button_release (GdkEventButton* ev)
 
 	stop_canvas_autoscroll();
 	
-	nframes_t where = leftmost_frame + pixel_to_frame (x);
+	nframes64_t where = leftmost_frame + pixel_to_frame (x);
 
 	switch (ev->button) {
 	case 1:
@@ -279,10 +336,10 @@ Editor::ruler_mouse_motion (GdkEventMotion* ev)
 	track_canvas->c2w (x, y, wcx, wcy);
 	track_canvas->w2c (wcx, wcy, cx, cy);
 	
-	nframes_t where = leftmost_frame + pixel_to_frame (x);
+	nframes64_t where = leftmost_frame + pixel_to_frame (x);
 
 	/// ripped from maybe_autoscroll, and adapted to work here
-	nframes_t rightmost_frame = leftmost_frame + current_page_frames ();
+	nframes64_t rightmost_frame = leftmost_frame + current_page_frames ();
 
 	if (autoscroll_timeout_tag < 0) {
 		if (where > rightmost_frame) {
@@ -333,7 +390,7 @@ Editor::ruler_mouse_motion (GdkEventMotion* ev)
 
 
 void
-Editor::popup_ruler_menu (nframes_t where, ItemType t)
+Editor::popup_ruler_menu (nframes64_t where, ItemType t)
 {
 	using namespace Menu_Helpers;
 
@@ -367,6 +424,8 @@ Editor::popup_ruler_menu (nframes_t where, ItemType t)
 	
 	case CdMarkerBarItem:
 		// TODO
+		ruler_items.push_back (MenuElem (_("Clear all locations"), mem_fun(*this, &Editor::clear_markers)));
+		ruler_items.push_back (MenuElem (_("Unhide locations"), mem_fun(*this, &Editor::unhide_markers)));
 		ruler_items.push_back (MenuElem (_("New CD track marker"), bind ( mem_fun(*this, &Editor::mouse_add_new_marker), where, true, false)));
 		break;
 		
@@ -469,66 +528,76 @@ Editor::restore_ruler_visibility ()
 
 	if (node) {
 		if ((prop = node->property ("smpte")) != 0) {
-			if (prop->value() == "yes") 
+			if (string_is_affirmative (prop->value())) {
 				ruler_timecode_action->set_active (true);
-			else 
+			} else {
 				ruler_timecode_action->set_active (false);
+			}
 		}
 		if ((prop = node->property ("bbt")) != 0) {
-			if (prop->value() == "yes") 
+			if (string_is_affirmative (prop->value())) {
 				ruler_bbt_action->set_active (true);
-			else 
+			} else {
 				ruler_bbt_action->set_active (false);
+			}
 		}
 		if ((prop = node->property ("frames")) != 0) {
-			if (prop->value() == "yes") 
+			if (string_is_affirmative (prop->value())) {
 				ruler_samples_action->set_active (true);
-			else 
+			} else {
 				ruler_samples_action->set_active (false);
+			}
 		}
 		if ((prop = node->property ("minsec")) != 0) {
-			if (prop->value() == "yes") 
+			if (string_is_affirmative (prop->value())) {
 				ruler_minsec_action->set_active (true);
-			else 
+			} else {
 				ruler_minsec_action->set_active (false);
+			}
 		}
 		if ((prop = node->property ("tempo")) != 0) {
-			if (prop->value() == "yes") 
+			if (string_is_affirmative (prop->value())) {
 				ruler_tempo_action->set_active (true);
-			else 
+			} else {
 				ruler_tempo_action->set_active (false);
+			}
 		}
 		if ((prop = node->property ("meter")) != 0) {
-			if (prop->value() == "yes") 
+			if (string_is_affirmative (prop->value())) {
 				ruler_meter_action->set_active (true);
-			else 
+			} else {
 				ruler_meter_action->set_active (false);
+			}
 		}
 		if ((prop = node->property ("marker")) != 0) {
-			if (prop->value() == "yes") 
+			if (string_is_affirmative (prop->value())) {
 				ruler_marker_action->set_active (true);
-			else 
+			} else {
 				ruler_marker_action->set_active (false);
+			}
 		}
 		if ((prop = node->property ("rangemarker")) != 0) {
-			if (prop->value() == "yes") 
+			if (string_is_affirmative (prop->value())) {
 				ruler_range_action->set_active (true);
-			else 
+			} else {
 				ruler_range_action->set_active (false);
+			}
 		}
 
 		if ((prop = node->property ("transportmarker")) != 0) {
-			if (prop->value() == "yes") 
+			if (string_is_affirmative (prop->value())) {
 				ruler_loop_punch_action->set_active (true);
-			else 
+			} else {
 				ruler_loop_punch_action->set_active (false);
+			}
 		}
 
 		if ((prop = node->property ("cdmarker")) != 0) {
-			if (prop->value() == "yes") 
+			if (string_is_affirmative (prop->value())) {
 				ruler_cd_marker_action->set_active (true);
-			else 
+			} else {
 				ruler_cd_marker_action->set_active (false);
+			}
 
 		} else {
 			// this session doesn't yet know about the cdmarker ruler
@@ -547,16 +616,13 @@ Editor::restore_ruler_visibility ()
 	}
 
 	no_ruler_shown_update = false;
-
 	update_ruler_visibility ();
 }
 
 void
 Editor::update_ruler_visibility ()
 {
-	using namespace Box_Helpers;
-	BoxList & lab_children =  time_button_vbox.children();
-	BoxList & ruler_children =  time_canvas_vbox.children();
+	int visible_rulers = 0;
 
 	if (no_ruler_shown_update) {
 		return;
@@ -564,190 +630,212 @@ Editor::update_ruler_visibility ()
 
 	visible_timebars = 0;
 
-	lab_children.clear();
-
-	// leave the last one (the time_canvas) intact
-	while (ruler_children.size() > 1) {
-		ruler_children.pop_front();
-	}
-
-	BoxList::iterator canvaspos = ruler_children.begin();
-	
-	_smpte_ruler = gtk_custom_hruler_new ();
-	smpte_ruler = Glib::wrap (_smpte_ruler);
-	smpte_ruler->set_name ("SMPTERuler");
-	smpte_ruler->set_size_request (-1, (int)timebar_height);
-	gtk_custom_ruler_set_metric (GTK_CUSTOM_RULER(_smpte_ruler), &ruler_metrics[ruler_metric_smpte]);
-	
-	_bbt_ruler = gtk_custom_hruler_new ();
-	bbt_ruler = Glib::wrap (_bbt_ruler);
-	bbt_ruler->set_name ("BBTRuler");
-	bbt_ruler->set_size_request (-1, (int)timebar_height);
-	gtk_custom_ruler_set_metric (GTK_CUSTOM_RULER(_bbt_ruler), &ruler_metrics[ruler_metric_bbt]);
-
-	_frames_ruler = gtk_custom_hruler_new ();
-	frames_ruler = Glib::wrap (_frames_ruler);
-	frames_ruler->set_name ("FramesRuler");
-	frames_ruler->set_size_request (-1, (int)timebar_height);
-	gtk_custom_ruler_set_metric (GTK_CUSTOM_RULER(_frames_ruler), &ruler_metrics[ruler_metric_frames]);
-
-	_minsec_ruler = gtk_custom_hruler_new ();
-	minsec_ruler = Glib::wrap (_minsec_ruler);
-	minsec_ruler->set_name ("MinSecRuler");
-	minsec_ruler->set_size_request (-1, (int)timebar_height);
-	gtk_custom_ruler_set_metric (GTK_CUSTOM_RULER(_minsec_ruler), &ruler_metrics[ruler_metric_minsec]);
-
-	smpte_ruler->add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK|Gdk::SCROLL_MASK);
-	bbt_ruler->add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK|Gdk::SCROLL_MASK);
-	frames_ruler->add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK|Gdk::SCROLL_MASK);
-	minsec_ruler->add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK|Gdk::SCROLL_MASK);
-
-	smpte_ruler->signal_button_release_event().connect (mem_fun(*this, &Editor::ruler_button_release));
-	bbt_ruler->signal_button_release_event().connect (mem_fun(*this, &Editor::ruler_button_release));
-	frames_ruler->signal_button_release_event().connect (mem_fun(*this, &Editor::ruler_button_release));
-	minsec_ruler->signal_button_release_event().connect (mem_fun(*this, &Editor::ruler_button_release));
-
-	smpte_ruler->signal_button_press_event().connect (mem_fun(*this, &Editor::ruler_button_press));
-	bbt_ruler->signal_button_press_event().connect (mem_fun(*this, &Editor::ruler_button_press));
-	frames_ruler->signal_button_press_event().connect (mem_fun(*this, &Editor::ruler_button_press));
-	minsec_ruler->signal_button_press_event().connect (mem_fun(*this, &Editor::ruler_button_press));
-	
-	smpte_ruler->signal_motion_notify_event().connect (mem_fun(*this, &Editor::ruler_mouse_motion));
-	bbt_ruler->signal_motion_notify_event().connect (mem_fun(*this, &Editor::ruler_mouse_motion));
-	frames_ruler->signal_motion_notify_event().connect (mem_fun(*this, &Editor::ruler_mouse_motion));
-	minsec_ruler->signal_motion_notify_event().connect (mem_fun(*this, &Editor::ruler_mouse_motion));
-
-	smpte_ruler->signal_scroll_event().connect (mem_fun(*this, &Editor::ruler_scroll));
-	bbt_ruler->signal_scroll_event().connect (mem_fun(*this, &Editor::ruler_scroll));
-	frames_ruler->signal_scroll_event().connect (mem_fun(*this, &Editor::ruler_scroll));
-	minsec_ruler->signal_scroll_event().connect (mem_fun(*this, &Editor::ruler_scroll));
-
-	ruler_children.insert (canvaspos, Element(*_ruler_separator, PACK_SHRINK, PACK_START));
-	
 	if (ruler_minsec_action->get_active()) {
-		lab_children.push_back (Element(minsec_label, PACK_SHRINK, PACK_START));
-		ruler_children.insert (canvaspos, Element(*minsec_ruler, PACK_SHRINK, PACK_START));
-		visible_timebars++;
+		visible_rulers++;
+		minsec_label.show ();
+		minsec_ruler->show ();
+	} else {
+		minsec_label.hide ();
+		minsec_ruler->hide ();
 	}
 
 	if (ruler_timecode_action->get_active()) {
-		lab_children.push_back (Element(smpte_label, PACK_SHRINK, PACK_START));
-		ruler_children.insert (canvaspos, Element(*smpte_ruler, PACK_SHRINK, PACK_START));
-		visible_timebars++;
+		visible_rulers++;
+		smpte_label.show ();
+		smpte_ruler->show ();
+	} else {
+		smpte_label.hide ();
+		smpte_ruler->hide ();
 	}
 
 	if (ruler_samples_action->get_active()) {
-		lab_children.push_back (Element(frame_label, PACK_SHRINK, PACK_START));
-		ruler_children.insert (canvaspos, Element(*frames_ruler, PACK_SHRINK, PACK_START));
-		visible_timebars++;
+		visible_rulers++;
+		frame_label.show ();
+		frames_ruler->show ();
+	} else {
+		frame_label.hide ();
+		frames_ruler->hide ();
 	}
 
 	if (ruler_bbt_action->get_active()) {
-		lab_children.push_back (Element(bbt_label, PACK_SHRINK, PACK_START));
-		ruler_children.insert (canvaspos, Element(*bbt_ruler, PACK_SHRINK, PACK_START));
-		visible_timebars++;
+		visible_rulers++;
+		bbt_label.show ();
+		bbt_ruler->show ();
+	} else {
+		bbt_label.hide ();
+		bbt_ruler->hide ();
 	}
 
-	double tbpos = 1.0;
-	double old_unit_pos ;
-	
-	if (ruler_meter_action->get_active()) {
-		lab_children.push_back (Element(meter_label, PACK_SHRINK, PACK_START));
+	double tbpos = 0.0;
+	double tbgpos = 0.0;
+	double old_unit_pos;
 
+#ifdef GTKOSX
+	/* gtk update probs require this (damn) */
+	meter_label.hide();
+	tempo_label.hide();
+	range_mark_label.hide();
+	transport_mark_label.hide();
+	cd_mark_label.hide();
+	mark_label.hide();
+#endif
+	if (ruler_meter_action->get_active()) {
 		old_unit_pos = meter_group->property_y();
 		if (tbpos != old_unit_pos) {
 			meter_group->move ( 0.0, tbpos - old_unit_pos);
 		}
+		old_unit_pos = meter_bar_group->property_y();
+		if (tbgpos != old_unit_pos) {
+			meter_bar_group->move ( 0.0, tbgpos - old_unit_pos);
+		}	
+		meter_bar_group->show();
 		meter_group->show();
+		meter_label.show();
 		tbpos += timebar_height;
+		tbgpos += timebar_height;
 		visible_timebars++;
-	}
-	else {
+	} else {
+		meter_bar_group->hide();
 		meter_group->hide();
+		meter_label.hide();
 	}
 	
 	if (ruler_tempo_action->get_active()) {
-		lab_children.push_back (Element(tempo_label, PACK_SHRINK, PACK_START));
 		old_unit_pos = tempo_group->property_y();
 		if (tbpos != old_unit_pos) {
 			tempo_group->move(0.0, tbpos - old_unit_pos);
 		}
+		old_unit_pos = tempo_bar_group->property_y();
+		if (tbgpos != old_unit_pos) {
+			tempo_bar_group->move ( 0.0, tbgpos - old_unit_pos);
+		}
+		tempo_bar_group->show();
 		tempo_group->show();
+		tempo_label.show();
 		tbpos += timebar_height;
+		tbgpos += timebar_height;
 		visible_timebars++;
-	}
-	else {
+	} else {
+		tempo_bar_group->hide();
 		tempo_group->hide();
+		tempo_label.hide();
 	}
 	
 	if (!Profile->get_sae() && ruler_range_action->get_active()) {
-		lab_children.push_back (Element(range_mark_label, PACK_SHRINK, PACK_START));
 		old_unit_pos = range_marker_group->property_y();
 		if (tbpos != old_unit_pos) {
 			range_marker_group->move (0.0, tbpos - old_unit_pos);
 		}
+		old_unit_pos = range_marker_bar_group->property_y();
+		if (tbgpos != old_unit_pos) {
+			range_marker_bar_group->move (0.0, tbgpos - old_unit_pos);
+		}
+		range_marker_bar_group->show();
 		range_marker_group->show();
+		range_mark_label.show();
 		tbpos += timebar_height;
+		tbgpos += timebar_height;
 		visible_timebars++;
 	} else {
+		range_marker_bar_group->hide();
 		range_marker_group->hide();
+		range_mark_label.hide();
 	}
 
 	if (ruler_loop_punch_action->get_active()) {
-		lab_children.push_back (Element(transport_mark_label, PACK_SHRINK, PACK_START));
 		old_unit_pos = transport_marker_group->property_y();
 		if (tbpos != old_unit_pos) {
 			transport_marker_group->move ( 0.0, tbpos - old_unit_pos);
 		}
+		old_unit_pos = transport_marker_bar_group->property_y();
+		if (tbgpos != old_unit_pos) {
+			transport_marker_bar_group->move ( 0.0, tbgpos - old_unit_pos);
+		}
+		transport_marker_bar_group->show();
 		transport_marker_group->show();
+		transport_mark_label.show();
 		tbpos += timebar_height;
+		tbgpos += timebar_height;
 		visible_timebars++;
-	}
-	else {
+	} else {
+		transport_marker_bar_group->hide();
 		transport_marker_group->hide();
+		transport_mark_label.hide();
 	}
 
 	if (ruler_cd_marker_action->get_active()) {
-		lab_children.push_back (Element(cd_mark_label, PACK_SHRINK, PACK_START));
 		old_unit_pos = cd_marker_group->property_y();
 		if (tbpos != old_unit_pos) {
 			cd_marker_group->move (0.0, tbpos - old_unit_pos);
 		}
+		old_unit_pos = cd_marker_bar_group->property_y();
+		if (tbgpos != old_unit_pos) {
+			cd_marker_bar_group->move (0.0, tbgpos - old_unit_pos);
+		}
+		cd_marker_bar_group->show();
 		cd_marker_group->show();
+		cd_mark_label.show();
 		tbpos += timebar_height;
+		tbgpos += timebar_height;
 		visible_timebars++;
 		// make sure all cd markers show up in their respective places
 		update_cd_marker_display();
-	}
-	else {
+	} else {
+		cd_marker_bar_group->hide();
 		cd_marker_group->hide();
+		cd_mark_label.hide();
 		// make sure all cd markers show up in their respective places
 		update_cd_marker_display();
 	}
 	
 	if (ruler_marker_action->get_active()) {
-		lab_children.push_back (Element(mark_label, PACK_SHRINK, PACK_START));
 		old_unit_pos = marker_group->property_y();
 		if (tbpos != old_unit_pos) {
 			marker_group->move ( 0.0, tbpos - old_unit_pos);
 		}
+		old_unit_pos = marker_bar_group->property_y();
+		if (tbgpos != old_unit_pos) {
+			marker_bar_group->move ( 0.0, tbgpos - old_unit_pos);
+		}
+		marker_bar_group->show();
 		marker_group->show();
+		mark_label.show();
 		tbpos += timebar_height;
+		tbgpos += timebar_height;
 		visible_timebars++;
-	}
-	else {
+	} else {
+		marker_bar_group->hide();
 		marker_group->hide();
+		mark_label.hide();
+	}
+
+	gdouble old_canvas_timebars_vsize = canvas_timebars_vsize;
+	canvas_timebars_vsize = (timebar_height * visible_timebars) - 1;
+	gdouble vertical_pos_delta = canvas_timebars_vsize - old_canvas_timebars_vsize;
+	vertical_adjustment.set_upper(vertical_adjustment.get_upper() + vertical_pos_delta);
+	full_canvas_height += vertical_pos_delta;
+
+	if (vertical_adjustment.get_value() != 0 && (vertical_adjustment.get_value() + canvas_height >= full_canvas_height)) {
+		/*if we're at the bottom of the canvas, don't move the _trackview_group*/
+		vertical_adjustment.set_value (full_canvas_height - canvas_height + 1);
+	} else {
+		_trackview_group->property_y () = - get_trackview_group_vertical_offset ();
+		_background_group->property_y () = - get_trackview_group_vertical_offset ();
+		_trackview_group->move (0, 0);
+		_background_group->move (0, 0);
+		last_trackview_group_vertical_offset = get_trackview_group_vertical_offset ();
 	}
 	
-	time_canvas_vbox.set_size_request (-1, (int)(timebar_height * visible_timebars));
-	time_canvas_event_box.queue_resize();
-	
-	update_fixed_rulers();
-	//update_tempo_based_rulers();
-	redisplay_tempo (false);
+	gdouble bottom_track_pos = vertical_adjustment.get_value() + canvas_height - canvas_timebars_vsize;
+	if (trackview_by_y_position(bottom_track_pos) != 0) {
+		trackview_by_y_position(bottom_track_pos)->clip_to_viewport ();
+	}
 
-	time_canvas_event_box.show_all();
-	time_button_frame.show_all();
+	ruler_label_vbox.set_size_request (-1, (int)(timebar_height * visible_rulers));
+	time_canvas_vbox.set_size_request (-1,-1);
+
+	update_fixed_rulers();
+	redisplay_tempo (false);
 }
 
 void
@@ -759,7 +847,7 @@ Editor::update_just_smpte ()
 		return;
 	}
 
-	nframes_t rightmost_frame = leftmost_frame + current_page_frames();
+	nframes64_t rightmost_frame = leftmost_frame + current_page_frames();
 
 	if (ruler_timecode_action->get_active()) {
 		gtk_custom_ruler_set_range (GTK_CUSTOM_RULER(_smpte_ruler), leftmost_frame, rightmost_frame,
@@ -770,7 +858,7 @@ Editor::update_just_smpte ()
 void
 Editor::update_fixed_rulers ()
 {
-	nframes_t rightmost_frame;
+	nframes64_t rightmost_frame;
 
 	if (session == 0) {
 		return;
@@ -846,10 +934,10 @@ Editor::_metric_get_minsec (GtkCustomRulerMark **marks, gdouble lower, gdouble u
 gint
 Editor::metric_get_smpte (GtkCustomRulerMark **marks, gdouble lower, gdouble upper, gint maxchars)
 {
-	nframes_t range;
 	nframes_t pos;
-	nframes_t spacer;
-	nframes_t fr;
+	nframes64_t range;
+	nframes64_t spacer;
+	nframes64_t fr;
 	SMPTE::Time smpte;
 	gchar buf[16];
 	gint nmarks = 0;
@@ -867,13 +955,13 @@ Editor::metric_get_smpte (GtkCustomRulerMark **marks, gdouble lower, gdouble upp
 
 	fr = session->frame_rate();
 
-	if (lower > (spacer = (nframes_t)(128 * Editor::get_current_zoom ()))) {
+	if (lower > (spacer = (nframes64_t)(128 * Editor::get_current_zoom ()))) {
 		lower = lower - spacer;
 	} else {
 		lower = 0;
 	}
 	upper = upper + spacer;
-	range = (nframes_t) floor (upper - lower);
+	range = (nframes64_t) floor (upper - lower);
 
 	if (range < (2 * session->frames_per_smpte_frame())) { /* 0 - 2 frames */
 		show_bits = true;
@@ -882,19 +970,19 @@ Editor::metric_get_smpte (GtkCustomRulerMark **marks, gdouble lower, gdouble upp
 	} else if (range <= (fr / 4)) { /* 2 frames - 0.250 second */
 		show_frames = true;
 		mark_modulo = 1;
-		nmarks = 1 + (range / (nframes_t)session->frames_per_smpte_frame());
+		nmarks = 1 + (range / (nframes64_t)session->frames_per_smpte_frame());
 	} else if (range <= (fr / 2)) { /* 0.25-0.5 second */
 		show_frames = true;
 		mark_modulo = 2;
-		nmarks = 1 + (range / (nframes_t)session->frames_per_smpte_frame());
+		nmarks = 1 + (range / (nframes64_t)session->frames_per_smpte_frame());
 	} else if (range <= fr) { /* 0.5-1 second */
 		show_frames = true;
 		mark_modulo = 5;
-		nmarks = 1 + (range / (nframes_t)session->frames_per_smpte_frame());
+		nmarks = 1 + (range / (nframes64_t)session->frames_per_smpte_frame());
 	} else if (range <= 2 * fr) { /* 1-2 seconds */
 		show_frames = true;
 		mark_modulo = 10;
-		nmarks = 1 + (range / (nframes_t)session->frames_per_smpte_frame());
+		nmarks = 1 + (range / (nframes64_t)session->frames_per_smpte_frame());
 	} else if (range <= 8 * fr) { /* 2-8 seconds */
 		show_seconds = true;
 		mark_modulo = 1;
@@ -945,7 +1033,7 @@ Editor::metric_get_smpte (GtkCustomRulerMark **marks, gdouble lower, gdouble upp
 		nmarks = 1 + 24;
 	} else {
     
-		/* not possible if nframes_t is a 32 bit quantity */
+		/* not possible if nframes64_t is a 32 bit quantity */
     
 		show_hours = true;
 		mark_modulo = 4;
@@ -1095,11 +1183,11 @@ Editor::metric_get_bbt (GtkCustomRulerMark **marks, gdouble lower, gdouble upper
 	gint nmarks;
         char buf[64];
         gint  n = 0;
-	nframes_t pos;
+	nframes64_t pos;
 	bool bar_helper_on = true;
        
 	BBT_Time next_beat;
-	nframes_t next_beat_pos;
+	nframes64_t next_beat_pos;
 
       	if ((desirable_marks = maxchars / 7) == 0) {
                return 0;
@@ -1152,7 +1240,7 @@ Editor::metric_get_bbt (GtkCustomRulerMark **marks, gdouble lower, gdouble upper
 		uint32_t tick = 0;
 		uint32_t skip;
 		uint32_t t;
-	        nframes_t frame_skip;
+	        nframes64_t frame_skip;
 		double frame_skip_error;
 		double accumulated_error;
 		double position_of_helper;
@@ -1228,7 +1316,7 @@ Editor::metric_get_bbt (GtkCustomRulerMark **marks, gdouble lower, gdouble upper
 				
 				next_beat_pos = session->tempo_map().frame_time(next_beat);
 
-      			        frame_skip = (nframes_t) floor (frame_skip_error = (session->frame_rate() *  60) / (bbt_beat_subdivision * (*i).tempo->beats_per_minute()));
+      			        frame_skip = (nframes64_t) floor (frame_skip_error = (session->frame_rate() *  60) / (bbt_beat_subdivision * (*i).tempo->beats_per_minute()));
 			        frame_skip_error -= frame_skip;
 			        skip = (uint32_t) (Meter::ticks_per_beat / bbt_beat_subdivision);
 
@@ -1254,7 +1342,7 @@ Editor::metric_get_bbt (GtkCustomRulerMark **marks, gdouble lower, gdouble upper
 
 					(*marks)[n].label = g_strdup (buf);
 
-					/* Error compensation for float to nframes_t*/
+					/* Error compensation for float to nframes64_t*/
 					accumulated_error += frame_skip_error;
 				        if (accumulated_error > 1) {
 					        pos += 1;
@@ -1382,10 +1470,10 @@ Editor::metric_get_bbt (GtkCustomRulerMark **marks, gdouble lower, gdouble upper
 gint
 Editor::metric_get_frames (GtkCustomRulerMark **marks, gdouble lower, gdouble upper, gint maxchars)
 {
-	nframes_t mark_interval;
-	nframes_t pos;
-	nframes_t ilower = (nframes_t) floor (lower);
-	nframes_t iupper = (nframes_t) floor (upper);
+	nframes64_t mark_interval;
+	nframes64_t pos;
+	nframes64_t ilower = (nframes64_t) floor (lower);
+	nframes64_t iupper = (nframes64_t) floor (upper);
 	gchar buf[16];
 	gint nmarks;
 	gint n;
@@ -1403,7 +1491,7 @@ Editor::metric_get_frames (GtkCustomRulerMark **marks, gdouble lower, gdouble up
 	nmarks = 5;
 	*marks = (GtkCustomRulerMark *) g_malloc (sizeof(GtkCustomRulerMark) * nmarks);
 	for (n = 0, pos = ilower; n < nmarks; pos += mark_interval, ++n) {
-		snprintf (buf, sizeof(buf), "%u", pos);
+		snprintf (buf, sizeof(buf), "%" PRIi64,  pos);
 		(*marks)[n].label = g_strdup (buf);
 		(*marks)[n].position = pos;
 		(*marks)[n].style = GtkCustomRulerMarkMajor;
@@ -1413,15 +1501,15 @@ Editor::metric_get_frames (GtkCustomRulerMark **marks, gdouble lower, gdouble up
 }
 
 static void
-sample_to_clock_parts ( nframes_t sample,
-			nframes_t sample_rate, 
+sample_to_clock_parts ( nframes64_t sample,
+			nframes64_t sample_rate, 
 			long *hrs_p,
 			long *mins_p,
 			long *secs_p,
 			long *millisecs_p)
 
 {
-	nframes_t left;
+	nframes64_t left;
 	long hrs;
 	long mins;
 	long secs;
@@ -1447,11 +1535,11 @@ sample_to_clock_parts ( nframes_t sample,
 gint
 Editor::metric_get_minsec (GtkCustomRulerMark **marks, gdouble lower, gdouble upper, gint maxchars)
 {
-	nframes_t range;
-	nframes_t fr;
-	nframes_t mark_interval;
-	nframes_t pos;
-	nframes_t spacer;
+	nframes64_t range;
+	nframes64_t fr;
+	nframes64_t mark_interval;
+	nframes64_t pos;
+	nframes64_t spacer;
 	long hrs, mins, secs, millisecs;
 	gchar buf[16];
 	gint nmarks;
@@ -1460,8 +1548,8 @@ Editor::metric_get_minsec (GtkCustomRulerMark **marks, gdouble lower, gdouble up
 	bool show_seconds = false;
 	bool show_minutes = false;
 	bool show_hours = false;
-	nframes_t ilower = (nframes_t) floor (lower);
-	nframes_t iupper = (nframes_t) floor (upper);
+	nframes64_t ilower = (nframes64_t) floor (lower);
+	nframes64_t iupper = (nframes64_t) floor (upper);
 
 	if (session == 0) {
 		return 0;
@@ -1470,7 +1558,7 @@ Editor::metric_get_minsec (GtkCustomRulerMark **marks, gdouble lower, gdouble up
 	fr = session->frame_rate();
 
 	/* to prevent 'flashing' */
-	if (lower > (spacer = (nframes_t)(128 * Editor::get_current_zoom ()))) {
+	if (lower > (spacer = (nframes64_t)(128 * Editor::get_current_zoom ()))) {
 		lower = lower - spacer;
 	} else {
 		lower = 0;
@@ -1542,7 +1630,7 @@ Editor::metric_get_minsec (GtkCustomRulerMark **marks, gdouble lower, gdouble up
 		mark_modulo = 2;
         } else {
                                                                                                                    
-                /* not possible if nframes_t is a 32 bit quantity */
+                /* not possible if nframes64_t is a 32 bit quantity */
                                                                                                                    
                 mark_interval = 4 * 60 * 60 * fr; /* show 4 hrs */
         }
