@@ -18,7 +18,7 @@
 */
 
 #include <algorithm>
-#include <stdlib.h>
+#include <cstdlib>
 
 #include <pbd/stacktrace.h>
 
@@ -160,24 +160,91 @@ Editor::extend_selection_to_track (TimeAxisView& view)
 void
 Editor::select_all_tracks ()
 {
-	selection->set (track_views);
+	TrackViewList visible_views;
+	for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
+		if ((*i)->marked_for_display()) {
+			visible_views.push_back (*i);
+		}
+	}
+	selection->set (visible_views);
 }
 
 void
-Editor::set_selected_track_as_side_effect (bool force)
+Editor::set_selected_track_as_side_effect (Selection::Operation op, bool force)
 {
 	if (!clicked_trackview) {
 		return;
 	}
 
-	if (!selection->tracks.empty()) {
-
-		if (!selection->selected (clicked_trackview)) {
-			selection->add (clicked_trackview);
-		}
-
-	} else if (force) {
+	 if (force) {
 		selection->set (clicked_trackview);
+		return;
+	}
+	
+	AudioTimeAxisView* atv = dynamic_cast<AudioTimeAxisView*>(clicked_trackview);
+	if (!atv) {
+		return;
+	}
+	RouteGroup* group = atv->route()->edit_group();
+
+	switch (op) {
+	case Selection::Toggle: {
+		if (selection->selected (clicked_trackview)) {
+			if (all_group_is_active) {
+				for (TrackViewList::iterator i = track_views.begin(); i != track_views.end (); ++i) {
+						selection->remove(*i);
+				}
+			} else if (group && group->is_active()) {
+				for (TrackViewList::iterator i = track_views.begin(); i != track_views.end (); ++i) {
+					if ( (*i)->edit_group() == group)
+						selection->remove(*i);
+				}
+			} else
+				selection->remove (clicked_trackview);
+		} else {
+			if (all_group_is_active) {
+				for (TrackViewList::iterator i = track_views.begin(); i != track_views.end (); ++i) {
+						selection->add(*i);
+				}
+			} else if (group && group->is_active())
+				for (TrackViewList::iterator i = track_views.begin(); i != track_views.end (); ++i) {
+					if ( (*i)->edit_group() == group)
+						selection->add(*i);
+				}
+			else
+				selection->add (clicked_trackview);
+		}
+	}
+	break;
+	
+	case Selection::Add: {
+		selection->clear();
+		cerr << ("Editor::set_selected_track_as_side_effect  case  Selection::Add  not yet implemented\n");
+	}
+	break;
+		
+	case Selection::Set:{
+		selection->clear();
+		if (all_group_is_active) {
+			for (TrackViewList::iterator i = track_views.begin(); i != track_views.end (); ++i) {
+					selection->add(*i);
+			}
+		} else if (group && group->is_active())
+			for (TrackViewList::iterator i  = track_views.begin(); i != track_views.end (); ++i) {
+				if ( (*i)->edit_group() == group)
+					selection->add(*i);
+			}
+		else
+			selection->set (clicked_trackview);
+	}
+	break;
+	
+	case Selection::Extend: {
+		selection->clear();
+		cerr << ("Editor::set_selected_track_as_side_effect  case  Selection::Add  not yet implemented\n");
+	}
+	break;
+	
 	}
 }
 
@@ -235,7 +302,7 @@ Editor::set_selected_control_point_from_click (Selection::Operation op, bool no_
 	/* select this point and any others that it represents */
 
 	double y1, y2;
-	nframes_t x1, x2;
+	nframes64_t x1, x2;
 
 	x1 = pixel_to_frame (clicked_control_point->get_x() - 10);
 	x2 = pixel_to_frame (clicked_control_point->get_x() + 10);
@@ -245,6 +312,16 @@ Editor::set_selected_control_point_from_click (Selection::Operation op, bool no_
 	return select_all_within (x1, x2, y1, y2, selection->tracks, op);
 }
 
+void
+Editor::get_onscreen_tracks (TrackViewList& tvl)
+{
+	for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
+		if ((*i)->y_position < canvas_height) {
+			tvl.push_back (*i);
+		}
+	}
+}
+
 void
 Editor::get_relevant_audio_tracks (set<AudioTimeAxisView*>& relevant_tracks)
 {
@@ -260,7 +337,14 @@ Editor::get_relevant_audio_tracks (set<AudioTimeAxisView*>& relevant_tracks)
 
 		RouteGroup* group = atv->route()->edit_group();
 
-		if (group && group->is_active()) {
+		if ( all_group_is_active ) {
+			for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
+				AudioTimeAxisView* tatv;
+				if ((tatv = dynamic_cast<AudioTimeAxisView*> (*i)) != 0) {
+					relevant_tracks.insert (tatv);
+				}
+			}
+		} else if (group && group->is_active()) {
 			
 			/* active group for this track, loop over all tracks and get every member of the group */
 
@@ -306,7 +390,7 @@ Editor::mapover_audio_tracks (slot<void,AudioTimeAxisView&,uint32_t> sl, TimeAxi
 	tracks.insert (audio_basis);
 
 	RouteGroup* group = audio_basis->route()->edit_group();
-	if (group && group->is_active()) {
+	if ( all_group_is_active || (group && group->is_active()) ) {
 
 		/* the basis is a member of an active edit group; find other members */
 		for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
@@ -452,8 +536,8 @@ Editor::set_selected_regionview_from_click (bool press, Selection::Operation op,
 	} else if (op == Selection::Extend) {
 
 		list<Selectable*> results;
-		nframes_t last_frame;
-		nframes_t first_frame;
+		nframes64_t last_frame;
+		nframes64_t first_frame;
 		bool same_track = false;
 
 		/* 1. find the last selected regionview in the track that was clicked in */
@@ -745,11 +829,7 @@ Editor::track_selection_changed ()
 	}
 
 	for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
-		if (find (selection->tracks.begin(), selection->tracks.end(), *i) != selection->tracks.end()) {
-			(*i)->set_selected (true);
-		} else {
-			(*i)->set_selected (false);
-		}
+			(*i)->show_selected (*selection);
 	}
 
 	ActionManager::set_sensitive (ActionManager::track_selection_sensitive_actions, !selection->tracks.empty());
@@ -816,8 +896,6 @@ Editor::region_selection_changed ()
 	}
 	
 	sensitize_the_right_region_actions (!selection->regions.empty());
-
-	zoomed_to_region = false;
 }
 
 void
@@ -913,7 +991,7 @@ Editor::invert_selection ()
 }
 
 bool
-Editor::select_all_within (nframes_t start, nframes_t end, double top, double bot, const TrackViewList& tracklist, Selection::Operation op)
+Editor::select_all_within (nframes64_t start, nframes64_t end, double top, double bot, const TrackViewList& tracklist, Selection::Operation op)
 {
 	list<Selectable*> touched;
 	list<Selectable*>::size_type n = 0;
@@ -1033,8 +1111,8 @@ Editor::select_all_selectables_using_time_selection ()
 		return;
 	}
 
-	nframes_t start = selection->time[clicked_selection].start;
-	nframes_t end = selection->time[clicked_selection].end;
+	nframes64_t start = selection->time[clicked_selection].start;
+	nframes64_t end = selection->time[clicked_selection].end;
 
 	if (end - start < 1)  {
 		return;
@@ -1126,8 +1204,8 @@ Editor::select_all_selectables_using_loop()
 void
 Editor::select_all_selectables_using_cursor (Cursor *cursor, bool after)
 {
-        nframes_t start;
-	nframes_t end;
+        nframes64_t start;
+	nframes64_t end;
 	list<Selectable *> touched;
 
 	if (after) {
@@ -1166,8 +1244,8 @@ Editor::select_all_selectables_using_cursor (Cursor *cursor, bool after)
 void
 Editor::select_all_selectables_using_edit (bool after)
 {
-        nframes_t start;
-	nframes_t end;
+        nframes64_t start;
+	nframes64_t end;
 	list<Selectable *> touched;
 
 	if (after) {
@@ -1237,7 +1315,11 @@ Editor::select_range_between ()
 {
         nframes64_t start;
 	nframes64_t end;
-	
+
+        if (mouse_mode == MouseRange && !selection->time.empty()) {
+                selection->clear_time ();
+        }
+
 	if (!get_edit_op_range (start, end)) {
 		return;
 	}
