@@ -73,6 +73,7 @@ EngineControl::EngineControl ()
 	int row = 0;
 
 	_used = false;
+	_interface_chosen = false;
 
 	strings.push_back (_("8000Hz"));
 	strings.push_back (_("22050Hz"));
@@ -215,6 +216,7 @@ EngineControl::EngineControl ()
 	options_packer.attach (realtime_button, 1, 2, row, row + 1, FILL|EXPAND, (AttachOptions) 0);
 	++row;
 
+	realtime_button.set_active (true); /* RT is active by default */
 	realtime_button.signal_toggled().connect (mem_fun (*this, &EngineControl::realtime_changed));
 	realtime_changed ();
 
@@ -276,23 +278,8 @@ EngineControl::EngineControl ()
 	++row;
 #endif
 
-	find_jack_servers (server_strings);
-
-	if (server_strings.empty()) {
-		fatal << _("No JACK server found anywhere on this system. Please install JACK and restart") << endmsg;
-		/*NOTREACHED*/
-	}
-	
-	set_popdown_strings (serverpath_combo, server_strings);
-	serverpath_combo.set_active_text (server_strings.front());
-
-	if (server_strings.size() > 1) {
-		label = manage (new Label (_("Server:")));
-		options_packer.attach (*label, 0, 1, row, row + 1, FILL|EXPAND, (AttachOptions) 0);
-		label->set_alignment (0.0, 0.5);
-		options_packer.attach (serverpath_combo, 1, 2, row, row + 1, FILL|EXPAND, (AttachOptions) 0);
-		++row;
-	}
+	/* defer server stuff till later */
+	server_row = row++;
 
 	/* device settings */
 
@@ -353,6 +340,27 @@ EngineControl::~EngineControl ()
 
 }
 
+void
+EngineControl::discover_servers ()
+{
+	find_jack_servers (server_strings);
+
+	if (server_strings.empty()) {
+		fatal << _("No JACK server found anywhere on this system. Please install JACK and restart") << endmsg;
+		/*NOTREACHED*/
+	}
+	
+	set_popdown_strings (serverpath_combo, server_strings);
+	serverpath_combo.set_active_text (server_strings.front());
+
+	if (server_strings.size() > 1) {
+		Gtk::Label* label = manage (new Label (_("Server:")));
+		options_packer.attach (*label, 0, 1, server_row, server_row + 1, FILL|EXPAND, (AttachOptions) 0);
+		label->set_alignment (0.0, 0.5);
+		options_packer.attach (serverpath_combo, 1, 2, server_row, server_row + 1, FILL|EXPAND, (AttachOptions) 0);
+	}
+}
+
 void
 EngineControl::build_command_line (vector<string>& cmd)
 {
@@ -375,7 +383,7 @@ EngineControl::build_command_line (vector<string>& cmd)
 	if (str != _("Ignore")) {
 		double secs;
 		uint32_t msecs;
-		atof (str);
+		secs = atof (str);
 		msecs = (uint32_t) floor (secs * 1000.0);
 		cmd.push_back ("-t");
 		cmd.push_back (to_string (msecs, std::dec));
@@ -452,11 +460,19 @@ EngineControl::build_command_line (vector<string>& cmd)
 			
 		} else if (str == _("Playback/Recording on 2 Devices")) {
 			
+			string input_device = get_device_name (driver, input_device_combo.get_active_text());
+			string output_device = get_device_name (driver, output_device_combo.get_active_text());
+
+			if (input_device.empty() || output_device.empty()) {
+				cmd.clear ();
+				return;
+			}
+
 			cmd.push_back ("-C");
-			cmd.push_back (get_device_name (driver, input_device_combo.get_active_text()));
+			cmd.push_back (input_device);
 			cmd.push_back ("-P");
-			cmd.push_back (get_device_name (driver, output_device_combo.get_active_text()));
-			
+			cmd.push_back (output_device);
+
 		} else if (str == _("Playback only")) {
 			cmd.push_back ("-P");
 		} else if (str == _("Recording only")) {
@@ -478,8 +494,16 @@ EngineControl::build_command_line (vector<string>& cmd)
 	if (using_alsa) {
 		
 		if (audio_mode_combo.get_active_text() != _("Playback/Recording on 2 Devices")) {
+
+			string device = get_device_name (driver, interface_combo.get_active_text());
+			if (device.empty()) {
+				cmd.clear ();
+				return;
+			}
+
 			cmd.push_back ("-d");
-			cmd.push_back (get_device_name (driver, interface_combo.get_active_text()));
+			cmd.push_back (device);
+			_interface_chosen = true;
 		} 
 
 		if (hw_meter_button.get_active()) {
@@ -513,8 +537,15 @@ EngineControl::build_command_line (vector<string>& cmd)
 
 #ifdef __APPLE__
 		// note: older versions of the CoreAudio JACK backend use -n instead of -d here
+		
+		string device = get_device_name (driver, interface_combo.get_active_text());
+		if (device.empty()) {
+			cmd.clear ();
+			return;
+		}
+
 		cmd.push_back ("-d");
-		cmd.push_back (get_device_name (driver, interface_combo.get_active_text()));
+		cmd.push_back (device);
 #endif
 
 	} else if (using_oss) {
@@ -544,6 +575,10 @@ EngineControl::setup_engine ()
 	std::string cwd = "/tmp";
 
 	build_command_line (args);
+	
+	if (args.empty()) {
+		return 1; // try again
+	}
 
 	Glib::ustring jackdrc_path = Glib::get_home_dir();
 	jackdrc_path += "/.jackdrc";
@@ -553,10 +588,12 @@ EngineControl::setup_engine ()
 		error << string_compose (_("cannot open JACK rc file %1 to store parameters"), jackdrc_path) << endmsg;
 		return -1;
 	}
-
+	cerr << "JACK COMMAND: ";
 	for (vector<string>::iterator i = args.begin(); i != args.end(); ++i) {
+		cerr << (*i) << ' ';
 		jackdrc << (*i) << ' ';
 	}
+	cerr << endl;
 	jackdrc << endl;
 	jackdrc.close ();
 
@@ -813,7 +850,7 @@ EngineControl::driver_changed ()
 
 	vector<string>& strings = devices[driver];
 
-	if (strings.empty()) {
+	if (strings.empty() && driver != "FFADO" && driver != "Dummy") {
 		error << string_compose (_("No devices found for driver \"%1\""), driver) << endmsg;
 		return;
 	}
@@ -833,7 +870,7 @@ EngineControl::driver_changed ()
 		interface_combo.set_active_text (strings.front());
 		input_device_combo.set_active_text (strings.front());
 		output_device_combo.set_active_text (strings.front());
-	}
+	} 
 	
 	if (driver == "ALSA") {
 		soft_mode_button.set_sensitive (true);
@@ -909,7 +946,7 @@ EngineControl::find_jack_servers (vector<string>& strings)
 
 	_NSGetExecutablePath (execpath, &pathsz);
 	
-	Glib::ustring path (Glib::path_get_dirname (execpath));
+	string path (Glib::path_get_dirname (execpath));
 	path += "/jackd";
 
 	if (Glib::file_test (path, FILE_TEST_EXISTS)) {
@@ -919,7 +956,7 @@ EngineControl::find_jack_servers (vector<string>& strings)
 	if (getenv ("ARDOUR_WITH_JACK")) {
 		/* no other options - only use the JACK we supply */
 		if (strings.empty()) {
-			fatal << _("JACK appears to be missing from the Ardour bundle") << endmsg;
+			fatal << string_compose (_("JACK appears to be missing from the %1 bundle"), PROGRAM_NAME) << endmsg;
 			/*NOTREACHED*/
 		}
 		return;
@@ -931,8 +968,36 @@ EngineControl::find_jack_servers (vector<string>& strings)
 	PathScanner scanner;
 	vector<string *> *jack_servers;
 	std::map<string,int> un;
-	
-	path = getenv ("PATH");
+	char *p;
+	bool need_minimal_path = false;
+
+	p = getenv ("PATH");
+
+	if (p && *p) {
+		path = p;
+	} else {
+		need_minimal_path = true;
+	}
+
+#ifdef __APPLE__
+	// many mac users don't have PATH set up to include
+	// likely installed locations of JACK
+	need_minimal_path = true;
+#endif
+
+	if (need_minimal_path) {
+		if (path.empty()) {
+			path = "/usr/bin:/bin:/usr/local/bin:/opt/local/bin";
+		} else {
+			path += ":/usr/local/bin:/opt/local/bin";
+		}
+	}
+
+#ifdef __APPLE__
+	// push it back into the environment so that auto-started JACK can find it.
+	// XXX why can't we just expect OS X users to have PATH set correctly? we can't ...
+	setenv ("PATH", path.c_str(), 1);
+#endif
 
 	jack_servers = scanner (path, jack_server_filter, 0, false, true);
 	
@@ -953,6 +1018,15 @@ EngineControl::get_device_name (const string& driver, const string& human_readab
 	vector<string>::iterator n;
 	vector<string>::iterator i;
 
+	if (human_readable.empty()) {
+		/* this can happen if the user's .ardourrc file has a device name from
+		   another computer system in it
+		*/
+		MessageDialog msg (_("You need to choose an audio device first."));
+		msg.run ();
+		return string();
+	}
+
 	if (backend_devs.empty()) {
 		return human_readable;
 	}
@@ -964,12 +1038,9 @@ EngineControl::get_device_name (const string& driver, const string& human_readab
 	}
 	
 	if (i == devices[driver].end()) {
-		fatal << string_compose (_("programming error: %1"), "true hardware name for ID missing") << endmsg;
-		/*NOTREACHED*/
+		warning << string_compose (_("Audio device \"%1\" not known on this computer."), human_readable) << endmsg;
 	}
 
-	/* keep gcc happy */
-
 	return string();
 }
 
@@ -1089,7 +1160,7 @@ EngineControl::set_state (const XMLNode& root)
 	XMLNodeList          clist;
 	XMLNodeConstIterator citer;
 	XMLNode* child;
-	XMLProperty* prop;
+	XMLProperty* prop = NULL;
 	bool using_dummy = false;
 	
 	int val;
@@ -1112,8 +1183,10 @@ EngineControl::set_state (const XMLNode& root)
 
 		if (!prop || prop->value().empty()) {
 
-			if ( using_dummy && ( child->name() == "interface" || child->name() == "inputdevice" || child->name() == "outputdevice" ))
+			if (using_dummy && ( child->name() == "interface" || child->name() == "inputdevice" || child->name() == "outputdevice" )) {
 				continue;
+			}
+
 			error << string_compose (_("AudioSetup value for %1 is missing data"), child->name()) << endmsg;
 			continue;
 		}
@@ -1180,28 +1253,42 @@ EngineControl::set_state (const XMLNode& root)
 		} else if (child->name() == "periodsize") {
 			period_size_combo.set_active_text(strval);
 		} else if (child->name() == "serverpath") {
-			/* do not allow us to use a server path that doesn't
-			   exist on this system. this handles cases where
-			   the user has an RC file listing a serverpath
-			   from some other machine.
+
+			/* only attempt to set this if we have bothered to look
+			   up server names already. otherwise this is all
+			   redundant (actually, all of this dialog/widget 
+			   is redundant in that case ...)
 			*/
-			vector<string>::iterator x;
-			for (x = server_strings.begin(); x != server_strings.end(); ++x) {
-				if (*x == strval) {
-					break;
+		       
+
+			if (!server_strings.empty()) {
+				/* do not allow us to use a server path that doesn't
+				   exist on this system. this handles cases where
+				   the user has an RC file listing a serverpath
+				   from some other machine.
+				*/
+				vector<string>::iterator x;
+				for (x = server_strings.begin(); x != server_strings.end(); ++x) {
+					if (*x == strval) {
+						break;
+					}
+				}
+				if (x != server_strings.end()) {
+					serverpath_combo.set_active_text (strval);
+				} else {
+					warning << string_compose (_("configuration files contain a JACK server path that doesn't exist (%1)"),
+								   strval)
+						<< endmsg;
 				}
 			}
-			if (x != server_strings.end()) {
-				serverpath_combo.set_active_text (strval);
-			} else {
-				warning << string_compose (_("configuration files contain a JACK server path that doesn't exist (%1)"),
-							     strval)
-					<< endmsg;
-			}
+
 		} else if (child->name() == "driver") {
 			driver_combo.set_active_text(strval);
 		} else if (child->name() == "interface") {
 			interface_combo.set_active_text(strval);
+			if (!strval.empty()) {
+				_interface_chosen = true;
+			}
 		} else if (child->name() == "timeout") {
 			timeout_combo.set_active_text(strval);
 		} else if (child->name() == "dither") {
