@@ -28,7 +28,6 @@
 #include <pbd/xml++.h>
 
 #include <gtkmm2ext/utils.h>
-#include <gtkmm2ext/window_title.h>
 
 #include <ardour/export.h>
 #include <ardour/sndfile_helpers.h>
@@ -43,6 +42,7 @@
 #include "ardour_ui.h"
 #include "public_editor.h"
 #include "keyboard.h"
+#include "nag.h"
 
 #include "i18n.h"
 
@@ -121,17 +121,15 @@ ExportDialog::ExportDialog(PublicEditor& e)
 	channel_count_selection_allowed = true;
 	export_cd_markers_allowed = true;
 
-	WindowTitle title(Glib::get_application_name());
-	title += _("Export");
-	
-	set_title (title.get_string());
-	set_wmclass (X_("ardour_export"), "Ardour");
+	set_title (_("Export"));
+	set_wmclass (X_("ardour_export"), PROGRAM_NAME);
 	set_name ("ExportWindow");
 	add_events (Gdk::KEY_PRESS_MASK|Gdk::KEY_RELEASE_MASK);
 	
 	spec.running = false;
 
 	file_entry.set_name ("ExportFileNameEntry");
+	file_entry.set_activates_default (true);
 
 	master_list = ListStore::create (exp_cols);
 	master_selector.set_model (master_list);
@@ -239,7 +237,7 @@ ExportDialog::ExportDialog(PublicEditor& e)
 	Gtkmm2ext::set_popdown_strings (cue_file_combo, pop_strings);
 	cue_file_combo.set_active_text (pop_strings.front());
 
-	/* this will re-sensitized as soon as a non RIFF/WAV
+	/* this will re-sensitized as soon as a something other than RIFF/WAV, AIFF or OGG
 	   header format is chosen.
 	*/
 
@@ -372,6 +370,7 @@ ExportDialog::ExportDialog(PublicEditor& e)
 	cancel_button = add_button (Stock::CANCEL, RESPONSE_CANCEL);
 	cancel_button->signal_clicked().connect (mem_fun(*this, &ExportDialog::end_dialog));
 	ok_button = add_button (_("Export"), RESPONSE_ACCEPT);
+	set_default_response (RESPONSE_ACCEPT);
 	ok_button->signal_clicked().connect (mem_fun(*this, &ExportDialog::do_export));
 	
 	file_browse_button.set_name ("EditorGTKButton");
@@ -489,6 +488,15 @@ ExportDialog::set_state()
 	channels_chosen();
 	sample_rate_chosen();
 
+	//header_chosen initializes the file_entry text.  we need to clear it so it will be set to the default, and/or recover the val that was stored in instant.xml
+	file_entry.set_text("");
+	if (node) {
+		if ((prop = node->property (X_("filename"))) != 0) {
+			file_entry.set_text(prop->value());
+		}
+	}
+
+
 	if (session->master_out()) {
 		track_scroll.hide ();
 	} else {
@@ -647,11 +655,18 @@ ExportDialog::export_toc_file (Locations::LocationList& locations, const string&
 		return;
 	}
 	
-	string filepath = path + ".toc";
-	ofstream out (filepath.c_str());
 	long unsigned int last_end_time = spec.start_frame, last_start_time = spec.start_frame;
 	gchar buf[18];
-
+	
+	/* Build the toc's file name from the specified audio file name. */
+	string basename = Glib::path_get_basename(path);	
+	size_t ext_pos = basename.rfind('.');
+	if (ext_pos != string::npos) {
+		basename = basename.substr(0, ext_pos); /* strip file extension, if there is one */
+	}
+	string filepath = Glib::build_filename(Glib::path_get_dirname(path), basename + ".toc");
+	
+	ofstream out (filepath.c_str());
 	if (!out) {
 		error << string_compose(_("Editor: cannot open \"%1\" as export file for CD toc file"), filepath) << endmsg;
 		return;
@@ -716,7 +731,7 @@ ExportDialog::export_toc_file (Locations::LocationList& locations, const string&
 				out << "  }" << endl << "}" << endl;
 				
 				frames_to_cd_frames_string (buf, last_end_time - spec.start_frame, session->frame_rate());
-				out << "FILE \"" << path << "\" " << buf;
+				out << "FILE \"" << Glib::path_get_basename(path) << "\"" << buf;
 				
 				if ((*i)->is_mark()) {
 					// a mark track location needs to look ahead to the next marker's start to determine length
@@ -780,12 +795,19 @@ ExportDialog::export_cue_file (Locations::LocationList& locations, const string&
 		return;
 	}
 	
-    string filepath = path + ".cue";
-	ofstream out (filepath.c_str());
 	gchar buf[18];
 	long unsigned int last_track_end = spec.start_frame;
 	int numtracks = 0, tracknum = 0, indexnum = 0;
-
+	
+	/* Build the cue sheet's file name from the specified audio file name. */
+	string basename = Glib::path_get_basename(path);	
+	size_t ext_pos = basename.rfind('.');
+	if (ext_pos != string::npos) {
+		basename = basename.substr(0, ext_pos); /* strip file extension, if there is one */
+	}
+	string filepath = Glib::build_filename(Glib::path_get_dirname(path), basename + ".cue");
+	
+	ofstream out (filepath.c_str());
 	if (!out) {
 		error << string_compose(_("Editor: cannot open \"%1\" as export file for CD cue file"), filepath) << endmsg;
 		return;
@@ -805,39 +827,66 @@ ExportDialog::export_cue_file (Locations::LocationList& locations, const string&
 	
 	out << "REM Cue file generated by Ardour" << endl;
 	out << "TITLE \"" << session->name() << "\"" << endl;
+	
+	out << "FILE \"" << Glib::path_get_basename(path) << "\" ";
+	
+	/*  The cue sheet syntax has originally five file types:
+			WAVE     : 44.1 kHz, 16 Bit (little endian)
+			AIFF     : 44.1 kHz, 16 Bit (big endian) 
+			BINARY   : 44.1 kHz, 16 Bit (little endian)
+			MOTOROLA : 44.1 kHz, 16 Bit (big endian)
+			MP3
+	
+		We want to use cue sheets not only as CD images but also as general playlyist
+		format, thus for WAVE and AIFF we don't care if it's really 44.1 kHz/16 Bit, the
+		soundfile's header shows it anyway.  But for the raw formats, i.e. BINARY 
+		and MOTOROLA we do care, because no header would tell us about a different format.
 
-	if ((header_format_combo.get_active_text() == N_("WAV"))) {
-		  out << "FILE " << path  << " WAVE" << endl;
+		For all other formats we just make up our own file type.  MP3 is not supported 
+		at the moment.
+	*/
+	int file_format = sndfile_header_format_from_string (header_format_combo.get_active_text ());
+	if ((file_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV) {
+		out << "WAVE";
+	} else if ((file_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_AIFF) {
+		out << "AIFF";
+	} else if ( ((file_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_RAW) 
+				&& (sndfile_bitdepth_format_from_string(bitdepth_format_combo.get_active_text()) == SF_FORMAT_PCM_16)
+				&& (sample_rate_combo.get_active_text() == _("44.1kHz")) ) {
+		/* raw audio, 16 Bit, 44.1 kHz */
+		if (sndfile_endian_format_from_string(endian_format_combo.get_active_text()) == SF_ENDIAN_LITTLE) {
+			out << "BINARY";
+		} else {
+			out << "MOTOROLA";
+		}
 	} else {
-		  out << "FILE " << path  << ' ' << (header_format_combo.get_active_text()) << endl;
+		out << (header_format_combo.get_active_text());
 	}
+	out << endl;
 
 	if (false && numtracks == 0) {
-		    /* the user has supplied no track markers.
-		       the entire export is treated as one track. 
-		    */
+		/* the user has supplied no track markers.
+		   the entire export is treated as one track. 
+		*/
 
-		  numtracks++;
-		  tracknum++;
-		  indexnum = 0;
-		  out << endl << "TRACK " << tracknum << " AUDIO" << endl;
-		  out << "FLAGS " ;
-		  
-		  out << "DCP " << endl;		   
-		  
-		  /* use the session name*/
-		  
-		  if (session->name() != "") {
-		    out << "TITLE \"" << session->name() << "\"" << endl;
-		  }	      
-		  
-		  /* no pregap in this case */
+		numtracks++;
+		tracknum++;
+		indexnum = 0;
 
-		  out << "INDEX 00 00:00:00" << endl;
-		  indexnum++;
-		  out << "INDEX 01 00:00:00" << endl;
-		  indexnum++;
-		  last_track_end = spec.end_frame;
+		snprintf (buf, sizeof(buf), "  TRACK %02d AUDIO", tracknum);
+		out << buf << endl;
+		out << "    FLAGS DCP" << endl;		   
+
+		/* use the session name*/
+
+		out << "    TITLE \"" << session->name() << "\"" << endl;
+
+		/* No pregap is specified in this case, adding the default pregap
+		   is left to the burning application. */
+
+		out << "    INDEX 01 00:00:00" << endl;
+		indexnum = 2;
+		last_track_end = spec.end_frame;
 	}
 
 	if (temp.size()) {
@@ -854,39 +903,43 @@ ExportDialog::export_cue_file (Locations::LocationList& locations, const string&
 				
 				tracknum++;
 				indexnum = 0;
-				out << endl << "TRACK " << tracknum << " AUDIO" << endl;
-				out << "FLAGS " ;
-				
+
+				snprintf (buf, sizeof(buf), "  TRACK %02d AUDIO", tracknum);
+				out << buf << endl;
+
+				out << "    FLAGS" ;
 				if ((*i)->cd_info.find("scms") != (*i)->cd_info.end())  {
-					out << "SCMS ";
+					out << " SCMS";
 				} else {
-					out << "DCP ";
+					out << " DCP";
 				}
-				
 				if ((*i)->cd_info.find("preemph") != (*i)->cd_info.end())  {
-					out << "PRE";
+					out << " PRE";
 				}
 				out << endl;
 				
 				if ((*i)->cd_info.find("isrc") != (*i)->cd_info.end())  {
-					out << "ISRC " << (*i)->cd_info["isrc"] << endl;
-					
+					out << "    ISRC " << (*i)->cd_info["isrc"] << endl;
 				}
+
 				if ((*i)->name() != "") {
-					out << "TITLE \"" << (*i)->name() << "\"" << endl;
+					out << "    TITLE \"" << (*i)->name() << "\"" << endl;
 				}	      
 				
 				if ((*i)->cd_info.find("performer") != (*i)->cd_info.end()) {
-					out << "PERFORMER \"" <<  (*i)->cd_info["performer"] << "\"" << endl;
+					out << "    PERFORMER \"" <<  (*i)->cd_info["performer"] << "\"" << endl;
 				}
 				
 				if ((*i)->cd_info.find("string_composer") != (*i)->cd_info.end()) {
-					out << "SONGWRITER \"" << (*i)->cd_info["string_composer"]  << "\"" << endl;
+					out << "    SONGWRITER \"" << (*i)->cd_info["string_composer"]  << "\"" << endl;
 				}
-				snprintf (buf, sizeof(buf), "INDEX %02d", indexnum);
-				out << buf;
-				frames_to_cd_frames_string (buf, last_track_end - spec.start_frame, session->frame_rate());
-				out << buf << endl;
+
+				/* only print "Index 00" if not at the same position as "Index 01" */
+				if (last_track_end != (*i)->start()) {
+					frames_to_cd_frames_string (buf, last_track_end - spec.start_frame, session->frame_rate());
+					out << "    INDEX 00" << buf << endl;
+				}
+
 				indexnum++;
 
 				if ((*i)->is_mark()) {
@@ -909,7 +962,7 @@ ExportDialog::export_cue_file (Locations::LocationList& locations, const string&
 	
 			if ((tracknum > 0) && ((*i)->start() < last_track_end)) {
 				/*this is an index and it lies within a track*/
-				snprintf (buf, sizeof(buf), "INDEX %02d", indexnum);
+				snprintf (buf, sizeof(buf), "    INDEX %02d", indexnum);
 				out << buf;
 				frames_to_cd_frames_string (buf,(*i)->start() - spec.start_frame, session->frame_rate());
 				out << buf << endl;
@@ -930,31 +983,100 @@ ExportDialog::do_export_cd_markers (const string& path,const string& cuefile_typ
 	}
 }
 
+string
+ExportDialog::get_suffixed_filepath ()
+{
+	string filepath = file_entry.get_text();
+
+	if (wants_dir()) {
+		return filepath;
+	}
+
+	string::size_type dotpos;
+	
+	/* maybe add suffix */
+	
+	int file_format = sndfile_header_format_from_string (header_format_combo.get_active_text ());
+	
+	if ((file_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV) {
+		if (filepath.find (".wav") != filepath.length() - 4) {
+			if ((dotpos = filepath.rfind ('.')) != string::npos) {
+				filepath = filepath.substr (0, dotpos);
+			}
+			filepath += ".wav";
+		}
+	} else if ((file_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_AIFF) {
+		if (filepath.find (".aiff") != filepath.length() - 5) {
+			if ((dotpos = filepath.rfind ('.')) != string::npos) {
+				filepath = filepath.substr (0, dotpos);
+			}
+			filepath += ".aiff";
+		}
+	} else if ((file_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_CAF) {
+		if (filepath.find (".caf") != filepath.length() - 4) {
+			if ((dotpos = filepath.rfind ('.')) != string::npos) {
+				filepath = filepath.substr (0, dotpos);
+			}
+			filepath += ".caf";
+		}
+	} else if ((file_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_W64) {
+		if (filepath.find (".w64") != filepath.length() - 4) {
+			if ((dotpos = filepath.rfind ('.')) != string::npos) {
+				filepath = filepath.substr (0, dotpos);
+			}
+			filepath += ".w64";
+		}
+	} else if ((file_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_FLAC) {
+		if (filepath.find (".flac") != filepath.length() - 5) {
+			if ((dotpos = filepath.rfind ('.')) != string::npos) {
+				filepath = filepath.substr (0, dotpos);
+			}
+			filepath += ".flac";
+		}
+	} else if ((file_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_OGG) {
+		if (filepath.find (".ogg") != filepath.length() - 4) {
+			if ((dotpos = filepath.rfind ('.')) != string::npos) {
+				filepath = filepath.substr (0, dotpos);
+			}
+			filepath += ".ogg";
+		}
+	} else if ((file_format & SF_FORMAT_TYPEMASK) == SF_FORMAT_RAW) {
+		if (filepath.find (".raw") != filepath.length() - 4) {
+			if ((dotpos = filepath.rfind ('.')) != string::npos) {
+				filepath = filepath.substr (0, dotpos);
+			}
+			filepath += ".raw";
+		}
+	}
+	return filepath;
+}
 
 void
 ExportDialog::do_export ()
 {
-	string filepath = file_entry.get_text();
-
 	if (!ARDOUR_UI::instance()->the_engine().connected()) {
 		MessageDialog msg (*this, 
 				   _("Not connected to audioengine"),
 				   true,
 				   MESSAGE_ERROR,
 				   BUTTONS_OK);
-		msg.set_secondary_text (_("Ardour cannot export audio when disconnected"));
+		msg.set_secondary_text (string_compose (_("%1 cannot export audio when disconnected"), PROGRAM_NAME));
 		msg.present ();
 		msg.run ();
 		return;
 	}
-		
+
+	string filepath;
+
+	filepath = get_suffixed_filepath ();
+
 	if(!is_filepath_valid(filepath)){
 		return;
 	}
 
 	if (!Profile->get_sae() && export_cd_markers_allowed) {
 		if (cue_file_combo.get_active_text () != _("None")) {
-			do_export_cd_markers (file_entry.get_text(), cue_file_combo.get_active_text ());
+			do_export_cd_markers (filepath, cue_file_combo.get_active_text ());
 		}
 
 		if (cuefile_only_checkbox.get_active()) {
@@ -974,10 +1096,24 @@ ExportDialog::do_export ()
 	progress_connection = Glib::signal_timeout().connect (mem_fun(*this, &ExportDialog::progress_timeout), 100);
 	cancel_label.set_text (_("Stop Export"));
 
+	session->pre_export ();
+
 	export_audio_data();
 	
   	progress_connection.disconnect ();
 	end_dialog ();
+
+	/* if not stopped early and not SAE, ask for money, maybe */
+
+	if (!spec.stop && !Profile->get_sae()) {
+
+		NagScreen* ns = NagScreen::maybe_nag (_("export"));
+		
+		if (ns) {
+			ns->nag ();
+			delete ns;
+		}
+	}
 }
 	
 void
@@ -1016,18 +1152,13 @@ ExportDialog::start_export ()
 	*/
 	
 	if (file_entry.get_text().length() == 0) {
-		string dir = session->export_dir();
-		string::size_type last_slash;
-		
-		if ((last_slash = dir.find_last_of ('/')) != string::npos && last_slash != 0) {
-			dir = dir.substr (0, last_slash+1);
-		}
+		Glib::ustring export_path = session->export_dir();
 
 		if (!wants_dir()) {
-			dir = dir + "export.wav";
+			export_path = Glib::build_filename (export_path, "export.wav");
 		}
 		
-		file_entry.set_text (dir);
+		file_entry.set_text (export_path);
 	}
 	
 	progress_bar.set_fraction (0);
@@ -1046,11 +1177,26 @@ ExportDialog::start_export ()
 void
 ExportDialog::header_chosen ()
 {
-	if (sndfile_header_format_from_string (header_format_combo.get_active_text ()) == SF_FORMAT_WAV) {
+        int fmt = sndfile_header_format_from_string (header_format_combo.get_active_text ());
+	
+	if ((fmt & SF_FORMAT_TYPEMASK) == SF_FORMAT_OGG) {
 		endian_format_combo.set_sensitive (false);
-	} else {
-		endian_format_combo.set_sensitive (true);
+		bitdepth_format_combo.set_sensitive (false);     
+        } else {
+                if ((fmt & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV) {
+                        endian_format_combo.set_active_text (sndfile_endian_formats_strings[0]);
+                        endian_format_combo.set_sensitive (false);
+                } else if ((fmt & SF_FORMAT_TYPEMASK) == SF_FORMAT_AIFF) {
+                        endian_format_combo.set_active_text (sndfile_endian_formats_strings[1]);
+                        endian_format_combo.set_sensitive (false);
+                } else {
+                        endian_format_combo.set_sensitive (true);
+                }
+
+		bitdepth_format_combo.set_sensitive (true);     
 	}
+
+	file_entry.set_text (get_suffixed_filepath());
 }
 
 void
@@ -1233,7 +1379,7 @@ ExportDialog::initSpec(string &filepath)
 {
 	spec.path = filepath;
 	spec.progress = 0;
-	spec.running = true;
+	spec.running = false;
 	spec.stop = false;
 	spec.port_map.clear();
 	
@@ -1247,14 +1393,22 @@ ExportDialog::initSpec(string &filepath)
 
 	spec.format |= sndfile_header_format_from_string (header_format_combo.get_active_text ());
 
+	/* if they picked Ogg, give them Ogg/Vorbis */
+
+	if ((spec.format & SF_FORMAT_TYPEMASK) == SF_FORMAT_OGG) {
+	  spec.format |= SF_FORMAT_VORBIS;
+	}
+
 	if (!Profile->get_sae()) {
-		if ((spec.format & SF_FORMAT_WAV) == 0) {
-			/* RIFF/WAV specifies endianess */
+                if ((spec.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_OGG) {
+			/* O/V has no concept of endianness */
 			spec.format |= sndfile_endian_format_from_string (endian_format_combo.get_active_text ());
 		}
 	}
 
-	spec.format |= sndfile_bitdepth_format_from_string (bitdepth_format_combo.get_active_text ());
+	if ((spec.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_OGG) {
+                spec.format |= sndfile_bitdepth_format_from_string (bitdepth_format_combo.get_active_text ());
+	}
 
 	string sr_str = sample_rate_combo.get_active_text();
 	if (sr_str == N_("22.05kHz")) {
