@@ -18,6 +18,7 @@
 */
 
 #include <cstdlib>
+#include <signal.h>
 
 #include <sigc++/bind.h>
 #include <gtkmm/settings.h>
@@ -30,15 +31,17 @@
 
 #include <jack/jack.h>
 
+#include <ardour/svn_revision.h>
 #include <ardour/version.h>
 #include <ardour/ardour.h>
 #include <ardour/audioengine.h>
 
 #include <gtkmm/main.h>
+
+#include <gtkmm2ext/application.h>
 #include <gtkmm2ext/popup.h>
 #include <gtkmm2ext/utils.h>
 
-#include "svn_revision.h"
 #include "version.h"
 #include "ardour_ui.h"
 #include "opts.h"
@@ -57,7 +60,7 @@ TextReceiver text_receiver ("ardour");
 extern int curvetest (string);
 
 static ARDOUR_UI  *ui = 0;
-static char* localedir = LOCALEDIR;
+static const char* localedir = LOCALEDIR;
 
 #ifdef __APPLE__
 
@@ -65,12 +68,16 @@ static char* localedir = LOCALEDIR;
 #include <sys/param.h>
 #include <fstream>
 
+extern void set_language_preference (); // cocoacarbon.mm
+
 void
 fixup_bundle_environment ()
 {
 	if (!getenv ("ARDOUR_BUNDLED")) {
 		return;
 	}
+	
+	set_language_preference ();
 
 	char execpath[MAXPATHLEN+1];
 	uint32_t pathsz = sizeof (execpath);
@@ -82,7 +89,10 @@ fixup_bundle_environment ()
 	Glib::ustring path;
 	const char *cstr = getenv ("PATH");
 
-	/* ensure that we find any bundled executables (e.g. JACK) */
+	/* ensure that we find any bundled executables (e.g. JACK),
+	   and find them before any instances of the same name
+	   elsewhere in PATH
+	*/
 
 	path = dir_path;
 	if (cstr) {
@@ -113,12 +123,18 @@ fixup_bundle_environment ()
 	setenv ("ARDOUR_CONFIG_PATH", path.c_str(), 1);
 	setenv ("ARDOUR_DATA_PATH", path.c_str(), 1);
 
+	path = dir_path;
+	path += "/../Resources";
+	setenv ("ARDOUR_INSTANT_XML_PATH", path.c_str(), 1);
+
 	cstr = getenv ("LADSPA_PATH");
 	if (cstr) {
 		path = cstr;
 		path += ':';
+	} else {
+		path = "";
 	}
-	path = dir_path;
+	path += dir_path;
 	path += "/../Plugins";
 	
 	setenv ("LADSPA_PATH", path.c_str(), 1);
@@ -127,8 +143,10 @@ fixup_bundle_environment ()
 	if (cstr) {
 		path = cstr;
 		path += ':';
+	} else {
+		path = "";
 	}
-	path = dir_path;
+	path += dir_path;
 	path += "/../Frameworks";
 	
 	setenv ("VAMP_PATH", path.c_str(), 1);
@@ -137,8 +155,10 @@ fixup_bundle_environment ()
 	if (cstr) {
 		path = cstr;
 		path += ':';
+	} else {
+		path = "";
 	}
-	path = dir_path;
+	path += dir_path;
 	path += "/../Surfaces";
 	
 	setenv ("ARDOUR_CONTROL_SURFACE_PATH", path.c_str(), 1);
@@ -147,8 +167,10 @@ fixup_bundle_environment ()
 	if (cstr) {
 		path = cstr;
 		path += ':';
+	} else {
+		path = "";
 	}
-	path = dir_path;
+	path += dir_path;
 	path += "/../Plugins";
 	
 	setenv ("LV2_PATH", path.c_str(), 1);
@@ -158,27 +180,45 @@ fixup_bundle_environment ()
 
 	setenv ("GTK_PATH", path.c_str(), 1);
 
-	path = dir_path;
-	path += "/../Resources/locale";
-	
-	localedir = strdup (path.c_str());
+	if (!ARDOUR::translations_are_disabled ()) {
 
-	/* write a pango.rc file and tell pango to use it */
-
-	path = dir_path;
-	path += "/../Resources/pango.rc";
-
-	std::ofstream pangorc (path.c_str());
-	if (!pangorc) {
-		error << string_compose (_("cannot open pango.rc file %1") , path) << endmsg;
-	} else {
-		pangorc << "[Pango]\nModuleFiles=";
-		Glib::ustring mpath = dir_path;
-		mpath += "/../Resources/pango.modules";
-		pangorc << mpath << endl;
+		path = dir_path;
+		path += "/../Resources/locale";
 		
-		pangorc.close ();
-		setenv ("PANGO_RC_FILE", path.c_str(), 1);
+		localedir = strdup (path.c_str());
+		setenv ("GTK_LOCALEDIR", localedir, 1);
+	}
+
+	/* write a pango.rc file and tell pango to use it. we'd love
+	   to put this into the Ardour.app bundle and leave it there,
+	   but the user may not have write permission. so ... 
+
+	   we also have to make sure that the user ardour directory
+	   actually exists ...
+	*/
+
+	if (g_mkdir_with_parents (ARDOUR::get_user_ardour_path().c_str(), 0755) < 0) {
+		error << string_compose (_("cannot create user ardour folder %1 (%2)"), ARDOUR::get_user_ardour_path(), strerror (errno))
+		      << endmsg;
+	} else {
+
+		path = Glib::build_filename (ARDOUR::get_user_ardour_path(), "pango.rc");
+		
+		std::ofstream pangorc (path.c_str());
+		if (!pangorc) {
+			error << string_compose (_("cannot open pango.rc file %1") , path) << endmsg;
+		} else {
+			pangorc << "[Pango]\nModuleFiles=";
+			Glib::ustring mpath;
+
+			mpath = dir_path;
+			mpath += "/../Resources/pango.modules";
+
+			pangorc << mpath << endl;
+			
+			pangorc.close ();
+			setenv ("PANGO_RC_FILE", path.c_str(), 1);
+		}
 	}
 
 	// gettext charset aliases
@@ -211,64 +251,60 @@ fixup_bundle_environment ()
 
 #endif
 
-static void
-setup_keybindings (ARDOUR_UI* ui)
+static gboolean
+tell_about_jack_death (void* /* ignored */)
 {
-	Glib::ustring path;
-
-	if (keybindings_path.empty()) {
-		keybindings_path = "ardour";
-	}
-
-	std::string kbpath;
-	
-	if (keybindings_path.find (".bindings") == string::npos) {
-
-		// just a style name - allow user to
-		// specify the layout type. 
+	if (AudioEngine::instance()->processed_frames() == 0) {
+		/* died during startup */
+		MessageDialog msg (_("JACK exited"), false, Gtk::MESSAGE_INFO, Gtk::BUTTONS_OK);
+		msg.set_position (Gtk::WIN_POS_CENTER);
+		msg.set_secondary_text (string_compose (_(
+"JACK exited unexpectedly, and without notifying %1.\n\
+\n\
+This could be due to misconfiguration or to an error inside JACK.\n\
+\n\
+Click OK to exit %1."), PROGRAM_NAME));
+    
+		msg.run ();
+		_exit (0);
 		
-		char* layout;
-		
-		if ((layout = getenv ("ARDOUR_KEYBOARD_LAYOUT")) != 0) {
-			keybindings_path += '-';
-			keybindings_path += layout;
-		}
-
-		keybindings_path += ".bindings";
-	} 
-
-	
-	// XXX timbyr - we need a portable test for "is-absolute" here 
-	
-	if (keybindings_path[0] != '/' && keybindings_path[0] != '.') {
-
-		/* not absolute - look in the usual places */
-		
-		path = find_config_file (keybindings_path);
-
-		if (path.empty()) {
-			warning << string_compose (_("Key bindings file \"%1\" not found. Default bindings used instead"), 
-						   keybindings_path)
-				<< endmsg;
-		}
-
 	} else {
 
-		// absolute path from user - use it as is
-
-		path = keybindings_path;
+		/* engine has already run, so this is a mid-session JACK death */
+		
+		MessageDialog* msg = manage (new MessageDialog (_("JACK exited"), false, Gtk::MESSAGE_INFO, Gtk::BUTTONS_NONE));
+		msg->set_secondary_text (string_compose (_(
+"JACK exited unexpectedly, and without notifying %1.\n\
+\n\
+This is probably due to an error inside JACK. You should restart JACK\n\
+and reconnect %1 to it, or exit %1 now. You cannot save your\n\
+session at this time, because we would lose your connection information.\n"), PROGRAM_NAME));
+		msg->present ();
 	}
+	return false; /* do not call again */
+}
 
-	if (!path.empty()) {
-		ui->set_keybindings_path (path);
+static void
+sigpipe_handler (int sig)
+{
+        static bool done_the_jack_thing = false;
+	
+	if (!done_the_jack_thing) {
+		AudioEngine::instance()->died ();
+		g_idle_add (tell_about_jack_death, 0);
+		done_the_jack_thing =  true;
 	}
 }
 
 #ifdef VST_SUPPORT
+
+extern int gui_init (int* argc, char** argv[]);
+
 /* this is called from the entry point of a wine-compiled
    executable that is linked against gtk2_ardour built
    as a shared library.
 */
+
 extern "C" {
 int ardour_main (int argc, char *argv[])
 #else
@@ -276,13 +312,19 @@ int main (int argc, char* argv[])
 #endif
 {
 	vector<Glib::ustring> null_file_list;
-
+	
 #ifdef __APPLE__
 	fixup_bundle_environment ();
 #endif
 
         Glib::thread_init();
-	gtk_set_locale ();
+
+#ifdef VST_SUPPORT
+	/* this does some magic that is needed to make GTK and Wine's own
+	   X11 client interact properly.
+	*/
+	gui_init (&argc, &argv);
+#endif
 
 	(void) bindtextdomain (PACKAGE, localedir);
 	/* our i18n translations are all in UTF-8, so make sure
@@ -290,6 +332,10 @@ int main (int argc, char* argv[])
 	   we use that when handling them.
 	*/
 	(void) bind_textdomain_codeset (PACKAGE,"UTF-8");
+
+	/* this really does nothing since we don't get gettext(),
+	   only dgettext(), but whatever ...
+	*/
 	(void) textdomain (PACKAGE);
 
 	pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, 0);
@@ -309,10 +355,10 @@ int main (int argc, char* argv[])
 		return curvetest (curvetest_file);
 	}
 	
-	cout << _("Ardour/GTK ") 
+	cout << PROGRAM_NAME << ' '
 	     << VERSIONSTRING
 	     << _("\n   (built using ")
-	     << ardour_svn_revision
+	     << svn_revision
 #ifdef __GNUC__
 	     << _(" and GCC version ") << __VERSION__ 
 #endif
@@ -324,10 +370,10 @@ int main (int argc, char* argv[])
 	}
 
 	if (no_splash) {
-		cerr << _("Copyright (C) 1999-2007 Paul Davis") << endl
+		cerr << _("Copyright (C) 1999-2008 Paul Davis") << endl
 		     << _("Some portions Copyright (C) Steve Harris, Ari Johnson, Brett Viren, Joel Baker") << endl
 		     << endl
-		     << _("Ardour comes with ABSOLUTELY NO WARRANTY") << endl
+		     << string_compose (_("%1 comes with ABSOLUTELY NO WARRANTY"), PROGRAM_NAME) << endl
 		     <<	_("not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.") << endl
 		     << _("This is free software, and you are welcome to redistribute it ") << endl
 		     << _("under certain conditions; see the source for copying conditions.")
@@ -338,6 +384,10 @@ int main (int argc, char* argv[])
 
 	PBD::ID::init ();
 
+	if (::signal (SIGPIPE, sigpipe_handler)) {
+		cerr << _("Cannot install SIGPIPE error handler") << endl;
+	}
+
         try { 
 		ui = new ARDOUR_UI (&argc, &argv);
 	} catch (failed_constructor& err) {
@@ -345,13 +395,13 @@ int main (int argc, char* argv[])
 		exit (1);
 	}
 
-	setup_keybindings (ui);
-
 	ui->run (text_receiver);
+	Gtkmm2ext::Application::instance()->cleanup();
 	ui = 0;
 
 	ARDOUR::cleanup ();
 	pthread_cancel_all ();
+
 	return 0;
 }
 #ifdef VST_SUPPORT
