@@ -83,15 +83,15 @@ speed_printer (char buf[32], Gtk::Adjustment& adj, void* arg)
 }
 #endif 
 
-MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt, bool in_mixer)
+MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, bool in_mixer)
 	: AxisView(sess),
-	  RouteUI (rt, sess, _("Mute"), _("Solo"), _("Record")),
+	  RouteUI (sess, _("Mute"), _("Solo"), _("Record")),
 	  _mixer(mx),
 	  _mixer_owned (in_mixer),
-	  pre_redirect_box (PreFader, sess, rt, mx.plugin_selector(), mx.selection(), in_mixer),
-	  post_redirect_box (PostFader, sess, rt, mx.plugin_selector(), mx.selection(), in_mixer),
-	  gpm (_route, sess),
-	  panners (_route, sess),
+	  pre_redirect_box (PreFader, sess, mx.plugin_selector(), mx.selection(), in_mixer),
+	  post_redirect_box (PostFader, sess, mx.plugin_selector(), mx.selection(), in_mixer),
+	  gpm (sess),
+	  panners (sess),
 	  button_table (3, 2),
 	  middle_button_table (1, 2),
 	  bottom_button_table (1, 2),
@@ -101,15 +101,48 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt
 	  speed_spinner (&speed_adjustment, "MixerStripSpeedBase", true)
 
 {
-	if (set_color_from_route()) {
-		set_color (unique_random_color());
-	}
+	init ();
 
+	if (!_mixer_owned) {
+		/* the editor mixer strip: don't destroy it every time
+		   the underlying route goes away.
+		*/
+
+		self_destruct = false;
+	}
+}
+
+MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt, bool in_mixer)
+	: AxisView(sess),
+	  RouteUI (sess, _("Mute"), _("Solo"), _("Record")),
+	  _mixer(mx),
+	  _mixer_owned (in_mixer),
+	  pre_redirect_box (PreFader, sess, mx.plugin_selector(), mx.selection(), in_mixer),
+	  post_redirect_box (PostFader, sess, mx.plugin_selector(), mx.selection(), in_mixer),
+	  gpm (sess),
+	  panners (sess),
+	  button_table (3, 2),
+	  middle_button_table (1, 2),
+	  bottom_button_table (1, 2),
+	  meter_point_label (_("pre")),
+	  comment_button (_("Comments")),
+	  speed_adjustment (1.0, 0.001, 4.0, 0.001, 0.1),
+	  speed_spinner (&speed_adjustment, "MixerStripSpeedBase", true)
+
+{
+	init ();
+	set_route (rt);
+}
+
+void
+MixerStrip::init ()
+{
 	input_selector = 0;
 	output_selector = 0;
 	group_menu = 0;
 	_marked_for_display = false;
 	route_ops_menu = 0;
+	rename_menu_item = 0;
 	ignore_comment_edit = false;
 	ignore_toggle = false;
 	ignore_speed_adjustment = false;
@@ -121,34 +154,22 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt
 	hide_button.add (*(manage (new Gtk::Image (::get_icon("hide")))));
 
 	input_label.set_text (_("Input"));
+	ARDOUR_UI::instance()->set_tip (&input_button, _("Click to choose inputs"), "");
 	input_button.add (input_label);
 	input_button.set_name ("MixerIOButton");
 	input_label.set_name ("MixerIOButtonLabel");
 
 	output_label.set_text (_("Output"));
+	ARDOUR_UI::instance()->set_tip (&output_button, _("Click to choose outputs"), "");
 	output_button.add (output_label);
 	output_button.set_name ("MixerIOButton");
 	output_label.set_name ("MixerIOButtonLabel");
 
-	_route->meter_change.connect (mem_fun(*this, &MixerStrip::meter_changed));
+	ARDOUR_UI::instance()->set_tip (&meter_point_button, _("Select metering point"), "");
 	meter_point_button.add (meter_point_label);
 	meter_point_button.set_name ("MixerStripMeterPreButton");
 	meter_point_label.set_name ("MixerStripMeterPreButton");
 	
-	switch (_route->meter_point()) {
-	case MeterInput:
-		meter_point_label.set_text (_("input"));
-		break;
-		
-	case MeterPreFader:
-		meter_point_label.set_text (_("pre"));
-		break;
-		
-	case MeterPostFader:
-		meter_point_label.set_text (_("post"));
-		break;
-	}
-	
 	/* TRANSLATORS: this string should be longest of the strings
 	   used to describe meter points. In english, it's "input".
 	*/
@@ -180,55 +201,20 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt
 	bottom_button_table.set_homogeneous (true);
 	bottom_button_table.attach (group_button, 0, 1, 0, 1);
 	
-	if (is_audio_track()) {
-		
-		rec_enable_button->signal_button_press_event().connect (mem_fun(*this, &RouteUI::rec_enable_press), false);
-		rec_enable_button->signal_button_release_event().connect (mem_fun(*this, &RouteUI::rec_enable_release));
-
-		rec_enable_button->set_name ("MixerRecordEnableButton");
-
-		boost::shared_ptr<AudioTrack> at = audio_track();
-
-		at->FreezeChange.connect (mem_fun(*this, &MixerStrip::map_frozen));
-
-#ifdef VARISPEED_IN_MIXER_STRIP
-		speed_adjustment.signal_value_changed().connect (mem_fun(*this, &MixerStrip::speed_adjustment_changed));
-		
-		speed_frame.set_name ("BaseFrame");
-		speed_frame.set_shadow_type (Gtk::SHADOW_IN);
-		speed_frame.add (speed_spinner);
-		
-		speed_spinner.set_print_func (speed_printer, 0);
-
-		ARDOUR_UI::instance()->tooltips().set_tip (speed_spinner, _("Varispeed"));
-
-		button_table.attach (speed_frame, 0, 2, 5, 6);
-#endif /* VARISPEED_IN_MIXER_STRIP */
-
-		button_table.attach (*rec_enable_button, 0, 2, 2, 3);
-	}
-
 	name_button.add (name_label);
 	name_button.set_name ("MixerNameButton");
 	Gtkmm2ext::set_size_request_to_display_given_text (name_button, "longest label", 2, 2);
-
 	name_label.set_name ("MixerNameButtonLabel");
-	if (_route->phase_invert()) {
-	        name_label.set_text (X_("Ø ") + name_label.get_text());
-	} else {
-	        name_label.set_text (_route->name());
-	}
 
+	ARDOUR_UI::instance()->set_tip (&group_button, _("Mix group"), "");
 	group_button.add (group_label);
 	group_button.set_name ("MixerGroupButton");
+	Gtkmm2ext::set_size_request_to_display_given_text (group_button, "Group", 2, 2);
+
 	group_label.set_name ("MixerGroupButtonLabel");
 
 	comment_button.set_name ("MixerCommentButton");
 
-	ARDOUR_UI::instance()->tooltips().set_tip (comment_button, _route->comment()==""	?
-							_("Click to Add/Edit Comments"):
-							_route->comment());
-
 	comment_button.signal_clicked().connect (mem_fun(*this, &MixerStrip::comment_button_clicked));
 	
 	global_vpacker.set_border_width (0);
@@ -263,19 +249,6 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt
 	global_vpacker.pack_start (output_button, Gtk::PACK_SHRINK);
 	global_vpacker.pack_start (comment_button, Gtk::PACK_SHRINK);
 
-	if (route()->master() || route()->control()) {
-		
-		if (scrollbar_height == 0) {
-			HScrollbar scrollbar;
-			Gtk::Requisition requisition(scrollbar.size_request ());
-			scrollbar_height = requisition.height;
-		}
-
-		EventBox* spacer = manage (new EventBox);
-		spacer->set_size_request (-1, scrollbar_height);
-		global_vpacker.pack_start (*spacer, false, false);
-	}
-
 	global_frame.add (global_vpacker);
 	global_frame.set_shadow_type (Gtk::SHADOW_IN);
 	global_frame.set_name ("BaseFrame");
@@ -292,22 +265,6 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt
 
 	_session.engine().Stopped.connect (mem_fun(*this, &MixerStrip::engine_stopped));
 	_session.engine().Running.connect (mem_fun(*this, &MixerStrip::engine_running));
-	_route->input_changed.connect (mem_fun(*this, &MixerStrip::input_changed));
-	_route->output_changed.connect (mem_fun(*this, &MixerStrip::output_changed));
-	_route->mute_changed.connect (mem_fun(*this, &RouteUI::mute_changed));
-	_route->solo_changed.connect (mem_fun(*this, &RouteUI::solo_changed));
-	_route->solo_safe_changed.connect (mem_fun(*this, &RouteUI::solo_changed));
-	_route->mix_group_changed.connect (mem_fun(*this, &MixerStrip::mix_group_changed));
-	_route->panner().Changed.connect (mem_fun(*this, &MixerStrip::connect_to_pan));
-
-	if (is_audio_track()) {
-		audio_track()->DiskstreamChanged.connect (mem_fun(*this, &MixerStrip::diskstream_changed));
-		get_diskstream()->SpeedChanged.connect (mem_fun(*this, &MixerStrip::speed_changed));
-	}
-
-	_route->name_changed.connect (mem_fun(*this, &RouteUI::name_changed));
-	_route->comment_changed.connect (mem_fun(*this, &MixerStrip::comment_changed));
-	_route->gui_changed.connect (mem_fun(*this, &MixerStrip::route_gui_changed));
 
 	input_button.signal_button_press_event().connect (mem_fun(*this, &MixerStrip::input_press), false);
 	output_button.signal_button_press_event().connect (mem_fun(*this, &MixerStrip::output_press), false);
@@ -317,11 +274,18 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt
 	mute_button->signal_button_press_event().connect (mem_fun(*this, &RouteUI::mute_press), false);
 	mute_button->signal_button_release_event().connect (mem_fun(*this, &RouteUI::mute_release), false);
 
+	/* we don't need this if its not an audio track, but we don't know that yet and it doesn't
+	   hurt (much).
+	*/
+
+	rec_enable_button->set_name ("MixerRecordEnableButton");
+	rec_enable_button->signal_button_press_event().connect (mem_fun(*this, &RouteUI::rec_enable_press), false);
+	rec_enable_button->signal_button_release_event().connect (mem_fun(*this, &RouteUI::rec_enable_release));
+
 	name_button.signal_button_press_event().connect (mem_fun(*this, &MixerStrip::name_button_button_press), false);
 	group_button.signal_button_press_event().connect (mem_fun(*this, &MixerStrip::select_mix_group), false);
 
 	_width = (Width) -1;
-	set_stuff_from_route ();
 
 	/* start off as a passthru strip. we'll correct this, if necessary,
 	   in update_diskstream_display().
@@ -329,6 +293,145 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt
 
 	set_name ("AudioTrackStripBase");
 
+	add_events (Gdk::BUTTON_RELEASE_MASK);
+}
+
+MixerStrip::~MixerStrip ()
+{
+	GoingAway(); /* EMIT_SIGNAL */
+
+	if (input_selector) {
+		delete input_selector;
+	}
+
+	if (output_selector) {
+		delete output_selector;
+	}
+	
+	if(comment_window) {
+		delete comment_window;
+	}
+}
+
+void
+MixerStrip::set_route (boost::shared_ptr<Route> rt)
+{
+	if (rec_enable_button->get_parent()) {
+		button_table.remove (*rec_enable_button);
+	}
+
+#ifdef VARISPEED_IN_MIXER_STRIP
+	if (speed_frame->get_parent()) {
+		button_table.remove (*speed_frame);
+	}
+#endif
+
+	RouteUI::set_route (rt);
+
+	if (input_selector) {
+		delete input_selector;
+		input_selector = 0;
+	}
+
+	if (output_selector) {
+		delete output_selector;
+		output_selector = 0;
+	}
+
+	panners.set_io (rt);
+	gpm.set_io (rt);
+	pre_redirect_box.set_route (rt);
+	post_redirect_box.set_route (rt);
+
+	if (set_color_from_route()) {
+		set_color (unique_random_color());
+	}
+
+	if (_mixer_owned && (route()->master() || route()->control())) {
+		
+		if (scrollbar_height == 0) {
+			HScrollbar scrollbar;
+			Gtk::Requisition requisition(scrollbar.size_request ());
+			scrollbar_height = requisition.height;
+		}
+
+		EventBox* spacer = manage (new EventBox);
+		spacer->set_size_request (-1, scrollbar_height);
+		global_vpacker.pack_start (*spacer, false, false);
+	}
+
+	if (is_audio_track()) {
+
+		boost::shared_ptr<AudioTrack> at = audio_track();
+
+		connections.push_back (at->FreezeChange.connect (mem_fun(*this, &MixerStrip::map_frozen)));
+
+#ifdef VARISPEED_IN_MIXER_STRIP
+		speed_adjustment.signal_value_changed().connect (mem_fun(*this, &MixerStrip::speed_adjustment_changed));
+		
+		speed_frame.set_name ("BaseFrame");
+		speed_frame.set_shadow_type (Gtk::SHADOW_IN);
+		speed_frame.add (speed_spinner);
+		
+		speed_spinner.set_print_func (speed_printer, 0);
+
+		ARDOUR_UI::instance()->tooltips().set_tip (speed_spinner, _("Varispeed"));
+
+		button_table.attach (speed_frame, 0, 2, 5, 6);
+#endif /* VARISPEED_IN_MIXER_STRIP */
+
+		button_table.attach (*rec_enable_button, 0, 2, 2, 3);
+		rec_enable_button->set_sensitive (_session.writable());
+		rec_enable_button->show();
+	}
+
+	if (_route->phase_invert()) {
+	        name_label.set_text (X_("Ø ") + name_label.get_text());
+	} else {
+	        name_label.set_text (_route->name());
+	}
+
+	switch (_route->meter_point()) {
+	case MeterInput:
+		meter_point_label.set_text (_("input"));
+		break;
+		
+	case MeterPreFader:
+		meter_point_label.set_text (_("pre"));
+		break;
+		
+	case MeterPostFader:
+		meter_point_label.set_text (_("post"));
+		break;
+	}
+
+	delete route_ops_menu;
+	route_ops_menu = 0;
+	
+	ARDOUR_UI::instance()->tooltips().set_tip (comment_button, _route->comment().empty() ?
+						   _("Click to Add/Edit Comments"):
+						   _route->comment());
+
+	connections.push_back (_route->meter_change.connect (mem_fun(*this, &MixerStrip::meter_changed)));
+	connections.push_back (_route->input_changed.connect (mem_fun(*this, &MixerStrip::input_changed)));
+	connections.push_back (_route->output_changed.connect (mem_fun(*this, &MixerStrip::output_changed)));
+	connections.push_back (_route->mute_changed.connect (mem_fun(*this, &RouteUI::mute_changed)));
+	connections.push_back (_route->solo_changed.connect (mem_fun(*this, &RouteUI::solo_changed)));
+	connections.push_back (_route->solo_safe_changed.connect (mem_fun(*this, &RouteUI::solo_changed)));
+	connections.push_back (_route->mix_group_changed.connect (mem_fun(*this, &MixerStrip::mix_group_changed)));
+	connections.push_back (_route->panner().Changed.connect (mem_fun(*this, &MixerStrip::connect_to_pan)));
+
+	if (is_audio_track()) {
+		connections.push_back (audio_track()->DiskstreamChanged.connect (mem_fun(*this, &MixerStrip::diskstream_changed)));
+		connections.push_back (get_diskstream()->SpeedChanged.connect (mem_fun(*this, &MixerStrip::speed_changed)));
+	}
+
+	connections.push_back (_route->name_changed.connect (mem_fun(*this, &RouteUI::name_changed)));
+	connections.push_back (_route->comment_changed.connect (mem_fun(*this, &MixerStrip::comment_changed)));
+	connections.push_back (_route->gui_changed.connect (mem_fun(*this, &MixerStrip::route_gui_changed)));
+
+	set_stuff_from_route ();
+
 	/* now force an update of all the various elements */
 
 	pre_redirect_box.update();
@@ -350,21 +453,6 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session& sess, boost::shared_ptr<Route> rt
 	update_diskstream_display ();
 	update_input_display ();
 	update_output_display ();
-
-	add_events (Gdk::BUTTON_RELEASE_MASK);
-}
-
-MixerStrip::~MixerStrip ()
-{
-	GoingAway(); /* EMIT_SIGNAL */
-
-	if (input_selector) {
-		delete input_selector;
-	}
-
-	if (output_selector) {
-		delete output_selector;
-	}
 }
 
 void
@@ -414,20 +502,19 @@ MixerStrip::set_width (Width w, void* owner)
 
 	switch (w) {
 	case Wide:
-		set_size_request (-1, -1);
-		
+
 		if (rec_enable_button)  {
-			((Gtk::Label*)rec_enable_button->get_child())->set_text (_("record"));
+			((Gtk::Label*)rec_enable_button->get_child())->set_text (_("Record"));
 		}
 		((Gtk::Label*)mute_button->get_child())->set_text  (_("Mute"));
 		((Gtk::Label*)solo_button->get_child())->set_text (_("Solo"));
 
 		if (_route->comment() == "") {
 		       comment_button.unset_bg (STATE_NORMAL);
-		       ((Gtk::Label*)comment_button.get_child())->set_text (_("comments"));
+		       ((Gtk::Label*)comment_button.get_child())->set_text (_("Comments"));
 		} else {
 		       comment_button.modify_bg (STATE_NORMAL, color());
-		       ((Gtk::Label*)comment_button.get_child())->set_text (_("*comments*"));
+		       ((Gtk::Label*)comment_button.get_child())->set_text (_("*Comments*"));
 		}
 
 		((Gtk::Label*)gpm.gain_automation_style_button.get_child())->set_text (gpm.astyle_string(_route->gain_automation_curve().automation_style()));
@@ -435,6 +522,7 @@ MixerStrip::set_width (Width w, void* owner)
 		((Gtk::Label*)panners.pan_automation_style_button.get_child())->set_text (panners.astyle_string(_route->panner().automation_style()));
 		((Gtk::Label*)panners.pan_automation_state_button.get_child())->set_text (panners.astate_string(_route->panner().automation_state()));
 		Gtkmm2ext::set_size_request_to_display_given_text (name_button, "long", 2, 2);
+		set_size_request (-1, -1);
 		break;
 
 	case Narrow:
@@ -460,12 +548,13 @@ MixerStrip::set_width (Width w, void* owner)
 		set_size_request (max (50, gpm.get_gm_width()), -1);
 		break;
 	}
-
 	update_input_display ();
 	update_output_display ();
 	mix_group_changed (0);
 	name_changed (0);
-
+#ifdef GTKOSX
+	WidthChanged();
+#endif
 }
 
 void
@@ -697,43 +786,229 @@ MixerStrip::connect_to_pan ()
 	panners.pan_changed (this);
 }
 
+/*
+ * Output port labelling
+ * =====================
+ * 
+ * Case 1: Each output has one connection, all connections are to system:playback_%i
+ *   out 1 -> system:playback_1
+ *   out 2 -> system:playback_2
+ *   out 3 -> system:playback_3
+ *   Display as: 1/2/3
+ *
+ * Case 2: Each output has one connection, all connections are to ardour:track_x/in 1
+ *   out 1 -> ardour:track_x/in 1
+ *   out 2 -> ardour:track_x/in 2
+ *   Display as: track_x
+ * 
+ * Case 3: Each output has one connection, all connections are to Jack client "program x"
+ *   out 1 -> program x:foo
+ *   out 2 -> program x:foo
+ *   Display as: program x
+ *
+ * Case 4: No connections (Disconnected)
+ *   Display as: -
+ * 
+ * Default case (unusual routing):
+ *   Display as: *number of connections*
+ *
+ * Tooltips
+ * ========
+ * .-----------------------------------------------.
+ * | Mixdown                                       |
+ * | out 1 -> ardour:master/in 1, jamin:input/in 1 |
+ * | out 2 -> ardour:master/in 2, jamin:input/in 2 |
+ * '-----------------------------------------------'
+ * .-----------------------------------------------.
+ * | Guitar SM58                                   |
+ * | Disconnected                                  |
+ * '-----------------------------------------------'
+ */
+
+void
+MixerStrip::update_io_button (boost::shared_ptr<ARDOUR::Route> route, Width width, bool for_input)
+{
+	uint32_t io_count;
+	uint32_t io_index;
+	Port *port;
+	const char **connections;
+	
+	uint32_t connection_index = 0;
+	uint32_t total_connection_count = 0;
+	uint32_t io_connection_count = 0;
+	uint32_t ardour_connection_count = 0;
+	uint32_t system_connection_count = 0;
+	uint32_t other_connection_count = 0;
+
+	ostringstream label;
+	string label_string;
+	char * label_cstr;
+
+	bool have_label = false;
+	bool each_io_has_one_connection = true;
+
+	string connection_name;
+	string ardour_track_name;
+	string other_connection_type;
+	string system_ports;
+	string system_port;
+	
+	ostringstream tooltip;
+	char * tooltip_cstr;
+	
+	tooltip << route->name();
+	
+	if (for_input) {
+		io_count = route->n_inputs();
+	} else {
+		io_count = route->n_outputs();
+	}    
+	
+	for (io_index = 0; io_index < io_count; ++io_index) {
+		if (for_input) {
+			port = route->input(io_index);
+		} else {
+			port = route->output(io_index);
+		}
+		
+		connections = port->get_connections();
+		io_connection_count = 0;
+		
+		if (connections) {
+			for (connection_index = 0; connections[connection_index]; ++connection_index) {
+				connection_name = connections[connection_index];
+				
+				if (connection_index == 0) {
+					tooltip << endl << port->name().substr(port->name().find("/") + 1) << " -> " << connection_name;
+				} else {
+					tooltip << ", " << connection_name;
+				}
+				
+				if (connection_name.find("ardour:") == 0) {
+					if (ardour_track_name.empty()) {
+						// "ardour:Master/in 1" -> "ardour:Master/"
+						string::size_type slash = connection_name.find("/");
+						if (slash != string::npos) {
+							ardour_track_name = connection_name.substr(0, slash + 1);
+						}
+					}
+					
+					if (connection_name.find(ardour_track_name) == 0) {
+						++ardour_connection_count;
+					}
+				} else if (connection_name.find("system:") == 0) {
+					if (for_input) {
+						// "system:capture_123" -> "123"
+						system_port = connection_name.substr(15);
+					} else {
+						// "system:playback_123" -> "123"
+						system_port = connection_name.substr(16);
+					}
+					
+					if (system_ports.empty()) {
+						system_ports += system_port;
+					} else {
+						system_ports += "/" + system_port;
+					}
+					
+					++system_connection_count;
+				} else {
+					if (other_connection_type.empty()) {
+						// "jamin:in 1" -> "jamin:"
+						other_connection_type = connection_name.substr(0, connection_name.find(":") + 1);
+					}
+					
+					if (connection_name.find(other_connection_type) == 0) {
+						++other_connection_count;
+					}
+				}
+				
+				++total_connection_count;
+				++io_connection_count;
+			}
+		} 
+		
+		if (io_connection_count != 1) {
+			each_io_has_one_connection = false;
+		}
+	}
+	
+	if (total_connection_count == 0) {
+		tooltip << endl << _("Disconnected");
+	}
+	
+	tooltip_cstr = new char[tooltip.str().size() + 1];
+	strcpy(tooltip_cstr, tooltip.str().c_str());
+	
+	if (for_input) {
+		ARDOUR_UI::instance()->set_tip (&input_button, tooltip_cstr, "");
+	} else {
+		ARDOUR_UI::instance()->set_tip (&output_button, tooltip_cstr, "");
+	}  
+	
+	if (each_io_has_one_connection) {
+		if ((total_connection_count == ardour_connection_count)) {
+			// all connections are to the same track in ardour
+			// "ardour:Master/" -> "Master"
+			string::size_type slash = ardour_track_name.find("/");
+			if (slash != string::npos) {
+				label << ardour_track_name.substr(7, slash - 7);
+				have_label = true;
+			}
+		}
+		else if (total_connection_count == system_connection_count) {
+			// all connections are to system ports
+			label << system_ports;
+			have_label = true;
+		}
+		else if (total_connection_count == other_connection_count) {
+			// all connections are to the same external program eg jamin
+			// "jamin:" -> "jamin"
+			label << other_connection_type.substr(0, other_connection_type.size() - 1);
+			have_label = true;
+		}
+	}
+	
+	if (!have_label) {
+		if (total_connection_count == 0) {
+			// Disconnected
+			label << "-";
+		} else {
+			// Odd configuration
+			label << "*" << total_connection_count << "*";
+		}
+	}
+	
+	switch (width) {
+	case Wide:
+		label_string = label.str().substr(0, 6);
+		break;
+	case Narrow:
+		label_string = label.str().substr(0, 3);
+		break;
+	}
+	
+	label_cstr = new char[label_string.size() + 1];
+	strcpy(label_cstr, label_string.c_str());
+	
+	if (for_input) {
+		input_label.set_text (label_cstr);
+	} else {
+		output_label.set_text (label_cstr);
+	}
+}
+
 void
 MixerStrip::update_input_display ()
 {
-	ARDOUR::Connection *c;
-
-	if ((c = _route->input_connection()) != 0) {
-		input_label.set_text (c->name());
-	} else {
-		switch (_width) {
-		case Wide:
-			input_label.set_text (_(" Input"));
-			break;
-		case Narrow:
-			input_label.set_text (_("I"));
-			break;
-		}
-	}
+	update_io_button (_route, _width, true);
 	panners.setup_pan ();
 }
 
 void
 MixerStrip::update_output_display ()
 {
-	ARDOUR::Connection *c;
-
-	if ((c = _route->output_connection()) != 0) {
-		output_label.set_text (c->name());
-	} else {
-		switch (_width) {
-		case Wide:
-			output_label.set_text (_("Output"));
-			break;
-		case Narrow:
-			output_label.set_text (_("O"));
-			break;
-		}
-	}
+    update_io_button (_route, _width, false);
 	gpm.setup_meters ();
 	panners.setup_pan ();
 }
@@ -923,7 +1198,7 @@ MixerStrip::mix_group_changed (void *ignored)
 	RouteGroup *rg = _route->mix_group();
 	
 	if (rg) {
-		group_label.set_text (rg->name());
+		group_label.set_text (PBD::short_version (rg->name(), 5));
 	} else {
 		switch (_width) {
 		case Wide:
@@ -967,12 +1242,14 @@ void
 MixerStrip::build_route_ops_menu ()
 {
 	using namespace Menu_Helpers;
-	route_ops_menu = manage (new Menu);
+	route_ops_menu = new Menu;
 	route_ops_menu->set_name ("ArdourContextMenu");
 
 	MenuList& items = route_ops_menu->items();
 
+	items.push_back (MenuElem (_("Save As Template"), mem_fun(*this, &RouteUI::save_as_template)));
 	items.push_back (MenuElem (_("Rename"), mem_fun(*this, &RouteUI::route_rename)));
+	rename_menu_item = &items.back();
 	items.push_back (SeparatorElem());
 	items.push_back (CheckMenuElem (_("Active"), mem_fun (*this, &RouteUI::toggle_route_active)));
 	route_active_menu_item = dynamic_cast<CheckMenuItem *> (&items.back());
@@ -985,11 +1262,10 @@ MixerStrip::build_route_ops_menu ()
 	denormal_menu_item = dynamic_cast<CheckMenuItem *> (&items.back());
 	denormal_menu_item->set_active (_route->denormal_protection());
 
-	build_remote_control_menu ();
-	
-	items.push_back (SeparatorElem());
 	if (!Profile->get_sae()) {
-              items.push_back (MenuElem (_("Remote Control ID"), *remote_control_menu));
+		build_remote_control_menu ();
+		items.push_back (SeparatorElem());
+		items.push_back (MenuElem (_("Remote Control ID"), *remote_control_menu));
         }
 
 	items.push_back (SeparatorElem());
@@ -999,13 +1275,10 @@ MixerStrip::build_route_ops_menu ()
 gint
 MixerStrip::name_button_button_press (GdkEventButton* ev)
 {
-	if (ev->button == 1) {
+	if (ev->button == 1 || ev->button == 3) {
 		list_route_operations ();
-
-		Menu_Helpers::MenuList& items = route_ops_menu->items();
 		/* do not allow rename if the track is record-enabled */
-		static_cast<MenuItem*> (&items.front())->set_sensitive (!_route->record_enabled());
-
+		rename_menu_item->set_sensitive (!_route->record_enabled());
 		route_ops_menu->popup (1, ev->time);
 	}
 	return FALSE;
@@ -1104,16 +1377,16 @@ MixerStrip::width_clicked ()
 void
 MixerStrip::hide_clicked ()
 {
-    // LAME fix to reset the button status for when it is redisplayed (part 1)
-    hide_button.set_sensitive(false);
-    
+	// LAME fix to reset the button status for when it is redisplayed (part 1)
+	hide_button.set_sensitive(false);
+	
 	if (_embedded) {
-		 Hiding(); /* EMIT_SIGNAL */
+		Hiding(); /* EMIT_SIGNAL */
 	} else {
 		_mixer.hide_strip (this);
 	}
 	
-    // (part 2)
+	// (part 2)
 	hide_button.set_sensitive(true);
 }
 
@@ -1213,21 +1486,23 @@ MixerStrip::meter_changed (void *src)
 
 	ENSURE_GUI_THREAD (bind (mem_fun(*this, &MixerStrip::meter_changed), src));
 
-		switch (_route->meter_point()) {
-		case MeterInput:
-			meter_point_label.set_text (_("input"));
-			break;
-			
-		case MeterPreFader:
-			meter_point_label.set_text (_("pre"));
-			break;
-			
-		case MeterPostFader:
-			meter_point_label.set_text (_("post"));
-			break;
-		}
-
-		gpm.setup_meters ();
-		set_width(_width, this);
+	switch (_route->meter_point()) {
+	case MeterInput:
+		meter_point_label.set_text (_("input"));
+		break;
+		
+	case MeterPreFader:
+		meter_point_label.set_text (_("pre"));
+		break;
+		
+	case MeterPostFader:
+		meter_point_label.set_text (_("post"));
+		break;
+	}
+	
+	gpm.setup_meters ();
+	// reset peak when meter point changes
+	gpm.reset_peak_display();
+	set_width (_width, this);
 }
 
