@@ -59,19 +59,18 @@ using namespace std;
 
 using PBD::atoi;
 
-
 Mixer_UI::Mixer_UI ()
 	: Window (Gtk::WINDOW_TOPLEVEL)
 {
 	session = 0;
-	Config->get_default_narrow_ms() ? _strip_width = Narrow : _strip_width = Wide;
+	_strip_width = Config->get_default_narrow_ms() ? Narrow : Wide;
 	track_menu = 0;
 	mix_group_context_menu = 0;
 	no_track_list_redisplay = false;
 	in_group_row_change = false;
 	_visible = false;
-	ignore_route_reorder = false;
-	ignore_sync = false;
+	strip_redisplay_does_not_reset_order_keys = false;
+	strip_redisplay_does_not_sync_order_keys = false;
 
 	Route::SyncOrderKeys.connect (mem_fun (*this, &Mixer_UI::sync_order_keys));
 
@@ -86,12 +85,18 @@ Mixer_UI::Mixer_UI ()
 
 	track_model = ListStore::create (track_columns);
 	track_display.set_model (track_model);
-	track_display.append_column (_("Strips"), track_columns.text);
-	track_display.append_column (_("Show"), track_columns.visible);
-	track_display.get_column (0)->set_data (X_("colnum"), GUINT_TO_POINTER(0));
-	track_display.get_column (1)->set_data (X_("colnum"), GUINT_TO_POINTER(1));
-	track_display.get_column (0)->set_expand(true);
-	track_display.get_column (1)->set_expand(false);
+	/* LT change column order based on value of TRACK_DISPLAY_... enum. */
+	if (TRACK_DISPLAY_SHOW_COLUMN < TRACK_DISPLAY_STRIPS_COLUMN) {
+	  track_display.append_column (_("Show"), track_columns.visible);
+	  track_display.append_column (_("Strips"), track_columns.text);
+	} else {
+	  track_display.append_column (_("Strips"), track_columns.text);
+	  track_display.append_column (_("Show"), track_columns.visible);
+	}
+	track_display.get_column (TRACK_DISPLAY_STRIPS_COLUMN)->set_data (X_("colnum"), GUINT_TO_POINTER(TRACK_DISPLAY_STRIPS_COLUMN));
+	track_display.get_column (TRACK_DISPLAY_SHOW_COLUMN)->set_data (X_("colnum"), GUINT_TO_POINTER(TRACK_DISPLAY_SHOW_COLUMN));
+	track_display.get_column (TRACK_DISPLAY_STRIPS_COLUMN)->set_expand(true);
+	track_display.get_column (TRACK_DISPLAY_SHOW_COLUMN)->set_expand(false);
 	track_display.set_name (X_("MixerTrackDisplayList"));
 	track_display.get_selection()->set_mode (Gtk::SELECTION_NONE);
 	track_display.set_reorderable (true);
@@ -101,7 +106,7 @@ Mixer_UI::Mixer_UI ()
 	track_model->signal_row_changed().connect (mem_fun (*this, &Mixer_UI::track_list_change));
 	track_model->signal_rows_reordered().connect (mem_fun (*this, &Mixer_UI::track_list_reorder));
 
-	CellRendererToggle* track_list_visible_cell = dynamic_cast<CellRendererToggle*>(track_display.get_column_cell_renderer (1));
+	CellRendererToggle* track_list_visible_cell = dynamic_cast<CellRendererToggle*>(track_display.get_column_cell_renderer (TRACK_DISPLAY_SHOW_COLUMN));
 	track_list_visible_cell->property_activatable() = true;
 	track_list_visible_cell->property_radio() = false;
 
@@ -112,15 +117,23 @@ Mixer_UI::Mixer_UI ()
 
 	group_model = ListStore::create (group_columns);
 	group_display.set_model (group_model);
-	group_display.append_column (_("Group"), group_columns.text);
-	group_display.append_column (_("Active"), group_columns.active);
-	group_display.append_column (_("Show"), group_columns.visible);
-	group_display.get_column (0)->set_data (X_("colnum"), GUINT_TO_POINTER(0));
-	group_display.get_column (1)->set_data (X_("colnum"), GUINT_TO_POINTER(1));
-	group_display.get_column (2)->set_data (X_("colnum"), GUINT_TO_POINTER(2));
-	group_display.get_column (0)->set_expand(true);
-	group_display.get_column (1)->set_expand(false);
-	group_display.get_column (2)->set_expand(false);
+	/* LT change column order based on value of GROUP_DISPLAY_... enum. */
+	/* LT Code only two possibilities : Group, Active, Show or Show, Active, Group. */
+	if (GROUP_DISPLAY_SHOW_COLUMN < GROUP_DISPLAY_ACTIVE_COLUMN) {
+	  group_display.append_column (_("Show"), group_columns.visible);
+	  group_display.append_column (_("Active"), group_columns.active);
+	  group_display.append_column (_("Group"), group_columns.text);
+	} else {
+	  group_display.append_column (_("Group"), group_columns.text);
+	  group_display.append_column (_("Active"), group_columns.active);
+	  group_display.append_column (_("Show"), group_columns.visible);
+	}
+	group_display.get_column (GROUP_DISPLAY_GROUP_COLUMN)->set_data (X_("colnum"), GUINT_TO_POINTER(GROUP_DISPLAY_GROUP_COLUMN));
+	group_display.get_column (GROUP_DISPLAY_ACTIVE_COLUMN)->set_data (X_("colnum"), GUINT_TO_POINTER(GROUP_DISPLAY_ACTIVE_COLUMN));
+	group_display.get_column (GROUP_DISPLAY_SHOW_COLUMN)->set_data (X_("colnum"), GUINT_TO_POINTER(GROUP_DISPLAY_SHOW_COLUMN));
+	group_display.get_column (GROUP_DISPLAY_GROUP_COLUMN)->set_expand(true);
+	group_display.get_column (GROUP_DISPLAY_ACTIVE_COLUMN)->set_expand(false);
+	group_display.get_column (GROUP_DISPLAY_SHOW_COLUMN)->set_expand(false);
 	group_display.set_name ("MixerGroupList");
 	group_display.get_selection()->set_mode (Gtk::SELECTION_SINGLE);
 	group_display.set_reorderable (true);
@@ -129,19 +142,19 @@ Mixer_UI::Mixer_UI ()
 
 	/* name is directly editable */
 
-	CellRendererText* name_cell = dynamic_cast<CellRendererText*>(group_display.get_column_cell_renderer (0));
+	CellRendererText* name_cell = dynamic_cast<CellRendererText*>(group_display.get_column_cell_renderer (GROUP_DISPLAY_GROUP_COLUMN));
 	name_cell->property_editable() = true;
 	name_cell->signal_edited().connect (mem_fun (*this, &Mixer_UI::mix_group_name_edit));
 
 	/* use checkbox for the active column */
 
-	CellRendererToggle* active_cell = dynamic_cast<CellRendererToggle*>(group_display.get_column_cell_renderer (1));
+	CellRendererToggle* active_cell = dynamic_cast<CellRendererToggle*>(group_display.get_column_cell_renderer (GROUP_DISPLAY_ACTIVE_COLUMN));
 	active_cell->property_activatable() = true;
 	active_cell->property_radio() = false;
 
 	/* use checkbox for the visible column */
 
-	active_cell = dynamic_cast<CellRendererToggle*>(group_display.get_column_cell_renderer (2));
+	active_cell = dynamic_cast<CellRendererToggle*>(group_display.get_column_cell_renderer (GROUP_DISPLAY_SHOW_COLUMN));
 	active_cell->property_activatable() = true;
 	active_cell->property_radio() = false;
 
@@ -192,8 +205,12 @@ Mixer_UI::Mixer_UI ()
 	list_vpacker.pack_start (rhs_pane1, true, true);
 
 	global_hpacker.pack_start (scroller, true, true);
+#ifdef GTKOSX
+	/* current gtk-quartz has dirty updates on borders like this one */
+	global_hpacker.pack_start (out_packer, false, false, 0);
+#else
 	global_hpacker.pack_start (out_packer, false, false, 12);
-
+#endif
 	list_hpane.add1(list_vpacker);
 	list_hpane.add2(global_hpacker);
 
@@ -202,10 +219,6 @@ Mixer_UI::Mixer_UI ()
 	list_hpane.signal_size_allocate().connect (bind (mem_fun(*this, &Mixer_UI::pane_allocation_handler), 
 							 static_cast<Gtk::Paned*> (&list_hpane)));
 	
-
-	rhs_pane1.set_data ("collapse-direction", (gpointer) 0);
-	list_hpane.set_data ("collapse-direction", (gpointer) 1);
-
 	global_vpacker.pack_start (list_hpane, true, true);
 
 	add (global_vpacker);
@@ -215,7 +228,7 @@ Mixer_UI::Mixer_UI ()
 	title += _("Mixer");
 	set_title (title.get_string());
 
-	set_wmclass (X_("ardour_mixer"), "Ardour");
+	set_wmclass (X_("ardour_mixer"), PROGRAM_NAME);
 
 	add_accel_group (ActionManager::ui_manager->get_accel_group());
 
@@ -227,6 +240,8 @@ Mixer_UI::Mixer_UI ()
 	signal_configure_event().connect (mem_fun (*ARDOUR_UI::instance(), &ARDOUR_UI::configure_handler));
 
 	_selection.RoutesChanged.connect (mem_fun(*this, &Mixer_UI::follow_strip_selection));
+
+	auto_rebinding = FALSE;
 }
 
 Mixer_UI::~Mixer_UI ()
@@ -243,18 +258,19 @@ void
 Mixer_UI::show_window ()
 {
 	show_all ();
+	if (!_visible) {
+		set_window_pos_and_size ();
 
-	set_window_pos_and_size ();
-
-	/* now reset each strips width so the right widgets are shown */
-	MixerStrip* ms;
-
-	TreeModel::Children rows = track_model->children();
-	TreeModel::Children::iterator ri;
-
-	for (ri = rows.begin(); ri != rows.end(); ++ri) {
-		ms = (*ri)[track_columns.strip];
-		ms->set_width (ms->get_width(), ms->width_owner());
+		/* now reset each strips width so the right widgets are shown */
+		MixerStrip* ms;
+		
+		TreeModel::Children rows = track_model->children();
+		TreeModel::Children::iterator ri;
+		
+		for (ri = rows.begin(); ri != rows.end(); ++ri) {
+			ms = (*ri)[track_columns.strip];
+			ms->set_width (ms->get_width(), ms->width_owner());
+		}
 	}
 	_visible = true;
 }
@@ -276,6 +292,9 @@ Mixer_UI::add_strip (Session::RouteList& routes)
 	
 	MixerStrip* strip;
 
+	no_track_list_redisplay = true;
+	strip_redisplay_does_not_sync_order_keys = true;
+
 	for (Session::RouteList::iterator x = routes.begin(); x != routes.end(); ++x) {
 		boost::shared_ptr<Route> route = (*x);
 
@@ -294,26 +313,30 @@ Mixer_UI::add_strip (Session::RouteList& routes)
 
 		show_strip (strip);
 		
-		no_track_list_redisplay = true;
-		
 		TreeModel::Row row = *(track_model->append());
 		row[track_columns.text] = route->name();
-		
 		row[track_columns.visible] = strip->marked_for_display();
 		row[track_columns.route] = route;
 		row[track_columns.strip] = strip;
-
-		strip->set_old_order_key (track_model->children().size() - 1);
-
-		no_track_list_redisplay = false;
-		redisplay_track_list ();
+		
+		if (route->order_key (N_("signal")) == -1) {
+			route->set_order_key (N_("signal"), track_model->children().size()-1);
+		}
 		
 		route->name_changed.connect (bind (mem_fun(*this, &Mixer_UI::strip_name_changed), strip));
 
 		strip->GoingAway.connect (bind (mem_fun(*this, &Mixer_UI::remove_strip), strip));
-		
+#ifdef GTKOSX
+		strip->WidthChanged.connect (mem_fun(*this, &Mixer_UI::queue_draw_all_strips));
+#endif	
 		strip->signal_button_release_event().connect (bind (mem_fun(*this, &Mixer_UI::strip_button_release_event), strip));
 	}
+
+	no_track_list_redisplay = false;
+
+	redisplay_track_list ();
+	
+	strip_redisplay_does_not_sync_order_keys = false;
 }
 
 void
@@ -329,13 +352,7 @@ Mixer_UI::remove_strip (MixerStrip* strip)
 		strips.erase (i);
 	}
 
-	/* Decrement old order keys for strips `above' the one that is being removed */
-	for (ri = rows.begin(); ri != rows.end(); ++ri) {
-		MixerStrip* s = (*ri)[track_columns.strip];
-		if (s->old_order_key() > strip->old_order_key()) {
-			s->set_old_order_key (s->old_order_key() - 1);
-		}
-	}
+	strip_redisplay_does_not_sync_order_keys = true;
 
 	for (ri = rows.begin(); ri != rows.end(); ++ri) {
 		if ((*ri)[track_columns.strip] == strip) {
@@ -343,27 +360,32 @@ Mixer_UI::remove_strip (MixerStrip* strip)
 			break;
 		}
 	}
+
+	strip_redisplay_does_not_sync_order_keys = false;
 }
 
 const char*
 Mixer_UI::get_order_key() 
 {
+	return X_("signal");
+
 	if (Config->get_sync_all_route_ordering()) {
 		return X_("editor");
 	} else {
 		return X_("signal");
 	}
+
 }
 
 
 void
-Mixer_UI::sync_order_keys ()
+Mixer_UI::sync_order_keys (const char *src)
 {
 	vector<int> neworder;
 	TreeModel::Children rows = track_model->children();
 	TreeModel::Children::iterator ri;
 
-	if (ignore_sync || !session || (session->state_of_the_state() & Session::Loading) || rows.empty()) {
+	if ((strcmp (src, get_order_key()) == 0) || !session || (session->state_of_the_state() & Session::Loading) || rows.empty()) {
 		return;
 	}
 
@@ -371,15 +393,26 @@ Mixer_UI::sync_order_keys ()
 		neworder.push_back (0);
 	}
 
-	for (ri = rows.begin(); ri != rows.end(); ++ri) {
+	bool changed = false;
+	int order;
+
+	for (order = 0, ri = rows.begin(); ri != rows.end(); ++ri, ++order) {
 		boost::shared_ptr<Route> route = (*ri)[track_columns.route];
-		MixerStrip* strip = (*ri)[track_columns.strip];
-		neworder[route->order_key (get_order_key())] = strip->old_order_key ();
+		int old_key = order;
+		int new_key = route->order_key (get_order_key());
+
+		neworder[new_key] = old_key;
+
+		if (new_key != old_key) {
+			changed = true;
+		}
 	}
 
-	ignore_route_reorder = true;
-	track_model->reorder (neworder);
-	ignore_route_reorder = false;
+	if (changed) {
+		strip_redisplay_does_not_reset_order_keys = true;
+		track_model->reorder (neworder);
+		strip_redisplay_does_not_reset_order_keys = false;
+	}
 }
 
 
@@ -620,29 +653,59 @@ Mixer_UI::show_all_audiotracks()
 {
 	set_all_audio_visibility (1, true);
 }
+
 void
 Mixer_UI::hide_all_audiotracks ()
 {
 	set_all_audio_visibility (1, false);
 }
 
+void
+Mixer_UI::show_tracks_with_regions_at_playhead ()
+{
+	boost::shared_ptr<Session::RouteList> const regions = session->get_routes_with_regions_at (session->transport_frame ());
+	
+	TreeModel::Children rows = track_model->children ();
+	for (TreeModel::Children::iterator i = rows.begin(); i != rows.end(); ++i) {
+		boost::shared_ptr<Route> route = (*i)[track_columns.route];
+
+		bool found = false;
+		for (Session::RouteList::iterator x = (*regions).begin(); x != (*regions).end(); ++x) {
+			if ((*x) == route)
+				found = true;
+		}
+				
+		(*i)[track_columns.visible] = found;
+	}
+
+	strip_redisplay_does_not_sync_order_keys = true;
+	redisplay_track_list ();
+	strip_redisplay_does_not_sync_order_keys = false;
+}
+
 void
 Mixer_UI::track_list_reorder (const TreeModel::Path& path, const TreeModel::iterator& iter, int* new_order)
 {
+	strip_redisplay_does_not_sync_order_keys = true;
 	session->set_remote_control_ids();
 	redisplay_track_list ();
+	strip_redisplay_does_not_sync_order_keys = false;
 }
 
 void
 Mixer_UI::track_list_change (const Gtk::TreeModel::Path& path,const Gtk::TreeModel::iterator& iter)
 {
+	// never reset order keys because of a property change
+	strip_redisplay_does_not_reset_order_keys = true; 
 	session->set_remote_control_ids();
 	redisplay_track_list ();
+	strip_redisplay_does_not_reset_order_keys = false;
 }
 
 void
 Mixer_UI::track_list_delete (const Gtk::TreeModel::Path& path)
 {
+	/* this could require an order sync */
 	session->set_remote_control_ids();
 	redisplay_track_list ();
 }
@@ -673,12 +736,10 @@ Mixer_UI::redisplay_track_list ()
 		if (visible) {
 			strip->set_marked_for_display (true);
 
-			if (!ignore_route_reorder) {
+			if (!strip_redisplay_does_not_reset_order_keys) {
 				strip->route()->set_order_key (get_order_key(), order);
 			} 
 
-			strip->set_old_order_key (order);
-
 			if (strip->packed()) {
 
 				if (strip->route()->master() || strip->route()->control()) {
@@ -711,13 +772,115 @@ Mixer_UI::redisplay_track_list ()
 		}
 	}
 	
-	if (Config->get_sync_all_route_ordering() && !ignore_route_reorder) {
-		ignore_sync = true;
-		Route::SyncOrderKeys (); // EMIT SIGNAL
-		ignore_sync = false;
+	if (!strip_redisplay_does_not_reset_order_keys && !strip_redisplay_does_not_sync_order_keys) {
+		session->sync_order_keys (get_order_key());
+	}
+
+	// Rebind all of the midi controls automatically
+	
+	if (auto_rebinding) {
+		auto_rebind_midi_controls ();
 	}
 }
 
+#ifdef GTKOSX
+void
+Mixer_UI::queue_draw_all_strips ()
+{
+	TreeModel::Children rows = track_model->children();
+	TreeModel::Children::iterator i;
+	long order;
+
+	for (order = 0, i = rows.begin(); i != rows.end(); ++i, ++order) {
+		MixerStrip* strip = (*i)[track_columns.strip];
+
+		if (strip == 0) {
+			continue;
+		}
+
+		bool visible = (*i)[track_columns.visible];
+		
+		if (visible) {
+			strip->queue_draw();
+		}
+	}
+}
+#endif
+
+void
+Mixer_UI::set_auto_rebinding( bool val )
+{
+	if( val == TRUE )
+	{
+		auto_rebinding = TRUE;
+		Session::AutoBindingOff();
+	}
+	else
+	{
+		auto_rebinding = FALSE;
+		Session::AutoBindingOn();
+	}
+}
+
+void 
+Mixer_UI::toggle_auto_rebinding() 
+{
+	set_auto_rebinding (!auto_rebinding);
+	auto_rebind_midi_controls();
+}
+
+void 
+Mixer_UI::auto_rebind_midi_controls () 
+{
+	TreeModel::Children rows = track_model->children();
+	TreeModel::Children::iterator i;
+	int pos;
+
+	// Create bindings for all visible strips and remove those that are not visible
+	pos = 1;  // 0 is reserved for the master strip
+	for (i = rows.begin(); i != rows.end(); ++i) {
+		MixerStrip* strip = (*i)[track_columns.strip];
+    
+		if ( (*i)[track_columns.visible] == true ) {  // add bindings for
+			// make the actual binding
+			//cout<<"Auto Binding:  Visible Strip Found: "<<strip->name()<<endl;
+
+			int controlValue = pos;
+			if( strip->route()->master() ) {
+				controlValue = 0;
+			}
+			else {
+				pos++;
+			}
+
+			PBD::Controllable::CreateBinding ( strip->solo_button->get_controllable(), controlValue, 0);
+			PBD::Controllable::CreateBinding ( strip->mute_button->get_controllable(), controlValue, 1);
+
+			if( strip->is_audio_track() ) {
+				PBD::Controllable::CreateBinding ( strip->rec_enable_button->get_controllable(), controlValue, 2);
+			}
+
+			PBD::Controllable::CreateBinding ( &(strip->gpm.get_controllable()), controlValue, 3);
+			PBD::Controllable::CreateBinding ( strip->panners.get_controllable(), controlValue, 4);
+
+		}
+		else {  // Remove any existing binding
+			PBD::Controllable::DeleteBinding ( strip->solo_button->get_controllable() );
+			PBD::Controllable::DeleteBinding ( strip->mute_button->get_controllable() );
+
+			if( strip->is_audio_track() ) {
+				PBD::Controllable::DeleteBinding ( strip->rec_enable_button->get_controllable() );
+			}
+
+			PBD::Controllable::DeleteBinding ( &(strip->gpm.get_controllable()) );
+			PBD::Controllable::DeleteBinding ( strip->panners.get_controllable() ); // This only takes the first panner if there are multiples...
+		}
+
+	} // for
+  
+}
+
+
 struct SignalOrderRouteSorter {
     bool operator() (boost::shared_ptr<Route> a, boost::shared_ptr<Route> b) {
 	    /* use of ">" forces the correct sort order */
@@ -774,11 +937,11 @@ Mixer_UI::track_display_button_press (GdkEventButton* ev)
 	}
 
 	switch (GPOINTER_TO_UINT (column->get_data (X_("colnum")))) {
-	case 0:
+	case TRACK_DISPLAY_STRIPS_COLUMN:
 		/* allow normal processing to occur */
 		return false;
 
-	case 1: /* visibility */
+	case TRACK_DISPLAY_SHOW_COLUMN: /* visibility */
 
 		if ((iter = track_model->get_iter (path))) {
 			MixerStrip* strip = (*iter)[track_columns.strip];
@@ -788,6 +951,9 @@ Mixer_UI::track_display_button_press (GdkEventButton* ev)
 					bool visible = (*iter)[track_columns.visible];
 					(*iter)[track_columns.visible] = !visible;
 				}
+#ifdef GTKOSX
+				track_display.queue_draw();
+#endif
 			}
 		}
 		return true;
@@ -816,7 +982,8 @@ Mixer_UI::build_track_menu ()
 	items.push_back (MenuElem (_("Hide All Audio Tracks"), mem_fun(*this, &Mixer_UI::hide_all_audiotracks)));
 	items.push_back (MenuElem (_("Show All Audio Busses"), mem_fun(*this, &Mixer_UI::show_all_audiobus)));
 	items.push_back (MenuElem (_("Hide All Audio Busses"), mem_fun(*this, &Mixer_UI::hide_all_audiobus)));
-
+	items.push_back (MenuElem (_("Hide All Audio Busses"), mem_fun(*this, &Mixer_UI::hide_all_audiobus)));
+	items.push_back (MenuElem (_("Show Tracks With Regions Under Playhead"), mem_fun (*this, &Mixer_UI::show_tracks_with_regions_at_playhead)));
 }
 
 void
@@ -850,6 +1017,9 @@ Mixer_UI::build_mix_group_context_menu ()
 	items.push_back (MenuElem (_("Activate All"), mem_fun(*this, &Mixer_UI::activate_all_mix_groups)));
 	items.push_back (MenuElem (_("Disable All"), mem_fun(*this, &Mixer_UI::disable_all_mix_groups)));
 	items.push_back (SeparatorElem());
+	items.push_back (MenuElem (_("Show All"), mem_fun(*this, &Mixer_UI::show_all_mix_groups)));
+	items.push_back (MenuElem (_("Hide All"), mem_fun(*this, &Mixer_UI::hide_all_mix_groups)));
+	items.push_back (SeparatorElem());
 	items.push_back (MenuElem (_("Add group"), mem_fun(*this, &Mixer_UI::new_mix_group)));
 	
 }
@@ -878,11 +1048,14 @@ Mixer_UI::group_display_button_press (GdkEventButton* ev)
 	}
 
 	switch (GPOINTER_TO_UINT (column->get_data (X_("colnum")))) {
-	case 0:
+	case GROUP_DISPLAY_GROUP_COLUMN:
 		if (Keyboard::is_edit_event (ev)) {
 			if ((iter = group_model->get_iter (path))) {
 				if ((group = (*iter)[group_columns.group]) != 0) {
 					// edit_mix_group (group);
+#ifdef GTKOSX
+					group_display.queue_draw();
+#endif
 					return true;
 				}
 			}
@@ -890,18 +1063,24 @@ Mixer_UI::group_display_button_press (GdkEventButton* ev)
 		} 
 		break;
 
-	case 1:
+	case GROUP_DISPLAY_ACTIVE_COLUMN:
 		if ((iter = group_model->get_iter (path))) {
 			bool active = (*iter)[group_columns.active];
 			(*iter)[group_columns.active] = !active;
+#ifdef GTKOSX
+			group_display.queue_draw();
+#endif
 			return true;
 		}
 		break;
 		
-	case 2:
+	case GROUP_DISPLAY_SHOW_COLUMN:
 		if ((iter = group_model->get_iter (path))) {
 			bool visible = (*iter)[group_columns.visible];
 			(*iter)[group_columns.visible] = !visible;
+#ifdef GTKOSX
+			group_display.queue_draw();
+#endif
 			return true;
 		}
 		break;
@@ -922,6 +1101,8 @@ Mixer_UI::activate_all_mix_groups ()
 	}
 }
 
+
+
 void
 Mixer_UI::disable_all_mix_groups ()
 {
@@ -931,6 +1112,24 @@ Mixer_UI::disable_all_mix_groups ()
 	}
 }
 
+void
+Mixer_UI::show_all_mix_groups ()
+{
+       Gtk::TreeModel::Children children = group_model->children();
+	for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter) {
+	        (*iter)[group_columns.visible] = true;
+	}
+}
+
+void
+Mixer_UI::hide_all_mix_groups ()
+{
+       Gtk::TreeModel::Children children = group_model->children();
+	for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter) {
+	        (*iter)[group_columns.visible] = false;
+	}
+}
+
 void
 Mixer_UI::mix_groups_changed ()
 {
@@ -940,15 +1139,6 @@ Mixer_UI::mix_groups_changed ()
 
 	group_model->clear ();
 
-	{
-		TreeModel::Row row;
-		row = *(group_model->append());
-		row[group_columns.active] = false;
-		row[group_columns.visible] = true;
-		row[group_columns.text] = (_("-all-"));
-		row[group_columns.group] = 0;
-	}
-
 	session->foreach_mix_group (mem_fun (*this, &Mixer_UI::add_mix_group));
 }
 
@@ -1073,6 +1263,7 @@ Mixer_UI::mix_group_row_change (const Gtk::TreeModel::Path& path,const Gtk::Tree
 	if (name != group->name()) {
 		group->set_name (name);
 	}
+
 }
 
 void
@@ -1086,7 +1277,18 @@ Mixer_UI::add_mix_group (RouteGroup* group)
 
 	TreeModel::Row row = *(group_model->append());
 	row[group_columns.active] = group->is_active();
-	row[group_columns.visible] = true;
+
+	row[group_columns.visible] = false;
+
+	for (list<MixerStrip *>::iterator i = strips.begin(); i != strips.end(); ++i) {
+		if ((*i)->mix_group() == group) {
+			if ((*i)->marked_for_display()) {
+				row[group_columns.visible] = true;
+			}
+			break;
+		}
+	}
+
 	row[group_columns.group] = group;
 	if (!group->name().empty()) {
 		row[group_columns.text] = group->name();
@@ -1098,8 +1300,8 @@ Mixer_UI::add_mix_group (RouteGroup* group)
 	group->FlagsChanged.connect (bind (mem_fun(*this, &Mixer_UI::group_flags_changed), group));
 	
 	if (focus) {
-		TreeViewColumn* col = group_display.get_column (0);
-		CellRendererText* name_cell = dynamic_cast<CellRendererText*>(group_display.get_column_cell_renderer (0));
+		TreeViewColumn* col = group_display.get_column (GROUP_DISPLAY_GROUP_COLUMN);
+		CellRendererText* name_cell = dynamic_cast<CellRendererText*>(group_display.get_column_cell_renderer (GROUP_DISPLAY_GROUP_COLUMN));
 		group_display.set_cursor (group_model->get_path (row), *col, *name_cell, true);
 	}
 
@@ -1149,25 +1351,47 @@ Mixer_UI::set_state (const XMLNode& node)
 	const XMLProperty* prop;
 	XMLNode* geometry;
 	
-	if ((geometry = find_named_node (node, "geometry")) == 0) {
+	m_width = default_width;
+	m_height = default_height;
+	m_root_x = 1;
+	m_root_y = 1;
+	
+	if ((geometry = find_named_node (node, "geometry")) != 0) {
 
-		m_width = default_width;
-		m_height = default_height;
-		m_root_x = 1;
-		m_root_y = 1;
+		XMLProperty* prop;
 
-	} else {
+		if ((prop = geometry->property("x_size")) == 0) {
+			prop = geometry->property ("x-size");
+		}
+		if (prop) {
+			m_width = atoi(prop->value());
+		}
+		if ((prop = geometry->property("y_size")) == 0) {
+			prop = geometry->property ("y-size");
+		}
+		if (prop) {
+			m_height = atoi(prop->value());
+		}
 
-		m_width = atoi(geometry->property("x_size")->value().c_str());
-		m_height = atoi(geometry->property("y_size")->value().c_str());
-		m_root_x = atoi(geometry->property("x_pos")->value().c_str());
-		m_root_y = atoi(geometry->property("y_pos")->value().c_str());
+		if ((prop = geometry->property ("x_pos")) == 0) {
+			prop = geometry->property ("x-pos");
+		}
+		if (prop) {
+			m_root_x = atoi (prop->value());
+			
+		}
+		if ((prop = geometry->property ("y_pos")) == 0) {
+			prop = geometry->property ("y-pos");
+		}
+		if (prop) {
+			m_root_y = atoi (prop->value());
+		}
 	}
 
 	set_window_pos_and_size ();
 
 	if ((prop = node.property ("narrow-strips"))) {
-		if (prop->value() == "yes") {
+		if (string_is_affirmative (prop->value())) {
 			set_strip_width (Narrow);
 		} else {
 			set_strip_width (Wide);
@@ -1175,7 +1399,7 @@ Mixer_UI::set_state (const XMLNode& node)
 	}
 
 	if ((prop = node.property ("show-mixer"))) {
-		if (prop->value() == "yes") {
+		if (string_is_affirmative (prop->value())) {
 		       _visible = true;
 		}
 	}
@@ -1237,12 +1461,24 @@ Mixer_UI::pane_allocation_handler (Allocation& alloc, Gtk::Paned* which)
 	int width, height;
 	static int32_t done[3] = { 0, 0, 0 };
 
-	if ((geometry = find_named_node (*node, "geometry")) == 0) {
-		width = default_width;
-		height = default_height;
-	} else {
-		width = atoi(geometry->property("x_size")->value());
-		height = atoi(geometry->property("y_size")->value());
+	width = default_width;
+	height = default_height;
+
+	if ((geometry = find_named_node (*node, "geometry")) != 0) {
+
+
+		if ((prop = geometry->property ("x_size")) == 0) {
+			prop = geometry->property ("x-size");
+		}
+		if (prop) {
+			width = atoi (prop->value());
+		}
+		if ((prop = geometry->property ("y_size")) == 0) {
+			prop = geometry->property ("y-size");
+		}
+		if (prop) {
+			height = atoi (prop->value());
+		}
 	}
 
 	if (which == static_cast<Gtk::Paned*> (&rhs_pane1)) {
@@ -1281,8 +1517,66 @@ Mixer_UI::pane_allocation_handler (Allocation& alloc, Gtk::Paned* which)
 	}
 }
 
+void
+Mixer_UI::scroll_left () 
+{
+	Adjustment* adj = scroller.get_hscrollbar()->get_adjustment();
+	/* stupid GTK: can't rely on clamping across versions */
+	scroller.get_hscrollbar()->set_value (max (adj->get_lower(), adj->get_value() - adj->get_step_increment()));
+}
+
+void
+Mixer_UI::scroll_right ()
+{
+	Adjustment* adj = scroller.get_hscrollbar()->get_adjustment();
+	/* stupid GTK: can't rely on clamping across versions */
+	scroller.get_hscrollbar()->set_value (min (adj->get_upper(), adj->get_value() + adj->get_step_increment()));
+}
+
 bool
 Mixer_UI::on_key_press_event (GdkEventKey* ev)
 {
+	switch (ev->keyval) {
+	case GDK_Left:
+		scroll_left ();
+		return true;
+
+	case GDK_Right:
+		scroll_right ();
+		return true;
+
+	default:
+		break;
+	}
+
 	return key_press_focus_accelerator_handler (*this, ev);
 }
+
+bool
+Mixer_UI::on_scroll_event (GdkEventScroll* ev)
+{
+	switch (ev->direction) {
+	case GDK_SCROLL_LEFT:
+		scroll_left ();
+		return true;
+	case GDK_SCROLL_UP:
+		if (ev->state & Keyboard::TertiaryModifier) {
+			scroll_left ();
+			return true;
+		}
+		return false;
+
+	case GDK_SCROLL_RIGHT:
+		scroll_right ();
+		return true;
+
+	case GDK_SCROLL_DOWN:
+		if (ev->state & Keyboard::TertiaryModifier) {
+			scroll_right ();
+			return true;
+		}
+		return false;
+	}
+
+	return false;
+}
