@@ -113,12 +113,15 @@ class Mixer_UI : public Gtk::Window
 	void get_window_pos_and_size ();
 
 	bool on_key_press_event (GdkEventKey*);
+	bool on_scroll_event (GdkEventScroll*);
 
 	void pane_allocation_handler (Gtk::Allocation&, Gtk::Paned*);
 	
 	list<MixerStrip *> strips;
 
 	bool strip_scroller_button_release (GdkEventButton*);
+	void scroll_left ();
+	void scroll_right ();
 
 	void add_strip (ARDOUR::Session::RouteList&);
 	void remove_strip (MixerStrip *);
@@ -152,11 +155,23 @@ class Mixer_UI : public Gtk::Window
 	void redisplay_track_list ();
 	bool no_track_list_redisplay;
 	bool track_display_button_press (GdkEventButton*);
+#ifdef GTKOSX
+	void queue_draw_all_strips ();
+#endif
 	
 	void track_list_change (const Gtk::TreeModel::Path&,const Gtk::TreeModel::iterator&);
 	void track_list_delete (const Gtk::TreeModel::Path&);
 	void track_list_reorder (const Gtk::TreeModel::Path& path, const Gtk::TreeModel::iterator& iter, int* new_order);
 
+	/* Column order in track_display frame */
+	enum {TRACK_DISPLAY_STRIPS_COLUMN = 1,
+	      TRACK_DISPLAY_SHOW_COLUMN = 0};
+
+	/* Column order in group_display frame */
+	enum {GROUP_DISPLAY_GROUP_COLUMN = 2,
+	      GROUP_DISPLAY_ACTIVE_COLUMN = 1,
+	      GROUP_DISPLAY_SHOW_COLUMN = 0};
+
 	void initial_track_display ();
 	void show_track_list_menu ();
 
@@ -169,6 +184,7 @@ class Mixer_UI : public Gtk::Window
 	void hide_all_audiobus ();
 	void show_all_audiotracks();
 	void hide_all_audiotracks ();
+	void show_tracks_with_regions_at_playhead ();
 
 	Gtk::Menu* mix_group_context_menu;
 	bool in_group_row_change;
@@ -181,6 +197,8 @@ class Mixer_UI : public Gtk::Window
 	void build_mix_group_context_menu ();
 	void activate_all_mix_groups ();
 	void disable_all_mix_groups ();
+	void show_all_mix_groups ();
+	void hide_all_mix_groups ();
 	void add_mix_group (ARDOUR::RouteGroup *);
 	void mix_groups_changed ();
 	void mix_group_name_edit (const Glib::ustring&, const Glib::ustring&);
@@ -242,9 +260,9 @@ class Mixer_UI : public Gtk::Window
 
 	Width _strip_width;
 
-	void sync_order_keys ();
-	bool ignore_route_reorder;
-	bool ignore_sync;
+	void sync_order_keys (const char *src);
+	bool strip_redisplay_does_not_reset_order_keys;
+	bool strip_redisplay_does_not_sync_order_keys;
 
 	static const int32_t default_width = 478;
 	static const int32_t default_height = 765;
