@@ -17,9 +17,6 @@
 
 */
 
-#include "i18n.h"
-#include "new_session_dialog.h"
-
 #include <pbd/error.h>
 
 #include <ardour/recent_sessions.h>
@@ -43,12 +40,15 @@ using namespace PBD;
 
 #include "opts.h"
 #include "utils.h"
+#include "i18n.h"
+#include "new_session_dialog.h"
 
 NewSessionDialog::NewSessionDialog()
 	: ArdourDialog ("session control")
 {
 	in_destructor = false;
 	session_name_label = new Gtk::Label(_("Name :"));
+	last_name_page = NewPage;
 	m_name = new Gtk::Entry();
 	m_name->set_text(ARDOUR_COMMAND_LINE::session_name);
 
@@ -83,17 +83,17 @@ NewSessionDialog::NewSessionDialog()
 	m_template = new Gtk::FileChooserButton();
 	m_create_control_bus = new Gtk::CheckButton(_("Create Monitor Bus"));
 	
-	Gtk::Adjustment *m_control_bus_channel_count_adj = Gtk::manage(new Gtk::Adjustment(2, 0, 100, 1, 10, 10));
+	Gtk::Adjustment *m_control_bus_channel_count_adj = Gtk::manage(new Gtk::Adjustment(2, 0, 100, 1, 10));
 	m_control_bus_channel_count = new Gtk::SpinButton(*m_control_bus_channel_count_adj, 1, 0);
 	
-	Gtk::Adjustment *m_master_bus_channel_count_adj = Gtk::manage(new Gtk::Adjustment(2, 0, 100, 1, 10, 10));
+	Gtk::Adjustment *m_master_bus_channel_count_adj = Gtk::manage(new Gtk::Adjustment(2, 0, 100, 1, 10));
 	m_master_bus_channel_count = new Gtk::SpinButton(*m_master_bus_channel_count_adj, 1, 0);
 	m_create_master_bus = new Gtk::CheckButton(_("Create Master Bus"));
 	advanced_table = new Gtk::Table(2, 2, true);
 	m_connect_inputs = new Gtk::CheckButton(_("Automatically Connect to Physical Inputs"));
 	m_limit_input_ports = new Gtk::CheckButton(_("Use only"));
 	
-	Gtk::Adjustment *m_input_limit_count_adj = Gtk::manage(new Gtk::Adjustment(1, 0, 100, 1, 10, 10));
+	Gtk::Adjustment *m_input_limit_count_adj = Gtk::manage(new Gtk::Adjustment(1, 0, 100, 1, 10));
 	m_input_limit_count = new Gtk::SpinButton(*m_input_limit_count_adj, 1, 0);
 	input_port_limit_hbox = new Gtk::HBox(false, 0);
 	input_port_vbox = new Gtk::VBox(false, 0);
@@ -106,7 +106,7 @@ NewSessionDialog::NewSessionDialog()
 	m_connect_outputs = new Gtk::CheckButton(_("Automatically Connect Outputs"));
 	m_limit_output_ports = new Gtk::CheckButton(_("Use only"));
 	
-	Gtk::Adjustment *m_output_limit_count_adj = Gtk::manage(new Gtk::Adjustment(1, 0, 100, 1, 10, 10));
+	Gtk::Adjustment *m_output_limit_count_adj = Gtk::manage(new Gtk::Adjustment(1, 0, 100, 1, 10));
 	m_output_limit_count = new Gtk::SpinButton(*m_output_limit_count_adj, 1, 0);
 	output_port_limit_hbox = new Gtk::HBox(false, 0);
 	output_port_vbox = new Gtk::VBox(false, 0);
@@ -375,9 +375,7 @@ NewSessionDialog::NewSessionDialog()
 		set_icon_list (window_icons);
 	}
 
-	WindowTitle title(Glib::get_application_name());
-	title += _("Session Control");
-	set_title(title.get_string());
+	set_title(_("Session Control"));
 
 	set_position (Gtk::WIN_POS_MOUSE);
 	set_resizable(false);
@@ -400,20 +398,28 @@ NewSessionDialog::NewSessionDialog()
 
 	const char * const template_dir_name = X_("templates");
 
-	if (!path.empty()) {
-		string user_template_path = path + template_dir_name;
+	//if SYSTEM template folder exists, add it to the file chooser
+	const std::string sys_templates_path = ARDOUR::get_system_data_path() + "/" + template_dir_name;
 
-		if (Glib::file_test(user_template_path, Glib::FILE_TEST_IS_DIR))
-		{
-			m_template->set_current_folder (user_template_path);
+	if (Glib::file_test(sys_templates_path, Glib::FILE_TEST_IS_DIR))
+	{
+		m_template->add_shortcut_folder(sys_templates_path);
+		m_template->set_current_folder (sys_templates_path);
+	}
+
+	//if USER template folder exists, add it to the file chooser
+	const std::string user_template_path = ARDOUR::get_user_ardour_path() + template_dir_name;
+	bool utp_exists = true;
+
+	if (!Glib::file_test(user_template_path, Glib::FILE_TEST_IS_DIR)) {
+		if (g_mkdir_with_parents (user_template_path.c_str(), 0755)) {
+			utp_exists = false;
 		}
 	}
 
-	const std::string sys_templates_dir = ARDOUR::get_system_data_path() + template_dir_name;
-	
-	if (Glib::file_test(sys_templates_dir, Glib::FILE_TEST_IS_DIR))
-	{
-		m_template->add_shortcut_folder(sys_templates_dir);
+	if (utp_exists) {
+		m_template->add_shortcut_folder(user_template_path);
+		m_template->set_current_folder (user_template_path);
 	}
 
 	m_template->set_title(_("select template"));
@@ -433,11 +439,18 @@ NewSessionDialog::NewSessionDialog()
 	m_folder->set_current_folder(getenv ("HOME"));
 	m_folder->set_title(_("select directory"));
 
+#ifdef GTKOSX
+	m_folder->add_shortcut_folder("/Volumes");
+	m_open_filechooser->add_shortcut_folder("/Volumes");
+#endif
+
 	on_new_session_page = true;
 	m_notebook->set_current_page(0);
 	m_notebook->show();
 	m_notebook->show_all_children();
 
+	engine_page_session_folder = X_("");
+	engine_page_session_name = X_("");
 
 	set_default_response (Gtk::RESPONSE_OK);
 	if (!ARDOUR_COMMAND_LINE::session_name.length()) {
@@ -478,21 +491,41 @@ NewSessionDialog::run ()
 		/* nothing to display */
 		return Gtk::RESPONSE_OK;
 	}
-
+	if (!(page_set & NewPage) && !(page_set & OpenPage)) {
+		set_response_sensitive (Gtk::RESPONSE_OK, true);
+	}
 	return ArdourDialog::run ();
 }
 
 void
 NewSessionDialog::set_have_engine (bool yn)
 {
-	if (yn) {
-		m_notebook->remove_page (engine_control);
-		page_set = Pages (page_set & ~EnginePage);
-	} else {
-		if (!(page_set & EnginePage)) {
+
+	m_notebook->remove_page (engine_control);
+	page_set = Pages (page_set & ~EnginePage);
+	
+	if (!yn) {
+
+		engine_control.discover_servers ();
+
+		if (engine_control.interface_chosen()) {
 			m_notebook->append_page (engine_control, _("Audio Setup"));
 			m_notebook->show_all_children();
 			page_set = Pages (page_set | EnginePage);
+		} else {
+			m_notebook->prepend_page (engine_control, _("Audio Setup"));
+			page_set = Pages (page_set | EnginePage);
+
+			/* no interface ever selected - make it the first and only page */
+			if (page_set & NewPage) {
+				m_notebook->remove_page (*new_session_table);
+				page_set = Pages (page_set & ~NewPage);
+			}
+			if (page_set & OpenPage) {
+				m_notebook->remove_page (*open_session_vbox);
+				page_set = Pages (page_set & ~OpenPage);
+			}
+			m_notebook->show_all_children();
 		}
 	}
 }
@@ -532,13 +565,13 @@ void
 NewSessionDialog::set_session_name (const Glib::ustring& name)
 {
 	m_name->set_text (name);
+	engine_page_session_name = name;
 }
 
 void
 NewSessionDialog::set_session_folder(const Glib::ustring& dir)
 {
 	Glib::ustring realdir = dir;
-	char* res;
 
 	/* this little tangled mess is a result of 4 things:
 
@@ -558,23 +591,24 @@ NewSessionDialog::set_session_folder(const Glib::ustring& dir)
 
 	char buf[PATH_MAX];
 
-	if((res = realpath (dir.c_str(), buf)) != 0) {
+	if(realpath (dir.c_str(), buf) != 0) {
 		if (!Glib::file_test (dir, Glib::FILE_TEST_IS_DIR)) {
 			realdir = Glib::path_get_dirname (realdir);
 		}
 		m_folder->set_current_folder (realdir);
+		engine_page_session_folder = realdir;
 	}
 
 	
 #else 
+	char* res;
 	if (!Glib::file_test (dir, Glib::FILE_TEST_IS_DIR)) {
 		realdir = Glib::path_get_dirname (realdir);
-		cerr << "didn't exist, use " << realdir << endl;
 	}
 
 	if ((res = canonicalize_file_name (realdir.c_str())) != 0) {
-		cerr << "canonical, use " << res << endl;
 		m_folder->set_current_folder (res);
+		engine_page_session_folder = res;
 		free (res);
 	}
 	
@@ -585,8 +619,8 @@ NewSessionDialog::set_session_folder(const Glib::ustring& dir)
 std::string
 NewSessionDialog::session_name() const
 {
-        std::string str = Glib::filename_from_utf8(m_open_filechooser->get_filename());
-	std::string::size_type position = str.find_last_of ('/');
+        std::string str = Glib::filename_from_utf8 (m_open_filechooser->get_filename());
+	std::string::size_type position = str.find_last_of (G_DIR_SEPARATOR);
 	str = str.substr (position+1);
 	position = str.find_last_of ('.');
 	str = str.substr (0, position);
@@ -602,9 +636,18 @@ NewSessionDialog::session_name() const
 
 	switch (which_page()) {
 	case NewPage:
-	case EnginePage:
-		/* new or audio setup pages */
 	        return Glib::filename_from_utf8(m_name->get_text());
+
+	case EnginePage:
+		if (!(page_set & (OpenPage|NewPage))) {
+			return engine_page_session_name;
+		} else if (last_name_page == NewPage) {
+			return Glib::filename_from_utf8(m_name->get_text());
+		} else {
+			/* relax and use the open page stuff at the end */
+		}
+		break;
+
 	default:
 		break;
 	} 
@@ -612,6 +655,7 @@ NewSessionDialog::session_name() const
 	if (m_treeview->get_selection()->count_selected_rows() == 0) {
 		return Glib::filename_from_utf8(str);
 	}
+
 	Gtk::TreeModel::iterator i = m_treeview->get_selection()->get_selected();
 	return (*i)[recent_columns.visible_name];
 }
@@ -624,9 +668,13 @@ NewSessionDialog::session_folder() const
 	        return Glib::filename_from_utf8(m_folder->get_filename());
 		
 	case EnginePage:
-		if (page_set == EnginePage) {
-			/* just engine page, nothing else : use m_folder since it should be set */
+		if (!(page_set & (OpenPage|NewPage))) {
+			return Glib::filename_from_utf8(engine_page_session_folder);
+		} else if (last_name_page == NewPage) {
+			/* use m_folder since it should be set */
 			return Glib::filename_from_utf8(m_folder->get_filename());
+		} else {
+			/* relax and use the open page stuff at the end */
 		}
 		break;
 
@@ -748,7 +796,7 @@ NewSessionDialog::which_page () const
 	} else if (page_set == EnginePage) {
 		return EnginePage;
 
-	} else if (page_set == NewPage|OpenPage) {
+	} else if (page_set == (NewPage|OpenPage)) {
 		switch (num) {
 		case 0:
 			return NewPage;
@@ -756,32 +804,63 @@ NewSessionDialog::which_page () const
 			return OpenPage;
 		}
 
-	} else if (page_set == NewPage|EnginePage) {
-		switch (num) {
-		case 0:
-			return NewPage;
-		default:
-			return EnginePage;
+	} else if (page_set == (NewPage|EnginePage)) {
+		if (engine_control.interface_chosen()) {
+			switch (num) {
+			case 0:
+				return NewPage;
+			default:
+				return EnginePage;
+			} 
+		} else {
+			switch (num) {
+			case 0:
+				return EnginePage;
+			default:
+				return NewPage;
+			} 
 		}
 
-	} else if (page_set == NewPage|EnginePage|OpenPage) {
-		switch (num) {
-		case 0:
-			return NewPage;
-		case 1:
-			return OpenPage;
-		default:
-			return EnginePage;
+	} else if (page_set == (NewPage|EnginePage|OpenPage)) {
+		if (engine_control.interface_chosen()) {
+			switch (num) {
+			case 0:
+				return NewPage;
+			case 1:
+				return OpenPage;
+			default:
+				return EnginePage;
+			}
+		} else {
+			switch (num) {
+			case 0:
+				return EnginePage;
+			case 1:
+				return NewPage;
+			default:
+				return OpenPage;
+			}
 		}
 
-	} else if (page_set == OpenPage|EnginePage) {
-		switch (num) {
-		case 0:
-			return OpenPage;
-		default:
-			return EnginePage;
+	} else if (page_set == (OpenPage|EnginePage)) {
+		if (engine_control.interface_chosen()) {
+			switch (num) {
+			case 0:
+				return OpenPage;
+			default:
+				return EnginePage;
+			}
+		} else {
+			switch (num) {
+			case 0:
+				return EnginePage;
+			default:
+				return OpenPage;
+			}
 		}
 	}
+
+	return NewPage; /* shouldn't get here */
 }
 
 void
@@ -827,17 +906,22 @@ NewSessionDialog::notebook_page_changed (GtkNotebookPage* np, uint pagenum)
 		} else {
 			set_response_sensitive (Gtk::RESPONSE_OK, true);
 		}
+		last_name_page = OpenPage;
 		break;
 
 	case EnginePage:
 		on_new_session_page = false;
-		m_okbutton->set_label(_("Open"));
+		if (!engine_control.interface_chosen()) {
+			m_okbutton->set_label(_("Start Audio Engine"));
+		} else {
+			m_okbutton->set_label(_("Start"));
+		}
 		m_okbutton->set_image (*(manage (new Gtk::Image (Gtk::Stock::OPEN, Gtk::ICON_SIZE_BUTTON))));
 		set_response_sensitive (Gtk::RESPONSE_NONE, false);
 		set_response_sensitive (Gtk::RESPONSE_OK, true);
 		break;
 
-	default:
+	case NewPage:
 		on_new_session_page = true;
 		m_okbutton->set_label(_("New"));
 		m_okbutton->set_image (*(new Gtk::Image (Gtk::Stock::NEW, Gtk::ICON_SIZE_BUTTON)));
@@ -847,6 +931,8 @@ NewSessionDialog::notebook_page_changed (GtkNotebookPage* np, uint pagenum)
 		} else {
 			set_response_sensitive (Gtk::RESPONSE_OK, true);
 		}
+		last_name_page = NewPage;
+		break;
 	}
 }
 
@@ -988,9 +1074,9 @@ NewSessionDialog::reset_recent()
 		std::vector<const gchar*> item;
 		std::string fullpath = *(*i);
 		
-		/* remove any trailing / */
+		/* remove any trailing separator */
 		
-		if (fullpath[fullpath.length()-1] == '/') {
+		if (fullpath[fullpath.length()-1] == G_DIR_SEPARATOR) {
 			fullpath = fullpath.substr (0, fullpath.length()-1);
 		}
 	    
