@@ -28,6 +28,7 @@
 #include <ardour/auditioner.h>
 #include <ardour/sndfilesource.h>
 #include <ardour/crossfade.h>
+#include <ardour/profile.h>
 #include <midi++/manager.h>
 #include <midi++/factory.h>
 #include <gtkmm2ext/stop_signal.h>
@@ -95,6 +96,8 @@ OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
 	  mmc_send_device_id_adjustment (0.0, 0.0, (double) 0x7f, 1.0, 16.0),
 	  mmc_send_device_id_spinner (mmc_send_device_id_adjustment),
 	  add_midi_port_button (_("Add new MIDI port")),
+	  initial_program_change_adjustment (0.0, -1.0, (double) 0x7f, 1.0, 16.0),
+	  initial_program_change_spinner (initial_program_change_adjustment),
 
 	  /* Click */
 
@@ -113,16 +116,15 @@ OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
 {
 	using namespace Notebook_Helpers;
 
+	first_click_setup = true;
 	click_io_selector = 0;
 	auditioner_io_selector = 0;
 	session = 0;
 	
-	WindowTitle title(Glib::get_application_name());
-	title += _("Preferences");
-	set_title(title.get_string());
+	set_title(_("Preferences"));
 
 	set_default_size (300, 300);
-	set_wmclass (X_("ardour_preferences"), "Ardour");
+	set_wmclass (X_("ardour_preferences"), PROGRAM_NAME);
 
 	set_name ("Preferences");
 	add_events (Gdk::KEY_PRESS_MASK|Gdk::KEY_RELEASE_MASK);
@@ -145,9 +147,11 @@ OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
 	setup_keyboard_options ();
 	setup_auditioner_editor ();
 
-	notebook.pages().push_back (TabElem (sync_packer, _("Sync")));
-	notebook.pages().push_back (TabElem (path_table, _("Paths/Files")));
-	notebook.pages().push_back (TabElem (keyboard_mouse_table, _("Kbd/Mouse")));
+	if (!Profile->get_sae()) {
+		notebook.pages().push_back (TabElem (sync_packer, _("Sync")));
+		notebook.pages().push_back (TabElem (path_table, _("Paths/Files")));
+		notebook.pages().push_back (TabElem (keyboard_mouse_table, _("Kbd/Mouse")));
+	}
 	notebook.pages().push_back (TabElem (click_packer, _("Click")));
 	notebook.pages().push_back (TabElem (audition_packer, _("Audition")));
 	notebook.pages().push_back (TabElem (misc_packer, _("Misc")));
@@ -253,12 +257,14 @@ OptionEditor::add_session_paths ()
 	session_raid_entry.set_text(session->raid_path());
 }
 
+#ifndef GTKOSX
 static void
 font_scale_changed (Gtk::Adjustment* adj)
 {
 	Config->set_font_scale((long)floor (adj->get_value() * 1024));
 	reset_dpi();
 }
+#endif
 
 void
 OptionEditor::setup_misc_options ()
@@ -526,9 +532,26 @@ OptionEditor::setup_midi_options ()
 
 	mmc_send_device_id_spinner.set_value(Config->get_mmc_send_device_id ());
 
+	hbox = manage (new HBox);
+	hbox->set_border_width (6);
+	hbox->set_spacing (6);
+	label = (manage (new Label (_("Startup program change")))); 
+	hbox->pack_start (initial_program_change_spinner, false, false);
+	hbox->pack_start (*label, false, false);
+	midi_packer.pack_start (*hbox, false, false);
+
+	initial_program_change_spinner.set_value (Config->get_initial_program_change());
+	initial_program_change_adjustment.signal_value_changed().connect (mem_fun (*this, &OptionEditor::initial_program_change_adjusted));
+
 	add_midi_port_button.signal_clicked().connect (mem_fun (*this, &OptionEditor::add_midi_port));
 }
 
+void
+OptionEditor::initial_program_change_adjusted ()
+{
+	Config->set_initial_program_change (((int32_t) floor (initial_program_change_adjustment.get_value())) & 0x7f);
+}
+
 void
 OptionEditor::redisplay_midi_ports ()
 {
@@ -994,8 +1017,8 @@ void
 OptionEditor::clear_click_editor ()
 {
 	if (click_io_selector) {
-		click_packer.remove (*click_io_selector);
-		click_packer.remove (*click_gpm);
+		click_hpacker.remove (*click_io_selector);
+		click_hpacker.remove (*click_gpm);
 		delete click_io_selector;
 		delete click_gpm;
 		click_io_selector = 0;
@@ -1007,51 +1030,59 @@ void
 OptionEditor::setup_click_editor ()
 {
 	Label* label;
-	HBox* hpacker = manage (new HBox);
+
+	if (first_click_setup) {
+		
+		click_path_entry.set_name ("OptionsEntry");
+		click_emphasis_path_entry.set_name ("OptionsEntry");
+		
+		click_path_entry.signal_activate().connect (mem_fun(*this, &OptionEditor::click_sound_changed));
+		click_emphasis_path_entry.signal_activate().connect (mem_fun(*this, &OptionEditor::click_emphasis_sound_changed));
+		
+		click_path_entry.signal_focus_out_event().connect (bind (mem_fun(*this, &OptionEditor::focus_out_event_handler), &OptionEditor::click_sound_changed));
+		click_emphasis_path_entry.signal_focus_out_event().connect (bind (mem_fun(*this, &OptionEditor::focus_out_event_handler), &OptionEditor::click_emphasis_sound_changed));
+		
+		click_browse_button.set_name ("EditorGTKButton");
+		click_emphasis_browse_button.set_name ("EditorGTKButton");
+
+		click_browse_button.signal_clicked().connect (mem_fun(*this, &OptionEditor::click_browse_clicked));
+		click_emphasis_browse_button.signal_clicked().connect (mem_fun(*this, &OptionEditor::click_emphasis_browse_clicked));
+
+		click_packer.set_border_width (12);
+		click_packer.set_spacing (5);
+
+		click_table.set_col_spacings (10);
+		
+		label = manage(new Label(_("Click audio file")));
+		label->set_name ("OptionsLabel");
+		click_table.attach (*label, 0, 1, 0, 1, FILL|EXPAND, FILL);
+		click_table.attach (click_path_entry, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, FILL);
+		click_table.attach (click_browse_button, 2, 3, 0, 1, FILL|EXPAND, FILL);
+		
+		label = manage(new Label(_("Click emphasis audiofile")));
+		label->set_name ("OptionsLabel");
+		click_table.attach (*label, 0, 1, 1, 2, FILL|EXPAND, FILL);
+		click_table.attach (click_emphasis_path_entry, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, FILL);
+		click_table.attach (click_emphasis_browse_button, 2, 3, 1, 2, FILL|EXPAND, FILL);
+
+		click_packer.pack_start (click_table, false, false);
+		click_packer.pack_start (click_hpacker, false, false);
+	
+
+		click_hpacker.set_spacing (10);
+
+		first_click_setup = false;
+	}
 
 	click_path_entry.set_sensitive (true);
 	click_emphasis_path_entry.set_sensitive (true);
 
-	click_path_entry.set_name ("OptionsEntry");
-	click_emphasis_path_entry.set_name ("OptionsEntry");
-	
-	click_path_entry.signal_activate().connect (mem_fun(*this, &OptionEditor::click_sound_changed));
-	click_emphasis_path_entry.signal_activate().connect (mem_fun(*this, &OptionEditor::click_emphasis_sound_changed));
-
-	click_path_entry.signal_focus_out_event().connect (bind (mem_fun(*this, &OptionEditor::focus_out_event_handler), &OptionEditor::click_sound_changed));
-	click_emphasis_path_entry.signal_focus_out_event().connect (bind (mem_fun(*this, &OptionEditor::focus_out_event_handler), &OptionEditor::click_emphasis_sound_changed));
-
-	click_browse_button.set_name ("EditorGTKButton");
-	click_emphasis_browse_button.set_name ("EditorGTKButton");
-	click_browse_button.signal_clicked().connect (mem_fun(*this, &OptionEditor::click_browse_clicked));
-	click_emphasis_browse_button.signal_clicked().connect (mem_fun(*this, &OptionEditor::click_emphasis_browse_clicked));
-
-	click_packer.set_border_width (12);
-	click_packer.set_spacing (5);
-
 	click_io_selector = new IOSelector (*session, session->click_io(), false);
-	click_gpm = new GainMeter (session->click_io(), *session);
+	click_gpm = new GainMeter (*session);
+	click_gpm->set_io (session->click_io());
 
-	click_table.set_col_spacings (10);
-	
-	label = manage(new Label(_("Click audio file")));
-	label->set_name ("OptionsLabel");
-	click_table.attach (*label, 0, 1, 0, 1, FILL|EXPAND, FILL);
-	click_table.attach (click_path_entry, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, FILL);
-	click_table.attach (click_browse_button, 2, 3, 0, 1, FILL|EXPAND, FILL);
-	
-	label = manage(new Label(_("Click emphasis audiofile")));
-	label->set_name ("OptionsLabel");
-	click_table.attach (*label, 0, 1, 1, 2, FILL|EXPAND, FILL);
-	click_table.attach (click_emphasis_path_entry, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, FILL);
-	click_table.attach (click_emphasis_browse_button, 2, 3, 1, 2, FILL|EXPAND, FILL);
-
-	hpacker->set_spacing (10);
-	hpacker->pack_start (*click_io_selector, false, false);
-	hpacker->pack_start (*click_gpm, false, false);
-
-	click_packer.pack_start (click_table, false, false);
-	click_packer.pack_start (*hpacker, false, false);
+	click_hpacker.pack_start (*click_io_selector, false, false);
+	click_hpacker.pack_start (*click_gpm, false, false);
 
 	click_packer.show_all ();
 }
@@ -1090,7 +1121,8 @@ void
 OptionEditor::connect_audition_editor ()
 {
 	auditioner_io_selector = new IOSelector (*session, session->the_auditioner(), false);
-	auditioner_gpm = new GainMeter (session->the_auditioner(), *session);
+	auditioner_gpm = new GainMeter (*session);
+	auditioner_gpm->set_io (session->the_auditioner());
 
 	audition_hpacker.pack_start (*auditioner_io_selector, false, false);
 	audition_hpacker.pack_start (*auditioner_gpm, false, false);
@@ -1320,35 +1352,6 @@ OptionEditor::edit_button_changed ()
 	Keyboard::set_edit_button ((guint) edit_button_adjustment.get_value());
 }
 
-void
-OptionEditor::fixup_combo_size (Gtk::ComboBoxText& combo, vector<string>& strings)
-{
-	/* find the widest string */
-
-	string::size_type maxlen = 0;
-	string maxstring;
-
-	for (vector<string>::iterator i = strings.begin(); i != strings.end(); ++i) {
-		string::size_type l;
-
-		if ((l = (*i).length()) > maxlen) {
-			maxlen = l;
-			maxstring = *i;
-		}
-	}
-
-	/* try to include ascenders and descenders */
-
-	if (maxstring.length() > 2) {
-		maxstring[0] = 'g';
-		maxstring[1] = 'l';
-	}
-
-	const guint32 FUDGE = 10; // Combo's are stupid - they steal space from the entry for the button
-
-	set_size_request_to_display_given_text (combo, maxstring.c_str(), 10 + FUDGE, 10);
-}
-
 void
 OptionEditor::parameter_changed (const char* parameter_name)
 {
