@@ -26,6 +26,7 @@
 #include <gtkmm2ext/selector.h>
 
 #include <ardour/plugin.h>
+#include "plugin_interest.h"
 
 namespace ARDOUR {
 	class Session;
@@ -36,16 +37,22 @@ class PluginSelector : public ArdourDialog
 {
   public:
 	PluginSelector (ARDOUR::PluginManager *);
-	sigc::signal<void,boost::shared_ptr<ARDOUR::Plugin> > PluginCreated;
 
+	void set_interested_object (PluginInterestedObject&);
+	
 	int run (); // XXX should we try not to overload the non-virtual Gtk::Dialog::run() ?
 
 	void set_session (ARDOUR::Session*);
 	void on_show ();
 
+	Gtk::Menu* plugin_menu ();
+	void show_manager ();
+
   private:
+	PluginInterestedObject* interested_object;
+
 	ARDOUR::Session* session;
-	Gtk::ScrolledWindow scroller;  // Available plugins
+	Gtk::ScrolledWindow scroller;   // Available plugins
 	Gtk::ScrolledWindow ascroller;  // Added plugins
 
 	Gtk::ComboBoxText filter_mode;
@@ -58,6 +65,8 @@ class PluginSelector : public ArdourDialog
 	
 	struct PluginColumns : public Gtk::TreeModel::ColumnRecord {
 		PluginColumns () {
+			add (favorite);
+			add (hidden);
 			add (name);
 			add (type_name);
 			add (category);
@@ -66,6 +75,8 @@ class PluginSelector : public ArdourDialog
 			add (outs);
 			add (plugin);
 		}
+		Gtk::TreeModelColumn<bool> favorite;
+		Gtk::TreeModelColumn<bool> hidden;
 		Gtk::TreeModelColumn<std::string> name;
 		Gtk::TreeModelColumn<std::string> type_name;
 		Gtk::TreeModelColumn<std::string> category;
@@ -108,10 +119,19 @@ class PluginSelector : public ArdourDialog
 	void added_list_selection_changed();
 	void display_selection_changed();
 	void btn_apply_clicked();
-	void use_plugin (ARDOUR::PluginInfoPtr);
-	void cleanup ();
+	ARDOUR::PluginPtr load_plugin (ARDOUR::PluginInfoPtr);
 	bool show_this_plugin (const ARDOUR::PluginInfoPtr&, const std::string&);
 	void setup_filter_string (std::string&);
+
+	void favorite_changed (const Glib::ustring& path);
+	void hidden_changed (const Glib::ustring& path);
+	bool in_row_change;
+
+	void plugin_chosen_from_menu (const ARDOUR::PluginInfoPtr&);
+
+	Gtk::Menu* create_favs_menu (ARDOUR::PluginInfoList&);
+	Gtk::Menu* create_by_creator_menu (ARDOUR::PluginInfoList&);
+	Gtk::Menu* create_by_category_menu (ARDOUR::PluginInfoList&);
 };
 
 #endif // __ardour_plugin_selector_h__
