@@ -94,11 +94,14 @@ class PublicEditor : public Gtk::Window, public PBD::StatefulThingWithGoingAway
 	virtual void set_mouse_mode (Editing::MouseMode, bool force = false) = 0;
 	virtual void step_mouse_mode (bool next) = 0;
 	virtual Editing::MouseMode current_mouse_mode () = 0;
+
+	/* CMT */
+	virtual void connect_to_image_compositor()  = 0;
 	virtual void add_imageframe_time_axis(const std::string & track_name, void*)  = 0;
 	virtual void add_imageframe_marker_time_axis(const std::string & track_name, TimeAxisView* marked_track, void*)  = 0;
-	virtual void connect_to_image_compositor()  = 0;
 	virtual void scroll_timeaxis_to_imageframe_item(const TimeAxisViewItem* item)  = 0;
 	virtual TimeAxisView* get_named_time_axis(const std::string & name)  = 0;
+
 	virtual void consider_auditioning (boost::shared_ptr<ARDOUR::Region>) = 0;
 	virtual void set_show_waveforms (bool yn) = 0;
 	virtual bool show_waveforms() const = 0;
@@ -106,10 +109,13 @@ class PublicEditor : public Gtk::Window, public PBD::StatefulThingWithGoingAway
 	virtual bool show_waveforms_recording() const = 0;
 	virtual void new_region_from_selection () = 0;
 	virtual void separate_region_from_selection () = 0;
-	virtual void toggle_playback (bool with_abort) = 0;
 	virtual void transition_to_rolling (bool fwd) = 0;
-	virtual nframes_t unit_to_frame (double unit) const = 0;
-	virtual double frame_to_unit (nframes_t frame) const = 0;
+	virtual nframes64_t unit_to_frame (double unit) const = 0;
+	// XXX remove me when libardour goes nframes64_t
+	double frame_to_unit (nframes_t frame) const { 
+		return frame_to_unit ((nframes64_t) frame);
+	}
+	virtual double frame_to_unit (nframes64_t frame) const = 0;
 	virtual double frame_to_unit (double frame) const = 0;
 	virtual nframes64_t pixel_to_frame (double pixel) const = 0;
 	virtual gulong frame_to_pixel (nframes64_t frame) const = 0;
@@ -137,38 +143,44 @@ class PublicEditor : public Gtk::Window, public PBD::StatefulThingWithGoingAway
 	virtual void select_all_tracks () = 0;
 	virtual void set_selected_track (TimeAxisView&, Selection::Operation op = Selection::Set, bool no_remove = false) = 0;
 	virtual void set_selected_mixer_strip (TimeAxisView&) = 0;
-	virtual void hide_track_in_display (TimeAxisView& tv) = 0;
+	virtual void hide_track_in_display (TimeAxisView& tv, bool temporary = false) = 0;
 	virtual void show_track_in_display (TimeAxisView& tv) = 0;
+	virtual void set_stationary_playhead (bool yn) = 0;
+	virtual void toggle_stationary_playhead () = 0;
+	virtual bool stationary_playhead() const = 0;
 	virtual void set_follow_playhead (bool yn) = 0;
 	virtual void toggle_follow_playhead () = 0;
 	virtual bool follow_playhead() const = 0;
 	virtual bool dragging_playhead() const = 0;
+	virtual double get_physical_screen_width() const = 0;
 	virtual void ensure_float (Gtk::Window&) = 0;
 	virtual void show_window () = 0;
 	virtual TrackViewList* get_valid_views (TimeAxisView*, ARDOUR::RouteGroup* grp = 0) = 0;
-	virtual nframes_t leftmost_position() const = 0;
-	virtual nframes_t current_page_frames() const = 0;
+	virtual nframes64_t leftmost_position() const = 0;
+	virtual nframes64_t current_page_frames() const = 0;
 	virtual void temporal_zoom_step (bool coarser) = 0;
 	virtual void scroll_tracks_down_line () = 0;
 	virtual void scroll_tracks_up_line () = 0;
 	virtual bool new_regionviews_display_gain () = 0;
 	virtual void prepare_for_cleanup () = 0;
-	virtual void reset_x_origin (nframes_t frame) = 0;
+	virtual void finish_cleanup () = 0;
+	virtual void reset_x_origin (nframes64_t frame) = 0;
 	virtual void remove_last_capture () = 0;
 	virtual void maximise_editing_space() = 0;
 	virtual void restore_editing_space() = 0;
 	virtual nframes64_t get_preferred_edit_position (bool ignore_playhead = false) = 0;
 	virtual void toggle_meter_updating() = 0;
 	virtual void split_region_at_points (boost::shared_ptr<ARDOUR::Region>, ARDOUR::AnalysisFeatureList&, bool can_ferret) = 0;
-	virtual void mouse_add_new_marker (nframes_t where, bool is_cd=false, bool is_xrun=false) = 0;
+	virtual void mouse_add_new_marker (nframes64_t where, bool is_cd=false, bool is_xrun=false) = 0;
 	virtual void foreach_time_axis_view (sigc::slot<void,TimeAxisView&>) = 0;
+	virtual void add_to_idle_resize (TimeAxisView*,uint32_t) = 0;
 
 	sigc::signal<void> ZoomFocusChanged;
 	sigc::signal<void> ZoomChanged;
 	sigc::signal<void> Resized;
 	sigc::signal<void> Realized;
 	sigc::signal<void> GoingAway;
-	sigc::signal<void,nframes_t> UpdateAllTransportClocks;
+	sigc::signal<void,nframes64_t> UpdateAllTransportClocks;
 
 	Glib::RefPtr<Gtk::ActionGroup> editor_actions;
 
@@ -209,9 +221,19 @@ class PublicEditor : public Gtk::Window, public PBD::StatefulThingWithGoingAway
 	virtual bool canvas_markerview_start_handle_event(GdkEvent* event, ArdourCanvas::Item*,MarkerView*) = 0;
 	virtual bool canvas_markerview_end_handle_event(GdkEvent* event, ArdourCanvas::Item*,MarkerView*) = 0;
 
+	static const int window_border_width;
+	static const int container_border_width;
+	static const int vertical_spacing;
+	static const int horizontal_spacing;
+
+	virtual gdouble get_trackview_group_vertical_offset () const = 0;
+	virtual gdouble get_canvas_timebars_vsize () const = 0;
+	virtual ArdourCanvas::Group* get_background_group () const = 0;
+	virtual ArdourCanvas::Group* get_trackview_group () const = 0;
+
 	static PublicEditor* _instance;
 
-	friend class PluginUIWindow;
+	friend bool relay_key_press (GdkEventKey* ev, Gtk::Window* win);
 };
 
 #endif // __gtk_ardour_public_editor_h__
