@@ -262,8 +262,10 @@ SoundFileBox::setup_labels (const ustring& filename)
 		samplerate.set_name ("NewSessionSR2Label");
 	}
 
-	length_clock.set (sf_info.length, true);
-	timecode_clock.set (sf_info.timecode, true);
+	double src_coef = (double) _session->nominal_frame_rate() / sf_info.samplerate;
+
+	length_clock.set (sf_info.length * src_coef + 0.5, true);
+	timecode_clock.set (sf_info.timecode * src_coef + 0.5, true);
 
 	// this is a hack that is fixed in trunk, i think (august 26th, 2007)
 
@@ -394,21 +396,24 @@ SoundFileBrowser::SoundFileBrowser (Gtk::Window& parent, string title, ARDOUR::S
 	  found_list (ListStore::create(found_list_columns)),
 	  freesound_list (ListStore::create(freesound_list_columns)),
 	  chooser (FILE_CHOOSER_ACTION_OPEN),
-	  found_list_view (found_list),
-	  freesound_list_view (freesound_list),
 	  preview (persistent),
 	  found_search_btn (_("Search")),
-	  freesound_search_btn (_("Start Downloading"))
-
+	  found_list_view (found_list),
+	  freesound_search_btn (_("Start Downloading")),
+	  freesound_list_view (freesound_list)
 {
 	resetting_ourselves = false;
 	gm = 0;
 
-	if (ARDOUR::Profile->get_sae()) {
-		chooser.add_shortcut_folder_uri("file:///Library/GarageBand/Apple Loops");
-		chooser.add_shortcut_folder_uri("file:///Library/Application Support/GarageBand/Instrument Library/Sampler/Sampler Files");
-	}
 	
+#ifdef GTKOSX
+	chooser.add_shortcut_folder_uri("file:///Library/GarageBand/Apple Loops");
+	chooser.add_shortcut_folder_uri("file:///Library/Audio/Apple Loops");
+	chooser.add_shortcut_folder_uri("file:///Library/Application Support/GarageBand/Instrument Library/Sampler/Sampler Files");
+	
+	chooser.add_shortcut_folder_uri("file:///Volumes");
+#endif
+
 	//add the file chooser
 	{
 		chooser.set_border_width (12);
@@ -423,6 +428,10 @@ SoundFileBrowser::SoundFileBrowser (Gtk::Window& parent, string title, ARDOUR::S
 		chooser.set_select_multiple (true);
 		chooser.signal_update_preview().connect(mem_fun(*this, &SoundFileBrowser::update_preview));
 		chooser.signal_file_activated().connect (mem_fun (*this, &SoundFileBrowser::chooser_file_activated));
+#ifdef GTKOSX
+		/* some broken redraw behaviour - this is a bandaid */
+		chooser.signal_selection_changed().connect (mem_fun (chooser, &Widget::queue_draw));
+#endif
 
 		if (!persistent_folder.empty()) {
 			chooser.set_current_folder (persistent_folder);
@@ -430,16 +439,17 @@ SoundFileBrowser::SoundFileBrowser (Gtk::Window& parent, string title, ARDOUR::S
 		notebook.append_page (chooser, _("Browse Files"));
 	}
 	
+	hpacker.set_spacing (6);
+	hpacker.pack_start (notebook, true, true);
+	hpacker.pack_start (preview, false, false);
+	
+	get_vbox()->pack_start (hpacker, true, true);
+
 	//add tag search
 	{
 		VBox* vbox;
 		HBox* hbox;
 
-		hpacker.set_spacing (6);
-		hpacker.pack_start (notebook, true, true);
-		hpacker.pack_start (preview, false, false);
-
-		get_vbox()->pack_start (hpacker, true, true);
 
 		hbox = manage(new HBox);
 		hbox->pack_start (found_entry);
@@ -472,12 +482,6 @@ SoundFileBrowser::SoundFileBrowser (Gtk::Window& parent, string title, ARDOUR::S
 		HBox* passbox;
 		Label* label;
 
-		hpacker.set_spacing (6);
-		hpacker.pack_start (notebook, true, true);
-		hpacker.pack_start (preview, false, false);
-
-		get_vbox()->pack_start (hpacker, true, true);
-
 		passbox = manage(new HBox);
 		passbox->set_border_width (12);
 		passbox->set_spacing (6);
@@ -588,7 +592,8 @@ SoundFileBrowser::add_gain_meter ()
 		delete gm;
 	}
 
-	gm = new GainMeter (session->the_auditioner(), *session);
+	gm = new GainMeter (*session);
+	gm->set_io (session->the_auditioner());
 
 	meter_packer.set_border_width (12);
 	meter_packer.pack_start (*gm, false, true);
@@ -717,7 +722,7 @@ SoundFileBrowser::found_search_clicked ()
 void*
 freesound_search_thread_entry (void* arg)
 {
-	PBD::ThreadCreated (pthread_self(), X_("Freesound Search"));
+	PBD::notify_gui_about_thread_creation (pthread_self(), X_("Freesound Search"));
 
 	static_cast<SoundFileBrowser*>(arg)->freesound_search_thread ();
 	
@@ -973,7 +978,7 @@ SoundFileOmega::reset_options ()
 				channel_strings.push_back (_("sequence files"));
 			}
 			if (same_size) {
-				channel_strings.push_back (_("all files in one region"));
+				channel_strings.push_back (_("all files in one track"));
 			}
 			
 		}
@@ -1040,7 +1045,7 @@ bool
 SoundFileOmega::bad_file_message()
 {
 	MessageDialog msg (*this, 
-			   _("One or more of the selected files\ncannot be used by Ardour"),
+			   string_compose (_("One or more of the selected files\ncannot be used by %1"), PROGRAM_NAME),
 			   true,
 			   Gtk::MESSAGE_INFO,
 			   Gtk::BUTTONS_OK);
@@ -1288,8 +1293,28 @@ SoundFileOmega::SoundFileOmega (Gtk::Window& parent, string title, ARDOUR::Sessi
 	disposition_map.insert (pair<ustring,ImportDisposition>(_("one region per file"), ImportDistinctFiles));
 	disposition_map.insert (pair<ustring,ImportDisposition>(_("one region per channel"), ImportDistinctChannels));
 	disposition_map.insert (pair<ustring,ImportDisposition>(_("all files in one region"), ImportMergeFiles));
+	disposition_map.insert (pair<ustring,ImportDisposition>(_("all files in one track"), ImportSerializeFiles));
 
 	chooser.signal_selection_changed().connect (mem_fun (*this, &SoundFileOmega::file_selection_changed));
+
+	/* set size requests for a couple of combos to allow them to display the longest text
+	   they will ever be asked to display.  This prevents them being resized when the user
+	   selects a file to import, which in turn prevents the size of the dialog from jumping
+	   around. */
+
+	vector<string> t;
+	t.push_back (_("one track per file"));
+	t.push_back (_("one track per channel"));
+	t.push_back (_("sequence files"));
+	t.push_back (_("all files in one region"));
+	set_size_request_to_display_given_text (channel_combo, t, COMBO_FUDGE + 10, 15);
+
+	t.clear ();
+	t.push_back (importmode2string (ImportAsTrack));
+	t.push_back (importmode2string (ImportToTrack));
+	t.push_back (importmode2string (ImportAsRegion));
+	t.push_back (importmode2string (ImportAsTapeTrack));
+	set_size_request_to_display_given_text (action_combo, t, COMBO_FUDGE + 10, 15);
 }
 
 void
