@@ -49,11 +49,11 @@ bool TimeAxisViewItem::have_name_font = false;
 const double TimeAxisViewItem::NAME_X_OFFSET = 15.0;
 const double TimeAxisViewItem::GRAB_HANDLE_LENGTH = 6 ;
 
+int TimeAxisViewItem::NAME_HEIGHT;
 double TimeAxisViewItem::NAME_Y_OFFSET;
 double TimeAxisViewItem::NAME_HIGHLIGHT_SIZE;
 double TimeAxisViewItem::NAME_HIGHLIGHT_THRESH;
 
-
 //---------------------------------------------------------------------------------------//
 // Constructor / Desctructor
 
@@ -69,16 +69,16 @@ double TimeAxisViewItem::NAME_HIGHLIGHT_THRESH;
  * @param duration the duration of this item
  */
 TimeAxisViewItem::TimeAxisViewItem(const string & it_name, ArdourCanvas::Group& parent, TimeAxisView& tv, double spu, Gdk::Color& base_color, 
-				   nframes_t start, nframes_t duration,
+				   nframes_t start, nframes_t duration, bool recording,
 				   Visibility vis)
-	: trackview (tv)
+	: trackview (tv), _height(1.0), _recregion(recording)
 {
 	if (!have_name_font) {
 
 		/* first constructed item sets up font info */
 
 		NAME_FONT = get_font_for_style (N_("TimeAxisViewItemName"));
-		
+
 		Gtk::Window win;
 		Gtk::Label foo;
 		win.add (foo);
@@ -90,7 +90,8 @@ TimeAxisViewItem::TimeAxisViewItem(const string & it_name, ArdourCanvas::Group&
 		layout->set_font_description (*NAME_FONT);
 		Gtkmm2ext::get_ink_pixel_size (layout, width, height);
 
-		NAME_Y_OFFSET = height + 6;
+		NAME_HEIGHT = height;
+		NAME_Y_OFFSET = (height / 2) + 8;
 		NAME_HIGHLIGHT_SIZE = height + 6;
 		NAME_HIGHLIGHT_THRESH = NAME_HIGHLIGHT_SIZE * 2;
 
@@ -105,6 +106,7 @@ TimeAxisViewItem::TimeAxisViewItem(const string & it_name, ArdourCanvas::Group&
 
 TimeAxisViewItem::TimeAxisViewItem (const TimeAxisViewItem& other)
 	: trackview (other.trackview)
+	, _height (other._height)
 {
 
 	Gdk::Color c;
@@ -122,13 +124,10 @@ TimeAxisViewItem::TimeAxisViewItem (const TimeAxisViewItem& other)
 	init (other.item_name, other.samples_per_unit, c, other.frame_position, other.item_duration, other.visibility);
 }
 
-
 void
 TimeAxisViewItem::init (const string& it_name, double spu, Gdk::Color& base_color, nframes_t start, nframes_t duration, Visibility vis)
 {
 	item_name = it_name ;
-	name_text_width = ::pixel_width (it_name, *NAME_FONT);
-	last_name_text_width = 0;
 	samples_per_unit = spu ;
 	should_show_selection = true;
 	frame_position = start ;
@@ -141,6 +140,8 @@ TimeAxisViewItem::init (const string& it_name, double spu, Gdk::Color& base_colo
 	show_vestigial = true;
 	visibility = vis;
 	_sensitive = true;
+	name_pixbuf_width = 0;
+	last_item_width = 0;
 
 	if (duration == 0) {
 		warning << "Time Axis Item Duration == 0" << endl ;
@@ -149,8 +150,8 @@ TimeAxisViewItem::init (const string& it_name, double spu, Gdk::Color& base_colo
 	vestigial_frame = new ArdourCanvas::SimpleRect (*group);
 	vestigial_frame->property_x1() = (double) 0.0;
 	vestigial_frame->property_y1() = (double) 1.0;
-	vestigial_frame->property_x2() = 2.0;
-	vestigial_frame->property_y2() = (double) trackview.height;
+	vestigial_frame->property_x2() = (double) 2.0;
+	vestigial_frame->property_y2() = (double) trackview.current_height();
 	vestigial_frame->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_VestigialFrame.get();
 	vestigial_frame->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_VestigialFrame.get();
 	vestigial_frame->hide ();
@@ -160,9 +161,8 @@ TimeAxisViewItem::init (const string& it_name, double spu, Gdk::Color& base_colo
 		frame->property_x1() = (double) 0.0;
 		frame->property_y1() = (double) 1.0;
 		frame->property_x2() = (double) trackview.editor.frame_to_pixel(duration);
-		frame->property_y2() = (double) trackview.height;
+		frame->property_y2() = (double) trackview.current_height();
 		frame->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_TimeAxisFrame.get();
-		frame->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_TimeAxisFrame.get();
 
 		/* by default draw all 4 edges */
 
@@ -195,10 +195,8 @@ TimeAxisViewItem::init (const string& it_name, double spu, Gdk::Color& base_colo
 			name_highlight->property_x1() = (double) 1.0;
 			name_highlight->property_x2() = (double) (trackview.editor.frame_to_pixel(item_duration)) - 1;
 		}
-		name_highlight->property_y1() = (double) (trackview.height - TimeAxisViewItem::NAME_HIGHLIGHT_SIZE);
-		name_highlight->property_y2() = (double) (trackview.height - 1);
-		name_highlight->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_NameHighlightFill.get();
-		name_highlight->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_NameHighlightOutline.get();
+		name_highlight->property_y1() = (double) (trackview.current_height() - TimeAxisViewItem::NAME_HIGHLIGHT_SIZE);
+		name_highlight->property_y2() = (double) (trackview.current_height() - 1);
 
 		name_highlight->set_data ("timeaxisviewitem", this);
 
@@ -206,22 +204,6 @@ TimeAxisViewItem::init (const string& it_name, double spu, Gdk::Color& base_colo
 		name_highlight = 0;
 	}
 
-	if (visibility & ShowNameText) {
-		name_text = new ArdourCanvas::Text (*group);
-		name_text->property_x() = (double) TimeAxisViewItem::NAME_X_OFFSET;
-		/* trackview.height is the bottom of the trackview. subtract 1 to get back to the bottom of the highlight,
-		   then NAME_Y_OFFSET to position the text in the vertical center of the highlight
-		*/
-		name_text->property_y() = (double) trackview.height - 1.0 - TimeAxisViewItem::NAME_Y_OFFSET;
-		name_text->property_font_desc() = *NAME_FONT;
-		name_text->property_anchor() = Gtk::ANCHOR_NW;
-
-		name_text->set_data ("timeaxisviewitem", this);
-		
-	} else {
-		name_text = 0;
-	}
-
 	/* create our grab handles used for trimming/duration etc */
 
 	if (visibility & ShowHandles) {
@@ -231,25 +213,34 @@ TimeAxisViewItem::init (const string& it_name, double spu, Gdk::Color& base_colo
 		frame_handle_start->property_y1() = (double) 1.0;
 		frame_handle_start->property_y2() = (double) TimeAxisViewItem::GRAB_HANDLE_LENGTH+1;
 		frame_handle_start->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_FrameHandle.get();
-		frame_handle_start->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_FrameHandle.get();
-		
+
 		frame_handle_end = new ArdourCanvas::SimpleRect (*group);
 		frame_handle_end->property_x1() = (double) (trackview.editor.frame_to_pixel(get_duration())) - (TimeAxisViewItem::GRAB_HANDLE_LENGTH);
 		frame_handle_end->property_x2() = (double) trackview.editor.frame_to_pixel(get_duration());
 		frame_handle_end->property_y1() = (double) 1;
 		frame_handle_end->property_y2() = (double) TimeAxisViewItem::GRAB_HANDLE_LENGTH + 1;
 		frame_handle_end->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_FrameHandle.get();
-		frame_handle_end->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_FrameHandle.get();
 
 	} else {
 		frame_handle_start = 0;
 		frame_handle_end = 0;
 	}
 
-	set_color (base_color) ;
+	if (visibility & ShowNameText) {
+		name_pixbuf = new ArdourCanvas::Pixbuf(*group);
+		name_pixbuf->property_x() = NAME_X_OFFSET;
+		name_pixbuf->property_y() = trackview.current_height() - 1.0 - NAME_Y_OFFSET;
+	
+	} else {
+		name_pixbuf = 0;
+	}
 
+	set_color (base_color) ;
+	
 	set_duration (item_duration, this) ;
 	set_position (start, this) ;
+
+	ColorsChanged.connect (mem_fun (*this, &TimeAxisViewItem::color_handler));
 }
 
 /**
@@ -492,7 +483,6 @@ TimeAxisViewItem::set_item_name(std::string new_name, void* src)
 	if (new_name != item_name) {
 		std::string temp_name = item_name ;
 		item_name = new_name ;
-		name_text_width = ::pixel_width (new_name, *NAME_FONT);
 		NameChanged (item_name, temp_name, src) ; /* EMIT_SIGNAL */
 	}
 }
@@ -560,11 +550,23 @@ TimeAxisViewItem::get_time_axis_view()
 void
 TimeAxisViewItem::set_name_text(const ustring& new_name)
 {
-	if (name_text) {
-		name_text->property_text() = new_name;
-		name_text_width = pixel_width (new_name, *NAME_FONT);
-		name_text_size_cache.clear ();
+	if (!name_pixbuf) {
+		return;
 	}
+
+	_name = new_name;
+
+	last_item_width = trackview.editor.frame_to_pixel(item_duration);
+	name_pixbuf_width = pixel_width (new_name, *NAME_FONT) + 2;
+	name_pixbuf->property_pixbuf() = pixbuf_from_ustring(new_name, NAME_FONT, 
+							     ARDOUR_UI::config()->canvasvar_TimeAxisViewItemName.get(),
+							     name_pixbuf_width, NAME_HEIGHT);
+}
+
+void
+TimeAxisViewItem::color_handler ()
+{
+	set_name_text (_name);
 }
 
 /**
@@ -575,37 +577,31 @@ TimeAxisViewItem::set_name_text(const ustring& new_name)
 void
 TimeAxisViewItem::set_height (double height)
 {
+	_height = height;
+
 	if (name_highlight) {
 		if (height < NAME_HIGHLIGHT_THRESH) {
 			name_highlight->hide();
-			if (name_text) {
-				name_text->hide();
-			}
 		} else {
-			name_highlight->show();
-			if (name_text) {
-				name_text->show();
-			}
+			name_highlight->show();	
 		}
 
 		if (height > NAME_HIGHLIGHT_SIZE) {
 			name_highlight->property_y1() = (double) height+1 - NAME_HIGHLIGHT_SIZE;
 			name_highlight->property_y2() = (double) height;
-		}
-		else {
+		} else {
 			/* it gets hidden now anyway */
-			name_highlight->property_y1() = (double) 1.0;
-			name_highlight->property_y2() = (double) height;
+			//name_highlight->property_y1() = (double) 1.0;
+			//name_highlight->property_y2() = (double) height;
 		}
 	}
 
-	if (name_text) {
-		name_text->property_y() = height+1 - NAME_Y_OFFSET;
+	if (name_pixbuf) {
 		if (height < NAME_HIGHLIGHT_THRESH) {
-			name_text->property_fill_color_rgba() =  fill_color;
-		}
-		else {
-			name_text->property_fill_color_rgba() = label_color;
+			name_pixbuf->hide();
+		} else {
+			name_pixbuf->property_y() = height+1 - NAME_Y_OFFSET;
+			name_pixbuf->show();
 		}
 	}
 
@@ -638,7 +634,7 @@ TimeAxisViewItem::get_canvas_frame()
 /**
  * 
  */
-ArdourCanvas::Item*
+ArdourCanvas::Group*
 TimeAxisViewItem::get_canvas_group()
 {
 	return (group) ;
@@ -656,10 +652,10 @@ TimeAxisViewItem::get_name_highlight()
 /**
  * 
  */
-ArdourCanvas::Text*
-TimeAxisViewItem::get_name_text()
+ArdourCanvas::Pixbuf*
+TimeAxisViewItem::get_name_pixbuf()
 {
-	return (name_text) ;
+	return (name_pixbuf) ;
 }
 
 /**
@@ -763,20 +759,6 @@ void
 TimeAxisViewItem::set_colors()
 {
 	set_frame_color() ;
-	if (name_text) {
-		double height = NAME_HIGHLIGHT_THRESH;
-
-		if (frame) {
-			height = frame->property_y2();
-		}
-
-		if (height < NAME_HIGHLIGHT_THRESH) {
-			name_text->property_fill_color_rgba() =  fill_color;
-		}
-		else {
-			name_text->property_fill_color_rgba() = label_color;
-		}
-	}
 
 	if (name_highlight) {
 		name_highlight->property_fill_color_rgba() = fill_color;
@@ -798,8 +780,13 @@ TimeAxisViewItem::set_frame_color()
 			UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_SelectedFrameBase.get(), &r, &g, &b, &a); 
 			frame->property_fill_color_rgba() = RGBA_TO_UINT(r, g, b, fill_opacity ? fill_opacity : a);// Lets still use the theme's opacity value if Opaque is not set
 		} else {
-			UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_FrameBase.get(), &r, &g, &b, &a);
-			frame->property_fill_color_rgba() = RGBA_TO_UINT(r, g, b, fill_opacity ? fill_opacity : a);
+			if (_recregion) {
+				UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_RecordingRect.get(), &r, &g, &b, &a);
+				frame->property_fill_color_rgba() = RGBA_TO_UINT(r, g, b, a);
+			} else {
+				UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_FrameBase.get(), &r, &g, &b, &a);
+				frame->property_fill_color_rgba() = RGBA_TO_UINT(r, g, b, fill_opacity ? fill_opacity : a);
+			}
 		}
 	}
 }
@@ -839,14 +826,7 @@ TimeAxisViewItem::set_samples_per_unit (double spu)
 void
 TimeAxisViewItem::reset_width_dependent_items (double pixel_width)
 {
-	if (pixel_width < GRAB_HANDLE_LENGTH * 2) {
-
-		if (frame_handle_start) {
-			frame_handle_start->hide();
-			frame_handle_end->hide();
-		}
-
-	} if (pixel_width < 2.0) {
+	if (pixel_width < 2.0) {
 
 		if (show_vestigial) {
 			vestigial_frame->show();
@@ -854,9 +834,10 @@ TimeAxisViewItem::reset_width_dependent_items (double pixel_width)
 
 		if (name_highlight) {
 			name_highlight->hide();
-			if (name_text) {
-				name_text->hide();
-			}
+		}
+
+		if (name_pixbuf) {
+			name_pixbuf->hide();
 		}
 
 		if (frame) {
@@ -873,19 +854,10 @@ TimeAxisViewItem::reset_width_dependent_items (double pixel_width)
 
 		if (name_highlight) {
 
-			double height = name_highlight->property_y2 ();
-
-			if (height < NAME_HIGHLIGHT_THRESH) {
+			if (_height < NAME_HIGHLIGHT_THRESH) {
 				name_highlight->hide();
-				if (name_text) {
-					name_text->hide();
-				}
 			} else {
 				name_highlight->show();
-				if (name_text && !get_item_name().empty()) {
-					name_text->show();
-					reset_name_width (pixel_width);
-				}
 			}
 
 			if (visibility & FullWidthNameHighlight) {
@@ -896,6 +868,14 @@ TimeAxisViewItem::reset_width_dependent_items (double pixel_width)
 
 		}
 
+		if (name_pixbuf) {
+			if (_height < NAME_HIGHLIGHT_THRESH) {
+				name_pixbuf->hide();
+			} else {
+				reset_name_width (pixel_width);
+			}
+		}
+
 		if (frame) {
 			frame->show();
 			frame->property_x2() = pixel_width;
@@ -905,102 +885,55 @@ TimeAxisViewItem::reset_width_dependent_items (double pixel_width)
 			if (pixel_width < (2*TimeAxisViewItem::GRAB_HANDLE_LENGTH)) {
 				frame_handle_start->hide();
 				frame_handle_end->hide();
+			} else {
+				frame_handle_start->show();
+				frame_handle_end->property_x1() = pixel_width - (TimeAxisViewItem::GRAB_HANDLE_LENGTH);
+				frame_handle_end->property_x2() = pixel_width;
+				frame_handle_end->show();
 			}
-			frame_handle_start->show();
-			frame_handle_end->property_x1() = pixel_width - (TimeAxisViewItem::GRAB_HANDLE_LENGTH);
-			frame_handle_end->show();
-			frame_handle_end->property_x2() = pixel_width;
 		}
 	}
 }
 
 void
-TimeAxisViewItem::reset_name_width (double pixel_width)
+TimeAxisViewItem::reset_name_width (double pix_width)
 {
-	if (name_text == 0) {
+	uint32_t it_width;
+	int pb_width;
+	bool pixbuf_holds_full_name;
+
+	if (!name_pixbuf) {
 		return;
 	}
 
-	int limit = (int) floor (pixel_width - NAME_X_OFFSET);
-	bool shrinking = (last_name_text_width > pixel_width);
-	int actual_width;
-	ustring ustr;
-	ustring::size_type n;
+	it_width = trackview.editor.frame_to_pixel(item_duration);
+	pb_width = name_pixbuf_width;
 
-	if ((last_name_text_width &&                                       // we did this once
-	     shrinking &&                                                  // we're getting smaller
-	     (name_text_width <= limit) &&                                 // fits the new size
-	     (name_text_width <= last_name_text_width - NAME_X_OFFSET))) { // fit into the old size too
-		last_name_text_width = pixel_width;
+	pixbuf_holds_full_name = last_item_width > pb_width + NAME_X_OFFSET;
+	last_item_width = it_width;
+
+	if (pixbuf_holds_full_name && (it_width >= pb_width + NAME_X_OFFSET)) {
+		/*
+		  we've previously had the full name length showing 
+		  and its still showing.
+		*/
 		return;
 	}
-
-	/* now check the cache of existing truncations */
-
-	Gtk::Label foo;
-	Glib::RefPtr<Pango::Layout> layout = foo.create_pango_layout ("");
-
-	for (n = item_name.length(); n > 0; --n) {
-		
-		map<ustring::size_type,int>::iterator i;
-
-		if ((i = name_text_size_cache.find (n)) != name_text_size_cache.end()) {
-
-			/* we know the length of this substring already */
-			
-			if ((actual_width = (*i).second) < limit) {
-
-				/* it fits, use it */
-				
-				ustr = item_name.substr (0, n);
-				break;
-			}
-						
-		} else {
-			
-			/* we don't know the length of this substring already, so compute
-			   it and put it into the cache.
-			 */
-
-			layout->set_text (item_name.substr (0, n));
-			
-			int width, height;
-			Gtkmm2ext::get_ink_pixel_size (layout, width, height);
-			
-			name_text_size_cache[n] = width;
-
-			if ((actual_width = width) < limit) {
-				ustr = item_name.substr (0, n);
-				break;
-			}
-		}
-	}
-
-	if (n == 0) {
-		name_text->property_text() = "";
-		last_name_text_width = pixel_width;
-		return;
-	} 
-
-	/* don't use name for event handling if it leaves no room
-	   for trimming to work.
-	*/
 	
-	if (pixel_width - actual_width < (NAME_X_OFFSET * 2.0)) {
-		if (name_connected) {
-			name_connected = false;
-		}
+	if (pb_width > it_width - NAME_X_OFFSET) {
+		pb_width = it_width - NAME_X_OFFSET;
+	}
+	
+	if (pb_width <= 0 || it_width <= NAME_X_OFFSET) {
+		name_pixbuf->hide();
+		return;
 	} else {
-		if (!name_connected) {
-			name_connected = true;
-		}
+		name_pixbuf->show();
 	}
-	
-	name_text->property_text() = ustr;
-	name_text_width = actual_width;
-	name_text->show();
-	last_name_text_width = pixel_width;
 
+	name_pixbuf->property_pixbuf() = pixbuf_from_ustring(item_name, NAME_FONT, 
+							     ARDOUR_UI::config()->canvasvar_TimeAxisViewItemName.get(),
+							     pb_width, NAME_HEIGHT);
 }
 
 
@@ -1043,3 +976,4 @@ TimeAxisViewItem::idle_remove_this_item(TimeAxisViewItem* item, void* src)
 	item = 0;
 	return false;
 }
+
