@@ -54,6 +54,10 @@ sigc::signal<void>  DPIReset;
 int
 pixel_width (const ustring& str, Pango::FontDescription& font)
 {
+	if (str.empty()) {
+		return 0;
+	}
+
 	Label foo;
 	Glib::RefPtr<Pango::Layout> layout = foo.create_pango_layout ("");
 
@@ -396,9 +400,80 @@ set_color (Gdk::Color& c, int rgb)
 	c.set_rgb((rgb >> 16)*256, ((rgb & 0xff00) >> 8)*256, (rgb & 0xff)*256);
 }
 
+bool
+relay_key_press (GdkEventKey* ev, Gtk::Window* win)
+{
+	if (!key_press_focus_accelerator_handler (*win, ev)) {
+		return PublicEditor::instance().on_key_press_event(ev);
+	} else {
+		return true;
+	}
+}
+
 #ifdef GTKOSX
-extern "C" {
-	gboolean gdk_quartz_possibly_forward (GdkEvent*);
+static guint
+osx_keyval_without_alt (guint accent_keyval)
+{
+	switch (accent_keyval) {
+	case GDK_oe:
+		return GDK_q;
+	case GDK_registered:
+		return GDK_r;
+	case GDK_dagger:
+		return GDK_t;
+	case GDK_yen:
+		return GDK_y;
+	case GDK_diaeresis:
+		return GDK_u;
+	case GDK_oslash:
+		return GDK_o;
+	case GDK_Greek_pi:
+		return GDK_p;
+	case GDK_leftdoublequotemark:
+		return GDK_bracketleft;
+	case GDK_leftsinglequotemark:
+		return GDK_bracketright;
+	case GDK_guillemotleft:
+		return GDK_backslash;
+	case GDK_aring:
+		return GDK_a;
+	case GDK_ssharp:
+		return GDK_s;
+	case GDK_partialderivative:
+		return GDK_d;
+	case GDK_function:
+		return GDK_f;
+	case GDK_copyright:
+		return GDK_g;
+	case GDK_abovedot:
+		return GDK_h;
+	case GDK_notsign:
+		return GDK_l;
+	case GDK_ellipsis:
+		return GDK_semicolon;
+	case GDK_ae:
+		return GDK_apostrophe;
+	case GDK_Greek_OMEGA:
+		return GDK_z;
+	case GDK_ccedilla:
+		return GDK_c;
+	case GDK_radical:
+		return GDK_v;
+	case GDK_integral:
+		return GDK_b;
+	case GDK_mu:
+		return GDK_m;
+	case GDK_lessthanequal:
+		return GDK_comma;
+	case GDK_greaterthanequal:
+		return GDK_period;
+	case GDK_division:
+		return GDK_slash;
+	default:
+		break;
+	}
+
+	return GDK_VoidSymbol;
 }
 #endif
 
@@ -408,21 +483,43 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 	GtkWindow* win = window.gobj();
 	GtkWidget* focus = gtk_window_get_focus (win);
 	bool special_handling_of_unmodified_accelerators = false;
+	bool allow_activating = true;
 
-#undef DEBUG_ACCELERATOR_HANDLING
+//#define DEBUG_ACCELERATOR_HANDLING
 #ifdef  DEBUG_ACCELERATOR_HANDLING
-	bool debug = (getenv ("ARDOUR_DEBUG_ACCELERATOR_HANDLING") != 0);
+	//bool debug = (getenv ("ARDOUR_DEBUG_ACCELERATOR_HANDLING") != 0);
+	bool debug=true;
 #endif
 	if (focus) {
 		if (GTK_IS_ENTRY(focus) || Keyboard::some_magic_widget_has_focus()) {
 			special_handling_of_unmodified_accelerators = true;
 		} 
+#ifdef DEBUG_ACCELERATOR_HANDLING
+		cerr << "Focus widget name " << gtk_widget_get_name(focus) << endl;
+#endif
 	} 
 
+#ifdef GTKOSX
+	/* should this be universally true? */
+	if (Keyboard::some_magic_widget_has_focus ()) {
+		allow_activating = false;
+	}
+#endif
+
 #ifdef DEBUG_ACCELERATOR_HANDLING
 	if (debug) {
-		cerr << "Win = " << win << " Key event: code = " << ev->keyval << " state = " << hex << ev->state << dec << " special handling ? " 
+		cerr << "Win = " << win << " Key event: code = " << ev->keyval << " name = " << gdk_keyval_name (ev->keyval) << " state = " << hex << ev->state << dec 
+		     << " ctrl " << ((ev->state & GDK_CONTROL_MASK) ? 1 : 0)
+		     << " alt " << ((ev->state & GDK_MOD1_MASK) ? 1 : 0)
+		     << " shift " << ((ev->state & GDK_SHIFT_MASK) ? 1 : 0)
+		     << " cmd/meta " << ((ev->state & GDK_META_MASK) ? 1 : 0)
+		     << " lock " << ((ev->state & GDK_LOCK_MASK) ? 1 : 0)
+		     << " special handling ? " 
 		     << special_handling_of_unmodified_accelerators
+		     << " magic widget focus ? "
+		     << Keyboard::some_magic_widget_has_focus()
+		     << " allow_activation ? "
+		     << allow_activating
 		     << endl;
 	}
 #endif
@@ -458,6 +555,24 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 	   all "normal text" accelerators.
 	*/
 
+#ifdef GTKOSX
+	if (!special_handling_of_unmodified_accelerators) {
+		if (ev->state & GDK_MOD1_MASK) {
+			/* we're not in a text entry or "magic focus" widget so we don't want OS X "special-character" 
+			   text-style handling of alt-<key>. change the keyval back to what it would be without
+			   the alt key. this way, we see <alt>-v rather than <alt>-radical and so on.
+			*/
+			guint keyval_without_alt = osx_keyval_without_alt (ev->keyval);
+
+			if (keyval_without_alt != GDK_VoidSymbol) {
+#ifdef DEBUG_ACCELERATOR_HANDLING
+				cerr << "Remapped " << gdk_keyval_name (ev->keyval) << " to " << gdk_keyval_name (keyval_without_alt) << endl;
+
+#endif				ev->keyval = keyval_without_alt;
+			}
+		}
+	}
+#endif
 
 	if (!special_handling_of_unmodified_accelerators) {
 
@@ -466,50 +581,22 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 		   it does allow.
 		*/
 
-		int fakekey = GDK_VoidSymbol;
-		int ret = false;
+		uint32_t fakekey = ev->keyval;
 
-		switch (ev->keyval) {
-		case GDK_Tab:
-		case GDK_ISO_Left_Tab:
-			fakekey = GDK_nabla;
-			break;
-
-		case GDK_Up:
-			fakekey = GDK_uparrow;
-			break;
-
-		case GDK_Down:
-			fakekey = GDK_downarrow;
-			break;
-
-		case GDK_Right:
-			fakekey = GDK_rightarrow;
-			break;
-
-		case GDK_Left:
-			fakekey = GDK_leftarrow;
-			break;
-
-		default:
-			break;
-		}
-
-		if (fakekey != GDK_VoidSymbol) {
-			ret = gtk_accel_groups_activate(G_OBJECT(win), fakekey, GdkModifierType(ev->state));
-			
-			if (ret) {
-				return true;
+		if (possibly_translate_keyval_to_make_legal_accelerator (fakekey)) {
+#ifdef DEBUG_ACCELERATOR_HANDLING
+			if (debug) {
+				cerr << "\tactivate (was " << ev->keyval << " now " << fakekey << ") without special handling of unmodified accels\n";
 			}
-
-#ifdef GTKOSX
-			int oldval = ev->keyval;
-			ev->keyval = fakekey;
-			if (gdk_quartz_possibly_forward ((GdkEvent*) ev)) {
-				return true;
-			}
-			ev->keyval = oldval;
 #endif
+			if (allow_activating && gtk_accel_groups_activate(G_OBJECT(win), fakekey, GdkModifierType(ev->state))) {
+#ifdef DEBUG_ACCELERATOR_HANDLING
+				if (debug) {
+					cerr << "\tactivation handled key\n";
+				}
+#endif
+				return true;
+			}
 		}
 	}
 
@@ -526,26 +613,25 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 			cerr << "\tactivate, then propagate\n";
 		}
 #endif
-#ifdef GTKOSX
-		if (gdk_quartz_possibly_forward ((GdkEvent*) ev)) {
-			return true;
+
+		if (allow_activating) {
+			if (gtk_window_activate_key (win, ev)) {
+#ifdef DEBUG_ACCELERATOR_HANDLING
+				if (debug) {
+					cerr << "\tactivation handled key " << ev->keyval << endl;
+				}
+#endif
+				
+				return true;
+			}
+		}
+
+#ifdef DEBUG_ACCELERATOR_HANDLING
+		if (debug) {
+			cerr << "\tnot accelerated, now propagate\n";
 		}
 #endif
-		if (!gtk_window_activate_key (win, ev)) {
-#ifdef DEBUG_ACCELERATOR_HANDLING
-			if (debug) {
-				cerr << "\tnot accelerated, now propagate\n";
-			}
-#endif
-			return gtk_window_propagate_key_event (win, ev);
-		} else {
-#ifdef DEBUG_ACCELERATOR_HANDLING
-			if (debug) {
-				cerr << "\taccelerated - done.\n";
-			}
-#endif
-			return true;
-		} 
+		return gtk_window_propagate_key_event (win, ev);
 	}
 	
 	/* no modifiers, propagate first */
@@ -561,12 +647,11 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 			cerr << "\tpropagation didn't handle, so activate\n";
 		}
 #endif
-#ifdef GTKOSX
-		if (gdk_quartz_possibly_forward ((GdkEvent*) ev)) {
-			return true;
-		}
-#endif
-		return gtk_window_activate_key (win, ev);
+
+		if (allow_activating) {
+			return gtk_window_activate_key (win, ev);
+		} 
+			
 	} else {
 #ifdef DEBUG_ACCELERATOR_HANDLING
 		if (debug) {
@@ -587,8 +672,14 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 Glib::RefPtr<Gdk::Pixbuf>	
 get_xpm (std::string name)
 {
+	//cerr << "xpm path = " << ARDOUR::find_data_file(name, "pixmaps") << endl;//DEBUG
 	if (!xpm_map[name]) {
-		xpm_map[name] = Gdk::Pixbuf::create_from_file (ARDOUR::find_data_file(name, "pixmaps"));
+		try {
+			xpm_map[name] = Gdk::Pixbuf::create_from_file (ARDOUR::find_data_file(name, "pixmaps"));
+		}
+		catch(const Glib::Error& e)	{
+    		warning << "Caught Glib::Error: " << e.what() << endmsg;
+  		}
 	}
 		
 	return (xpm_map[name]);
@@ -703,13 +794,13 @@ set_pango_fontsize ()
 {
 	long val = ARDOUR::Config->get_font_scale();
 
-	/* FT2 rendering */
+	/* FT2 rendering - used by GnomeCanvas, sigh */
 
 	pango_ft2_font_map_set_resolution ((PangoFT2FontMap*) pango_ft2_font_map_for_display(), val/1024, val/1024);
 
 	/* Cairo rendering, in case there is any */
 	
-	pango_cairo_font_map_set_resolution ((PangoCairoFontMap*) pango_cairo_font_map_get_default(), val/1024);
+	// pango_cairo_font_map_set_resolution ((PangoCairoFontMap*) pango_cairo_font_map_get_default(), val/1024);
 }
 
 void
@@ -724,3 +815,105 @@ reset_dpi ()
 	DPIReset();//Emit Signal
 }
 
+bool
+possibly_translate_keyval_to_make_legal_accelerator (uint32_t& keyval)
+{
+	int fakekey = GDK_VoidSymbol;
+
+	switch (keyval) {
+	case GDK_Tab:
+	case GDK_ISO_Left_Tab:
+		fakekey = GDK_nabla;
+		break;
+		
+	case GDK_Up:
+		fakekey = GDK_uparrow;
+		break;
+		
+	case GDK_Down:
+		fakekey = GDK_downarrow;
+		break;
+		
+	case GDK_Right:
+		fakekey = GDK_rightarrow;
+		break;
+		
+	case GDK_Left:
+		fakekey = GDK_leftarrow;
+		break;
+		
+	default:
+		break;
+	}
+	
+	if (fakekey != GDK_VoidSymbol) {
+		keyval = fakekey;
+		return true;
+	} 
+
+	return false;
+}
+		
+inline guint8
+convert_color_channel (guint8 src,
+		       guint8 alpha)
+{
+	return alpha ? ((guint (src) << 8) - src) / alpha : 0;
+}
+
+void
+convert_bgra_to_rgba (guint8 const* src,
+		      guint8*       dst,
+		      int           width,
+		      int           height)
+{
+	guint8 const* src_pixel = src;
+	guint8*       dst_pixel = dst;
+	
+	for (int y = 0; y < height; y++)
+		for (int x = 0; x < width; x++)
+		{
+			dst_pixel[0] = convert_color_channel (src_pixel[2],
+							      src_pixel[3]);
+			dst_pixel[1] = convert_color_channel (src_pixel[1],
+							      src_pixel[3]);
+			dst_pixel[2] = convert_color_channel (src_pixel[0],
+							      src_pixel[3]);
+			dst_pixel[3] = src_pixel[3];
+			
+			dst_pixel += 4;
+			src_pixel += 4;
+		}
+}
+
+Glib::RefPtr<Gdk::Pixbuf>
+pixbuf_from_ustring(const ustring& name, Pango::FontDescription* font, uint32_t rgba, int clip_width, int clip_height)
+{
+	Glib::RefPtr<Gdk::Pixbuf> buf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, true, 8, clip_width, clip_height);
+	cairo_surface_t* surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, clip_width, clip_height);
+	cairo_t* cr = cairo_create (surface);
+	cairo_text_extents_t te;
+
+	float fr, fg, fb, fa;
+
+	fr = ((rgba & 0xff000000) >> 24) / 255.0f;
+	fg = ((rgba & 0xff0000) >> 16) / 255.0f;
+	fb = ((rgba & 0xff00) >> 8) / 255.0f;
+	fa = (rgba & 0xff) / 255.0f;
+
+	cairo_set_source_rgba (cr, fr, fg, fb, fa);
+	cairo_select_font_face (cr, font->get_family().c_str(),
+				CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
+	cairo_set_font_size (cr,  font->get_size() / Pango::SCALE);
+	cairo_text_extents (cr, name.c_str(), &te);
+	
+	cairo_move_to (cr, 0.5, 0.5 - te.height / 2 - te.y_bearing + clip_height / 2);
+	cairo_show_text (cr, name.c_str());
+	
+	convert_bgra_to_rgba(cairo_image_surface_get_data (surface), buf->get_pixels(), clip_width, clip_height);
+
+	cairo_destroy(cr);
+	cairo_surface_destroy(surface);
+
+	return buf;
+}
