@@ -57,7 +57,8 @@ class AudioSource : public Source, public boost::enable_shared_from_this<ARDOUR:
 
 	virtual nframes_t natural_position() const { return 0; }
 	
-	/* returns the number of items in this `audio_source' */
+	nframes64_t readable_length() const { return _length; }
+	uint32_t    n_channels() const { return 1; }
 
 	virtual nframes_t length() const {
 		return _length;
@@ -65,6 +66,15 @@ class AudioSource : public Source, public boost::enable_shared_from_this<ARDOUR:
 
 	virtual nframes_t available_peaks (double zoom) const;
 
+	/* stopgap until nframes_t becomes nframes64_t. this function is needed by the Readable interface */
+
+	virtual nframes64_t read (Sample *dst, nframes64_t start, nframes64_t cnt, int channel) const {
+		/* XXX currently ignores channel, assuming that source is always mono, which
+		   historically has been true.
+		*/
+		return read (dst, (nframes_t) start, (nframes_t) cnt);
+	}
+
 	virtual nframes_t read (Sample *dst, nframes_t start, nframes_t cnt) const;
 	virtual nframes_t write (Sample *src, nframes_t cnt);
 
@@ -80,6 +90,7 @@ class AudioSource : public Source, public boost::enable_shared_from_this<ARDOUR:
 
 	uint32_t read_data_count() const { return _read_data_count; }
 	uint32_t write_data_count() const { return _write_data_count; }
+	void dec_read_data_count(nframes_t);
 
  	int read_peaks (PeakData *peaks, nframes_t npeaks, nframes_t start, nframes_t cnt, double samples_per_visual_peak) const;
 
@@ -112,16 +123,23 @@ class AudioSource : public Source, public boost::enable_shared_from_this<ARDOUR:
 	int prepare_for_peakfile_writes ();
 	void done_with_peakfile_writes (bool done = true);
 
+	AnalysisFeatureList transients;
+	std::string get_transients_path() const;
+
+	void set_been_analysed (bool yn);
+	bool check_for_analysis_data_on_disk ();
+
   protected:
 	static bool _build_missing_peakfiles;
 	static bool _build_peakfiles;
 
 	bool                 _peaks_built;
+	bool                 _analysed;
 	mutable Glib::Mutex  _lock;
 	mutable Glib::Mutex  _peaks_ready_lock;
 	nframes_t            _length;
-	Glib::ustring               peakpath;
-	Glib::ustring              _captured_for;
+	Glib::ustring         peakpath;
+	Glib::ustring        _captured_for;
 
 	mutable uint32_t _read_data_count;  // modified in read()
 	mutable uint32_t _write_data_count; // modified in write()
@@ -146,6 +164,8 @@ class AudioSource : public Source, public boost::enable_shared_from_this<ARDOUR:
 	int compute_and_write_peaks (Sample* buf, nframes_t first_frame, nframes_t cnt, bool force, 
 				     bool intermediate_peaks_ready_signal, nframes_t frames_per_peak);
 
+	int load_transients (const std::string&);
+
   private:
 	int peakfile;
 	nframes_t peak_leftover_cnt;
