@@ -90,18 +90,17 @@ class IO : public PBD::StatefulDestructible
 	const string& name() const { return _name; }
 	virtual int set_name (string str, void *src);
 	
-	virtual void silence  (nframes_t, nframes_t offset);
+	virtual void silence  (nframes_t);
 
 	// These should be moved in to a separate object that manipulates an IO
 	
-	void pan (vector<Sample*>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t offset, gain_t gain_coeff);
+	void pan (vector<Sample*>& bufs, uint32_t nbufs, nframes_t nframes, gain_t gain_coeff);
 	void pan_automated (vector<Sample*>& bufs, uint32_t nbufs, nframes_t start_frame, nframes_t end_frame, 
-			    nframes_t nframes, nframes_t offset);
-	void collect_input  (vector<Sample*>&, uint32_t nbufs, nframes_t nframes, nframes_t offset);
-	void deliver_output (vector<Sample*>&, uint32_t nbufs, nframes_t nframes, nframes_t offset);
-	void deliver_output_no_pan (vector<Sample*>&, uint32_t nbufs, nframes_t nframes, nframes_t offset);
-	void just_meter_input (nframes_t start_frame, nframes_t end_frame, 
-			       nframes_t nframes, nframes_t offset);
+			    nframes_t nframes);
+	void collect_input  (vector<Sample*>&, uint32_t nbufs, nframes_t nframes);
+	void deliver_output (vector<Sample*>&, uint32_t nbufs, nframes_t nframes);
+	void deliver_output_no_pan (vector<Sample*>&, uint32_t nbufs, nframes_t nframes);
+	void just_meter_input (nframes_t start_frame, nframes_t end_frame, nframes_t nframes);
 
 	virtual uint32_t n_process_buffers () { return 0; }
 
@@ -161,6 +160,9 @@ class IO : public PBD::StatefulDestructible
 	uint32_t n_inputs () const { return _ninputs; }
 	uint32_t n_outputs () const { return _noutputs; }
 
+	Sample* get_input_buffer (int n, nframes_t);
+	Sample* get_output_buffer (int n, nframes_t);
+
 	sigc::signal<void>                active_changed;
 
 	sigc::signal<void,IOChange,void*> input_changed;
@@ -214,8 +216,11 @@ class IO : public PBD::StatefulDestructible
 
 	void reset_max_peak_meters ();
 
+	static sigc::signal<void,nframes_t> CycleStart;
 	
 	static void update_meters();
+	static std::string name_from_state (const XMLNode&);
+	static void set_name_in_state (XMLNode&, const std::string&);
 
   private: 
 
@@ -239,13 +244,11 @@ class IO : public PBD::StatefulDestructible
 	 
 	 void clear_automation ();
 	 
-	 bool gain_automation_recording() const { 
-		 return (_gain_automation_curve.automation_state() & (Write|Touch));
-	 }
+	 bool gain_automation_recording();
 	 
 	 bool gain_automation_playback() const {
-		 return (_gain_automation_curve.automation_state() & Play) ||
-			 ((_gain_automation_curve.automation_state() & Touch) && 
+		 return (_gain_automation_curve.automation_state() & Auto_Play) ||
+			 ((_gain_automation_curve.automation_state() & Auto_Touch) && 
 			  !_gain_automation_curve.touching());
 	 }
 
@@ -263,6 +266,8 @@ class IO : public PBD::StatefulDestructible
 	}
 	virtual void automation_snapshot (nframes_t now, bool force);
 
+        Session& session() const { return _session; }
+
 	ARDOUR::Curve& gain_automation_curve () { return _gain_automation_curve; }
 
 	void start_gain_touch ();
@@ -346,6 +351,10 @@ class IO : public PBD::StatefulDestructible
 	int set_inputs (const string& str);
 	int set_outputs (const string& str);
 
+	void increment_output_offset (nframes_t);
+
+	void cycle_start (nframes_t);
+
 	static bool connecting_legal;
 	static bool ports_legal;
 
@@ -353,6 +362,8 @@ class IO : public PBD::StatefulDestructible
 
 	uint32_t _ninputs;
 	uint32_t _noutputs;
+	nframes_t _output_offset;
+
 
 	/* are these the best variable names ever, or what? */
 
@@ -395,6 +406,7 @@ class IO : public PBD::StatefulDestructible
 
 	int create_ports (const XMLNode&);
 	int make_connections (const XMLNode&);
+	Connection *find_possible_connection(const string &desired_name, const string &default_name, const string &connection_type_name);
 
 	void setup_peak_meters ();
 	void meter ();
