@@ -52,15 +52,6 @@ typedef boost::shared_ptr<Plugin> PluginPtr;
 class PluginInfo {
   public:
 	PluginInfo () { }
-	PluginInfo (const PluginInfo &o)
-		: name(o.name),
-		category (o.category), 
-		creator (o.creator),
-		path (o.path), 
-		n_inputs(o.n_inputs), 
-		n_outputs(o.n_outputs),
-		unique_id(o.unique_id), 
-		index(o.index) {}
 	virtual ~PluginInfo () { }
 
 	string name;
@@ -122,7 +113,8 @@ class Plugin : public PBD::StatefulDestructible
 	virtual uint32_t nth_parameter (uint32_t which, bool& ok) const = 0;
 	virtual void activate () = 0;
 	virtual void deactivate () = 0;
-	virtual void set_block_size (nframes_t nframes) = 0;
+	virtual void flush () { deactivate(); activate(); }
+	virtual int set_block_size (nframes_t nframes) = 0;
 
 	virtual int connect_and_run (vector<Sample*>& bufs, uint32_t maxbuf, int32_t& in, int32_t& out, nframes_t nframes, nframes_t offset) = 0;
 	virtual std::set<uint32_t> automatable() const = 0;
@@ -138,17 +130,19 @@ class Plugin : public PBD::StatefulDestructible
 	virtual bool save_preset(string name) = 0;
 	virtual bool load_preset (const string preset_label);
 	virtual std::vector<std::string> get_presets();
+	virtual std::string current_preset() const { return std::string(); }
+
+	static sigc::signal<bool> PresetFileExists;
 
 	virtual bool has_editor() const = 0;
 
 	sigc::signal<void,uint32_t,float> ParameterChanged;
 
-	virtual bool fixed_io() const { return true; }
-	virtual int32_t can_support_input_configuration (int32_t in);
-	virtual int32_t compute_output_streams (int32_t nplugins);
+	virtual int32_t can_do (int32_t in, int32_t& out);
 	virtual uint32_t output_streams() const;
 	virtual uint32_t input_streams() const;
-	
+	virtual int32_t configure_io (int32_t in, int32_t out);
+
 	PBD::Controllable *get_nth_control (uint32_t, bool do_not_create = false);
 	void make_nth_control (uint32_t, const XMLNode&);
 
