@@ -135,8 +135,7 @@ class Session : public PBD::StatefulDestructible
 		    SetSlaveSource,
 		    Audition,
 		    InputConfigurationChange,
-		    SetAudioRange,
-		    SetPlayRange,
+		    SetPlayAudioRange,
 		    
 		    /* only one of each of these events
 		       can be queued at any one time
@@ -155,29 +154,34 @@ class Session : public PBD::StatefulDestructible
 
 	    Type           type;
 	    Action         action;
-	    nframes_t action_frame;
-	    nframes_t target_frame;
+	    nframes_t      action_frame;
+	    nframes_t      target_frame;
 	    float          speed;
 
 	    union {
-			void*                ptr;
-			bool                 yes_or_no;
-		        nframes_t            target2_frame;
-			SlaveSource slave;
+		void*                ptr;
+		bool                 yes_or_no;
+		nframes_t            target2_frame;
+		SlaveSource slave;
 	    };
 
+	    union {
+		bool second_yes_or_no;
+	    };
+	    
 	    boost::shared_ptr<Region>     region;
 
 	    list<AudioRange>     audio_range;
 	    list<MusicRange>     music_range;
 
-	    Event(Type t, Action a, nframes_t when, nframes_t where, float spd, bool yn = false)
-		    : type (t), 
-		      action (a),
-		      action_frame (when),
-		      target_frame (where),
-		      speed (spd),
-		      yes_or_no (yn) {}
+	Event(Type t, Action a, nframes_t when, nframes_t where, float spd, bool yn = false, bool yn2 = false)
+		    : type (t)
+		    , action (a)
+		    , action_frame (when)
+		    , target_frame (where)
+		    , speed (spd)
+		    , yes_or_no (yn)
+		    , second_yes_or_no (yn2) {}
 
 	    void set_ptr (void* p) { 
 		    ptr = p;
@@ -241,7 +245,8 @@ class Session : public PBD::StatefulDestructible
 	string export_dir () const;
 
 	void set_snap_name ();
-
+	
+	bool writable() const { return _writable; }
 	void set_dirty ();
 	void set_clean ();
 	bool dirty() const { return _state_of_the_state & Dirty; }
@@ -267,8 +272,17 @@ class Session : public PBD::StatefulDestructible
 	static string suffixed_search_path (std::string suffix, bool data);
 	static string control_protocol_path ();
 	static string template_path ();
+	static string route_template_path ();
 	static string template_dir ();
-	static void get_template_list (list<string>&);
+	static string route_template_dir ();
+	static void get_template_list (std::list<std::string>&);
+
+	struct RouteTemplateInfo {
+	  std::string name;
+	  std::string path;
+	};
+
+	static void get_route_templates (std::vector<RouteTemplateInfo>&);
 	
 	static string change_audio_path_by_name (string oldpath, string oldname, string newname, bool destructive);
 	string audio_path_from_name (string, uint32_t nchans, uint32_t chan, bool destructive);
@@ -298,6 +312,8 @@ class Session : public PBD::StatefulDestructible
 		return routes.reader ();
 	}
 
+	boost::shared_ptr<RouteList> get_routes_with_regions_at (nframes64_t const) const;
+
 	uint32_t nroutes() const { return routes.reader()->size(); }
 	uint32_t ntracks () const;
 	uint32_t nbusses () const;
@@ -306,11 +322,15 @@ class Session : public PBD::StatefulDestructible
 	    bool operator() (boost::shared_ptr<Route>, boost::shared_ptr<Route> b);
 	};
 	
+	void sync_order_keys (const char* base);
+
 	template<class T> void foreach_route (T *obj, void (T::*func)(Route&));
 	template<class T> void foreach_route (T *obj, void (T::*func)(boost::shared_ptr<Route>));
 	template<class T, class A> void foreach_route (T *obj, void (T::*func)(Route&, A), A arg);
 
-	boost::shared_ptr<Route> route_by_name (string);
+	bool io_name_is_legal (const std::string&);
+
+	boost::shared_ptr<Route> route_by_name (const std::string&);
 	boost::shared_ptr<Route> route_by_id (PBD::ID);
 	boost::shared_ptr<Route> route_by_remote_id (uint32_t id);
 
@@ -359,10 +379,10 @@ class Session : public PBD::StatefulDestructible
 
 	void request_roll_at_and_return (nframes_t start, nframes_t return_to);
 	void request_bounded_roll (nframes_t start, nframes_t end);
-	void request_stop (bool abort = false);
+	void request_stop (bool abort = false, bool clear_state = true);
 	void request_locate (nframes_t frame, bool with_roll = false);
-
-	void request_play_loop (bool yn);
+	void request_transport_speed (float speed);
+	void request_play_loop (bool yn, bool leave_rolling = false);
 	bool get_play_loop () const { return play_loop; }
 
 	nframes_t  last_transport_start() const { return _last_roll_location; }
@@ -372,7 +392,6 @@ class Session : public PBD::StatefulDestructible
 	void set_session_end (nframes_t end) { end_location->set_start(end); _end_location_is_free = false; }
 	void use_rf_shuttle_speed ();
 	void allow_auto_play (bool yn);
-	void request_transport_speed (float speed);
 	void request_overwrite_buffer (Diskstream*);
 	void request_diskstream_speed (Diskstream&, float speed);
 	void request_input_change_handling ();
@@ -396,6 +415,10 @@ class Session : public PBD::StatefulDestructible
 	double frames_per_smpte_frame() const { return _frames_per_smpte_frame; }
 	nframes_t smpte_frames_per_hour() const { return _smpte_frames_per_hour; }
 
+	MIDI::byte get_mtc_smpte_bits() const { 
+		return mtc_smpte_bits;   /* encoding of SMTPE type for MTC */
+	}
+
 	float smpte_frames_per_second() const;
 	bool smpte_drop_frames() const;
 
@@ -422,7 +445,7 @@ class Session : public PBD::StatefulDestructible
 	nframes_t worst_input_latency () const { return _worst_input_latency; }
 	nframes_t worst_track_latency () const { return _worst_track_latency; }
 
-	int save_state (string snapshot_name, bool pending = false);
+	int save_state (string snapshot_name, bool pending = false, bool switch_to_snapshot = false);
 	int restore_state (string snapshot_name);
 	int save_template (string template_name);
         int save_history (string snapshot_name = "");
@@ -432,7 +455,6 @@ class Session : public PBD::StatefulDestructible
 	void remove_pending_capture_state ();
 
 	static int rename_template (string old_name, string new_name);
-
 	static int delete_template (string name);
 	
 	sigc::signal<void,string> StateSaved;
@@ -489,6 +511,13 @@ class Session : public PBD::StatefulDestructible
 
 	std::list<boost::shared_ptr<AudioTrack> > new_audio_track (int input_channels, int output_channels, TrackMode mode = Normal, uint32_t how_many = 1);
 	RouteList new_audio_route (int input_channels, int output_channels, uint32_t how_many);
+	RouteList new_route_from_template (uint32_t how_many, const std::string& template_path);
+
+	/* this is a hack put in to support work on video tracks by Robin Gareus. Nobody 
+	   should assume that this works yet (February 2009)
+	*/
+
+ 	boost::shared_ptr<Route> new_video_track (string name);
 
 	void   remove_route (boost::shared_ptr<Route>);
 
@@ -566,7 +595,7 @@ class Session : public PBD::StatefulDestructible
 	sigc::signal<void,std::vector<boost::weak_ptr<AudioRegion> >& > AudioRegionsAdded;
 	sigc::signal<void,boost::weak_ptr<AudioRegion> > AudioRegionRemoved;
 
-	int region_name (string& result, string base = string(""), bool newlevel = false) const;
+	int region_name (string& result, string base = string(""), bool newlevel = false);
 	string new_region_name (string);
 	string path_from_region_name (string name, string identifier);
 
@@ -583,6 +612,8 @@ class Session : public PBD::StatefulDestructible
 	    string doing_what;
 	    
 	    /* control info */
+	    uint32_t total;
+	    uint32_t count;
 	    SrcQuality quality;
 	    volatile bool freeze;
 	    std::vector<Glib::ustring> paths;
@@ -590,7 +621,6 @@ class Session : public PBD::StatefulDestructible
 	    
 	    /* result */
 	    SourceList sources;
-	    
 	};
 
 	void import_audiofiles (import_status&);
@@ -600,9 +630,11 @@ class Session : public PBD::StatefulDestructible
 	SlaveSource post_export_slave;
 	nframes_t post_export_position;
 
-	int start_audio_export (ARDOUR::AudioExportSpecification&);
-	int stop_audio_export (ARDOUR::AudioExportSpecification&);
+	int  pre_export ();
+	int  start_audio_export (ARDOUR::AudioExportSpecification&);
+	int  stop_audio_export (ARDOUR::AudioExportSpecification&);
 	void finalize_audio_export ();
+	static sigc::signal<void, std::string, std::string> Exported;
 
 	void add_source (boost::shared_ptr<Source>);
 	void remove_source (boost::weak_ptr<Source>);
@@ -633,6 +665,12 @@ class Session : public PBD::StatefulDestructible
 
 	static sigc::signal<int,nframes_t, nframes_t> AskAboutSampleRateMismatch;
 
+	class SRMismatchRejected : public std::exception {
+	  public:
+		SRMismatchRejected () {}
+		const char* what() const throw() { return "Sample rate mismatch rejected"; }
+	};
+
 	/* handlers should return !0 for use pending state, 0 for
 	   ignore it.
 	*/
@@ -688,8 +726,7 @@ class Session : public PBD::StatefulDestructible
 
 	/* flattening stuff */
 
-	int write_one_audio_track (AudioTrack&, nframes_t start, nframes_t cnt, bool overwrite, vector<boost::shared_ptr<AudioSource> >&,
-				   InterThreadInfo& wot);
+	boost::shared_ptr<Region> write_one_audio_track (AudioTrack&, nframes_t start, nframes_t end, bool overwrite, vector<boost::shared_ptr<AudioSource> >&, InterThreadInfo& wot, bool enable_processing = true);
 	int freeze (InterThreadInfo&);
 
 	/* session-wide solo/mute/rec-enable */
@@ -888,22 +925,18 @@ class Session : public PBD::StatefulDestructible
 	void reset_playback_load_min ();
 	void reset_capture_load_min ();
 	
-	float read_data_rate () const;
+	float read_data_rate () const; // in usec
 	float write_data_rate () const;
 
 	/* ranges */
 
-	void set_audio_range (list<AudioRange>&);
-	void set_music_range (list<MusicRange>&);
-
-	void request_play_range (bool yn);
+	void request_play_range (list<AudioRange>*, bool leave_rolling = false);
 	bool get_play_range () const { return _play_range; }
 
 	/* favorite dirs */
 	typedef vector<string> FavoriteDirs;
 
 	static int read_favorite_dirs (FavoriteDirs&);
-
 	static int write_favorite_dirs (FavoriteDirs&);
 	
 	/* file suffixes */
@@ -975,6 +1008,8 @@ class Session : public PBD::StatefulDestructible
 
 	nframes_t compute_initial_length ();
 
+	bool   _writable;
+
 	static const char* _template_suffix;
 	static const char* _statefile_suffix;
 	static const char* _pending_suffix;
@@ -1028,13 +1063,14 @@ class Session : public PBD::StatefulDestructible
 	float                   _meter_hold;
 	float                   _meter_falloff;
 	bool                    _end_location_is_free;
+	bool                    _was_seamless;
 
 	void set_worst_io_latencies ();
 	void set_worst_io_latencies_x (IOChange asifwecare, void *ignored) {
 		set_worst_io_latencies ();
 	}
 
-	void update_latency_compensation_proxy (void* ignored);
+	void route_redirects_changed (void* ignored);
 
 	void ensure_passthru_buffers (uint32_t howmany);
 	
@@ -1043,7 +1079,11 @@ class Session : public PBD::StatefulDestructible
 	void process_with_events    (nframes_t);
 	void process_audition       (nframes_t);
 	int  process_export         (nframes_t, ARDOUR::AudioExportSpecification*);
-	
+
+	void block_processing() { g_atomic_int_set (&processing_prohibited, 1); }
+	void unblock_processing() { g_atomic_int_set (&processing_prohibited, 0); }
+	bool processing_blocked() const { return g_atomic_int_get (&processing_prohibited); }
+
 	/* slave tracking */
 
 	static const int delta_accumulator_size = 25;
@@ -1063,16 +1103,16 @@ class Session : public PBD::StatefulDestructible
 	nframes_t slave_wait_end;
 
 	void reset_slave_state ();
-	bool follow_slave (nframes_t, nframes_t);
-	void set_slave_source (SlaveSource);
+	bool follow_slave (nframes_t);
+	void set_slave_source (SlaveSource, bool stop_transport = true);
 
 	bool _exporting;
-	int prepare_to_export (ARDOUR::AudioExportSpecification&);
+	int  prepare_to_export (ARDOUR::AudioExportSpecification&);
 
 	void prepare_diskstreams ();
 	void commit_diskstreams (nframes_t, bool& session_requires_butler);
-	int  process_routes (nframes_t, nframes_t);
-	int  silent_process_routes (nframes_t, nframes_t);
+	int  process_routes (nframes_t);
+	int  silent_process_routes (nframes_t);
 
 	bool get_rec_monitors_input () {
 		if (actively_recording()) {
@@ -1098,15 +1138,9 @@ class Session : public PBD::StatefulDestructible
 		}
 	}
 
-	bool maybe_stop (nframes_t limit) {
-		if ((_transport_speed > 0.0f && _transport_frame >= limit) || (_transport_speed < 0.0f && _transport_frame == 0)) {
-			stop_transport ();
-			return true;
-		}
-		return false;
-	}
+	bool maybe_stop (nframes_t limit);
 
-	bool maybe_sync_start (nframes_t&, nframes_t&);
+	bool maybe_sync_start (nframes_t&);
 
 	void check_declick_out ();
 
@@ -1142,16 +1176,16 @@ class Session : public PBD::StatefulDestructible
 	bool     save_config_options_predicate (ConfigVariableBase::Owner owner) const;
 
 	nframes_t   _last_roll_location;
+ 	nframes_t _last_roll_or_reversal_location;
 	nframes_t   _last_record_location;
 	bool              pending_locate_roll;
 	nframes_t    pending_locate_frame;
 
 	bool              pending_locate_flush;
 	bool              pending_abort;
+	bool              pending_clear_substate;
 	bool              pending_auto_loop;
 	
-	Sample*           butler_mixdown_buffer;
-	float*            butler_gain_buffer;
 	pthread_t         butler_thread;
 	Glib::Mutex       butler_request_lock;
         Glib::Cond        butler_paused;
@@ -1185,7 +1219,8 @@ class Session : public PBD::StatefulDestructible
 		PostTransportScrub              = 0x8000,
 		PostTransportReverse            = 0x10000,
 		PostTransportInputChange        = 0x20000,
-		PostTransportCurveRealloc       = 0x40000
+		PostTransportCurveRealloc       = 0x40000,
+		PostTransportClearSubstate      = 0x80000
 	};
 	
 	static const PostTransportWork ProcessCannotProceedMask = 
@@ -1196,7 +1231,8 @@ class Session : public PBD::StatefulDestructible
 				   PostTransportScrub|
 				   PostTransportAudition|
 				   PostTransportLocate|
-				   PostTransportStop);
+				   PostTransportStop|
+				   PostTransportClearSubstate);
 	
 	PostTransportWork post_transport_work;
 
@@ -1341,6 +1377,8 @@ class Session : public PBD::StatefulDestructible
 	nframes_t  last_smpte_when;
 	SMPTE::Time last_smpte;
 
+	bool _send_smpte_update; ///< Flag to send a full frame (SMPTE) MTC message this cycle
+	
 	int send_full_time_code ();
 	int send_midi_time_code ();
 
@@ -1353,7 +1391,8 @@ class Session : public PBD::StatefulDestructible
 	
 	void reset_record_status ();
 	
-	int no_roll (nframes_t nframes, nframes_t offset);
+	int fail_roll (nframes_t nframes);
+	int no_roll (nframes_t nframes);
 	
 	bool non_realtime_work_pending() const { return static_cast<bool>(post_transport_work); }
 	bool process_can_proceed() const { return !(post_transport_work & ProcessCannotProceedMask); }
@@ -1416,27 +1455,26 @@ class Session : public PBD::StatefulDestructible
 	void          change_midi_ports ();
 	int           use_config_midi_ports ();
 
-	bool waiting_to_start;
-
 	void set_play_loop (bool yn);
+	void unset_play_loop ();
 	void overwrite_some_buffers (Diskstream*);
 	void flush_all_redirects ();
-	void locate (nframes_t, bool with_roll, bool with_flush, bool with_loop=false);
-	void start_locate (nframes_t, bool with_roll, bool with_flush, bool with_loop=false);
+	int  micro_locate (nframes_t distance);
+	void locate (nframes_t, bool with_roll, bool with_flush, bool with_loop=false, bool force=false);
+	void start_locate (nframes_t, bool with_roll, bool with_flush, bool with_loop=false, bool force=false);
 	void force_locate (nframes_t frame, bool with_roll = false);
 	void set_diskstream_speed (Diskstream*, float speed);
-	void set_transport_speed (float speed, bool abort = false);
-	void stop_transport (bool abort = false);
+	void set_transport_speed (float speed, bool abort = false, bool clear_state = false);
+	void stop_transport (bool abort = false, bool clear_state = false);
 	void start_transport ();
-	void actually_start_transport ();
-	void realtime_stop (bool abort);
+	void realtime_stop (bool abort, bool clear_state);
 	void non_realtime_start_scrub ();
 	void non_realtime_set_speed ();
 	void non_realtime_stop (bool abort, int entry_request_count, bool& finished);
 	void non_realtime_overwrite (int entry_request_count, bool& finished);
 	void butler_transport_work ();
 	void post_transport ();
-	void engine_halted ();
+	void engine_halted (const char*);
 	void xrun_recovery ();
 
 	TempoMap    *_tempo_map;
@@ -1477,12 +1515,16 @@ class Session : public PBD::StatefulDestructible
 	void route_mute_changed (void *src);
 	void route_solo_changed (void *src, boost::weak_ptr<Route>);
 	void catch_up_on_solo ();
+	void catch_up_on_solo_mute_override ();
 	void update_route_solo_state ();
 	void modify_solo_mute (bool, bool);
 	void strip_portname_for_solo (string& portname);
 
 	/* REGION MANAGEMENT */
 
+	std::map<std::string,uint32_t> region_name_map;
+	void update_region_name_map (boost::shared_ptr<Region>);
+
 	mutable Glib::Mutex region_lock;
 	typedef map<PBD::ID,boost::shared_ptr<AudioRegion> > AudioRegionList;
 	AudioRegionList audio_regions;
@@ -1688,7 +1730,7 @@ class Session : public PBD::StatefulDestructible
 	Click *get_click();
 	void   setup_click_sounds (int which);
 	void   clear_clicks ();
-	void   click (nframes_t start, nframes_t nframes, nframes_t offset);
+	void   click (nframes_t start, nframes_t nframes);
 
 	vector<Route*> master_outs;
 	
@@ -1696,8 +1738,8 @@ class Session : public PBD::StatefulDestructible
 
 	list<AudioRange> current_audio_range;
 	bool _play_range;
-	void set_play_range (bool yn);
-	void setup_auto_play ();
+	void set_play_range (std::list<AudioRange>&, bool leave_rolling);
+	void unset_play_range ();
 
 	/* main outs */
 	uint32_t main_outs;
@@ -1745,9 +1787,14 @@ class Session : public PBD::StatefulDestructible
 	XMLNode& get_control_protocol_state ();
 	
 	void set_history_depth (uint32_t depth);
-	void sync_order_keys ();
 
 	static bool _disable_all_loaded_plugins;
+
+	/* used in ::audible_frame() */
+
+	mutable bool have_looped;
+
+	static void get_templates_from (const std::string& path, std::list<std::string> &template_names);
 };
 
 } // namespace ARDOUR
