@@ -26,13 +26,16 @@
 #include <climits>
 #include <fcntl.h>
 #include <cstdlib>
+#include <sys/time.h>
 #include <ctime>
 #include <sys/stat.h>
 #include <sys/mman.h>
 
 #include <pbd/error.h>
 #include <pbd/basename.h>
+#include <pbd/localeguard.h>
 #include <glibmm/thread.h>
+#include <glibmm/fileutils.h>
 #include <pbd/xml++.h>
 #include <pbd/memento_command.h>
 #include <pbd/enumwriter.h>
@@ -384,6 +387,7 @@ AudioDiskstream::check_record_status (nframes_t transport_frame, nframes_t nfram
 	const int transport_rolling = 0x4;
 	const int track_rec_enabled = 0x2;
 	const int global_rec_enabled = 0x1;
+        const int fully_rec_enabled = (transport_rolling|track_rec_enabled|global_rec_enabled);
 
 	/* merge together the 3 factors that affect record status, and compute
 	   what has changed.
@@ -393,76 +397,74 @@ AudioDiskstream::check_record_status (nframes_t transport_frame, nframes_t nfram
 	possibly_recording = (rolling << 2) | (record_enabled() << 1) | can_record;
 	change = possibly_recording ^ last_possibly_recording;
 
-	if (possibly_recording == last_possibly_recording) {
-		return;
-	}
+        if (possibly_recording == fully_rec_enabled) {
 
-	/* change state */
-
-	/* if per-track or global rec-enable turned on while the other was already on, we've started recording */
-
-	if ((change & track_rec_enabled) && record_enabled() && (!(change & global_rec_enabled) && can_record) || 
-	    ((change & global_rec_enabled) && can_record && (!(change & track_rec_enabled) && record_enabled()))) {
-		
-		/* starting to record: compute first+last frames */
+                if (last_possibly_recording == fully_rec_enabled) {
+                        return;
+                }
 
+                /* we transitioned to recording. lets see if its transport based or a punch */
+                
 		first_recordable_frame = transport_frame + _capture_offset;
 		last_recordable_frame = max_frames;
 		capture_start_frame = transport_frame;
 
-		if (!(last_possibly_recording & transport_rolling) && (possibly_recording & transport_rolling)) {
-
-			/* was stopped, now rolling (and recording) */
+                if (change & transport_rolling) {
 
+                        /* transport-change (started rolling) */
+                        
 			if (_alignment_style == ExistingMaterial) {
-				first_recordable_frame += _session.worst_output_latency();
-			} else {
+                                
+                                /* there are two delays happening:
+                                   
+                                   1) inbound, represented by _capture_offset
+                                   2) outbound, represented by _session.worst_output_latency()
+
+                                   the first sample to record occurs when the larger of these
+                                   two has elapsed, since they occur in parallel.
+
+                                   since we've already added _capture_offset, just add the
+                                   difference if _session.worst_output_latency() is larger.
+                                */
+
+                                if (_capture_offset < _session.worst_output_latency()) {
+                                        first_recordable_frame += (_session.worst_output_latency() - _capture_offset);
+                                } 
+                        } else {
 				first_recordable_frame += _roll_delay;
   			}
+                        
+                } else {
 
-		} else {
-
-			/* was rolling, but record state changed */
+                        /* punch in */
 
 			if (_alignment_style == ExistingMaterial) {
 
-				if (!Config->get_punch_in()) {
+				/* There are two kinds of punch:
+                                   
+                                   manual punch in happens at the correct transport frame
+                                   because the user hit a button. but to get alignment correct 
+                                   we have to back up the position of the new region to the 
+                                   appropriate spot given the roll delay.
 
-					/* manual punch in happens at the correct transport frame
-					   because the user hit a button. but to get alignment correct 
-					   we have to back up the position of the new region to the 
-					   appropriate spot given the roll delay.
-					*/
+                                   autopunch toggles recording at the precise
+				   transport frame, and then the DS waits
+				   to start recording for a time that depends
+				   on the output latency.
 
-					capture_start_frame -= _roll_delay;
+                                   XXX: BUT THIS CODE DOESN'T DIFFERENTIATE !!!
 
-					/* XXX paul notes (august 2005): i don't know why
-					   this is needed.
-					*/
+				*/
 
-					first_recordable_frame += _capture_offset;
-
-				} else {
-
-					/* autopunch toggles recording at the precise
-					   transport frame, and then the DS waits
-					   to start recording for a time that depends
-					   on the output latency.
-					*/
-
-					first_recordable_frame += _session.worst_output_latency();
-				}
+                                if (_capture_offset < _session.worst_output_latency()) {
+                                        /* see comment in ExistingMaterial block above */
+                                        first_recordable_frame += (_session.worst_output_latency() - _capture_offset);
+                                }
 
 			} else {
-
-				if (Config->get_punch_in()) {
-					first_recordable_frame += _roll_delay;
-				} else {
-					capture_start_frame -= _roll_delay;
-				}
+				capture_start_frame -= _roll_delay;
 			}
-			
-		}
+                }
 
 		if (recordable() && destructive()) {
 			boost::shared_ptr<ChannelList> c = channels.reader();
@@ -484,24 +486,37 @@ AudioDiskstream::check_record_status (nframes_t transport_frame, nframes_t nfram
  			}
 		}
 
-	} else if (!record_enabled() || !can_record) {
-		
-		/* stop recording */
 
-		last_recordable_frame = transport_frame + _capture_offset;
-		
-		if (_alignment_style == ExistingMaterial) {
-			last_recordable_frame += _session.worst_output_latency();
-		} else {
-			last_recordable_frame += _roll_delay;
-		}
-	}
+        } else {
+
+                if (last_possibly_recording == fully_rec_enabled) {
+
+                        /* we were recording last time */
+                        
+                        if (change & transport_rolling) {
+                                /* transport-change (stopped rolling): last_recordable_frame was set in ::prepare_to_stop() */
+                                
+                        } else {
+                                /* punch out */
+                                
+                                last_recordable_frame = transport_frame + _capture_offset;
+                                
+                                if (_alignment_style == ExistingMaterial) {
+                                        if (_session.worst_output_latency() > _capture_offset) {
+                                                last_recordable_frame += (_session.worst_output_latency() - _capture_offset);
+                                        }
+                                } else {
+                                        last_recordable_frame += _roll_delay;
+                                }
+                        }
+                }
+        }
 
 	last_possibly_recording = possibly_recording;
 }
 
 int
-AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, nframes_t offset, bool can_record, bool rec_monitors_input)
+AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, bool can_record, bool rec_monitors_input)
 {
 	uint32_t n;
 	boost::shared_ptr<ChannelList> c = channels.reader();
@@ -509,8 +524,6 @@ AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, nframes_
 	int ret = -1;
 	nframes_t rec_offset = 0;
 	nframes_t rec_nframes = 0;
-	bool nominally_recording;
-	bool re = record_enabled ();
 	bool collect_playback = false;
 
 	/* if we've already processed the frames corresponding to this call,
@@ -527,14 +540,18 @@ AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, nframes_
 
 	commit_should_unlock = false;
 
-	if (!_io->active()) {
+	if (!_io || !_io->active()) {
 		_processed = true;
 		return 0;
 	}
 
 	check_record_status (transport_frame, nframes, can_record);
 
-	nominally_recording = (can_record && re);
+#if 0
+        cerr << _name << " can record " << can_record << " re " << record_enabled() 
+             << " FRF " << first_recordable_frame << " LRF " << last_recordable_frame
+             << endl;
+#endif
 
 	if (nframes == 0) {
 		_processed = true;
@@ -558,69 +575,67 @@ AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, nframes_
 		(*chan)->current_playback_buffer  = 0;
 	}
 
-	if (nominally_recording || (_session.get_record_enabled() && Config->get_punch_in())) {
-		OverlapType ot;
+        OverlapType ot;
 		
-		// Safeguard against situations where process() goes haywire when autopunching and last_recordable_frame < first_recordable_frame
-		if (last_recordable_frame < first_recordable_frame) {
-                        last_recordable_frame = max_frames;
-
-		}
+        // Safeguard against situations where process() goes haywire 
+        // when autopunching and last_recordable_frame < first_recordable_frame
+        
+        if (last_recordable_frame < first_recordable_frame) {
+                last_recordable_frame = max_frames;
+        }
 		
-		ot = coverage (first_recordable_frame, last_recordable_frame, transport_frame, transport_frame + nframes);
-
-		switch (ot) {
-		case OverlapNone:
-			rec_nframes = 0;
-			break;
-			
-		case OverlapInternal:
+        ot = coverage (first_recordable_frame, last_recordable_frame, transport_frame, transport_frame + nframes);
+        
+        switch (ot) {
+        case OverlapNone:
+                rec_nframes = 0;
+                break;
+		
+        case OverlapInternal:
 		/*     ----------    recrange
-                         |---|       transrange
+                       |---|       transrange
 		*/
-			rec_nframes = nframes;
-			rec_offset = 0;
-			break;
-			
-		case OverlapStart:
-			/*    |--------|    recrange
-                            -----|          transrange
-			*/
-			rec_nframes = transport_frame + nframes - first_recordable_frame;
-			if (rec_nframes) {
-				rec_offset = first_recordable_frame - transport_frame;
-			}
-			break;
-			
-		case OverlapEnd:
-			/*    |--------|    recrange
-                                 |--------  transrange
-			*/
-			rec_nframes = last_recordable_frame - transport_frame;
-			rec_offset = 0;
-			break;
-			
-		case OverlapExternal:
-			/*    |--------|    recrange
-                            --------------  transrange
-			*/
-			rec_nframes = last_recordable_frame - first_recordable_frame;
-			rec_offset = first_recordable_frame - transport_frame;
-			break;
-		}
-
-		if (rec_nframes && !was_recording) {
-			capture_captured = 0;
-			was_recording = true;
-		}
-	}
-
-
+                rec_nframes = nframes;
+                rec_offset = 0;
+                break;
+		
+        case OverlapStart:
+                /*    |--------|    recrange
+                      -----|          transrange
+                */
+                rec_nframes = transport_frame + nframes - first_recordable_frame;
+                if (rec_nframes) {
+                        rec_offset = first_recordable_frame - transport_frame;
+                }
+                break;
+		
+        case OverlapEnd:
+                /*    |--------|    recrange
+                      |--------  transrange
+                */
+                rec_nframes = last_recordable_frame - transport_frame;
+                rec_offset = 0;
+                break;
+		
+        case OverlapExternal:
+                /*    |--------|    recrange
+                      --------------  transrange
+                */
+                rec_nframes = last_recordable_frame - first_recordable_frame;
+                rec_offset = first_recordable_frame - transport_frame;
+                break;
+        }
+        
+        if (rec_nframes && !was_recording) {
+                capture_captured = 0;
+                was_recording = true;
+        }
+                
 	if (can_record && !_last_capture_regions.empty()) {
 		_last_capture_regions.clear ();
 	}
 
-	if (nominally_recording || rec_nframes) {
+	if (rec_nframes) {
 
 		uint32_t limit = _io->n_inputs ();
 
@@ -645,7 +660,7 @@ AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, nframes_
 				   rec_offset
 				*/
 
-				memcpy (chaninfo->current_capture_buffer, _io->input(n)->get_buffer (rec_nframes) + offset + rec_offset, sizeof (Sample) * rec_nframes);
+				memcpy (chaninfo->current_capture_buffer, _io->get_input_buffer (n, rec_nframes) + rec_offset, sizeof (Sample) * rec_nframes);
 
 			} else {
 
@@ -656,7 +671,7 @@ AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, nframes_
 					goto out;
 				}
 
-				Sample* buf = _io->input (n)->get_buffer (nframes) + offset;
+				Sample* buf = _io->get_input_buffer (n, nframes);
 				nframes_t first = chaninfo->capture_vector.len[0];
 
 				memcpy (chaninfo->capture_wrap_buffer, buf, sizeof (Sample) * first);
@@ -701,7 +716,7 @@ AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, nframes_
 
 		adjust_capture_position = rec_nframes;
 
-	} else if (nominally_recording) {
+	} else if (can_record && record_enabled()) {
 
 		/* can't do actual capture yet - waiting for latency effects to finish before we start*/
 
@@ -832,7 +847,7 @@ AudioDiskstream::commit (nframes_t nframes)
 {
 	bool need_butler = false;
 
-	if (!_io->active()) {
+	if (!_io || !_io->active()) {
 		return false;
 	}
 
@@ -975,7 +990,7 @@ AudioDiskstream::seek (nframes_t frame, bool complete_refill)
 		(*chan)->playback_buf->reset ();
 		(*chan)->capture_buf->reset ();
 	}
-	
+
 	/* can't rec-enable in destructive mode if transport is before start */
 	
 	if (destructive() && record_enabled() && frame < _session.current_start_frame()) {
@@ -984,7 +999,7 @@ AudioDiskstream::seek (nframes_t frame, bool complete_refill)
 	
 	playback_sample = frame;
 	file_frame = frame;
-	
+
 	if (complete_refill) {
 		while ((ret = do_refill_with_alloc ()) > 0) ;
 	} else {
@@ -1178,6 +1193,7 @@ AudioDiskstream::_do_refill (Sample* mixdown_buffer, float* gain_buffer)
 	ChannelList::iterator i;
 	boost::shared_ptr<ChannelList> c = channels.reader();
 	nframes_t ts;
+        bool debug = Glib::file_test ("/tmp/debug_ardour_disk_io", Glib::FILE_TEST_EXISTS);
 
 	if (c->empty()) {
 		return 0;
@@ -1193,6 +1209,15 @@ AudioDiskstream::_do_refill (Sample* mixdown_buffer, float* gain_buffer)
 
 	c->front()->playback_buf->get_write_vector (&vector);
 	
+        if (debug) {
+                cerr << "***************\n";
+                cerr << _name << " do_refill: write spac = " << vector.len[0] << " + " << vector.len[1]
+                     << " = " << vector.len[0] + vector.len[1] 
+                     << " (" << ((vector.len[0] + vector.len[1]) / (double) c->front()->playback_buf->bufsize()) * 100.0
+                     << "%"
+                     << endl;
+        }
+
 	if ((total_space = vector.len[0] + vector.len[1]) == 0) {
 		return 0;
 	}
@@ -1203,6 +1228,11 @@ AudioDiskstream::_do_refill (Sample* mixdown_buffer, float* gain_buffer)
 	*/
 	
 	if (total_space >= (_slaved?3:2) * disk_io_chunk_frames) {
+                if (debug) {
+                        cerr << _name << " do_refill: more than " << ((_slaved?3:2) * disk_io_chunk_frames)
+                             << " frames available in write buffer - will ask for more I/O when done"
+                             << endl;
+                }
 		ret = 1;
 	}
 	
@@ -1214,7 +1244,10 @@ AudioDiskstream::_do_refill (Sample* mixdown_buffer, float* gain_buffer)
 	*/
 	
 	if ((total_space < disk_io_chunk_frames) && fabs (_actual_speed) < 2.0f) {
-		return 0;
+                if (debug) {
+                        cerr << _name << " insufficient space in write buffer - do refill will do nothing\n";
+                }
+                return 0;
 	}
 	
 	/* when slaved, don't try to get too close to the read pointer. this
@@ -1297,6 +1330,10 @@ AudioDiskstream::_do_refill (Sample* mixdown_buffer, float* gain_buffer)
 	
 	nframes_t file_frame_tmp = 0;
 
+        if (debug) {
+                cerr << _name << " do refill : zero fill = " << zero_fill << " total to read = " << total_space << endl;
+        }
+
 	for (chan_n = 0, i = c->begin(); i != c->end(); ++i, ++chan_n) {
 
 		ChannelInfo* chan (*i);
@@ -1341,11 +1378,29 @@ AudioDiskstream::_do_refill (Sample* mixdown_buffer, float* gain_buffer)
 		to_read = min (to_read, disk_io_chunk_frames);
 
 		if (to_read) {
+                        struct timeval before;
+                        struct timeval after;
+                        struct timeval diff;
+                        double usecs;
+
+                        if (debug) {
+                                cerr << _name << " do refill1: read " << to_read << " samples for chan " << chan_n << endl;
+                        }
+                        gettimeofday (&before, 0);
 
 			if (read (buf1, mixdown_buffer, gain_buffer, file_frame_tmp, to_read, chan, chan_n, reversed)) {
 				ret = -1;
 				goto out;
 			}
+                        gettimeofday (&after, 0);
+                        timersub (&after, &before, &diff);
+                        usecs = diff.tv_sec * 1000000.0 + diff.tv_usec;
+
+                        if (debug) {
+                                cerr << _name << " usecs = " << usecs << " throughput = " 
+                                     << ((to_read/usecs) * 1000000.0 * sizeof (Sample)/1048576.0) << " MiB/sec"
+                                     << endl;
+                        }
 
 			chan->playback_buf->increment_write_ptr (to_read);
 			ts -= to_read;
@@ -1354,6 +1409,16 @@ AudioDiskstream::_do_refill (Sample* mixdown_buffer, float* gain_buffer)
 		to_read = min (ts, len2);
 
 		if (to_read) {
+                        struct timeval before;
+                        struct timeval after;
+                        struct timeval diff;
+                        double usecs;
+
+                        if (debug) {
+                                cerr << _name << " do refill2: read " << to_read << " samples for chan " << chan_n << endl;
+                        }
+                        gettimeofday (&before, 0);
+
 
 			/* we read all of vector.len[0], but it wasn't an entire disk_io_chunk_frames of data,
 			   so read some or all of vector.len[1] as well.
@@ -1363,14 +1428,23 @@ AudioDiskstream::_do_refill (Sample* mixdown_buffer, float* gain_buffer)
 				ret = -1;
 				goto out;
 			}
-		
+                        gettimeofday (&after, 0);
+                        timersub (&after, &before, &diff);
+                        usecs = diff.tv_sec * 1000000.0 + diff.tv_usec;
+                        
+                        if (debug) {
+                                cerr << _name << " usecs = " << usecs << " throughput = " 
+                                     << ((to_read/usecs) * 1000000.0 * sizeof (Sample)/1048576.0) << " MiB/sec"
+                                     << endl;
+                        }
+                        
 			chan->playback_buf->increment_write_ptr (to_read);
 		}
 
 		if (zero_fill) {
 			/* do something */
 		}
-
+                
 	}
 	
 	file_frame = file_frame_tmp;
@@ -1378,7 +1452,7 @@ AudioDiskstream::_do_refill (Sample* mixdown_buffer, float* gain_buffer)
   out:
 
 	return ret;
-}	
+}
 
 /** Flush pending data to disk.
  *
@@ -1407,7 +1481,8 @@ AudioDiskstream::do_flush (Session::RunContext context, bool force_flush)
 	vector.buf[1] = 0;
 
 	boost::shared_ptr<ChannelList> c = channels.reader();
-	for (ChannelList::iterator chan = c->begin(); chan != c->end(); ++chan) {
+	int nn = 0;
+	for (ChannelList::iterator chan = c->begin(); chan != c->end(); ++chan, ++nn) {
 	
 		(*chan)->capture_buf->get_read_vector (&vector);
 
@@ -1583,10 +1658,15 @@ AudioDiskstream::transport_stopped (struct tm& when, time_t twhen, bool abort_ca
 		goto out;
 	} 
 
+        // cerr << _name << " check capture info ...\n";
+
 	for (total_capture = 0, ci = capture_info.begin(); ci != capture_info.end(); ++ci) {
+                // cerr << "\tCI frames = " << (*ci)->frames << endl;
 		total_capture += (*ci)->frames;
 	}
 
+        // cerr << "\ttotal for this take = " << total_capture << endl;
+
 	/* figure out the name for this take */
 
 	for (n = 0, chan = c->begin(); chan != c->end(); ++chan, ++n) {
@@ -1753,7 +1833,9 @@ void
 AudioDiskstream::finish_capture (bool rec_monitors_input, boost::shared_ptr<ChannelList> c)
 {
 	was_recording = false;
-	
+        first_recordable_frame = max_frames;
+        last_recordable_frame = max_frames;
+
 	if (capture_captured == 0) {
 		return;
 	}
@@ -2086,7 +2168,7 @@ AudioDiskstream::reset_write_sources (bool mark_write_complete, bool force)
 	boost::shared_ptr<ChannelList> c = channels.reader();
 	uint32_t n;
 
-	if (!recordable()) {
+	if (!_session.writable() || !recordable()) {
 		return;
 	}
 	
