@@ -96,6 +96,7 @@ AudioFileSource::AudioFileSource (Session& s, ustring path, Flag flags)
 		throw failed_constructor ();
 	}
 
+	fix_writable_flags ();
 }
 
 AudioFileSource::AudioFileSource (Session& s, ustring path, Flag flags, SampleFormat samp_format, HeaderFormat hdr_format)
@@ -108,6 +109,8 @@ AudioFileSource::AudioFileSource (Session& s, ustring path, Flag flags, SampleFo
 	if (init (path, false)) {
 		throw failed_constructor ();
 	}
+
+	fix_writable_flags ();
 }
 
 AudioFileSource::AudioFileSource (Session& s, const XMLNode& node, bool must_exist)
@@ -125,6 +128,8 @@ AudioFileSource::AudioFileSource (Session& s, const XMLNode& node, bool must_exi
 	if (init (foo, must_exist)) {
 		throw failed_constructor ();
 	}
+
+	fix_writable_flags ();
 }
 
 AudioFileSource::~AudioFileSource ()
@@ -135,6 +140,14 @@ AudioFileSource::~AudioFileSource ()
 	}
 }
 
+void
+AudioFileSource::fix_writable_flags ()
+{
+	if (!_session.writable()) {
+		_flags = Flag (_flags & ~(Writable|Removable|RemovableIfEmpty|RemoveAtDestroy|CanRename));
+	}
+}
+
 bool
 AudioFileSource::determine_embeddedness (ustring path)
 {
@@ -147,6 +160,12 @@ AudioFileSource::removable () const
 	return (_flags & Removable) && ((_flags & RemoveAtDestroy) || ((_flags & RemovableIfEmpty) && length() == 0));
 }
 
+bool
+AudioFileSource::writable() const
+{
+	return (_flags & Writable);
+}
+
 int
 AudioFileSource::init (ustring pathstr, bool must_exist)
 {
@@ -154,7 +173,11 @@ AudioFileSource::init (ustring pathstr, bool must_exist)
 	timeline_position = 0;
 	_peaks_built = false;
 
-	if (!find (pathstr, must_exist, file_is_new, _channel)) {
+	/* is_embedded() can't work yet, because our _path is not set */
+
+	bool embedded = determine_embeddedness (pathstr);
+
+	if (!find (pathstr, must_exist, embedded, file_is_new, _channel, _path, _name)) {
 		throw non_existent_source ();
 	}
 
@@ -222,13 +245,7 @@ AudioFileSource::find_broken_peakfile (ustring peak_path, ustring audio_path)
 ustring
 AudioFileSource::broken_peak_path (ustring audio_path)
 {
-	ustring res;
-
-	res = _session.peak_dir ();
-	res += PBD::basename_nosuffix (audio_path);
-	res += ".peak";
-
-	return res;
+	return Glib::build_filename(_session.peak_dir (), PBD::basename_nosuffix (audio_path) + ".peak");
 }
 
 ustring
@@ -260,16 +277,20 @@ AudioFileSource::old_peak_path (ustring audio_path)
 bool
 AudioFileSource::get_soundfile_info (ustring path, SoundFileInfo& _info, string& error_msg)
 {
+	/* try sndfile first because it gets timecode info from .wav (BWF) if it exists,
+	   which at present, ExtAudioFile from Apple seems unable to do.
+	*/
+
+	if (SndFileSource::get_soundfile_info (path, _info, error_msg) != 0) {
+		return true;
+	}
+
 #ifdef HAVE_COREAUDIO
 	if (CoreAudioSource::get_soundfile_info (path, _info, error_msg) == 0) {
 		return true;
 	}
 #endif // HAVE_COREAUDIO
 
-	if (SndFileSource::get_soundfile_info (path, _info, error_msg) != 0) {
-		return true;
-	}
-
 	return false;
 }
 
@@ -300,6 +321,8 @@ AudioFileSource::set_state (const XMLNode& node)
 
 	}
 
+	fix_writable_flags ();
+
 	if ((prop = node.property (X_("channel"))) != 0) {
 		_channel = atoi (prop->value());
 	} else {
@@ -326,7 +349,7 @@ AudioFileSource::mark_for_remove ()
 	// This operation is not allowed for sources for destructive tracks or embedded files.
 	// Fortunately mark_for_remove() is never called for embedded files. This function
 	// must be fixed if that ever happens.
-	if (_flags & Destructive) {
+	if (!_session.writable() || (_flags & Destructive)) {
 		return;
 	}
 
@@ -383,12 +406,14 @@ AudioFileSource::move_to_trash (const ustring& trash_dir_name)
 
 	cerr << "from " << _path << " dead dir looks like " << newpath << endl;
 
-	newpath += '/';
-	newpath += trash_dir_name;
-	newpath += '/';
-	newpath += Glib::path_get_basename (_path);
+        vector<string> p;
+        p.push_back (newpath);
+        p.push_back (trash_dir_name);
+        p.push_back (Glib::path_get_basename (_path));
 
-	if (access (newpath.c_str(), F_OK) == 0) {
+	newpath = Glib::build_filename (p);
+
+	if (Glib::file_test (newpath, Glib::FILE_TEST_EXISTS)) {
 
 		/* the new path already exists, try versioning */
 		
@@ -399,7 +424,7 @@ AudioFileSource::move_to_trash (const ustring& trash_dir_name)
 		snprintf (buf, sizeof (buf), "%s.%d", newpath.c_str(), version);
 		newpath_v = buf;
 
-		while (access (newpath_v.c_str(), F_OK) == 0 && version < 999) {
+		while (Glib::file_test (newpath_v, Glib::FILE_TEST_EXISTS) && version < 999) {
 			snprintf (buf, sizeof (buf), "%s.%d", newpath.c_str(), ++version);
 			newpath_v = buf;
 		}
@@ -419,7 +444,7 @@ AudioFileSource::move_to_trash (const ustring& trash_dir_name)
 	}
 
 	if (::rename (_path.c_str(), newpath.c_str()) != 0) {
-		error << string_compose (_("cannot rename audio file source from %1 to %2 (%3)"),
+		error << string_compose (_("cannot 1 rename audio file source from %1 to %2 (%3)"),
 				  _path, newpath, strerror (errno))
 		      << endmsg;
 		return -1;
@@ -445,14 +470,16 @@ AudioFileSource::move_to_trash (const ustring& trash_dir_name)
 }
 
 bool
-AudioFileSource::find (ustring& pathstr, bool must_exist, bool& isnew, uint16_t& chan)
+AudioFileSource::find (ustring pathstr, bool must_exist, bool embedded,
+		       bool& isnew, uint16_t& chan, 
+		       ustring& path, std::string& name)
 {
 	ustring::size_type pos;
 	bool ret = false;
 
 	isnew = false;
 
-	if (pathstr[0] != '/') {
+	if (!Glib::path_is_absolute (pathstr)) {
 
 		/* non-absolute pathname: find pathstr in search path */
 
@@ -472,12 +499,7 @@ AudioFileSource::find (ustring& pathstr, bool must_exist, bool& isnew, uint16_t&
 		
 		for (vector<ustring>::iterator i = dirs.begin(); i != dirs.end(); ++i) {
 
-			fullpath = *i;
-			if (fullpath[fullpath.length()-1] != '/') {
-				fullpath += '/';
-			}
-
-			fullpath += pathstr;
+			fullpath = Glib::build_filename (*i, pathstr);
 
 			/* i (paul) made a nasty design error by using ':' as a special character in
 			   Ardour 0.99 .. this hack tries to make things sort of work.
@@ -501,13 +523,7 @@ AudioFileSource::find (ustring& pathstr, bool must_exist, bool& isnew, uint16_t&
 						 */
 						
 						ustring shorter = pathstr.substr (0, pos);
-						fullpath = *i;
-
-						if (fullpath[fullpath.length()-1] != '/') {
-							fullpath += '/';
-						}
-
-						fullpath += shorter;
+						fullpath = Glib::build_filename (*i, shorter);
 
 						if (Glib::file_test (pathstr, Glib::FILE_TEST_EXISTS|Glib::FILE_TEST_IS_REGULAR)) {
 							chan = atoi (pathstr.substr (pos+1));
@@ -547,8 +563,8 @@ AudioFileSource::find (ustring& pathstr, bool must_exist, bool& isnew, uint16_t&
 			}
 		}
 
-		_name = pathstr;
-		_path = keeppath;
+		name = pathstr;
+		path = keeppath;
 		ret = true;
 
 	} else {
@@ -567,12 +583,12 @@ AudioFileSource::find (ustring& pathstr, bool must_exist, bool& isnew, uint16_t&
 			}
 		}
 		
-		_path = pathstr;
+		path = pathstr;
 
-		if (is_embedded()) {
-			_name = pathstr;
+		if (embedded) {
+			name = pathstr;
 		} else {
-			_name = pathstr.substr (pathstr.find_last_of ('/') + 1);
+			name = Glib::path_get_basename (pathstr);
 		}
 
 		if (!Glib::file_test (pathstr, Glib::FILE_TEST_EXISTS|Glib::FILE_TEST_IS_REGULAR)) {
@@ -580,12 +596,12 @@ AudioFileSource::find (ustring& pathstr, bool must_exist, bool& isnew, uint16_t&
 			/* file does not exist or we cannot read it */
 			
 			if (must_exist) {
-				error << string_compose(_("Filesource: cannot find required file (%1): %2"), _path, strerror (errno)) << endmsg;
+				error << string_compose(_("Filesource: cannot find required file (%1): %2"), pathstr, strerror (errno)) << endmsg;
 				goto out;
 			}
 			
 			if (errno != ENOENT) {
-				error << string_compose(_("Filesource: cannot check for existing file (%1): %2"), _path, strerror (errno)) << endmsg;
+				error << string_compose(_("Filesource: cannot check for existing file (%1): %2"), pathstr, strerror (errno)) << endmsg;
 				goto out;
 			}
 			
@@ -638,6 +654,8 @@ AudioFileSource::set_allow_remove_if_empty (bool yn)
 	} else {
 		_flags = Flag (_flags & ~RemovableIfEmpty);
 	}
+
+	fix_writable_flags ();
 }
 
 int
@@ -654,12 +672,12 @@ AudioFileSource::set_name (ustring newname, bool destructive)
 
 	// Test whether newpath exists, if yes notify the user but continue. 
 	if (access(newpath.c_str(),F_OK) == 0) {
-		error << _("Programming error! Ardour tried to rename a file over another file! It's safe to continue working, but please report this to the developers.") << endmsg;
+		error << _("Programming error! Tried to rename a file over another file! It's safe to continue working, but please report this to the developers.") << endmsg;
 		return -1;
 	}
 
 	if (rename (oldpath.c_str(), newpath.c_str()) != 0) {
-		error << string_compose (_("cannot rename audio file %1 to %2"), _name, newpath) << endmsg;
+		error << string_compose (_("cannot 2 rename audio file %1 to %2"), _name, newpath) << endmsg;
 		return -1;
 	}
 
@@ -697,29 +715,43 @@ bool
 AudioFileSource::safe_file_extension(ustring file)
 {
 	const char* suffixes[] = {
-		".wav", ".WAV",
-		".aiff", ".AIFF",
-		".caf", ".CAF",
 		".aif", ".AIF",
+		".aifc", ".AIFC",
+		".aiff", ".AIFF",
 		".amb", ".AMB",
-		".snd", ".SND",
 		".au", ".AU",
-		".raw", ".RAW",
-		".sf", ".SF",
+		".caf", ".CAF",
 		".cdr", ".CDR",
-		".smp", ".SMP",
-		".maud", ".MAUD",
-		".vwe", ".VWE",
-		".paf",
-#ifdef HAVE_FLAC
 		".flac", ".FLAC",
-#endif // HAVE_FLAC
-#ifdef HAVE_COREAUDIO
-		".mp3", ".MP3",
-		".aac", ".AAC",
-		".mp4", ".MP4",
-#endif // HAVE_COREAUDIO
+		".htk", ".HTK",
+		".iff", ".IFF",
+		".mat", ".MAT",
+		".oga", ".OGA",
+		".ogg", ".OGG",
+		".paf", ".PAF",
+		".pvf", ".PVF",
+		".sf", ".SF",
+		".smp", ".SMP",
+		".snd", ".SND",
+		".maud", ".MAUD",
 		".voc", ".VOC"
+		".vwe", ".VWE",
+		".w64", ".W64",
+		".wav", ".WAV",
+#ifdef HAVE_COREAUDIO
+		".aac", ".AAC",
+		".adts", ".ADTS",
+		".ac3", ".AC3",
+		".amr", ".AMR",
+		".mpa", ".MPA",
+		".mpeg", ".MPEG",
+		".mp1", ".MP1",
+		".mp2", ".MP2",
+		".mp3", ".MP3",
+		".mp4", ".MP4",
+		".m4a", ".M4A",
+		".sd2", ".SD2", 	// libsndfile supports sd2 also, but the resource fork is required to open.
+#endif // HAVE_COREAUDIO
 	};
 
 	for (size_t n = 0; n < sizeof(suffixes)/sizeof(suffixes[0]); ++n) {
