@@ -21,9 +21,10 @@
 #include <climits>
 #include <cfloat>
 #include <algorithm>
-
 #include <set>
 
+#include <boost/scoped_array.hpp>
+
 #include <sigc++/bind.h>
 #include <sigc++/class_slot.h>
 
@@ -34,6 +35,7 @@
 #include <pbd/stacktrace.h>
 #include <pbd/enumwriter.h>
 #include <pbd/convert.h>
+#include <pbd/localeguard.h>
 
 #include <ardour/audioregion.h>
 #include <ardour/session.h>
@@ -181,7 +183,6 @@ AudioRegion::AudioRegion (boost::shared_ptr<const AudioRegion> other, nframes_t
 	_fade_in_disabled = 0;
 	_fade_out_disabled = 0;
 
-
 	if (_flags & LeftOfSplit) {
 		if (_fade_in.back()->when >= _length) {
 			set_default_fade_in ();
@@ -212,7 +213,8 @@ AudioRegion::AudioRegion (boost::shared_ptr<const AudioRegion> other)
 	: Region (other),
 	  _fade_in (other->_fade_in),
 	  _fade_out (other->_fade_out),
-	  _envelope (other->_envelope) 
+	  _envelope (other->_envelope)
+	  
 {
 	/* Pure copy constructor */
 
@@ -241,11 +243,42 @@ AudioRegion::AudioRegion (boost::shared_ptr<const AudioRegion> other)
 	}
 
 	_scale_amplitude = other->_scale_amplitude;
-	_envelope = other->_envelope;
 
 	_fade_in_disabled = 0;
 	_fade_out_disabled = 0;
-	
+
+	listen_to_my_curves ();
+	listen_to_my_sources ();
+}
+
+AudioRegion::AudioRegion (boost::shared_ptr<const AudioRegion> other, const SourceList& srcs,
+			  nframes_t length, const string& name, layer_t layer, Flag flags)
+	: Region (other, length, name, layer, flags),
+	  _fade_in (other->_fade_in),
+	  _fade_out (other->_fade_out),
+	  _envelope (other->_envelope)
+	  
+{
+	/* make-a-sort-of-copy-with-different-sources constructor (used by audio filter) */
+
+	set<boost::shared_ptr<AudioSource> > unique_srcs;
+
+	for (SourceList::const_iterator i=srcs.begin(); i != srcs.end(); ++i) {
+		sources.push_back (*i);
+		master_sources.push_back (*i);
+		(*i)->GoingAway.connect (mem_fun (*this, &AudioRegion::source_deleted));
+		
+		boost::shared_ptr<AudioFileSource> afs = boost::dynamic_pointer_cast<AudioFileSource> ((*i));
+		if (afs) {
+			afs->HeaderPositionOffsetChanged.connect (mem_fun (*this, &AudioRegion::source_offset_changed));
+		}
+	}
+
+	_scale_amplitude = other->_scale_amplitude;
+
+	_fade_in_disabled = 0;
+	_fade_out_disabled = 0;
+
 	listen_to_my_curves ();
 	listen_to_my_sources ();
 }
@@ -388,6 +421,7 @@ AudioRegion::verify_start_and_length (nframes_t new_start, nframes_t& new_length
 
 	return true;
 }
+
 bool
 AudioRegion::verify_start (nframes_t pos)
 {
@@ -458,26 +492,30 @@ AudioRegion::read_peaks (PeakData *buf, nframes_t npeaks, nframes_t offset, nfra
 }
 
 nframes64_t
-AudioRegion::read (Sample* buf, nframes64_t position, nframes64_t cnt, int channel) const
+AudioRegion::read (Sample* buf, nframes64_t timeline_position, nframes64_t cnt, int channel) const
 {
 	/* raw read, no fades, no gain, nada */
-	return _read_at (sources, _length, buf, 0, 0, _position + position, cnt, channel, 0, 0, true);
+	return _read_at (sources, _length, buf, 0, 0, _position + timeline_position, cnt, channel, 0, 0, ReadOps (0));
 }
 
 nframes_t
-AudioRegion::read_at (Sample *buf, Sample *mixdown_buffer, float *gain_buffer, nframes_t position, 
+AudioRegion::read_at (Sample *buf, Sample *mixdown_buffer, float *gain_buffer, nframes_t file_position, 
 		      nframes_t cnt, 
 		      uint32_t chan_n, nframes_t read_frames, nframes_t skip_frames) const
 {
 	/* regular diskstream/butler read complete with fades etc */
-	return _read_at (sources, _length, buf, mixdown_buffer, gain_buffer, position, cnt, chan_n, read_frames, skip_frames, false);
+	return _read_at (sources, _length, buf, mixdown_buffer, gain_buffer, file_position, cnt, 
+			 chan_n, read_frames, skip_frames, ReadOps (~0));
 }
 
 nframes_t
 AudioRegion::master_read_at (Sample *buf, Sample *mixdown_buffer, float *gain_buffer, nframes_t position, 
 			     nframes_t cnt, uint32_t chan_n) const
 {
-	return _read_at (master_sources, master_sources.front()->length(), buf, mixdown_buffer, gain_buffer, position, cnt, chan_n, 0, 0);
+	/* do not read gain/scaling/fades and do not count this disk i/o in statistics */
+
+	return _read_at (master_sources, master_sources.front()->length(), buf, mixdown_buffer, 
+			 gain_buffer, position, cnt, chan_n, 0, 0, ReadOps (0));
 }
 
 nframes_t
@@ -487,11 +525,16 @@ AudioRegion::_read_at (const SourceList& srcs, nframes_t limit,
 		       uint32_t chan_n, 
 		       nframes_t read_frames, 
 		       nframes_t skip_frames,
-		       bool raw) const
+		       ReadOps rops) const
 {
 	nframes_t internal_offset;
 	nframes_t buf_offset;
 	nframes_t to_read;
+	bool raw = (rops == ReadOpsNone);
+
+        if (n_channels() == 0) {
+                return 0;
+        }
 
 	if (muted() && !raw) {
 		return 0; /* read nothing */
@@ -523,41 +566,46 @@ AudioRegion::_read_at (const SourceList& srcs, nframes_t limit,
 		mixdown_buffer += buf_offset;
 	}
 
-	if (!raw) {
+	if (rops & ReadOpsCount) {
 		_read_data_count = 0;
 	}
 
 	if (chan_n < n_channels()) {
 
 		if (srcs[chan_n]->read (mixdown_buffer, _start + internal_offset, to_read) != to_read) {
-			
 			return 0; /* "read nothing" */
 		}
 		
-		if (!raw) {
+		if (rops & ReadOpsCount) {
 			_read_data_count += srcs[chan_n]->read_data_count();
 		}
 
 	} else {
 		
-		/* track is N-channel, this region has less channels; silence the ones
-		   we don't have.
-		*/
+                if (Config->get_replicate_missing_region_channels()) {
+                        /* track is N-channel, this region has less channels, so use a relevant channel
+                         */
+                        
+                        uint32_t channel = n_channels() % chan_n;
+                        
+                        if (srcs[channel]->read (mixdown_buffer, _start + internal_offset, to_read) != to_read) {
+                                return 0; /* "read nothing" */
+                        }
+                        
+                        /* adjust read data count appropriately since this was a duplicate read */
+                        srcs[channel]->dec_read_data_count (to_read);
 
-		memset (mixdown_buffer, 0, sizeof (Sample) * cnt);
-
-		/* no fades required */
-
-		if (!raw) {
-			goto merge;
-		}
+                } else {
+                        /* region has no data for requested channel, so make it silent */
+                        memset (mixdown_buffer, 0, to_read * sizeof (Sample));
+                }
 	}
 
-	/* fade in */
-
-	if (!raw) {
+	if (rops & ReadOpsFades) {
 	
-		if (_flags & FadeIn) {
+		/* fade in */
+
+		if ((_flags & FadeIn) && Config->get_use_region_fades()) {
 			
 			nframes_t fade_in_length = (nframes_t) _fade_in.back()->when;
 			
@@ -565,13 +613,13 @@ AudioRegion::_read_at (const SourceList& srcs, nframes_t limit,
 			
 			if (internal_offset < fade_in_length) {
 				
-				nframes_t limit;
+				nframes_t fi_limit;
 				
-				limit = min (to_read, fade_in_length - internal_offset);
+				fi_limit = min (to_read, fade_in_length - internal_offset);
 				
-				_fade_in.get_vector (internal_offset, internal_offset+limit, gain_buffer, limit);
+				_fade_in.get_vector (internal_offset, internal_offset+fi_limit, gain_buffer, fi_limit);
 				
-				for (nframes_t n = 0; n < limit; ++n) {
+				for (nframes_t n = 0; n < fi_limit; ++n) {
 					mixdown_buffer[n] *= gain_buffer[n];
 				}
 			}
@@ -579,17 +627,17 @@ AudioRegion::_read_at (const SourceList& srcs, nframes_t limit,
 		
 		/* fade out */
 		
-		if (_flags & FadeOut) {
+		if ((_flags & FadeOut) && Config->get_use_region_fades()) {
 
 			/* see if some part of this read is within the fade out */
 			
 		/* .................        >|            REGION
-		                            limit
+		                             limit
  					    
                                  {           }            FADE
 				             fade_out_length
                                  ^					     
-                                limit - fade_out_length
+                                 limit - fade_out_length
                         |--------------|
                         ^internal_offset
                                        ^internal_offset + to_read
@@ -619,39 +667,37 @@ AudioRegion::_read_at (const SourceList& srcs, nframes_t limit,
 			} 
 			
 		}
-		
-		/* Regular gain curves */
-		
-		if (envelope_active())  {
-			_envelope.get_vector (internal_offset, internal_offset + to_read, gain_buffer, to_read);
-			
-			if (_scale_amplitude != 1.0f) {
-				for (nframes_t n = 0; n < to_read; ++n) {
-					mixdown_buffer[n] *= gain_buffer[n] * _scale_amplitude;
-				}
-			} else {
-				for (nframes_t n = 0; n < to_read; ++n) {
-					mixdown_buffer[n] *= gain_buffer[n];
-				}
-			}
-		} else if (_scale_amplitude != 1.0f) {
-			Session::apply_gain_to_buffer (mixdown_buffer, to_read, _scale_amplitude);
-		}
-	
-	  merge:
-		
-		if (!opaque()) {
-			
-			/* gack. the things we do for users.
-			 */
-			
-			buf += buf_offset;
-			
-			for (nframes_t n = 0; n < to_read; ++n) {
-				buf[n] += mixdown_buffer[n];
-			}
-		} 
 	}
+		
+        /* Regular gain curves and scaling */
+	
+	if ((rops & ReadOpsOwnAutomation) && envelope_active())  {
+		_envelope.get_vector (internal_offset, internal_offset + to_read, gain_buffer, to_read);
+		
+		if ((rops & ReadOpsOwnScaling) && _scale_amplitude != 1.0f) {
+			for (nframes_t n = 0; n < to_read; ++n) {
+				mixdown_buffer[n] *= gain_buffer[n] * _scale_amplitude;
+			}
+		} else {
+			for (nframes_t n = 0; n < to_read; ++n) {
+				mixdown_buffer[n] *= gain_buffer[n];
+			}
+		}
+	} else if ((rops & ReadOpsOwnScaling) && _scale_amplitude != 1.0f) {
+		Session::apply_gain_to_buffer (mixdown_buffer, to_read, _scale_amplitude);
+	}
+	
+	if (!opaque() && (buf != mixdown_buffer)) {
+		
+		/* gack. the things we do for users.
+		 */
+		
+		buf += buf_offset;
+			
+		for (nframes_t n = 0; n < to_read; ++n) {
+			buf[n] += mixdown_buffer[n];
+		}
+	} 
 
 	return to_read;
 }
@@ -812,7 +858,7 @@ AudioRegion::set_live_state (const XMLNode& node, Change& what_changed, bool sen
 			}
 
 			if ((prop = child->property ("active")) != 0) {
-				if (prop->value() == "yes") {
+				if (string_is_affirmative (prop->value())) {
 					set_fade_in_active (true);
 				} else {
 					set_fade_in_active (true);
@@ -833,7 +879,7 @@ AudioRegion::set_live_state (const XMLNode& node, Change& what_changed, bool sen
 			}
 
 			if ((prop = child->property ("active")) != 0) {
-				if (prop->value() == "yes") {
+				if (string_is_affirmative (prop->value())) {
 					set_fade_out_active (true);
 				} else {
 					set_fade_out_active (false);
@@ -1106,15 +1152,18 @@ AudioRegion::recompute_at_end ()
 	_envelope.set_max_xval (_length);
 	_envelope.thaw ();
 
+	if (_flags & LeftOfSplit) {
+		set_default_fade_out ();
+		_flags = Flag (_flags & ~Region::LeftOfSplit);
+	} else if (_fade_out.back()->when > _length) {
+		_fade_out.extend_to (_length);
+		send_change (FadeOutChanged);
+	}
+
 	if (_fade_in.back()->when > _length) {
 		_fade_in.extend_to (_length);
 		send_change (FadeInChanged);
 	}
-
-	if (_fade_out.back()->when > _length) {
-		_fade_out.extend_to (_length);
-		send_change (FadeOutChanged);
-	}
 }	
 
 void
@@ -1124,7 +1173,10 @@ AudioRegion::recompute_at_start ()
 
 	_envelope.truncate_start (_length);
 
-	if (_fade_in.back()->when > _length) {
+	if (_flags & RightOfSplit) {
+		set_default_fade_in ();
+		_flags = Flag (_flags & ~Region::RightOfSplit);
+	} else if (_fade_in.back()->when > _length) {
 		_fade_in.extend_to (_length);
 		send_change (FadeInChanged);
 	}
@@ -1214,11 +1266,17 @@ AudioRegion::source_equivalent (boost::shared_ptr<const Region> o) const
 {
 	boost::shared_ptr<const AudioRegion> other = boost::dynamic_pointer_cast<const AudioRegion>(o);
 
-	if (!other)
+	if (!other) {
 		return false;
+        }
 
 	SourceList::const_iterator i;
 	SourceList::const_iterator io;
+        
+        if ((sources.size() != other->sources.size()) ||
+            (master_sources.size() != other->master_sources.size())) {
+                return false;
+        }
 
 	for (i = sources.begin(), io = other->sources.begin(); i != sources.end() && io != other->sources.end(); ++i, ++io) {
 		if ((*i)->id() != (*io)->id()) {
@@ -1247,7 +1305,9 @@ AudioRegion::exportme (Session& session, AudioExportSpecification& spec)
 {
 	const nframes_t blocksize = 4096;
 	nframes_t to_read;
+        nframes_t nread;
 	int status = -1;
+        boost::scoped_array<Sample> gain_buffer (new Sample[blocksize]);
 
 	spec.channels = sources.size();
 
@@ -1255,29 +1315,33 @@ AudioRegion::exportme (Session& session, AudioExportSpecification& spec)
 		goto out;
 	}
 
-	spec.pos = 0;
-	spec.total_frames = _length;
+        /* the ::read_at() methods expect a starting position that is absolute, not relative
+           to our bounds. So we begin at our _position on the timeline ...
+        */
 
-	while (spec.pos < _length && !spec.stop) {
-		
+	spec.pos = _position;
+	spec.total_frames = _length;
+        nread = 0;
+
+	while (spec.pos < last_frame() && !spec.stop) {
 		
 		/* step 1: interleave */
 		
-		to_read = min (_length - spec.pos, blocksize);
-		
+		to_read = min (_length - nread, blocksize);
+                
 		if (spec.channels == 1) {
-
-			if (sources.front()->read (spec.dataF, _start + spec.pos, to_read) != to_read) {
+                        
+			if (read_at (spec.dataF, spec.dataF, gain_buffer.get(), spec.pos, to_read) != to_read) {
 				goto out;
 			}
 
 		} else {
 
-			Sample buf[blocksize];
+                        boost::scoped_array<Sample> buf (new Sample[blocksize]);
 
 			for (uint32_t chan = 0; chan < spec.channels; ++chan) {
 				
-				if (sources[chan]->read (buf, _start + spec.pos, to_read) != to_read) {
+				if (read_at (buf.get(), buf.get(), gain_buffer.get(), spec.pos, to_read, chan) != to_read) {
 					goto out;
 				}
 				
@@ -1292,8 +1356,8 @@ AudioRegion::exportme (Session& session, AudioExportSpecification& spec)
 		}
 		
 		spec.pos += to_read;
-		spec.progress = (double) spec.pos /_length;
-		
+                nread += to_read;
+		spec.progress = (double) nread /_length;
 	}
 	
 	status = 0;
@@ -1345,7 +1409,7 @@ void
 AudioRegion::normalize_to (float target_dB)
 {
 	const nframes_t blocksize = 64 * 1024;
-	Sample buf[blocksize];
+        boost::scoped_array<Sample> buf (new Sample[blocksize]);
 	nframes_t fpos;
 	nframes_t fend;
 	nframes_t to_read;
@@ -1374,11 +1438,11 @@ AudioRegion::normalize_to (float target_dB)
 
 			/* read it in */
 
-			if (source (n)->read (buf, fpos, to_read) != to_read) {
+			if (source (n)->read (buf.get(), fpos, to_read) != to_read) {
 				return;
 			}
 			
-			maxamp = Session::compute_peak (buf, to_read, maxamp);
+			maxamp = Session::compute_peak (buf.get(), to_read, maxamp);
 		}
 
 		fpos += to_read;
@@ -1481,6 +1545,15 @@ AudioRegion::speed_mismatch (float sr) const
 void
 AudioRegion::source_offset_changed ()
 {
+	/* XXX this fixes a crash that should not occur. It does occur
+	   becauses regions are not being deleted when a session
+	   is unloaded. That bug must be fixed.
+	*/
+
+	if (sources.empty()) {
+		return;
+	}
+
 	boost::shared_ptr<AudioFileSource> afs = boost::dynamic_pointer_cast<AudioFileSource>(sources.front());
 
 	if (afs && afs->destructive()) {
@@ -1592,14 +1665,21 @@ AudioRegion::get_transients (AnalysisFeatureList& results, bool force_new)
 
 	/* no existing/complete transient info */
 
+	static bool analyse_dialog_shown = false;
 	if (!Config->get_auto_analyse_audio()) {
-		pl->session().Dialog (_("\
+		if ( !analyse_dialog_shown ) {
+			pl->session().Dialog (_("\
 You have requested an operation that requires audio analysis.\n\n\
 You currently have \"auto-analyse-audio\" disabled, which means\n\
 that transient data must be generated every time it is required.\n\n\
 If you are doing work that will require transient data on a\n\
 regular basis, you should probably enable \"auto-analyse-audio\"\n\
-then quit ardour and restart."));
+then quit ardour and restart.\n\n\
+This dialog will not display again.  But you may notice a slight delay\n\
+in this and future transient-detection operations.\n\
+"));
+			analyse_dialog_shown = true;  //only show this dialog once 
+			}
 	}
 
 	TransientDetector t (pl->session().frame_rate());
