@@ -175,7 +175,7 @@ AudioSource::rename_peakfile (ustring newpath)
 
 	if (access (oldpath.c_str(), F_OK) == 0) {
 		if (rename (oldpath.c_str(), newpath.c_str()) != 0) {
-			error << string_compose (_("cannot rename peakfile for %1 from %2 to %3 (%4)"), _name, oldpath, newpath, strerror (errno)) << endmsg;
+			error << string_compose (_("cannot 3 rename peakfile for %1 from %2 to %3 (%4)"), _name, oldpath, newpath, strerror (errno)) << endmsg;
 			return -1;
 		}
 	}
@@ -214,7 +214,7 @@ AudioSource::initialize_peakfile (bool newfile, ustring audio_path)
 		
 		/* we found it in the peaks dir, so check it out */
 		
-		if (statbuf.st_size == 0 || (statbuf.st_size < ((length() / _FPP) * sizeof (PeakData)))) {
+		if (statbuf.st_size == 0 || ((nframes_t) statbuf.st_size < ((length() / _FPP) * sizeof (PeakData)))) {
 			// empty
 			_peaks_built = false;
 		} else {
@@ -523,21 +523,28 @@ AudioSource::read_peaks_with_fpp (PeakData *peaks, nframes_t npeaks, nframes_t s
 
 			if (i == frames_read) {
 				
-				to_read = min (chunksize, (_length - current_frame));
+				if (current_frame >= _length) {
 
-				if (to_read == 0) {
-					/* XXX ARGH .. out by one error ... need to figure out why this happens
-					   and fix it rather than do this band-aid move.
+					/* hmm, error condition - we've reached the end of the file
+					   without generating all the peak data. cook up a zero-filled
+					   data buffer and then use it. this is simpler than
+					   adjusting zero_fill and npeaks and then breaking out of 
+					   this loop early 
 					*/
-					zero_fill = npeaks - nvisual_peaks;
-					break;
-				}
 
-				if ((frames_read = read_unlocked (raw_staging, current_frame, to_read)) == 0) {
-					error << string_compose(_("AudioSource[%1]: peak read - cannot read %2 samples at offset %3 of %4 (%5)"),
-								_name, to_read, current_frame, _length, strerror (errno)) 
-					      << endmsg;
-					goto out;
+					memset (raw_staging, 0, sizeof (Sample) * chunksize);
+
+				} else {
+					
+					to_read = min (chunksize, (_length - current_frame));
+					
+					
+					if ((frames_read = read_unlocked (raw_staging, current_frame, to_read)) == 0) {
+						error << string_compose(_("AudioSource[%1]: peak read - cannot read %2 samples at offset %3 of %4 (%5)"),
+									_name, to_read, current_frame, _length, strerror (errno)) 
+						      << endmsg;
+						goto out;
+					}
 				}
 					
 				i = 0;
@@ -1008,3 +1015,15 @@ AudioSource::check_for_analysis_data_on_disk ()
 
 	return ok;
 }
+
+void
+AudioSource::dec_read_data_count (nframes_t cnt)
+{
+        uint32_t val = cnt * sizeof (Sample);
+
+        if (val < _read_data_count) {
+                _read_data_count -= val;
+        } else { 
+                _read_data_count = 0;
+        }
+}
