@@ -22,10 +22,12 @@
 #include <float.h>
 #include <cmath>
 #include <sstream>
+#include <iomanip>
 #include <algorithm>
 #include <sigc++/bind.h>
 #include <ardour/automation_event.h>
 #include <pbd/stacktrace.h>
+#include <pbd/localeguard.h>
 
 #include "i18n.h"
 
@@ -56,15 +58,14 @@ AutomationList::AutomationList (double defval)
 {
 	_frozen = 0;
 	changed_when_thawed = false;
-	_state = Off;
-	_style = Absolute;
-	_touching = false;
+	_state = Auto_Off;
+	_style = Auto_Absolute;
+	g_atomic_int_set (&_touching, 0);
 	min_yval = FLT_MIN;
 	max_yval = FLT_MAX;
 	max_xval = 0; // means "no limit" 
 	default_value = defval;
 	_dirty = false;
-	rt_insertion_point = events.end();
 	lookup_cache.left = -1;
 	lookup_cache.range.first = events.end();
 	sort_pending = false;
@@ -82,9 +83,8 @@ AutomationList::AutomationList (const AutomationList& other)
 	max_xval = other.max_xval;
 	default_value = other.default_value;
 	_state = other._state;
-	_touching = other._touching;
+	g_atomic_int_set (&_touching, 0);
 	_dirty = false;
-	rt_insertion_point = events.end();
 	lookup_cache.left = -1;
 	lookup_cache.range.first = events.end();
 	sort_pending = false;
@@ -110,9 +110,8 @@ AutomationList::AutomationList (const AutomationList& other, double start, doubl
 	max_xval = other.max_xval;
 	default_value = other.default_value;
 	_state = other._state;
-	_touching = other._touching;
+	g_atomic_int_set (&_touching, other.touching());
 	_dirty = false;
-	rt_insertion_point = events.end();
 	lookup_cache.left = -1;
 	lookup_cache.range.first = events.end();
 	sort_pending = false;
@@ -138,14 +137,13 @@ AutomationList::AutomationList (const XMLNode& node)
 {
 	_frozen = 0;
 	changed_when_thawed = false;
-	_touching = false;
+	g_atomic_int_set (&_touching, 0);
 	min_yval = FLT_MIN;
 	max_yval = FLT_MAX;
 	max_xval = 0; // means "no limit" 
 	_dirty = false;
-	_state = Off;
-	_style = Absolute;
-	rt_insertion_point = events.end();
+	_state = Auto_Off;
+	_style = Auto_Absolute;
 	lookup_cache.left = -1;
 	lookup_cache.range.first = events.end();
 	sort_pending = false;
@@ -162,6 +160,13 @@ AutomationList::~AutomationList()
 	for (AutomationEventList::iterator x = events.begin(); x != events.end(); ++x) {
 		delete (*x);
 	}
+
+	for (list<NascentInfo*>::iterator n = nascent.begin(); n != nascent.end(); ++n) {
+                for (AutomationEventList::iterator x = (*n)->events.begin(); x != (*n)->events.end(); ++x) {
+                        delete *x;
+                }
+		delete (*n);
+	}
 }
 
 bool
@@ -185,7 +190,9 @@ AutomationList::operator= (const AutomationList& other)
 		max_yval = other.max_yval;
 		max_xval = other.max_xval;
 		default_value = other.default_value;
-		
+
+		lookup_cache.range.first = events.end();
+	
 		mark_dirty ();
 		maybe_signal_changed ();
 	}
@@ -210,6 +217,12 @@ AutomationList::set_automation_state (AutoState s)
 {
 	if (s != _state) {
 		_state = s;
+
+                if (_state == Auto_Write) {
+                        Glib::Mutex::Lock lm (lock);
+                        nascent.push_back (new NascentInfo (false));
+                }
+
 		automation_state_changed (); /* EMIT SIGNAL */
 	}
 }
@@ -224,17 +237,37 @@ AutomationList::set_automation_style (AutoStyle s)
 }
 
 void
-AutomationList::start_touch ()
+AutomationList::start_touch (double when)
 {
-	_touching = true;
-	_new_touch = true;
+        if (_state == Auto_Touch) {
+                Glib::Mutex::Lock lm (lock);
+                nascent.push_back (new NascentInfo (true, when));
+        }
+
+	g_atomic_int_set (&_touching, 1);
 }
 
 void
-AutomationList::stop_touch ()
+AutomationList::stop_touch (bool mark, double when)
 {
-	_touching = false;
-	_new_touch = false;
+	g_atomic_int_set (&_touching, 0);
+
+        if (_state == Auto_Touch) {
+                Glib::Mutex::Lock lm (lock);
+                
+                if (mark) {
+                        nascent.back()->end_time = when;
+                        
+                } else {
+                        
+                        /* nascent info created in start touch but never used. just get rid of it.
+                         */
+                        
+                        NascentInfo* ninfo = nascent.back ();
+                        nascent.erase (nascent.begin());
+                        delete ninfo;
+                }
+        }
 }
 
 void
@@ -278,100 +311,163 @@ void AutomationList::_x_scale (double factor)
 }
 
 void
-AutomationList::reposition_for_rt_add (double when)
+AutomationList::write_pass_finished (double when)
 {
-	rt_insertion_point = events.end();
+        merge_nascent (when);
 }
 
-#define last_rt_insertion_point rt_insertion_point
-
 void
 AutomationList::rt_add (double when, double value)
 {
 	/* this is for automation recording */
 
-	if ((_state & Touch) && !_touching) {
-		return;
-	}
+	if ((_state & Auto_Touch) && !touching()) {
+                return;
+        }
 
-	// cerr << "RT: alist @ " << this << " add " << value << " @ " << when << endl;
+        Glib::Mutex::Lock lm (lock, Glib::TRY_LOCK);
 
-	{
-		Glib::Mutex::Lock lm (lock);
+        if (lm.locked()) {
+                assert (!nascent.empty());
+                if (!nascent.back()->events.empty()) {
+                        assert (when > nascent.back()->events.back()->when);
+                }
+                nascent.back()->events.push_back (point_factory (when, value));
+        }
+}
 
-		iterator where;
-		TimeComparator cmp;
-		ControlEvent cp (when, 0.0);
-		bool done = false;
+void
+AutomationList::merge_nascent (double when)
+{
+        {
+                Glib::Mutex::Lock lm (lock);
 
-		if ((last_rt_insertion_point != events.end()) && ((*last_rt_insertion_point)->when < when) ) {
+                if (nascent.empty()) {
+                        return;
+                }
 
-			/* we have a previous insertion point, so we should delete
-			   everything between it and the position where we are going
-			   to insert this point.
-			*/
+                for (list<NascentInfo*>::iterator n = nascent.begin(); n != nascent.end(); ++n) {
 
-			iterator after = last_rt_insertion_point;
+                        NascentInfo* ninfo = *n;
+                        AutomationEventList& nascent_events (ninfo->events);
+                        bool need_adjacent_start_clamp;
+                        bool need_adjacent_end_clamp;
 
-			if (++after != events.end()) {
-				iterator far = after;
+                        if (nascent_events.empty()) {
+                                delete ninfo;
+                                continue;
+                        }
+                        
+                        if (ninfo->start_time < 0.0) {
+                                ninfo->start_time = nascent_events.front()->when;
+                        }
+                        
+                        if (ninfo->end_time < 0.0) {
+                                ninfo->end_time = when;
+                        }
 
-				while (far != events.end()) {
-					if ((*far)->when > when) {
-						break;
-					}
-					++far;
-				}
+                        bool preexisting = !events.empty();
 
-                                if(_new_touch) {
-                                        where = far;
-                                        last_rt_insertion_point = where;
-                                                                                             
-                                        if((*where)->when == when) {
-                                                (*where)->value = value;
-                                                done = true;
+                        if (!preexisting) {
+                                
+                                events = nascent_events;
+                                
+                        } else if (ninfo->end_time < events.front()->when) {
+                                
+                                /* all points in nascent are before the first existing point */
+
+                                events.insert (events.begin(), nascent_events.begin(), nascent_events.end());
+                                
+                        } else if (ninfo->start_time > events.back()->when) {
+                                
+                                /* all points in nascent are after the last existing point */
+
+                                events.insert (events.end(), nascent_events.begin(), nascent_events.end());
+                                
+                        } else {
+                                
+                                /* find the range that overaps with nascent events,
+                                   and insert the contents of nascent events.
+                                */
+                                
+                                iterator i;
+                                iterator range_begin = events.end();
+                                iterator range_end = events.end();
+                                double end_value = unlocked_eval (ninfo->end_time);
+                                double start_value = unlocked_eval (ninfo->start_time - 1);
+
+                                need_adjacent_end_clamp = true;
+                                need_adjacent_start_clamp = true;
+
+                                for (i = events.begin(); i != events.end(); ++i) {
+
+                                        if ((*i)->when == ninfo->start_time) {
+                                                /* existing point at same time, remove it
+                                                   and the consider the next point instead.
+                                                */
+                                                i = events.erase (i);
+
+                                                if (i == events.end()) {
+                                                        break;
+                                                }
+
+                                                if (range_begin == events.end()) {
+                                                        range_begin = i;
+                                                        need_adjacent_start_clamp = false;
+                                                } else {
+                                                        need_adjacent_end_clamp = false;
+                                                }
+                                                
+                                                if ((*i)->when > ninfo->end_time) {
+                                                        range_end = i;
+                                                        break;
+                                                }   
+
+                                        } else if ((*i)->when > ninfo->start_time) {
+                                                
+                                                if (range_begin == events.end()) {
+                                                        range_begin = i;
+                                                }
+                                                
+                                                if ((*i)->when > ninfo->end_time) {
+                                                        range_end = i;
+                                                        break;
+                                                }
+                                        }
+                                }
+                                
+                                assert (range_begin != events.end());
+                                
+                                if (range_begin != events.begin()) {
+                                        /* clamp point before */
+                                        if (need_adjacent_start_clamp) {
+                                                events.insert (range_begin, point_factory (ninfo->start_time, start_value));
                                         }
-                                } else {
-                                        where = events.erase (after, far);
                                 }
 
-			} else {
+                                events.insert (range_begin, nascent_events.begin(), nascent_events.end());
 
-				where = after;
+                                if (range_end != events.end()) {
+                                        /* clamp point after */
+                                        if (need_adjacent_end_clamp) {
+                                                events.insert (range_begin, point_factory (ninfo->end_time, end_value));
+                                        }
+                                }
+                                
+                                events.erase (range_begin, range_end);
+                        }
 
-			}
-			
-			iterator previous = last_rt_insertion_point;
-                        --previous;
-			
-			if (last_rt_insertion_point != events.begin() && (*last_rt_insertion_point)->value == value && (*previous)->value == value) {
-				(*last_rt_insertion_point)->when = when;
-				done = true;
-				
-			}
-			
-		} else {
+                        delete ninfo;
+                }
 
-			where = lower_bound (events.begin(), events.end(), &cp, cmp);
+                nascent.clear ();
 
-			if (where != events.end()) {
-				if ((*where)->when == when) {
-					(*where)->value = value;
-					done = true;
-				}
-			}
-		}
-		
-		if (!done) {
-			last_rt_insertion_point = events.insert (where, point_factory (when, value));
-			// cerr << "\tINSERTED\n";
-		}
-		
-		_new_touch = false;
-		mark_dirty ();
-	}
+                if (_state == Auto_Write) {
+                        nascent.push_back (new NascentInfo (false));
+                }
+        }
 
-	maybe_signal_changed ();
+        maybe_signal_changed ();
 }
 
 void
@@ -381,12 +477,10 @@ AutomationList::fast_simple_add (double when, double value)
 	events.insert (events.end(), point_factory (when, value));
 }
 
-#undef last_rt_insertion_point
-
 void
 AutomationList::add (double when, double value)
 {
-	/* this is for graphical editing */
+	/* this is for making changes from some kind of user interface or control surface (GUI, MIDI, OSC etc) */
 
 	{
 		Glib::Mutex::Lock lm (lock);
@@ -411,10 +505,7 @@ AutomationList::add (double when, double value)
 		}
 
 		if (insert) {
-			
 			events.insert (insertion_point, point_factory (when, value));
-			reposition_for_rt_add (0);
-
 		} 
 
 		mark_dirty ();
@@ -429,7 +520,6 @@ AutomationList::erase (AutomationList::iterator i)
 	{
 		Glib::Mutex::Lock lm (lock);
 		events.erase (i);
-		reposition_for_rt_add (0);
 		mark_dirty ();
 	}
 	maybe_signal_changed ();
@@ -441,7 +531,6 @@ AutomationList::erase (AutomationList::iterator start, AutomationList::iterator
 	{
 		Glib::Mutex::Lock lm (lock);
 		events.erase (start, end);
-		reposition_for_rt_add (0);
 		mark_dirty ();
 	}
 	maybe_signal_changed ();
@@ -495,7 +584,6 @@ AutomationList::erase_range (double start, double endt)
 			cp.when = endt;
 			e = upper_bound (events.begin(), events.end(), &cp, cmp);
 			events.erase (s, e);
-			reposition_for_rt_add (0);
 			erased = true;
 			mark_dirty ();
 		}
@@ -674,7 +762,7 @@ AutomationList::truncate_end (double last_coordinate)
 	{
 		Glib::Mutex::Lock lm (lock);
 		ControlEvent cp (last_coordinate, 0);
-		list<ControlEvent*>::reverse_iterator i;
+		AutomationList::reverse_iterator i;
 		double last_val;
 
 		if (events.empty()) {
@@ -743,7 +831,7 @@ AutomationList::truncate_end (double last_coordinate)
 			uint32_t sz = events.size();
 			
 			while (i != events.rend() && sz > 2) {
-				list<ControlEvent*>::reverse_iterator tmp;
+				AutomationList::reverse_iterator tmp;
 				
 				tmp = i;
 				++tmp;
@@ -762,7 +850,6 @@ AutomationList::truncate_end (double last_coordinate)
 			events.back()->value = last_val;
 		}
 
-		reposition_for_rt_add (0);
 		mark_dirty();
 	}
 
@@ -840,7 +927,7 @@ AutomationList::truncate_start (double overall_length)
 			i = events.begin();
 			
 			while (i != events.end() && !events.empty()) {
-				list<ControlEvent*>::iterator tmp;
+				AutomationList::iterator tmp;
 				
 				tmp = i;
 				++tmp;
@@ -868,8 +955,6 @@ AutomationList::truncate_start (double overall_length)
 			events.push_front (point_factory (0, first_legal_value));
 		}	    
 
-		reposition_for_rt_add (0);
-
 		mark_dirty();
 	}
 
@@ -940,6 +1025,9 @@ AutomationList::shared_eval (double x)
 		return multipoint_eval (x);
 		break;
 	}
+
+	/*NOTREACHED*/ /* stupid gcc */
+	return 0.0;
 }
 
 double
@@ -1005,36 +1093,6 @@ AutomationList::multipoint_eval (double x)
 	return (*range.first)->value;
 }
 
-AutomationList*
-AutomationList::cut (iterator start, iterator end)
-{
-	AutomationList* nal = new AutomationList (default_value);
-
-	{
-		Glib::Mutex::Lock lm (lock);
-
-		for (iterator x = start; x != end; ) {
-			iterator tmp;
-			
-			tmp = x;
-			++tmp;
-			
-			nal->events.push_back (point_factory (**x));
-			events.erase (x);
-			
-			reposition_for_rt_add (0);
-
-			x = tmp;
-		}
-
-		mark_dirty ();
-	}
-
-	maybe_signal_changed ();
-
-	return nal;
-}
-
 AutomationList*
 AutomationList::cut_copy_clear (double start, double end, int op)
 {
@@ -1042,11 +1100,14 @@ AutomationList::cut_copy_clear (double start, double end, int op)
 	iterator s, e;
 	ControlEvent cp (start, 0.0);
 	TimeComparator cmp;
-	bool changed = false;
-	
+
 	{
 		Glib::Mutex::Lock lm (lock);
 
+                /* first, determine s & e, two iterators that define the range of points
+                   affected by this operation
+                */
+
 		if ((s = lower_bound (events.begin(), events.end(), &cp, cmp)) == events.end()) {
 			return nal;
 		}
@@ -1054,18 +1115,39 @@ AutomationList::cut_copy_clear (double start, double end, int op)
 		cp.when = end;
 		e = upper_bound (events.begin(), events.end(), &cp, cmp);
 
-		if (op != 2 && (*s)->when != start) {
-			nal->events.push_back (point_factory (0, unlocked_eval (start)));
-		}
+                /* if "start" isn't the location of an existing point,
+                   evaluate the curve to get a value for the start. Add a point to
+                   both the existing event list, and if its not a "clear" operation,
+                   to the copy ("nal") as well. 
+
+                   Note that the time positions of the points in each list are different 
+                   because we want the copy ("nal") to have a zero time reference.
+                */
+
+                        
+                /* before we begin any cut/clear operations, get the value of the curve
+                   at "end".
+                */
+
+                double end_value = unlocked_eval (end);
+
+                if ((*s)->when != start) {
+                        
+                        double val = unlocked_eval (start);
+
+			if (op == 0) { // cut
+				if (start > events.front()->when) {
+					events.insert (s, (point_factory (start, val)));
+				}
+			}
+                        
+                        if (op != 2) { // ! clear
+                                nal->events.push_back (point_factory (0, val));
+                        }
+                }
 
 		for (iterator x = s; x != e; ) {
-			iterator tmp;
-			
-			tmp = x;
-			++tmp;
 
-			changed = true;
-			
 			/* adjust new points to be relative to start, which
 			   has been set to zero.
 			*/
@@ -1075,48 +1157,32 @@ AutomationList::cut_copy_clear (double start, double end, int op)
 			}
 
 			if (op != 1) {
-				events.erase (x);
-			}
-			
-			x = tmp;
+				x = events.erase (x);
+			} else {
+                                ++x;
+                        }
+		}
+                
+                if (e == events.end() || (*e)->when != end) {
+
+                        /* only add a boundary point if there is a point after "end"
+                         */
+
+                        if (op == 0 && (e != events.end() && end < (*e)->when)) { // cut
+                                events.insert (e, point_factory (end, end_value));
+                        }
+
+                        if (op != 2 && (e != events.end() && end < (*e)->when)) { // cut/copy
+                                nal->events.push_back (point_factory (end - start, end_value));
+                        }
 		}
 
-		if (op != 2 && nal->events.back()->when != end - start) {
-			nal->events.push_back (point_factory (end - start, unlocked_eval (end)));
-		}
-
-		if (changed) {
-			reposition_for_rt_add (0);
-		}
-
-		mark_dirty ();
+                mark_dirty ();
 	}
 
-	maybe_signal_changed ();
-
-	return nal;
-
-}
-
-AutomationList*
-AutomationList::copy (iterator start, iterator end)
-{
-	AutomationList* nal = new AutomationList (default_value);
-
-	{
-		Glib::Mutex::Lock lm (lock);
-		
-		for (iterator x = start; x != end; ) {
-			iterator tmp;
-			
-			tmp = x;
-			++tmp;
-			
-			nal->events.push_back (point_factory (**x));
-			
-			x = tmp;
-		}
-	}
+        if (op != 1) {
+                maybe_signal_changed ();
+        }
 
 	return nal;
 }
@@ -1179,7 +1245,6 @@ AutomationList::paste (AutomationList& alist, double pos, float times)
 			}
 		}
 
-		reposition_for_rt_add (0);
 		mark_dirty ();
 	}
 
@@ -1224,10 +1289,17 @@ AutomationList::state (bool full)
 	root->add_property ("max_xval", buf);
 
 	if (full) {
-		root->add_property ("state", auto_state_to_string (_state));
+                /* never serialize state with Write enabled - too dangerous 
+                   for the user's data
+                */
+                if (_state != Auto_Write) {
+                        root->add_property ("state", auto_state_to_string (_state));
+                } else {
+                        root->add_property ("state", auto_state_to_string (Auto_Off));
+                }
 	} else {
 		/* never save anything but Off for automation state to a template */
-		root->add_property ("state", auto_state_to_string (Off));
+		root->add_property ("state", auto_state_to_string (Auto_Off));
 	}
 
 	root->add_property ("style", auto_style_to_string (_style));
@@ -1244,6 +1316,8 @@ AutomationList::serialize_events ()
 {
 	XMLNode* node = new XMLNode (X_("events"));
 	stringstream str;
+	
+	str.precision(15);  //10 digits is enough digits for 24 hours at 96kHz
 
 	for (iterator xx = events.begin(); xx != events.end(); ++xx) {
 		str << (double) (*xx)->when;
@@ -1302,7 +1376,6 @@ AutomationList::deserialize_events (const XMLNode& node)
 		error << _("automation list: cannot load coordinates from XML, all points ignored") << endmsg;
 	} else {
 		mark_dirty ();
-		reposition_for_rt_add (0);
 		maybe_signal_changed ();
 	}
 
@@ -1384,13 +1457,16 @@ AutomationList::set_state (const XMLNode& node)
 	if ((prop = node.property (X_("style"))) != 0) {
 		_style = string_to_auto_style (prop->value());
 	} else {
-		_style = Absolute;
+		_style = Auto_Absolute;
 	}
 
 	if ((prop = node.property (X_("state"))) != 0) {
 		_state = string_to_auto_state (prop->value());
+                if (_state == Auto_Write) {
+                        _state = Auto_Off;
+                }
 	} else {
-		_state = Off;
+		_state = Auto_Off;
 	}
 
 	if ((prop = node.property (X_("min_yval"))) != 0) {
@@ -1420,3 +1496,20 @@ AutomationList::set_state (const XMLNode& node)
 	return 0;
 }
 
+void
+AutomationList::shift (nframes64_t pos, nframes64_t frames)
+{
+	{
+		Glib::Mutex::Lock lm (lock);
+
+		for (iterator i = begin (); i != end (); ++i) {
+			if ((*i)->when >= pos) {
+				(*i)->when += frames;
+			}
+		}
+
+		mark_dirty ();
+	}
+
+	maybe_signal_changed ();
+}
