@@ -19,7 +19,7 @@
 
 #include <sigc++/bind.h>
 
-#include <pbd/stacktrace.h>
+#include <pbd/localeguard.h>
 
 #include <ardour/types.h>
 #include <ardour/crossfade.h>
@@ -81,7 +81,7 @@ Crossfade::Crossfade (boost::shared_ptr<AudioRegion> in, boost::shared_ptr<Audio
 {
 	_in = in;
 	_out = out;
-	
+
 	_length = length;
 	_position = position;
 	_anchor_point = ap;
@@ -100,6 +100,7 @@ Crossfade::Crossfade (boost::shared_ptr<AudioRegion> a, boost::shared_ptr<AudioR
 {
 	_in_update = false;
 	_fixed = false;
+	_follow_overlap = false;
 
 	if (compute (a, b, model)) {
 		throw failed_constructor();
@@ -208,22 +209,57 @@ Crossfade::initialize ()
 
 	_fade_out.freeze ();
 	_fade_out.clear ();
-	_fade_out.add (0.0, 1.0);
-	_fade_out.add ((_length * 0.1), 0.99);
-	_fade_out.add ((_length * 0.2), 0.97);
-	_fade_out.add ((_length * 0.8), 0.03);
-	_fade_out.add ((_length * 0.9), 0.01);
-	_fade_out.add (_length, 0.0);
+
+#define EQUAL_POWER_MINUS_3DB
+#ifdef  EQUAL_POWER_MINUS_3DB
+
+	_fade_out.add ((_length * 0.000000), 1.000000);
+	_fade_out.add ((_length * 0.166667), 0.948859);
+	_fade_out.add ((_length * 0.333333), 0.851507);
+	_fade_out.add ((_length * 0.500000), 0.707946);
+	_fade_out.add ((_length * 0.666667), 0.518174);
+	_fade_out.add ((_length * 0.833333), 0.282192);
+	_fade_out.add ((_length * 1.000000), 0.000000);
+
+#else // EQUAL_POWER_MINUS_6DB
+
+	_fade_out.add ((_length * 0.000000), 1.000000);
+	_fade_out.add ((_length * 0.166667), 0.833033);
+	_fade_out.add ((_length * 0.333333), 0.666186);
+	_fade_out.add ((_length * 0.500000), 0.499459);
+	_fade_out.add ((_length * 0.666667), 0.332853);
+	_fade_out.add ((_length * 0.833333), 0.166366);
+	_fade_out.add ((_length * 1.000000), 0.000000);
+#endif
+
 	_fade_out.thaw ();
 	
 	_fade_in.freeze ();
 	_fade_in.clear ();
-	_fade_in.add (0.0, 0.0);
-	_fade_in.add ((_length * 0.1),  0.01);
-	_fade_in.add ((_length * 0.2),  0.03);
-	_fade_in.add ((_length * 0.8),  0.97);
-	_fade_in.add ((_length * 0.9),  0.99);
-	_fade_in.add (_length, 1.0);
+
+#define EQUAL_POWER_MINUS_3DB
+#ifdef  EQUAL_POWER_MINUS_3DB
+
+	_fade_in.add ((_length * 0.000000), 0.000000);
+	_fade_in.add ((_length * 0.166667), 0.282192);
+	_fade_in.add ((_length * 0.333333), 0.518174);
+	_fade_in.add ((_length * 0.500000), 0.707946);
+	_fade_in.add ((_length * 0.666667), 0.851507);
+	_fade_in.add ((_length * 0.833333), 0.948859);
+	_fade_in.add ((_length * 1.000000), 1.000000);
+
+#else // EQUAL_POWER_MINUS_SIX_DB
+
+	_fade_in.add ((_length * 0.000000), 0.000000);
+	_fade_in.add ((_length * 0.166667), 0.166366);
+	_fade_in.add ((_length * 0.333333), 0.332853);
+	_fade_in.add ((_length * 0.500000), 0.499459);
+	_fade_in.add ((_length * 0.666667), 0.666186);
+	_fade_in.add ((_length * 0.833333), 0.833033);
+	_fade_in.add ((_length * 1.000000), 1.000000);
+
+#endif
+
 	_fade_in.thaw ();
 
 	overlap_type = _in->coverage (_out->position(), _out->last_frame());
@@ -268,6 +304,13 @@ Crossfade::read_at (Sample *buf, Sample *mixdown_buffer,
 
 	offset = start - _position;
 
+	/* Prevent data from piling up inthe crossfade buffers when reading a transparent region */
+	if (!(_out->opaque())) {
+		memset (crossfade_buffer_out, 0, sizeof (Sample) * to_write);
+	} else if (!(_in->opaque())) {
+		memset (crossfade_buffer_in, 0, sizeof (Sample) * to_write);
+	}
+
 	_out->read_at (crossfade_buffer_out, mixdown_buffer, gain_buffer, start, to_write, chan_n, read_frames, skip_frames);
 	_in->read_at (crossfade_buffer_in, mixdown_buffer, gain_buffer, start, to_write, chan_n, read_frames, skip_frames);
 
@@ -332,6 +375,13 @@ Crossfade::refresh ()
 		return false;
 	}
 
+	/* Top layer shouldn't be transparent */
+
+	if (!((layer_relation > 0 ? _in : _out)->opaque())) {
+		Invalidated (shared_from_this());
+		return false;
+	}
+
 	/* layer ordering cannot change */
 
 	int32_t new_layer_relation = (int32_t) (_in->layer() - _out->layer());
@@ -419,11 +469,11 @@ Crossfade::update ()
 		break;
 		
 	case EndOfIn:
-		_position = _in->last_frame() - _length;
+		_position = _in->last_frame() - _length + 1;
 		break;
 		
 	case EndOfOut:
-		_position = _out->last_frame() - _length;
+		_position = _out->last_frame() - _length + 1;
 	}
 
 	return true;
@@ -672,7 +722,7 @@ Crossfade::set_state (const XMLNode& node)
 	}
 
 	if ((prop = node.property ("active")) != 0) {
-		bool x = (prop->value() == "yes");
+		bool x = string_is_affirmative (prop->value());
 		if (x != _active) {
 			_active = x;
 			what_changed = Change (what_changed | ActiveChanged);
@@ -682,13 +732,13 @@ Crossfade::set_state (const XMLNode& node)
 	}
 
 	if ((prop = node.property ("follow-overlap")) != 0) {
-		_follow_overlap = (prop->value() == "yes");
+		_follow_overlap = string_is_affirmative (prop->value());
 	} else {
 		_follow_overlap = false;
 	}
 
 	if ((prop = node.property ("fixed")) != 0) {
-		_fixed = (prop->value() == "yes");
+		_fixed = string_is_affirmative (prop->value());
 	} else {
 		_fixed = false;
 	}
@@ -748,6 +798,9 @@ Crossfade::set_state (const XMLNode& node)
 		}
 	}
 
+	_fade_in.front()->value=0.0;
+	_fade_in.back()->value=1.0;
+
 	_fade_in.thaw ();
 	
         /* fade out */
@@ -773,6 +826,9 @@ Crossfade::set_state (const XMLNode& node)
 		}
 	}
 
+	_fade_out.front()->value=1.0;
+	_fade_out.back()->value=0.0;
+
 	_fade_out.thaw ();
 
 	StateChanged (what_changed); /* EMIT SIGNAL */
@@ -807,7 +863,7 @@ Crossfade::set_follow_overlap (bool yn)
 nframes_t
 Crossfade::set_length (nframes_t len)
 {
-	nframes_t limit;
+	nframes_t limit = 0;
 
 	switch (_anchor_point) {
 	case StartOfIn:
