@@ -80,6 +80,7 @@ setup_enum_writer ()
 	Location::Flags _Location_Flags;
 	RouteGroup::Flag _RouteGroup_Flag;
 	Region::Flag _Region_Flag;
+	Region::PositionLockStyle _Region_PositionLockStyle;
 	Track::FreezeState _Track_FreezeState;
 
 #define REGISTER(e) enum_writer->register_distinct (typeid(e).name(), i, s); i.clear(); s.clear()
@@ -106,14 +107,14 @@ setup_enum_writer ()
 	REGISTER_ENUM (MuteAutomation);
 	REGISTER_BITS (_AutomationType);
 
-	REGISTER_ENUM (Off);
-	REGISTER_ENUM (Write);
-	REGISTER_ENUM (Touch);
-	REGISTER_ENUM (Play);
+	REGISTER_ENUM (Auto_Off);
+	REGISTER_ENUM (Auto_Write);
+	REGISTER_ENUM (Auto_Touch);
+	REGISTER_ENUM (Auto_Play);
 	REGISTER_BITS (_AutoState);
 
-	REGISTER_ENUM (Absolute);
-	REGISTER_ENUM (Trim);
+	REGISTER_ENUM (Auto_Absolute);
+	REGISTER_ENUM (Auto_Trim);
 	REGISTER_BITS (_AutoStyle);
 
 	REGISTER_ENUM (CaptureTime);
@@ -241,8 +242,7 @@ setup_enum_writer ()
 	REGISTER_CLASS_ENUM (Session::Event, SetSlaveSource);
 	REGISTER_CLASS_ENUM (Session::Event, Audition);
 	REGISTER_CLASS_ENUM (Session::Event, InputConfigurationChange);
-	REGISTER_CLASS_ENUM (Session::Event, SetAudioRange);
-	REGISTER_CLASS_ENUM (Session::Event, SetPlayRange);
+	REGISTER_CLASS_ENUM (Session::Event, SetPlayAudioRange);
 	REGISTER_CLASS_ENUM (Session::Event, StopOnce);
 	REGISTER_CLASS_ENUM (Session::Event, AutoLoop);
 	REGISTER (_Session_Event_Type);
@@ -352,6 +352,10 @@ setup_enum_writer ()
 	REGISTER_CLASS_ENUM (Region, DoNotSaveState);
 	REGISTER_BITS (_Region_Flag);
 
+	REGISTER_CLASS_ENUM (Region, AudioTime);
+	REGISTER_CLASS_ENUM (Region, MusicTime);
+	REGISTER_BITS (_Region_PositionLockStyle);
+
 	REGISTER_CLASS_ENUM (Track, NoFreeze);
 	REGISTER_CLASS_ENUM (Track, Frozen);
 	REGISTER_CLASS_ENUM (Track, UnFrozen);
