@@ -24,13 +24,16 @@
 #include <sys/resource.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <locale.h>
 #include <errno.h>
 
 #ifdef VST_SUPPORT
 #include <fst.h>
 #endif
 
+#ifdef HAVE_AUDIOUNITS
+#include <ardour/audio_unit.h>
+#endif
+
 #ifdef __SSE__
 #include <xmmintrin.h>
 #endif
@@ -86,9 +89,9 @@ using namespace ARDOUR;
 using namespace std;
 using namespace PBD;
 
-MIDI::Port *default_mmc_port = 0;
-MIDI::Port *default_mtc_port = 0;
-MIDI::Port *default_midi_port = 0;
+MIDI::Port *ARDOUR::default_mmc_port = 0;
+MIDI::Port *ARDOUR::default_mtc_port = 0;
+MIDI::Port *ARDOUR::default_midi_port = 0;
 
 Change ARDOUR::StartChanged = ARDOUR::new_change ();
 Change ARDOUR::LengthChanged = ARDOUR::new_change ();
@@ -115,10 +118,11 @@ setup_osc ()
 		return 0;
 	}
 }
+
 #endif
 
-static int 
-setup_midi ()
+int
+ARDOUR::setup_midi ()
 {
 	if (Config->midi_ports.size() == 0) {
 		warning << _("no MIDI ports specified: no MMC or MTC control possible") << endmsg;
@@ -182,16 +186,17 @@ setup_midi ()
 			<< endmsg;
 		return 0;
 	} 
+ 
 
 	if (default_mtc_port == 0) {
 		warning << string_compose (_("No MTC support (MIDI port \"%1\" not available)"), Config->get_mtc_port_name()) 
 			<< endmsg;
-	}
+	} 
 
 	if (default_midi_port == 0) {
 		warning << string_compose (_("No MIDI parameter support (MIDI port \"%1\" not available)"), Config->get_midi_port_name()) 
 			<< endmsg;
-	}
+	} 
 
 	return 0;
 }
@@ -278,7 +283,7 @@ lotsa_files_please ()
 			if (rl.rlim_cur == RLIM_INFINITY) {
 				info << _("Removed open file count limit. Excellent!") << endmsg;
 			} else {
-				info << string_compose (_("Ardour will be limited to %1 open files"), rl.rlim_cur) << endmsg;
+				info << string_compose (_("%1 will be limited to %2 open files"), PROGRAM_NAME, rl.rlim_cur) << endmsg;
 			}
 		}
 	} else {
@@ -324,11 +329,15 @@ ARDOUR::init (bool use_vst, bool try_optimization)
 #endif
 
 #ifdef VST_SUPPORT
-	if (Config->get_use_vst() && fst_init ()) {
-		return -1;
+	if (Config->get_use_vst() && fst_init (0)) {
+	        return -1;
 	}
 #endif
-	
+
+#ifdef HAVE_AUDIOUNITS
+	AUPluginInfo::load_cached_info ();
+#endif
+
 	/* Make VAMP look in our library ahead of anything else */
 
 	char *p = getenv ("VAMP_PATH");
@@ -368,6 +377,9 @@ ARDOUR::cleanup ()
 	delete Library;
 	lrdf_cleanup ();
 	delete &ControlProtocolManager::instance();
+#ifdef VST_SUPPORT
+	fst_exit ();
+#endif
 	return 0;
 }
 
@@ -375,11 +387,7 @@ ARDOUR::cleanup ()
 microseconds_t
 ARDOUR::get_microseconds ()
 {
-	/* XXX need JACK to export its functionality */
-
-	struct timeval now;
-	gettimeofday (&now, 0);
-	return now.tv_sec * 1000000ULL + now.tv_usec;
+	return (microseconds_t) jack_get_time ();
 }
 
 ARDOUR::Change
@@ -406,7 +414,7 @@ ARDOUR::new_change ()
 string
 ARDOUR::get_ardour_revision ()
 {
-	return "$Rev: 3142 $";
+	return "$Rev: 7261 $";
 }
 
 string
@@ -425,7 +433,9 @@ ARDOUR::get_user_ardour_path ()
 	/* create it if necessary */
 
 	if (g_mkdir_with_parents (path.c_str (), 0755)) {
-		throw exception ();
+                cerr << "\n\n\nYour home folder is not writable (Ardour cannot create its settings folder there). Please fix this before running Ardour again."
+                     << endl;
+                _exit (1);
 	}
 
 	return path;
@@ -493,11 +503,12 @@ find_file (string name, string dir, string subdir = "")
 	path = get_user_ardour_path();
 		
 	if (subdir.length()) {
-		path += subdir + "/";
+		path = Glib::build_filename (path, subdir);
 	}
 		
-	path += name;
-	if (access (path.c_str(), R_OK) == 0) {
+	path = Glib::build_filename (path, name);
+
+	if (Glib::file_test (path.c_str(), Glib::FILE_TEST_EXISTS)) {
 		return path;
 	}
 
@@ -543,6 +554,8 @@ ARDOUR::find_bindings_files (map<string,string>& files)
 	vector<string*> *found;
 	string full_path;
 	
+	/* XXX building path, so separators are fixed */
+
 	full_path = get_user_ardour_path ();
 	full_path += ':';
 	full_path = get_system_data_path ();
@@ -594,20 +607,6 @@ ARDOUR::find_data_file (string name, string subdir)
 	return find_file (name, envvar, subdir);
 }
 
-ARDOUR::LocaleGuard::LocaleGuard (const char* str)
-{
-	old = strdup (setlocale (LC_NUMERIC, NULL));
-	if (strcmp (old, str)) {
-		setlocale (LC_NUMERIC, str);
-	} 
-}
-
-ARDOUR::LocaleGuard::~LocaleGuard ()
-{
-	setlocale (LC_NUMERIC, old);
-	free ((char*)old);
-}
-
 void
 ARDOUR::setup_fpu ()
 {
@@ -745,6 +744,19 @@ ARDOUR::coverage (nframes_t sa, nframes_t ea,
 	return OverlapNone;
 }
 
+string
+ARDOUR::translation_kill_path ()
+{
+	return Glib::build_filename (get_user_ardour_path(), ".love_is_the_language_of_audio");
+}
+
+bool
+ARDOUR::translations_are_disabled ()
+{
+	/* if file does not exist, we don't translate (bundled ardour only) */
+	return Glib::file_test (translation_kill_path(), Glib::FILE_TEST_EXISTS) == false;
+}
+
 /* not sure where to put these */
 
 template<class T>
