@@ -26,6 +26,7 @@
 #include <pbd/stacktrace.h>
 
 #include <ardour/insert.h>
+#include <ardour/mtdm.h>
 #include <ardour/plugin.h>
 #include <ardour/port.h>
 #include <ardour/route.h>
@@ -80,11 +81,6 @@ PluginInsert::PluginInsert (Session& s, boost::shared_ptr<Plugin> plug, Placemen
 	
 	init ();
 
-	if (_plugins[0]->fixed_io()) {
-		Glib::Mutex::Lock em (_session.engine().process_lock());
-		IO::MoreOutputs (output_streams ());
-	}
-
 	RedirectCreated (this); /* EMIT SIGNAL */
 }
 
@@ -95,14 +91,7 @@ PluginInsert::PluginInsert (Session& s, const XMLNode& node)
 		throw failed_constructor();
 	}
 
-	set_automatable ();
-
 	_plugins[0]->ParameterChanged.connect (mem_fun (*this, &PluginInsert::parameter_changed));
-
-	if (_plugins[0]->fixed_io()) {
-		Glib::Mutex::Lock em (_session.engine().process_lock());
-		IO::MoreOutputs (output_streams());
-	}
 }
 
 PluginInsert::PluginInsert (const PluginInsert& other)
@@ -159,8 +148,6 @@ void
 PluginInsert::init ()
 {
 	set_automatable ();
-
-	set<uint32_t>::iterator s;
 }
 
 PluginInsert::~PluginInsert ()
@@ -179,7 +166,10 @@ PluginInsert::auto_state_changed (uint32_t which)
 {
 	AutomationList& alist (automation_list (which));
 
-	if (alist.automation_state() != Off) {
+	/* don't reset automation if we're moving to Off or Write mode;
+	   if we're moving to Write, the user may have manually set up automation
+	   that they don't want to lose */		
+	if (alist.automation_state() != Auto_Off && alist.automation_state() != Auto_Write) {
 		_plugins[0]->set_parameter (which, alist.eval (_session.transport_frame()));
 	}
 }
@@ -200,6 +190,7 @@ uint32_t
 PluginInsert::input_streams() const
 {
 	int32_t in = _plugins[0]->get_info()->n_inputs;
+
 	if (in < 0) {
 		return _plugins[0]->input_streams ();
 	} else {
@@ -232,6 +223,10 @@ PluginInsert::is_generator() const
 void
 PluginInsert::set_automatable ()
 {
+	/* fill the parameter automation list with null AutomationLists */
+
+	parameter_automation.assign (_plugins.front()->parameter_count(), (AutomationList*) 0);
+
 	set<uint32_t> a;
 	
 	a = _plugins.front()->automatable ();
@@ -256,12 +251,18 @@ PluginInsert::parameter_changed (uint32_t which, float val)
 	}
 }
 
-void
+int
 PluginInsert::set_block_size (nframes_t nframes)
 {
+	int ret = 0;
+
 	for (vector<boost::shared_ptr<Plugin> >::iterator i = _plugins.begin(); i != _plugins.end(); ++i) {
-		(*i)->set_block_size (nframes);
+		if ((*i)->set_block_size (nframes)) {
+			ret = -1;
+		}
 	}
+
+	return ret;
 }
 
 void
@@ -280,6 +281,14 @@ PluginInsert::deactivate ()
 	}
 }
 
+void
+PluginInsert::flush ()
+{
+	for (vector<boost::shared_ptr<Plugin> >::iterator i = _plugins.begin(); i != _plugins.end(); ++i) {
+		(*i)->flush ();
+	}
+}
+
 void
 PluginInsert::connect_and_run (vector<Sample*>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t offset, bool with_auto, nframes_t now)
 {
@@ -290,47 +299,50 @@ PluginInsert::connect_and_run (vector<Sample*>& bufs, uint32_t nbufs, nframes_t
 	   be able to handle in-place processing.
 	*/
 
+	// cerr << "Connect and run for " << _plugins[0]->name() << " auto ? " << with_auto << " nf = " << nframes << " off = " << offset << endl;
+	
 	if (with_auto) {
 
-		map<uint32_t,AutomationList*>::iterator li;
+		vector<AutomationList*>::iterator li;
 		uint32_t n;
 		
 		for (n = 0, li = parameter_automation.begin(); li != parameter_automation.end(); ++li, ++n) {
 			
-			AutomationList& alist (*((*li).second));
+			AutomationList* alist = *li;
 
-			if (alist.automation_playback()) {
+			if (alist && alist->automation_playback()) {
 				bool valid;
-
-				float val = alist.rt_safe_eval (now, valid);				
+				
+				float val = alist->rt_safe_eval (now, valid);				
 
 				if (valid) {
 					/* set the first plugin, the others will be set via signals */
-					_plugins[0]->set_parameter ((*li).first, val);
+					// cerr << "\t@ " << now << " param[" << n << "] = " << val << endl;
+					_plugins[0]->set_parameter (n, val);
 				}
 
 			} 
 		}
-	}
+	} 
 
 	for (vector<boost::shared_ptr<Plugin> >::iterator i = _plugins.begin(); i != _plugins.end(); ++i) {
 		(*i)->connect_and_run (bufs, nbufs, in_index, out_index, nframes, offset);
 	}
-
-	/* leave remaining channel buffers alone */
 }
 
 void
 PluginInsert::automation_snapshot (nframes_t now, bool force)
 {
-	map<uint32_t,AutomationList*>::iterator li;
-	
-	for (li = parameter_automation.begin(); li != parameter_automation.end(); ++li) {
+	vector<AutomationList*>::iterator li;
+	uint32_t n;
+
+	for (n = 0, li = parameter_automation.begin(); li != parameter_automation.end(); ++li, ++n) {
 		
-		AutomationList *alist = ((*li).second);
-		if (alist != 0 && alist->automation_write ()) {
+		AutomationList *alist = *li;
+
+		if (alist && alist->automation_write () && _session.transport_rolling()) {
 			
-			float val = _plugins[0]->get_parameter ((*li).first);
+			float val = _plugins[0]->get_parameter (n);
 			alist->rt_add (now, val);
 			last_automation_snapshot = now;
 		}
@@ -340,20 +352,24 @@ PluginInsert::automation_snapshot (nframes_t now, bool force)
 void
 PluginInsert::transport_stopped (nframes_t now)
 {
-	map<uint32_t,AutomationList*>::iterator li;
+	vector<AutomationList*>::iterator li;
+	uint32_t n;
 
-	for (li = parameter_automation.begin(); li != parameter_automation.end(); ++li) {
-		AutomationList& alist (*(li->second));
-		alist.reposition_for_rt_add (now);
+	for (n = 0, li = parameter_automation.begin(); li != parameter_automation.end(); ++li, ++n) {
 
-		if (alist.automation_state() != Off) {
-			_plugins[0]->set_parameter (li->first, alist.eval (now));
+		AutomationList* alist = *li;
+
+		if (alist) {
+			alist->write_pass_finished (now);
+			if (alist->automation_state() == Auto_Touch || alist->automation_state() == Auto_Play) {
+				_plugins[0]->set_parameter (n, alist->eval (now));
+			}
 		}
 	}
 }
 
 void
-PluginInsert::silence (nframes_t nframes, nframes_t offset)
+PluginInsert::silence (nframes_t nframes)
 {
 	int32_t in_index = 0;
 	int32_t out_index = 0;
@@ -362,20 +378,20 @@ PluginInsert::silence (nframes_t nframes, nframes_t offset)
 	if (active()) {
 		for (vector<boost::shared_ptr<Plugin> >::iterator i = _plugins.begin(); i != _plugins.end(); ++i) {
 			n = input_streams();
-			(*i)->connect_and_run (_session.get_silent_buffers (n), n, in_index, out_index, nframes, offset);
+			(*i)->connect_and_run (_session.get_silent_buffers (n), n, in_index, out_index, nframes, 0);
 		}
 	}
 }
 	
 void
-PluginInsert::run (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t offset)
+PluginInsert::run (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes)
 {
 	if (active()) {
 
 		if (_session.transport_rolling()) {
-			automation_run (bufs, nbufs, nframes, offset);
+			automation_run (bufs, nbufs, nframes);
 		} else {
-			connect_and_run (bufs, nbufs, nframes, offset, false);
+			connect_and_run (bufs, nbufs, nframes, 0, false);
 		}
 
 	} else {
@@ -401,9 +417,15 @@ PluginInsert::set_parameter (uint32_t port, float val)
 {
 	/* the others will be set from the event triggered by this */
 
+	float last_val = _plugins[0]->get_parameter (port);
+	Plugin::ParameterDescriptor desc;
+	_plugins[0]->get_parameter_descriptor(port, desc);
+	
 	_plugins[0]->set_parameter (port, val);
 	
-	if (automation_list (port).automation_write()) {
+	if (automation_list (port).automation_write() && _session.transport_rolling()) {
+		if ( desc.toggled )  //store the previous value just before this so any interpolation works right 
+			automation_list (port).add (_session.audible_frame()-1, last_val);
 		automation_list (port).add (_session.audible_frame(), val);
 	}
 
@@ -411,47 +433,46 @@ PluginInsert::set_parameter (uint32_t port, float val)
 }
 
 void
-PluginInsert::automation_run (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t offset)
+PluginInsert::automation_run (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes)
 {
 	ControlEvent next_event (0, 0.0f);
 	nframes_t now = _session.transport_frame ();
 	nframes_t end = now + nframes;
+	nframes_t offset = 0;
 
 	Glib::Mutex::Lock lm (_automation_lock, Glib::TRY_LOCK);
 
 	if (!lm.locked()) {
-		connect_and_run (bufs, nbufs, nframes, offset, false);
+		connect_and_run (bufs, nbufs, nframes, 0, false, now);
 		return;
 	}
-	
-	if (!find_next_event (now, end, next_event)) {
-		
+
+	if (!find_next_event (now, end, next_event) || requires_fixed_size_buffers()) {
  		/* no events have a time within the relevant range */
-		
- 		connect_and_run (bufs, nbufs, nframes, offset, true, now);
+ 		connect_and_run (bufs, nbufs, nframes, 0, true, now);
  		return;
- 	}
+	}
 	
- 	while (nframes) {
-
+	while (nframes) {
 		nframes_t cnt = min (((nframes_t) ceil (next_event.when) - now), nframes);
-  
- 		connect_and_run (bufs, nbufs, cnt, offset, true, now);
- 		
- 		nframes -= cnt;
- 		offset += cnt;
+		
+		connect_and_run (bufs, nbufs, cnt, offset, true, now);
+		
+		nframes -= cnt;
 		now += cnt;
-
+		offset += cnt;
+		
 		if (!find_next_event (now, end, next_event)) {
 			break;
 		}
-  	}
-  
+	}
+
  	/* cleanup anything that is left to do */
   
  	if (nframes) {
  		connect_and_run (bufs, nbufs, nframes, offset, true, now);
   	}
+
 }	
 
 float
@@ -486,7 +507,7 @@ PluginInsert::get_port_automation_state (uint32_t port)
 	if (port < _plugins[0]->parameter_count()) {
 		return automation_list (port).automation_state();
 	} else {
-		return Off;
+		return Auto_Off;
 	}
 }
 
@@ -502,11 +523,11 @@ PluginInsert::protect_automation ()
 		AutomationList& al = automation_list (*i);
 
 		switch (al.automation_state()) {
-		case Write:
-			al.set_automation_state (Off);
+		case Auto_Write:
+			al.set_automation_state (Auto_Off);
 			break;
-		case Touch:
-			al.set_automation_state (Play);
+		case Auto_Touch:
+			al.set_automation_state (Auto_Play);
 			break;
 		default:
 			break;
@@ -551,68 +572,29 @@ PluginInsert::plugin_factory (boost::shared_ptr<Plugin> other)
 	return boost::shared_ptr<Plugin> ((Plugin*) 0);
 }
 
-int32_t
-PluginInsert::compute_output_streams (int32_t cnt) const
-{
-	int32_t outputs;
-
-	if ((outputs = _plugins[0]->get_info()->n_outputs) < 0) {
-		// have to ask the plugin itself, because it has flexible I/O
-		return _plugins[0]->compute_output_streams (cnt);
-	}
-
-	return outputs * cnt;
-}
-
 int32_t
 PluginInsert::configure_io (int32_t magic, int32_t in, int32_t out)
 {
-	return set_count (magic);
+	int32_t ret;
+
+	if ((ret = set_count (magic)) < 0) {
+		return ret;
+	}
+
+	/* if we're running replicated plugins, each plugin has
+	   the same i/o configuration and we may need to announce how many
+	   output streams there are.
+
+	   if we running a single plugin, we need to configure it.
+	*/
+
+	return _plugins[0]->configure_io (in, out);
 }
 
 int32_t 
-PluginInsert::can_support_input_configuration (int32_t in) const
+PluginInsert::can_do (int32_t in, int32_t& out)
 {
-	int32_t outputs = _plugins[0]->get_info()->n_outputs;
-	int32_t inputs = _plugins[0]->get_info()->n_inputs;
-
-	if (outputs < 0 || inputs < 0) {
-		/* have to ask the plugin because its got reconfigurable I/O
-		 */
-		return _plugins[0]->can_support_input_configuration (in);
-	}
-
-	if (inputs == 0) {
-
-		/* instrument plugin, always legal, but it throws
-		   away any existing active streams.
-		*/
-
-		return 1;
-	}
-
-	if (outputs == 1 && inputs == 1) {
-		/* mono plugin, replicate as needed */
-		return in;
-	}
-
-	if (inputs == in) {
-		/* exact match */
-		return 1;
-	}
-
-	if ((inputs < in) && (inputs % in == 0)) {
-
-		/* number of inputs is a factor of the requested input
-		   configuration, so we can replicate.
-		*/
-
-		return in/inputs;
-	}
-
-	/* sorry */
-
-	return -1;
+	return _plugins[0]->can_do (in, out);
 }
 
 XMLNode&
@@ -658,6 +640,13 @@ PluginInsert::state (bool full)
 		snprintf(buf, sizeof(buf), "%" PRIu32, *x);
 		child->add_property("number", string(buf));
 
+#ifdef HAVE_SLV2
+		LV2Plugin* lv2p = dynamic_cast<LV2Plugin*>(_plugins[0].get());
+		if (lv2p) {
+			child->add_property("symbol", string(lv2p->port_symbol(*x)));
+		}
+#endif
+
 		child->add_child_nocopy (automation_list (*x).state (full));
 		autonode->add_child_nocopy (*child);
 	}
@@ -703,8 +692,7 @@ PluginInsert::set_state(const XMLNode& node)
 		 */
 		
 		if (type == ARDOUR::VST) {
-			if (prop = node.property ("id")) {
-			}
+			prop = node.property ("id");
 		}
 #endif		
 		/* recheck  */
@@ -740,7 +728,7 @@ PluginInsert::set_state(const XMLNode& node)
 			_plugins.push_back (plugin_factory (plugin));
 		}
 	}
-	
+
 	for (niter = nlist.begin(); niter != nlist.end(); ++niter) {
 		if ((*niter)->name() == plugin->state_node_name()) {
 			for (vector<boost::shared_ptr<Plugin> >::iterator i = _plugins.begin(); i != _plugins.end(); ++i) {
@@ -791,6 +779,8 @@ PluginInsert::set_state(const XMLNode& node)
 		break;
 	}
 		
+	set_automatable ();
+	
 	/* look for port automation node */
 	
 	for (niter = nlist.begin(); niter != nlist.end(); ++niter) {
@@ -841,7 +831,7 @@ PluginInsert::set_state(const XMLNode& node)
 					
 					/* missing */
 					
-					automation_list (port_id).set_automation_state (Off);
+					automation_list (port_id).set_automation_state (Auto_Off);
 				}
 			}
 
@@ -877,30 +867,7 @@ PluginInsert::latency()
 ARDOUR::PluginType
 PluginInsert::type ()
 {
-	boost::shared_ptr<LadspaPlugin> lp;
-#ifdef VST_SUPPORT
-	boost::shared_ptr<VSTPlugin> vp;
-#endif
-#ifdef HAVE_AUDIOUNITS
-	boost::shared_ptr<AUPlugin> ap;
-#endif
-	
-	PluginPtr other = plugin ();
-
-	if ((lp = boost::dynamic_pointer_cast<LadspaPlugin> (other)) != 0) {
-		return ARDOUR::LADSPA;
-#ifdef VST_SUPPORT
-	} else if ((vp = boost::dynamic_pointer_cast<VSTPlugin> (other)) != 0) {
-		return ARDOUR::VST;
-#endif
-#ifdef HAVE_AUDIOUNITS
-	} else if ((ap = boost::dynamic_pointer_cast<AUPlugin> (other)) != 0) {
-		return ARDOUR::AudioUnit;
-#endif
-	} else {
-		/* NOT REACHED */
-		return (ARDOUR::PluginType) 0;
-	}
+	return plugin()->get_info()->type;
 }
 
 /***************************************************************
@@ -925,12 +892,19 @@ PortInsert::PortInsert (const PortInsert& other)
 void
 PortInsert::init ()
 {
+	_mtdm = 0;
+	_latency_detect = false;
+	_latency_flush_frames = false;
+	_measured_latency = 0;
 }
 
 PortInsert::PortInsert (Session& s, const XMLNode& node)
 	: Insert (s, "will change", PreFader)
 {
+	init ();
+
 	bitslot = 0xffffffff;
+
 	if (set_state (node)) {
 		throw failed_constructor();
 	}
@@ -940,37 +914,98 @@ PortInsert::PortInsert (Session& s, const XMLNode& node)
 
 PortInsert::~PortInsert ()
 {
+	delete _mtdm;
 	GoingAway ();
 }
 
 void
-PortInsert::run (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t offset)
+PortInsert::start_latency_detection ()
+{
+	if (_mtdm != 0) {
+		delete _mtdm;
+	}
+
+	_mtdm = new MTDM;
+	_latency_flush_frames = false;
+	_latency_detect = true;
+	_measured_latency = 0;
+}
+
+void
+PortInsert::stop_latency_detection ()
+{
+	_latency_flush_frames = latency() + _session.engine().frames_per_cycle();
+	_latency_detect = false;
+}
+
+void
+PortInsert::set_measured_latency (nframes_t n)
+{
+	_measured_latency = n;
+}
+
+void
+PortInsert::run (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes)
 {
 	if (n_outputs() == 0) {
 		return;
 	}
 
-	if (!active()) {
-		/* deliver silence */
-		silence (nframes, offset);
+	vector<Port*>::iterator o;
+
+	if (_latency_detect) {
+
+		if (n_inputs() != 0) {
+			Sample* in = get_input_buffer (0, nframes);
+			Sample* out = get_output_buffer (0, nframes);
+
+			_mtdm->process (nframes, in, out);
+			
+			for (o = _outputs.begin(); o != _outputs.end(); ++o) {
+				(*o)->mark_silence (false);
+			}
+		}
+
+		return;
+
+	} else if (_latency_flush_frames) {
+
+		/* wait for the entire input buffer to drain before picking up input again so that we can't
+		   hear the remnants of whatever MTDM pumped into the pipeline.
+		*/
+
+		silence (nframes);
+
+		if (_latency_flush_frames > nframes) {
+			_latency_flush_frames -= nframes;
+		} else {
+			_latency_flush_frames = 0;
+		}
+
 		return;
 	}
 
-	uint32_t n;
-	vector<Port*>::iterator o;
-	vector<Port*>::iterator i;
+	if (!active()) {
+		/* deliver silence */
+		silence (nframes);
+		return;
+	}
 
 	/* deliver output */
 
+	uint32_t n;
+
 	for (o = _outputs.begin(), n = 0; o != _outputs.end(); ++o, ++n) {
-		memcpy ((*o)->get_buffer (nframes) + offset, bufs[min(nbufs,n)], sizeof (Sample) * nframes);
+		memcpy (get_output_buffer (n, nframes), bufs[min(nbufs,n)], sizeof (Sample) * nframes);
 		(*o)->mark_silence (false);
 	}
 	
+	vector<Port*>::iterator i;
+
 	/* collect input */
 	
 	for (i = _inputs.begin(), n = 0; i != _inputs.end(); ++i, ++n) {
-		memcpy (bufs[min(nbufs,n)], (*i)->get_buffer (nframes) + offset, sizeof (Sample) * nframes);
+		memcpy (bufs[min(nbufs,n)], get_input_buffer (n, nframes), sizeof (Sample) * nframes);
 	}
 }
 
@@ -989,6 +1024,10 @@ PortInsert::state (bool full)
 	node->add_property ("type", "port");
 	snprintf (buf, sizeof (buf), "%" PRIu32, bitslot);
 	node->add_property ("bitslot", buf);
+	snprintf (buf, sizeof (buf), "%u", _measured_latency);
+	node->add_property("latency", buf);
+	snprintf (buf, sizeof (buf), "%u", _session.get_block_size());
+	node->add_property("block_size", buf);
 
 	return *node;
 }
@@ -1011,6 +1050,18 @@ PortInsert::set_state(const XMLNode& node)
 		return -1;
 	}
 
+	uint32_t blocksize = 0;
+	if ((prop = node.property ("block_size")) != 0) {
+		sscanf (prop->value().c_str(), "%u", &blocksize);
+	}
+		
+	//if the jack period is the same as when the value was saved, we can recall our latency..
+	if ( (_session.get_block_size() == blocksize) && (prop = node.property ("latency")) != 0) {
+		uint32_t latency = 0;
+		sscanf (prop->value().c_str(), "%u", &latency);
+		_measured_latency = latency;
+	}
+
 	if ((prop = node.property ("bitslot")) == 0) {
 		bitslot = _session.next_insert_id();
 	} else {
@@ -1047,17 +1098,22 @@ PortInsert::latency()
 	   need to take that into account too.
 	*/
 
-	return _session.engine().frames_per_cycle() + input_latency();
+	if (_measured_latency == 0) {
+		return _session.engine().frames_per_cycle() + input_latency();
+	} else {
+		return _measured_latency;
+	}
 }
 
 int32_t
-PortInsert::can_support_input_configuration (int32_t in) const
+PortInsert::can_do (int32_t in, int32_t& out) 
 {
 	if (input_maximum() == -1 && output_maximum() == -1) {
 
 		/* not configured yet */
 
-		return in; /* we can support anything the first time we're asked */
+		out = in;
+		return 1;
 
 	} else {
 
@@ -1066,7 +1122,8 @@ PortInsert::can_support_input_configuration (int32_t in) const
 		*/
 
 		if (output_maximum() == in) {
-			return in;
+			out = in;
+			return 1;
 		} 
 	}
 
@@ -1105,18 +1162,6 @@ PortInsert::configure_io (int32_t ignored_magic, int32_t in, int32_t out)
 	return ensure_io (out, in, false, this);
 }
 
-int32_t
-PortInsert::compute_output_streams (int32_t cnt) const
-{
-	if (n_inputs() == 0) {
-		/* not configured yet */
-		return cnt;
-	}
-
-	/* puzzling, eh? think about it ... */
-	return n_inputs ();
-}
-
 uint32_t
 PortInsert::output_streams() const
 {
