@@ -25,10 +25,12 @@
 
 #include <sigc++/bind.h>
 
+#include <glibmm.h>
 #include <glibmm/thread.h>
 
 #include <pbd/xml++.h>
 #include <pbd/replace_all.h>
+#include <pbd/localeguard.h>
 
 #include <ardour/audioengine.h>
 #include <ardour/io.h>
@@ -44,6 +46,8 @@
 
 #include <cmath>
 
+#include <cstdlib>
+
 /*
   A bug in OS X's cmath that causes isnan() and isinf() to be 
   "undeclared". the following works around that
@@ -71,6 +75,7 @@ sigc::signal<int>                 IO::PortsLegal;
 sigc::signal<int>                 IO::PannersLegal;
 sigc::signal<void,uint32_t>  IO::MoreOutputs;
 sigc::signal<int>                 IO::PortsCreated;
+sigc::signal<void,nframes_t>      IO::CycleStart;
 
 Glib::StaticMutex       IO::m_meter_signal_lock = GLIBMM_STATIC_MUTEX_INIT;
 
@@ -91,9 +96,39 @@ static double direct_gain_to_control (gain_t gain) {
 	return pow((6.0*log(gain)/log(2.0)+192.0)/198.0, 8.0);
 }
 
-static bool sort_ports_by_name (Port* a, Port* b)
-{
-	return a->name() < b->name();
+static bool sort_ports_by_name (Port* a, Port* b) {
+
+	std::string const a_name = a->name();
+	unsigned int	last_digit_position_a = a_name.size();
+	std::string::const_reverse_iterator	r_iterator = a_name.rbegin();
+
+	while (r_iterator != a_name.rend() and Glib::Unicode::isdigit(*r_iterator)) {
+		r_iterator++; last_digit_position_a--;
+	}
+
+	std::string const b_name = b->name();
+	unsigned int	last_digit_position_b = b_name.size();
+	r_iterator = b_name.rbegin();
+
+	while (r_iterator != b_name.rend() and Glib::Unicode::isdigit(*r_iterator)) {
+		r_iterator++; last_digit_position_b--;
+	}
+
+	// if some of the names don't have a number as posfix, compare as strings
+	if (last_digit_position_a == a_name.size() or last_digit_position_b == b_name.size()) {
+		return a_name < b_name;
+	}
+
+	const std::string 	prefix_a = a_name.substr(0, last_digit_position_a - 1);
+	const unsigned int 	posfix_a = std::atoi(a_name.substr(last_digit_position_a, a_name.size() - last_digit_position_a).c_str());
+	const std::string 	prefix_b = b_name.substr(0, last_digit_position_b - 1);
+	const unsigned int 	posfix_b = std::atoi(b_name.substr(last_digit_position_b, b_name.size() - last_digit_position_b).c_str());
+
+	if (prefix_a != prefix_b) {
+		return a_name < b_name;
+	} else {
+		return posfix_a < posfix_b;
+	}
 }
 
 
@@ -130,8 +165,8 @@ IO::IO (Session& s, string name,
 	
 	last_automation_snapshot = 0;
 
-	_gain_automation_state = Off;
-	_gain_automation_style = Absolute;
+	_gain_automation_state = Auto_Off;
+	_gain_automation_style = Auto_Absolute;
 
 	{
 		// IO::Meter is emitted from another thread so the
@@ -140,6 +175,9 @@ IO::IO (Session& s, string name,
 		m_meter_connection = Meter.connect (mem_fun (*this, &IO::meter));
 	}
 
+	_output_offset = 0;
+	CycleStart.connect (mem_fun (*this, &IO::cycle_start));
+
 	_session.add_controllable (&_gain_control);
 }
 
@@ -172,6 +210,9 @@ IO::IO (Session& s, const XMLNode& node, DataType dt)
 		m_meter_connection = Meter.connect (mem_fun (*this, &IO::meter));
 	}
 
+	_output_offset = 0;
+	CycleStart.connect (mem_fun (*this, &IO::cycle_start));
+
 	_session.add_controllable (&_gain_control);
 }
 
@@ -198,12 +239,12 @@ IO::~IO ()
 }
 
 void
-IO::silence (nframes_t nframes, nframes_t offset)
+IO::silence (nframes_t nframes)
 {
 	/* io_lock, not taken: function must be called from Session::process() calltree */
 
 	for (vector<Port *>::iterator i = _outputs.begin(); i != _outputs.end(); ++i) {
-		(*i)->silence (nframes, offset);
+		(*i)->silence (nframes);
 	}
 }
 
@@ -264,7 +305,7 @@ IO::apply_declick (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, ga
 }
 
 void
-IO::pan_automated (vector<Sample*>& bufs, uint32_t nbufs, nframes_t start, nframes_t end, nframes_t nframes, nframes_t offset)
+IO::pan_automated (vector<Sample*>& bufs, uint32_t nbufs, nframes_t start, nframes_t end, nframes_t nframes)
 {
 	Sample* dst;
 
@@ -276,7 +317,7 @@ IO::pan_automated (vector<Sample*>& bufs, uint32_t nbufs, nframes_t start, nfram
 
 	if (_noutputs == 1) {
 
-		dst = output(0)->get_buffer (nframes) + offset;
+		dst = get_output_buffer (0, nframes);
 
 		for (uint32_t n = 0; n < nbufs; ++n) {
 			if (bufs[n] != dst) {
@@ -298,7 +339,7 @@ IO::pan_automated (vector<Sample*>& bufs, uint32_t nbufs, nframes_t start, nfram
 	/* the terrible silence ... */
 
 	for (out = _outputs.begin(), o = 0; out != _outputs.end(); ++out, ++o) {
-		obufs[o] = (*out)->get_buffer (nframes) + offset;
+		obufs[o] = get_output_buffer (o, nframes);
 		memset (obufs[o], 0, sizeof (Sample) * nframes);
 		(*out)->mark_silence (false);
 	}
@@ -311,7 +352,7 @@ IO::pan_automated (vector<Sample*>& bufs, uint32_t nbufs, nframes_t start, nfram
 }
 
 void
-IO::pan (vector<Sample*>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t offset, gain_t gain_coeff)
+IO::pan (vector<Sample*>& bufs, uint32_t nbufs, nframes_t nframes, gain_t gain_coeff)
 {
 	Sample* dst;
 	Sample* src;
@@ -327,13 +368,13 @@ IO::pan (vector<Sample*>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t off
 	*/
 
 	if (_panner->bypassed() || _panner->empty()) {
-		deliver_output_no_pan (bufs, nbufs, nframes, offset);
+		deliver_output_no_pan (bufs, nbufs, nframes);
 		return;
 	}
 
 	if (_noutputs == 1) {
 
-		dst = output(0)->get_buffer (nframes) + offset;
+		dst = get_output_buffer (0, nframes);
 
 		if (gain_coeff == 0.0f) {
 
@@ -388,7 +429,7 @@ IO::pan (vector<Sample*>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t off
 	/* XXX this is wasteful but i see no way to avoid it */
 	
 	for (out = _outputs.begin(), o = 0; out != _outputs.end(); ++out, ++o) {
-		obufs[o] = (*out)->get_buffer (nframes) + offset;
+		obufs[o] = get_output_buffer (o, nframes);
 		memset (obufs[o], 0, sizeof (Sample) * nframes);
 		(*out)->mark_silence (false);
 	}
@@ -410,7 +451,7 @@ IO::pan (vector<Sample*>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t off
 }
 
 void
-IO::deliver_output (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t offset)
+IO::deliver_output (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes)
 {
 	/* io_lock, not taken: function must be called from Session::process() calltree */
 
@@ -419,7 +460,7 @@ IO::deliver_output (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, n
 	}
 	
 	if (_panner->bypassed() || _panner->empty()) {
-		deliver_output_no_pan (bufs, nbufs, nframes, offset);
+		deliver_output_no_pan (bufs, nbufs, nframes);
 		return;
 	}
 
@@ -446,14 +487,14 @@ IO::deliver_output (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, n
 	/* simple, non-automation panning to outputs */
 
 	if (_session.transport_speed() > 1.5f || _session.transport_speed() < -1.5f) {
-		pan (bufs, nbufs, nframes, offset, pangain * speed_quietning);
+		pan (bufs, nbufs, nframes, pangain * speed_quietning);
 	} else {
-		pan (bufs, nbufs, nframes, offset, pangain);
+		pan (bufs, nbufs, nframes, pangain);
 	}
 }
 
 void
-IO::deliver_output_no_pan (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t offset)
+IO::deliver_output_no_pan (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes)
 {
 	/* io_lock, not taken: function must be called from Session::process() calltree */
 
@@ -503,7 +544,7 @@ IO::deliver_output_no_pan (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nfr
 	
 	for (o = _outputs.begin(), i = 0; o != _outputs.end(); ++o, ++i) {
 
-		dst = (*o)->get_buffer (nframes) + offset;
+		dst = get_output_buffer (i, nframes);
 		src = bufs[min(nbufs,i)];
 
 		if (dg != _gain) {
@@ -535,7 +576,7 @@ IO::deliver_output_no_pan (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nfr
 }
 
 void
-IO::collect_input (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t offset)
+IO::collect_input (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes)
 {
 	/* io_lock, not taken: function must be called from Session::process() calltree */
 
@@ -549,19 +590,8 @@ IO::collect_input (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, nf
 		if (i == _inputs.end()) {
 			break;
 		}
-		
-		/* XXX always read the full extent of the port buffer that
-		   we need. One day, we may use jack_port_get_buffer_at_offset()
-		   or something similar. For now, this simple hack will
-		   have to do.
 
-		   Hack? Why yes .. we only need to read nframes-worth of
-		   data, but the data we want is at `offset' within the
-		   buffer.
-		*/
-
-		last = (*i)->get_buffer (nframes+offset) + offset;
-		// the dest buffer's offset has already been applied
+		last = get_input_buffer (n, nframes);
 		memcpy (bufs[n], last, sizeof (Sample) * nframes);
 	}
 
@@ -569,7 +599,6 @@ IO::collect_input (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, nf
 	
 	if (last) {
 		while (n < nbufs) {
-			// the dest buffer's offset has already been applied
 			memcpy (bufs[n], last, sizeof (Sample) * nframes);
 			++n;
 		}
@@ -582,13 +611,12 @@ IO::collect_input (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, nf
 }
 
 void
-IO::just_meter_input (nframes_t start_frame, nframes_t end_frame, 
-		      nframes_t nframes, nframes_t offset)
+IO::just_meter_input (nframes_t start_frame, nframes_t end_frame, nframes_t nframes)
 {
 	vector<Sample*>& bufs = _session.get_passthru_buffers ();
 	uint32_t nbufs = n_process_buffers ();
 
-	collect_input (bufs, nbufs, nframes, offset);
+	collect_input (bufs, nbufs, nframes);
 
 	for (uint32_t n = 0; n < nbufs; ++n) {
 		_peak_power[n] = Session::compute_peak (bufs[n], nframes, _peak_power[n]);
@@ -1537,7 +1565,7 @@ IO::state (bool full_state)
 		snprintf (buf, sizeof (buf), "0x%x", (int) _gain_automation_curve.automation_state());
 	} else {
 		/* never store anything except Off for automation state in a template */
-		snprintf (buf, sizeof (buf), "0x%x", ARDOUR::Off); 
+		snprintf (buf, sizeof (buf), "0x%x", ARDOUR::Auto_Off); 
 	}
 
 	return *node;
@@ -1586,7 +1614,7 @@ IO::set_state (const XMLNode& node)
 	}
 
 	if ((prop = node.property (X_("active"))) != 0) {
-		set_active (prop->value() == "yes");
+		set_active (string_is_affirmative (prop->value()));
 	}
 
 	for (iter = node.children().begin(); iter != node.children().end(); ++iter) {
@@ -1669,16 +1697,12 @@ IO::load_automation (string path)
 	float version;
 	LocaleGuard lg (X_("POSIX"));
 
-	fullpath = _session.automation_dir();
-	fullpath += path;
+	fullpath = Glib::build_filename(_session.automation_dir(), path);
 
 	in.open (fullpath.c_str());
 
 	if (!in) {
-		fullpath = _session.automation_dir();
-		fullpath += _session.snap_name();
-		fullpath += '-';
-		fullpath += path;
+		fullpath = Glib::build_filename(_session.automation_dir(), _session.snap_name() + '-' + path);
 
 		in.open (fullpath.c_str());
 
@@ -1782,6 +1806,98 @@ IO::ports_became_legal ()
 	return ret;
 }
 
+Connection *
+IO::find_possible_connection(const string &desired_name, const string &default_name, const string &connection_type_name) 
+{
+	static const string digits = "0123456789";
+
+	Connection* c = _session.connection_by_name (desired_name);
+
+	if (!c) {
+		int connection_number, mask;
+		string possible_name;
+		bool stereo = false;
+		string::size_type last_non_digit_pos;
+
+		error << string_compose(_("Unknown connection \"%1\" listed for %2 of %3"), desired_name, connection_type_name, _name)
+		      << endmsg;
+
+		// find numeric suffix of desired name
+		connection_number = 0;
+		
+		last_non_digit_pos = desired_name.find_last_not_of(digits);
+
+		if (last_non_digit_pos != string::npos) {
+			stringstream s;
+			s << desired_name.substr(last_non_digit_pos);
+			s >> connection_number;
+		}
+	
+		// see if it's a stereo connection e.g. "in 3+4"
+
+		if (last_non_digit_pos > 1 && desired_name[last_non_digit_pos] == '+') {
+			int left_connection_number = 0;
+			string::size_type left_last_non_digit_pos;
+
+			left_last_non_digit_pos = desired_name.find_last_not_of(digits, last_non_digit_pos-1);
+
+			if (left_last_non_digit_pos != string::npos) {
+				stringstream s;
+				s << desired_name.substr(left_last_non_digit_pos, last_non_digit_pos-1);
+				s >> left_connection_number;
+
+				if (left_connection_number > 0 && left_connection_number + 1 == connection_number) {
+					connection_number--;
+					stereo = true;
+				}
+			}
+		}
+
+		// make 0-based
+		if (connection_number)
+			connection_number--;
+
+		// find highest set bit
+		mask = 1;
+		while ((mask <= connection_number) && (mask <<= 1));
+		
+		// "wrap" connection number into largest possible power of 2 
+		// that works...
+
+		while (mask) {
+
+			if (connection_number & mask) {
+				connection_number &= ~mask;
+				
+				stringstream s;
+				s << default_name << " " << connection_number + 1;
+
+				if (stereo) {
+					s << "+" << connection_number + 2;
+				}
+				
+				possible_name = s.str();
+
+				if ((c = _session.connection_by_name (possible_name)) != 0) {
+					break;
+				}
+			}
+			mask >>= 1;
+		}
+		if (c) {
+			info << string_compose (_("Connection %1 was not available - \"%2\" used instead"), desired_name, possible_name)
+			     << endmsg;
+		} else {
+			error << string_compose(_("No %1 connections available as a replacement"), connection_type_name)
+			      << endmsg;
+		}
+
+	}
+
+	return c;
+
+}
+
 int
 IO::create_ports (const XMLNode& node)
 {
@@ -1791,19 +1907,10 @@ IO::create_ports (const XMLNode& node)
 
 	if ((prop = node.property ("input-connection")) != 0) {
 
-		Connection* c = _session.connection_by_name (prop->value());
+		Connection* c = find_possible_connection(prop->value(), _("in"), _("input"));
 		
 		if (c == 0) {
-			error << string_compose(_("Unknown connection \"%1\" listed for input of %2"), prop->value(), _name) << endmsg;
-
-			if ((c = _session.connection_by_name (_("in 1"))) == 0) {
-				error << _("No input connections available as a replacement")
-				      << endmsg;
-				return -1;
-			}  else {
-				info << string_compose (_("Connection %1 was not available - \"in 1\" used instead"), prop->value())
-				     << endmsg;
-			}
+			return -1;
 		} 
 
 		num_inputs = c->nports();
@@ -1814,19 +1921,10 @@ IO::create_ports (const XMLNode& node)
 	}
 	
 	if ((prop = node.property ("output-connection")) != 0) {
-		Connection* c = _session.connection_by_name (prop->value());
+		Connection* c = find_possible_connection(prop->value(), _("out"), _("output"));
 
 		if (c == 0) {
-			error << string_compose(_("Unknown connection \"%1\" listed for output of %2"), prop->value(), _name) << endmsg;
-
-			if ((c = _session.connection_by_name (_("out 1"))) == 0) {
-				error << _("No output connections available as a replacement")
-				      << endmsg;
-				return -1;
-			}  else {
-				info << string_compose (_("Connection %1 was not available - \"out 1\" used instead"), prop->value())
-				     << endmsg;
-			}
+			return -1;
 		} 
 
 		num_outputs = c->nports ();
@@ -1857,19 +1955,10 @@ IO::make_connections (const XMLNode& node)
 	const XMLProperty* prop;
 
 	if ((prop = node.property ("input-connection")) != 0) {
-		Connection* c = _session.connection_by_name (prop->value());
+		Connection* c = find_possible_connection (prop->value(), _("in"), _("input"));
 		
 		if (c == 0) {
-			error << string_compose(_("Unknown connection \"%1\" listed for input of %2"), prop->value(), _name) << endmsg;
-
-			if ((c = _session.connection_by_name (_("in 1"))) == 0) {
-				error << _("No input connections available as a replacement")
-				      << endmsg;
-				return -1;
-			} else {
-				info << string_compose (_("Connection %1 was not available - \"in 1\" used instead"), prop->value())
-				     << endmsg;
-			}
+			return -1;
 		} 
 
 		use_input_connection (*c, this);
@@ -1882,19 +1971,10 @@ IO::make_connections (const XMLNode& node)
 	}
 	
 	if ((prop = node.property ("output-connection")) != 0) {
-		Connection* c = _session.connection_by_name (prop->value());
+		Connection* c = find_possible_connection (prop->value(), _("out"), _("output"));
 		
 		if (c == 0) {
-			error << string_compose(_("Unknown connection \"%1\" listed for output of %2"), prop->value(), _name) << endmsg;
-
-			if ((c = _session.connection_by_name (_("out 1"))) == 0) {
-				error << _("No output connections available as a replacement")
-				      << endmsg;
-				return -1;
-			}  else {
-				info << string_compose (_("Connection %1 was not available - \"out 1\" used instead"), prop->value())
-				     << endmsg;
-			}
+			return -1;
 		} 
 
 		use_output_connection (*c, this);
@@ -2468,7 +2548,7 @@ IO::meter ()
 		/* compute new visible value using falloff */
 
 		if (new_peak > 0.0f) {
-			new_peak = coefficient_to_dB (new_peak);
+			new_peak = fast_coefficient_to_dB (new_peak);
 		} else {
 			new_peak = -INFINITY;
 		}
@@ -2487,6 +2567,13 @@ IO::meter ()
 	}
 }
 
+bool
+IO::gain_automation_recording ()
+{ 
+	return (_session.transport_rolling() &&	_gain_automation_curve.automation_write() );
+}
+
+
 void
 IO::clear_automation ()
 {
@@ -2507,8 +2594,11 @@ IO::set_gain_automation_state (AutoState state)
 			changed = true;
 			last_automation_snapshot = 0;
 			_gain_automation_curve.set_automation_state (state);
-			
-			if (state != Off) {
+
+			/* don't reset gain if we're moving to Off or Write mode;
+			   if we're moving to Write, the user may have manually set up gains
+			   that they don't want to lose */
+			if (state != Auto_Off && state != Auto_Write) {
 				set_gain (_gain_automation_curve.eval (_session.transport_frame()), this);
 			}
 		}
@@ -2551,7 +2641,10 @@ void
 IO::set_gain (gain_t val, void *src)
 {
 	// max gain at about +6dB (10.0 ^ ( 6 dB * 0.05))
-	if (val>1.99526231f) val=1.99526231f;
+
+	if (val > 1.99526231f) {
+		val = 1.99526231f;
+	}
 
 	{
 		Glib::Mutex::Lock dm (declick_lock);
@@ -2560,7 +2653,7 @@ IO::set_gain (gain_t val, void *src)
 
 	if (_session.transport_stopped()) {
 		_effective_gain = val;
-		_gain = val;
+		// _gain = val;
 	}
 
 	gain_changed (src);
@@ -2576,20 +2669,28 @@ IO::set_gain (gain_t val, void *src)
 void
 IO::start_gain_touch ()
 {
-	_gain_automation_curve.start_touch ();
+	_gain_automation_curve.start_touch (_session.transport_frame());
 }
 
 void
 IO::end_gain_touch ()
 {
-	_gain_automation_curve.stop_touch ();
+        bool mark = false;
+        double when = 0;
+
+        if (_session.transport_rolling() && _gain_automation_curve.automation_state() == Auto_Touch) {
+                mark = true;
+                when = _session.transport_frame();
+        }
+
+	_gain_automation_curve.stop_touch (mark, when);
 }
 
 void
 IO::start_pan_touch (uint32_t which)
 {
 	if (which < _panner->size()) {
-		(*_panner)[which]->automation().start_touch();
+		(*_panner)[which]->automation().start_touch (_session.transport_frame());
 	}
 }
 
@@ -2597,9 +2698,16 @@ void
 IO::end_pan_touch (uint32_t which)
 {
 	if (which < _panner->size()) {
-		(*_panner)[which]->automation().stop_touch();
+                bool mark = false;
+                double when = 0;
+                
+                if (_session.transport_rolling() && (*_panner)[which]->automation().automation_state() == Auto_Touch) {
+                        mark = true;
+                        when = _session.transport_frame();
+                }
+                
+		(*_panner)[which]->automation().stop_touch(mark, when);
 	}
-
 }
 
 void
@@ -2616,9 +2724,9 @@ IO::automation_snapshot (nframes_t now, bool force)
 void
 IO::transport_stopped (nframes_t frame)
 {
-	_gain_automation_curve.reposition_for_rt_add (frame);
+	_gain_automation_curve.write_pass_finished (frame);
 
-	if (_gain_automation_curve.automation_state() != Off) {
+	if (_gain_automation_curve.automation_state() == Auto_Touch || _gain_automation_curve.automation_state() == Auto_Play) {
 		
 		/* the src=0 condition is a special signal to not propagate 
 		   automation gain changes into the mix group when locating.
@@ -2635,14 +2743,28 @@ IO::build_legal_port_name (bool in)
 {
 	const int name_size = jack_port_name_size();
 	int limit;
-	char* suffix;
+	const char* suffix;
 	int maxports;
 
+	/* note that if "in" or "out" are translated it will break a session
+	   across locale switches because a port's connection list will
+	   show (old) translated names, but the current port name will
+	   use the (new) translated name.
+	*/
+
 	if (in) {
-		suffix = _("in");
+		if (getenv ("ARDOUR_RETAIN_PORT_NAME_SUFFIX_TRANSLATION") != 0) {
+			suffix = _("in");
+		} else {
+			suffix = X_("in");
+		}
 		maxports = _input_maximum;
 	} else {
-		suffix = _("out");
+		if (getenv ("ARDOUR_RETAIN_PORT_NAME_SUFFIX_TRANSLATION") != 0) {
+			suffix = _("out");
+		} else {
+			suffix = X_("out");
+		}
 		maxports = _output_maximum;
 	}
 	
@@ -2750,3 +2872,61 @@ IO::set_active (bool yn)
 	 active_changed(); /* EMIT SIGNAL */
 }
 
+string
+IO::name_from_state (const XMLNode& node)
+{
+	const XMLProperty* prop;
+	
+	if ((prop = node.property ("name")) != 0) {
+		return prop->value();
+	} 
+	
+	return string();
+}
+
+void
+IO::set_name_in_state (XMLNode& node, const string& new_name)
+{
+	const XMLProperty* prop;
+	
+	if ((prop = node.property ("name")) != 0) {
+		node.add_property ("name", new_name);
+	} 
+}
+
+void
+IO::cycle_start (nframes_t nframes)
+{
+	_output_offset = 0;
+}
+
+void
+IO::increment_output_offset (nframes_t n)
+{
+	_output_offset += n;
+}
+
+Sample*
+IO::get_input_buffer (int n, nframes_t nframes)
+{
+	Port* port;
+
+	if (!(port = input (n))) {
+		return 0;
+	}
+
+	return port->get_buffer (nframes);
+}
+
+Sample*
+IO::get_output_buffer (int n, nframes_t nframes)
+{
+	Port* port;
+
+	if (!(port = output (n))) {
+		return 0;
+	}
+
+	return port->get_buffer (nframes) + _output_offset;
+}
+	       
