@@ -25,6 +25,7 @@
 #include <pbd/xml++.h>
 #include <pbd/enumwriter.h>
 #include <pbd/stacktrace.h>
+#include <pbd/memento_command.h>
 
 #include <ardour/timestamps.h>
 #include <ardour/buffer.h>
@@ -51,7 +52,7 @@ using namespace ARDOUR;
 using namespace PBD;
 
 uint32_t Route::order_key_cnt = 0;
-sigc::signal<void> Route::SyncOrderKeys;
+sigc::signal<void,const char*> Route::SyncOrderKeys;
 
 Route::Route (Session& sess, string name, int input_min, int input_max, int output_min, int output_max, Flag flg, DataType default_type)
 	: IO (sess, name, input_min, input_max, output_min, output_max, default_type),
@@ -169,20 +170,32 @@ Route::set_order_key (const char* name, long n)
 }
 
 void
-Route::sync_order_keys ()
+Route::sync_order_keys (const char* base)
 {
-	uint32_t key;
-	
 	if (order_keys.empty()) {
 		return;
 	}
-	
-	OrderKeys::iterator x = order_keys.begin();
-	key = x->second;
-	++x;
 
-	for (; x != order_keys.end(); ++x) {
-		x->second = key;
+	OrderKeys::iterator i;
+	uint32_t key;
+
+	if ((i = order_keys.find (base)) == order_keys.end()) {
+		/* key doesn't exist, use the first existing
+		   key (this is done during session initialization)
+		*/
+		i = order_keys.begin();
+		key = i->second;
+		++i;
+	} else {
+		/* key exists - use it and reset all others
+		   (actually, itself included)
+		*/
+		i = order_keys.begin();
+		key = i->second;
+	}
+
+	for (; i != order_keys.end(); ++i) {
+		i->second = key;
 	}
 }
 
@@ -190,16 +203,14 @@ string
 Route::ensure_track_or_route_name(string name, Session &session)
 {
 	string newname = name;
-
-	while (session.route_by_name (newname)!=NULL)
-	{
+	
+	while (!session.io_name_is_legal (newname)) {
 		newname = bump_name_once (newname);
 	}
 
 	return newname;
 }
 
-
 void
 Route::inc_gain (gain_t fraction, void *src)
 {
@@ -264,7 +275,7 @@ Route::set_gain (gain_t val, void *src)
 void
 Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 			       nframes_t start_frame, nframes_t end_frame, 
-			       nframes_t nframes, nframes_t offset, bool with_redirects, int declick,
+			       nframes_t nframes, bool with_redirects, int declick,
 			       bool meter)
 {
 	uint32_t n;
@@ -372,11 +383,11 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 
 			) {
 			
-			co->silence (nframes, offset);
+			co->silence (nframes);
 			
 		} else {
 
-			co->deliver_output (bufs, nbufs, nframes, offset);
+			co->deliver_output (bufs, nbufs, nframes);
 			
 		} 
 	} 
@@ -390,7 +401,7 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 		for (n = 0; n < nbufs; ++n)  {
 			Sample *sp = bufs[n];
 			
-			for (nframes_t nx = offset; nx < nframes + offset; ++nx) {
+			for (nframes_t nx = 0; nx < nframes; ++nx) {
 				sp[nx] += 1.0e-27f;
 			}
 		}
@@ -410,10 +421,10 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 					case PreFader:
 						if (dsg == 0) {
 							if (boost::dynamic_pointer_cast<Send>(*i) || boost::dynamic_pointer_cast<PortInsert>(*i)) {
-								(*i)->silence (nframes, offset);
+								(*i)->silence (nframes);
 							}
 						} else {
-							(*i)->run (bufs, nbufs, nframes, offset);
+							(*i)->run (bufs, nbufs, nframes);
 						}
 						break;
 					case PostFader:
@@ -425,7 +436,7 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 				for (i = _redirects.begin(); i != _redirects.end(); ++i) {
 					switch ((*i)->placement()) {
 					case PreFader:
-						(*i)->silence (nframes, offset);
+						(*i)->silence (nframes);
 						break;
 					case PostFader:
 						post_fader_work = true;
@@ -473,11 +484,11 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 
 			) {
 			
-			co->silence (nframes, offset);
+			co->silence (nframes);
 			
 		} else {
 
-			co->deliver_output_no_pan (bufs, nbufs, nframes, offset);
+			co->deliver_output_no_pan (bufs, nbufs, nframes);
 			
 		} 
 	} 
@@ -583,10 +594,10 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 					case PostFader:
 						if (dsg == 0) {
 							if (boost::dynamic_pointer_cast<Send>(*i) || boost::dynamic_pointer_cast<PortInsert>(*i)) {
-								(*i)->silence (nframes, offset);
+								(*i)->silence (nframes);
 							}
 						} else {
-							(*i)->run (bufs, nbufs, nframes, offset);
+							(*i)->run (bufs, nbufs, nframes);
 						}
 						break;
 					}
@@ -597,7 +608,7 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 					case PreFader:
 						break;
 					case PostFader:
-						(*i)->silence (nframes, offset);
+						(*i)->silence (nframes);
 						break;
 					}
 				}
@@ -638,11 +649,11 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 
 			) {
 			
-			co->silence (nframes, offset);
+			co->silence (nframes);
 			
 		} else {
 
-			co->deliver_output_no_pan (bufs, nbufs, nframes, offset);
+			co->deliver_output_no_pan (bufs, nbufs, nframes);
 		} 
 	} 
 
@@ -669,7 +680,7 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 
 	} else if (no_monitor && record_enabled() && (!Config->get_auto_input() || _session.actively_recording())) {
 		
-		IO::silence (nframes, offset);
+		IO::silence (nframes);
 		
 	} else {
 
@@ -695,22 +706,22 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 				reset_peak_meters ();
 			}
 
-			IO::silence (nframes, offset);
+			IO::silence (nframes);
 			
 		} else {
 			
 			if ((_session.transport_speed() > 1.5f || 
 			     _session.transport_speed() < -1.5f) &&
 			    Config->get_quieten_at_speed()) {
-				pan (bufs, nbufs, nframes, offset, speed_quietning); 
+				pan (bufs, nbufs, nframes, speed_quietning); 
 			} else {
 				// cerr << _name << " panner state = " << _panner->automation_state() << endl;
 				if (!_panner->empty() &&
-				    (_panner->automation_state() & Play ||
-				     ((_panner->automation_state() & Touch) && !_panner->touching()))) {
-					pan_automated (bufs, nbufs, start_frame, end_frame, nframes, offset);
+				    (_panner->automation_state() & Auto_Play ||
+				     ((_panner->automation_state() & Auto_Touch) && !_panner->touching()))) {
+					pan_automated (bufs, nbufs, start_frame, end_frame, nframes);
 				} else {
-					pan (bufs, nbufs, nframes, offset, 1.0); 
+					pan (bufs, nbufs, nframes, 1.0); 
 				}
 			}
 		}
@@ -731,7 +742,7 @@ Route::process_output_buffers (vector<Sample*>& bufs, uint32_t nbufs,
 		} else {
 			uint32_t no = n_outputs();
 			for (n = 0; n < no; ++n) {
-				_peak_power[n] = Session::compute_peak (output(n)->get_buffer (nframes) + offset, nframes, _peak_power[n]);
+				_peak_power[n] = Session::compute_peak (get_output_buffer (n, nframes), nframes, _peak_power[n]);
 			}
 		}
 	}
@@ -745,14 +756,14 @@ Route::n_process_buffers ()
 
 void
 
-Route::passthru (nframes_t start_frame, nframes_t end_frame, nframes_t nframes, nframes_t offset, int declick, bool meter_first)
+Route::passthru (nframes_t start_frame, nframes_t end_frame, nframes_t nframes, int declick, bool meter_first)
 {
 	vector<Sample*>& bufs = _session.get_passthru_buffers();
 	uint32_t limit = n_process_buffers ();
 
 	_silent = false;
 
-	collect_input (bufs, limit, nframes, offset);
+	collect_input (bufs, limit, nframes);
 
 #define meter_stream meter_first
 
@@ -765,7 +776,7 @@ Route::passthru (nframes_t start_frame, nframes_t end_frame, nframes_t nframes,
 		meter_stream = true;
 	}
 		
-	process_output_buffers (bufs, limit, start_frame, end_frame, nframes, offset, true, declick, meter_stream);
+	process_output_buffers (bufs, limit, start_frame, end_frame, nframes, true, declick, meter_stream);
 
 #undef meter_stream
 }
@@ -804,6 +815,31 @@ Route::set_solo (bool yn, void *src)
 		_soloed = yn;
 		solo_changed (src); /* EMIT SIGNAL */
 		_solo_control.Changed (); /* EMIT SIGNAL */
+	}	
+	
+	catch_up_on_solo_mute_override ();
+}
+
+void
+Route::catch_up_on_solo_mute_override ()
+{
+	if (Config->get_solo_model() != InverseMute) {
+		return;
+	}
+	
+	{
+
+		Glib::Mutex::Lock lm (declick_lock);
+		
+		if (_muted) {
+			if (Config->get_solo_mute_override()) {
+				desired_mute_gain = (_soloed?1.0:0.0);
+			} else {
+				desired_mute_gain = 0.0;
+			}
+		} else {
+			desired_mute_gain = 1.0;
+		}
 	}
 }
 
@@ -843,7 +879,12 @@ Route::set_mute (bool yn, void *src)
 		_mute_control.Changed (); /* EMIT SIGNAL */
 		
 		Glib::Mutex::Lock lm (declick_lock);
-		desired_mute_gain = (yn?0.0f:1.0f);
+		
+		if (_soloed && Config->get_solo_mute_override()){
+			desired_mute_gain = 1.0f;
+		} else {
+			desired_mute_gain = (yn?0.0f:1.0f);
+		}
 	}
 }
 
@@ -873,7 +914,6 @@ Route::add_redirect (boost::shared_ptr<Redirect> redirect, void *src, uint32_t*
 		uint32_t potential_max_streams = 0;
 
 		if ((pi = boost::dynamic_pointer_cast<PluginInsert>(redirect)) != 0) {
-			pi->set_count (1);
 
 			if (pi->input_streams() == 0) {
 				/* instrument plugin */
@@ -910,7 +950,6 @@ Route::add_redirect (boost::shared_ptr<Redirect> redirect, void *src, uint32_t*
 			_max_peak_power.push_back(-INFINITY);
 		}
 
-		redirect->activate ();
 		redirect->active_changed.connect (mem_fun (*this, &Route::redirect_active_proxy));
 	}
 	
@@ -974,7 +1013,6 @@ Route::add_redirects (const RedirectList& others, void *src, uint32_t* err_strea
 				return -1;
 			}
 			
-			(*i)->activate ();
 			(*i)->active_changed.connect (mem_fun (*this, &Route::redirect_active_proxy));
 		}
 	}
@@ -1093,19 +1131,18 @@ Route::remove_redirect (boost::shared_ptr<Redirect> redirect, void *src, uint32_
 			return -1;
 		}
 
-		bool foo = false;
+		_have_internal_generator = false;
 
 		for (i = _redirects.begin(); i != _redirects.end(); ++i) {
 			boost::shared_ptr<PluginInsert> pi;
 			
 			if ((pi = boost::dynamic_pointer_cast<PluginInsert>(*i)) != 0) {
 				if (pi->is_generator()) {
-					foo = true;
+					_have_internal_generator = true;
+					break;
 				}
 			}
 		}
-
-		_have_internal_generator = foo;
 	}
 
 	if (old_rmo != redirect_max_outs) {
@@ -1113,8 +1150,8 @@ Route::remove_redirect (boost::shared_ptr<Redirect> redirect, void *src, uint32_
 	}
 
 	redirect->drop_references ();
-
 	redirects_changed (src); /* EMIT SIGNAL */
+
 	return 0;
 }
 
@@ -1134,12 +1171,15 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 	uint32_t send_cnt = 0;
 	map<Placement,list<InsertCount> > insert_map;
 	RedirectList::iterator prev;
-	nframes_t initial_streams;
+	int32_t initial_streams = n_inputs ();;
+	int32_t previous_initial_streams = n_inputs ();
 	int ret = -1;
 
 	redirect_max_outs = 0;
 
-	/* divide inserts up by placement so we get the signal flow
+	/* Step 1: build a map that links each insert to an in/out channel count 
+
+	   Divide inserts up by placement so we get the signal flow
 	   properly modelled. we need to do this because the _redirects
 	   list is not sorted by placement, and because other reasons may 
 	   exist now or in the future for this separate treatment.
@@ -1191,10 +1231,13 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 	/* figure out the streams that will feed into PreFader */
 
 	if (!insert_map[PreFader].empty()) {
-		InsertCount& ic (insert_map[PreFader].back());
-		initial_streams = ic.insert->compute_output_streams (ic.cnt);
-	} else {
-		initial_streams = n_inputs ();
+		previous_initial_streams = n_inputs ();
+		for (list<InsertCount>::iterator i = insert_map[PreFader].begin(); i != insert_map[PreFader].end(); i++) {
+			if (i->insert->can_do (previous_initial_streams, initial_streams) < 0) {
+				goto streamcount;
+			}
+			previous_initial_streams = initial_streams;
+		}
 	}
 
 	/* B: PostFader */
@@ -1203,6 +1246,15 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 		goto streamcount;
 	}
 
+	if (!insert_map[PostFader].empty()) {
+		for (list<InsertCount>::iterator i = insert_map[PostFader].begin(); i != insert_map[PostFader].end(); i++) {
+			if (i->insert->can_do (previous_initial_streams, initial_streams) < 0) {
+				goto streamcount;
+			}
+			previous_initial_streams = initial_streams;
+		}
+	}
+
 	/* OK, everything can be set up correctly, so lets do it */
 
 	apply_some_plugin_counts (insert_map[PreFader]);
@@ -1269,8 +1321,8 @@ Route::check_some_plugin_counts (list<InsertCount>& iclist, int32_t required_inp
 	list<InsertCount>::iterator i;
 	
 	for (i = iclist.begin(); i != iclist.end(); ++i) {
-
-		if (((*i).cnt = (*i).insert->can_support_input_configuration (required_inputs)) < 0) {
+		
+		if (((*i).cnt = (*i).insert->can_do (required_inputs, (*i).out)) < 0) {
 			if (err_streams) {
 				*err_streams = required_inputs;
 			}
@@ -1278,14 +1330,6 @@ Route::check_some_plugin_counts (list<InsertCount>& iclist, int32_t required_inp
 		}
 		
 		(*i).in = required_inputs;
-
-		if (((*i).out = (*i).insert->compute_output_streams ((*i).cnt)) < 0) {
-			if (err_streams) {
-				*err_streams = required_inputs;
-			}
-			return -1;
-		}
-
 		required_inputs = (*i).out;
 	}
 
@@ -1477,7 +1521,8 @@ Route::state(bool full_state)
 	node->add_property("mute-affects-pre-fader", _mute_affects_pre_fader?"yes":"no"); 
 	node->add_property("mute-affects-post-fader", _mute_affects_post_fader?"yes":"no"); 
 	node->add_property("mute-affects-control-outs", _mute_affects_control_outs?"yes":"no"); 
-	node->add_property("mute-affects-main-outs", _mute_affects_main_outs?"yes":"no"); 
+	node->add_property("mute-affects-main-outs", _mute_affects_main_outs?"yes":"no");
+	node->add_property("meter-point", enum_2_string (_meter_point));
 
 	if (_edit_group) {
 		node->add_property("edit-group", _edit_group->name());
@@ -1649,15 +1694,15 @@ Route::_set_state (const XMLNode& node, bool call_base)
 	}
 
 	if ((prop = node.property (X_("phase-invert"))) != 0) {
-		set_phase_invert (prop->value()=="yes"?true:false, this);
+		set_phase_invert (string_is_affirmative (prop->value()), this);
 	}
 
 	if ((prop = node.property (X_("denormal-protection"))) != 0) {
-		set_denormal_protection (prop->value()=="yes"?true:false, this);
+		set_denormal_protection (string_is_affirmative (prop->value()), this);
 	}
 
 	if ((prop = node.property (X_("muted"))) != 0) {
-		bool yn = prop->value()=="yes"?true:false; 
+		bool yn = string_is_affirmative (prop->value());
 
 		/* force reset of mute status */
 
@@ -1667,7 +1712,7 @@ Route::_set_state (const XMLNode& node, bool call_base)
 	}
 
 	if ((prop = node.property (X_("soloed"))) != 0) {
-		bool yn = prop->value()=="yes"?true:false; 
+		bool yn = string_is_affirmative (prop->value());
 
 		/* force reset of solo status */
 
@@ -1677,21 +1722,25 @@ Route::_set_state (const XMLNode& node, bool call_base)
 	}
 
 	if ((prop = node.property (X_("mute-affects-pre-fader"))) != 0) {
-		_mute_affects_pre_fader = (prop->value()=="yes")?true:false;
+		_mute_affects_pre_fader = string_is_affirmative (prop->value());
 	}
 
 	if ((prop = node.property (X_("mute-affects-post-fader"))) != 0) {
-		_mute_affects_post_fader = (prop->value()=="yes")?true:false;
+		_mute_affects_post_fader = string_is_affirmative (prop->value());
 	}
 
 	if ((prop = node.property (X_("mute-affects-control-outs"))) != 0) {
-		_mute_affects_control_outs = (prop->value()=="yes")?true:false;
+		_mute_affects_control_outs = string_is_affirmative (prop->value());
 	}
 
 	if ((prop = node.property (X_("mute-affects-main-outs"))) != 0) {
-		_mute_affects_main_outs = (prop->value()=="yes")?true:false;
+		_mute_affects_main_outs = string_is_affirmative (prop->value());
 	}
 
+	if ((prop = node.property (X_("meter-point"))) != 0) {
+		_meter_point = MeterPoint (string_2_enum (prop->value (), _meter_point));
+	}
+	
 	if ((prop = node.property (X_("edit-group"))) != 0) {
 		RouteGroup* edit_group = _session.edit_group_by_name(prop->value());
 		if(edit_group == 0) {
@@ -1783,9 +1832,22 @@ Route::_set_state (const XMLNode& node, bool call_base)
 			string coutname = _name;
 			coutname += _("[control]");
 
+			delete _control_outs;
 			_control_outs = new IO (_session, coutname);
-			_control_outs->set_state (**(child->children().begin()));
 
+			/* fix up the control out name in the XML before setting it. 
+			   Otherwise track templates don't work because the control
+			   outs end up with the stored template name, rather than
+			   the new name of the track based on the template.
+			*/
+
+			XMLProperty* prop = (*child->children().begin())->property ("name");
+			if (prop) {
+				prop->set_value (coutname);
+			}
+
+			_control_outs->set_state (**(child->children().begin()));
+			
 		} else if (child->name() == X_("Comment")) {
 
 			/* XXX this is a terrible API design in libxml++ */
@@ -1941,16 +2003,16 @@ Route::curve_reallocate ()
 }
 
 void
-Route::silence (nframes_t nframes, nframes_t offset)
+Route::silence (nframes_t nframes)
 {
 	if (!_silent) {
 
 		// reset_peak_meters ();
 		
-		IO::silence (nframes, offset);
+		IO::silence (nframes);
 
 		if (_control_outs) {
-			_control_outs->silence (nframes, offset);
+			_control_outs->silence (nframes);
 		}
 
 		{ 
@@ -1958,16 +2020,17 @@ Route::silence (nframes_t nframes, nframes_t offset)
 			
 			if (lm.locked()) {
 				for (RedirectList::iterator i = _redirects.begin(); i != _redirects.end(); ++i) {
-					boost::shared_ptr<PluginInsert> pi;
-					if (!_active && (pi = boost::dynamic_pointer_cast<PluginInsert> (*i)) != 0) {
+					boost::shared_ptr<PluginInsert> pi = boost::dynamic_pointer_cast<PluginInsert> (*i);
+
+					if (!_active && pi) {
 						// skip plugins, they don't need anything when we're not active
 						continue;
 					}
 
-					(*i)->silence (nframes, offset);
+					(*i)->silence (nframes);
 				}
 
-				if (nframes == _session.get_block_size() && offset == 0) {
+				if (nframes == _session.get_block_size()) {
 					// _silent = true;
 				}
 			}
@@ -2202,9 +2265,8 @@ Route::handle_transport_stopped (bool abort_ignored, bool did_locate, bool can_f
 		for (RedirectList::iterator i = _redirects.begin(); i != _redirects.end(); ++i) {
 			
 			if (Config->get_plugins_stop_with_transport() && can_flush_redirects) {
-				(*i)->deactivate ();
-				(*i)->activate ();
-			}
+				(*i)->flush ();
+			} 
 			
 			(*i)->transport_stopped (now);
 		}
@@ -2246,36 +2308,51 @@ Route::pans_required () const
 }
 
 int 
-Route::no_roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, nframes_t offset, 
-		   bool session_state_changing, bool can_record, bool rec_monitors_input)
+Route::no_roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame,  
+		bool session_state_changing, bool can_record, bool rec_monitors_input)
 {
 	if (n_outputs() == 0) {
 		return 0;
 	}
 
-	if (session_state_changing || !_active)  {
-		silence (nframes, offset);
+	if (!_active)  {
+		silence (nframes);
 		return 0;
 	}
 
+	if (session_state_changing) {
+		if (_session.transport_speed() != 0.0f) {
+			/* we're rolling but some state is changing (e.g. our diskstream contents)
+			   so we cannot use them. Be silent till this is over.
+			   
+			   XXX note the absurdity of ::no_roll() being called when we ARE rolling!
+			*/
+			silence (nframes);
+			return 0;
+		}
+		/* we're really not rolling, so we're either delivery silence or actually
+		   monitoring, both of which are safe to do while session_state_changing is true.
+		*/
+	}
+
 	apply_gain_automation = false;
 	
 	if (n_inputs()) {
-		passthru (start_frame, end_frame, nframes, offset, 0, false);
+		passthru (start_frame, end_frame, nframes, 0, false);
 	} else {
-		silence (nframes, offset);
+		silence (nframes);
 	}
 
 	return 0;
 }
 
 nframes_t
-Route::check_initial_delay (nframes_t nframes, nframes_t& offset, nframes_t& transport_frame)
+Route::check_initial_delay (nframes_t nframes, nframes_t& transport_frame)
 {
 	if (_roll_delay > nframes) {
 
 		_roll_delay -= nframes;
-		silence (nframes, offset);
+		silence (nframes);
 		/* transport frame is not legal for caller to use */
 		return 0;
 
@@ -2283,9 +2360,15 @@ Route::check_initial_delay (nframes_t nframes, nframes_t& offset, nframes_t& tra
 
 		nframes -= _roll_delay;
 
-		silence (_roll_delay, offset);
+		silence (_roll_delay);
+
+		/* we've written _roll_delay of samples into the 
+		   output ports, so make a note of that for
+		   future reference.
+		*/
+
+		increment_output_offset (_roll_delay);
 
-		offset += _roll_delay;
 		transport_frame += _roll_delay;
 
 		_roll_delay = 0;
@@ -2295,7 +2378,7 @@ Route::check_initial_delay (nframes_t nframes, nframes_t& offset, nframes_t& tra
 }
 
 int
-Route::roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, nframes_t offset, int declick,
+Route::roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, int declick,
 	     bool can_record, bool rec_monitors_input)
 {
 	{
@@ -2308,13 +2391,13 @@ Route::roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, nfra
 	}
 
 	if ((n_outputs() == 0 && _redirects.empty()) || n_inputs() == 0 || !_active) {
-		silence (nframes, offset);
+		silence (nframes);
 		return 0;
 	}
 	
 	nframes_t unused = 0;
 
-	if ((nframes = check_initial_delay (nframes, offset, unused)) == 0) {
+	if ((nframes = check_initial_delay (nframes, unused)) == 0) {
 		return 0;
 	}
 
@@ -2335,16 +2418,16 @@ Route::roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, nfra
 		}
 	}
 
-	passthru (start_frame, end_frame, nframes, offset, declick, false);
+	passthru (start_frame, end_frame, nframes, declick, false);
 
 	return 0;
 }
 
 int
-Route::silent_roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, nframes_t offset, 
+Route::silent_roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, 
 		    bool can_record, bool rec_monitors_input)
 {
-	silence (nframes, offset);
+	silence (nframes);
 	return 0;
 }
 
@@ -2393,8 +2476,7 @@ Route::flush_redirects ()
 	Glib::RWLock::ReaderLock lm (redirect_lock);
 
 	for (RedirectList::iterator i = _redirects.begin(); i != _redirects.end(); ++i) {
-		(*i)->deactivate ();
-		(*i)->activate ();
+		(*i)->flush ();
 	}
 }
 
@@ -2419,6 +2501,11 @@ Route::update_total_latency ()
 		}
 	}
 
+#undef DEBUG_LATENCY
+#ifdef DEBUG_LATENCY
+	cerr << _name << ": internal redirect latency = " << _own_latency << endl;
+#endif
+
 	set_port_latency (_own_latency);
 
 	/* this (virtual) function is used for pure Routes,
@@ -2430,6 +2517,11 @@ Route::update_total_latency ()
 
 	_own_latency += input_latency ();
 
+#ifdef DEBUG_LATENCY
+	cerr << _name << ": input latency = " << input_latency() << " total = "
+	     << _own_latency << endl;
+#endif
+
 	return _own_latency;
 }
 
@@ -2517,21 +2609,21 @@ void
 Route::protect_automation ()
 {
 	switch (gain_automation_state()) {
-	case Write:
-		set_gain_automation_state (Off);
-	case Touch:
-		set_gain_automation_state (Play);
+	case Auto_Write:
+		set_gain_automation_state (Auto_Off);
+	case Auto_Touch:
+		set_gain_automation_state (Auto_Play);
 		break;
 	default:
 		break;
 	}
 
 	switch (panner().automation_state ()) {
-	case Write:
-		panner().set_automation_state (Off);
+	case Auto_Write:
+		panner().set_automation_state (Auto_Off);
 		break;
-	case Touch:
-		panner().set_automation_state (Play);
+	case Auto_Touch:
+		panner().set_automation_state (Auto_Play);
 		break;
 	default:
 		break;
@@ -2559,3 +2651,94 @@ Route::set_pending_declick (int declick)
 	}
 
 }
+
+/** Shift automation forwards from a particular place, thereby inserting time.
+ *  Adds undo commands for any shifts that are performed.
+ *
+ * @param pos Position to start shifting from.
+ * @param frames Amount to shift forwards by.
+ */
+
+void
+Route::shift (nframes64_t pos, nframes64_t frames)
+{
+	/* gain automation */
+	XMLNode &before = _gain_automation_curve.get_state ();
+	_gain_automation_curve.shift (pos, frames);
+	XMLNode &after = _gain_automation_curve.get_state ();
+	_session.add_command (new MementoCommand<AutomationList> (_gain_automation_curve, &before, &after));
+
+	/* pan automation */
+	for (std::vector<StreamPanner*>::iterator i = _panner->begin (); i != _panner->end (); ++i) {
+		Curve & c = (*i)->automation ();
+		XMLNode &before = c.get_state ();
+		c.shift (pos, frames);
+		XMLNode &after = c.get_state ();
+		_session.add_command (new MementoCommand<AutomationList> (c, &before, &after));
+	}
+
+	/* redirect automation */
+	{
+		Glib::RWLock::ReaderLock lm (redirect_lock);
+		for (RedirectList::iterator i = _redirects.begin (); i != _redirects.end (); ++i) {
+			
+			set<uint32_t> a;
+			(*i)->what_has_automation (a);
+			
+			for (set<uint32_t>::const_iterator j = a.begin (); j != a.end (); ++j) {
+				AutomationList & al = (*i)->automation_list (*j);
+				XMLNode &before = al.get_state ();
+				al.shift (pos, frames);
+				XMLNode &after = al.get_state ();
+				_session.add_command (new MementoCommand<AutomationList> (al, &before, &after));
+			}
+		}
+	}
+}
+
+int
+Route::save_as_template (const string& path, const string& name)
+{
+	XMLNode& node (state (false));
+	XMLTree tree;
+	
+	IO::set_name_in_state (*node.children().front(), name);
+	
+	tree.set_root (&node);
+	return tree.write (path.c_str());
+}
+
+int
+Route::set_name (string str, void* src)
+{
+	int ret;
+
+	if ((ret = IO::set_name (str, src)) == 0) {
+		if (_control_outs) {
+			string coutname = _name;
+			coutname += _("[control]");
+			cerr << _name << " reset control outs to " << coutname << endl;
+			return _control_outs->set_name (coutname, src);
+		}
+		return 0;
+	}
+	return ret;
+}
+
+bool
+Route::has_io_redirect_named (const string& name)
+{
+	Glib::RWLock::ReaderLock lm (redirect_lock);
+	RedirectList::iterator i;
+
+	for (i = _redirects.begin(); i != _redirects.end(); ++i) {
+		if (boost::dynamic_pointer_cast<Send> (*i) ||
+		    boost::dynamic_pointer_cast<PortInsert> (*i)) {
+			if ((*i)->name() == name) {
+				return true;
+			}
+		}
+	}
+
+	return false;
+}
