@@ -55,6 +55,7 @@
 #include <pbd/strsplit.h>
 #include <pbd/stacktrace.h>
 #include <pbd/copyfile.h>
+#include <pbd/localeguard.h>
 
 #include <ardour/audioengine.h>
 #include <ardour/configuration.h>
@@ -113,8 +114,16 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 
 	_path = string(buf);
 
-	if (_path[_path.length()-1] != '/') {
-		_path += '/';
+	if (_path[_path.length()-1] != G_DIR_SEPARATOR) {
+		_path += G_DIR_SEPARATOR;
+	}
+
+	if (Glib::file_test (_path, Glib::FILE_TEST_EXISTS) && ::access (_path.c_str(), W_OK)) {
+		cerr << "Session non-writable based on " << _path << endl;
+		_writable = false;
+	} else {
+		cerr << "Session writable based on " << _path << endl;
+		_writable = true;	
 	}
 
 	set_history_depth (Config->get_history_depth());
@@ -134,6 +143,7 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 	_tempo_map->StateChanged.connect (mem_fun (*this, &Session::tempo_map_changed));
 
 	g_atomic_int_set (&processing_prohibited, 0);
+	post_transport_work = PostTransportWork (0);
 	insert_cnt = 0;
 	_transport_speed = 0;
 	_last_transport_speed = 0;
@@ -148,13 +158,14 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 	g_atomic_int_set (&_record_status, Disabled);
 	loop_changing = false;
 	play_loop = false;
+	have_looped = false;
 	_last_roll_location = 0;
+	_last_roll_or_reversal_location = 0;
 	_last_record_location = 0;
 	pending_locate_frame = 0;
 	pending_locate_roll = false;
 	pending_locate_flush = false;
 	dstream_buffer_size = 0;
-	state_tree = 0;
 	state_was_pending = false;
 	set_next_event ();
 	outbound_mtc_smpte_frame = 0;
@@ -162,17 +173,15 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 	current_block_size = 0;
 	solo_update_disabled = false;
 	currently_soloing = false;
+	_was_seamless = Config->get_seamless_loop ();
 	_have_captured = false;
 	_worst_output_latency = 0;
 	_worst_input_latency = 0;
 	_worst_track_latency = 0;
 	_state_of_the_state = StateOfTheState(CannotSave|InitialConnecting|Loading);
-
+	
 	_slave = 0;
 	_silent = false;
-	butler_mixdown_buffer = 0;
-	butler_gain_buffer = 0;
-	mmc = 0;
 	session_send_mmc = false;
 	session_send_mtc = false;
 	post_transport_work = PostTransportWork (0);
@@ -183,18 +192,17 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 	g_atomic_int_set (&_playback_load_min, 100);
 	g_atomic_int_set (&_capture_load_min, 100);
 	_play_range = false;
-	waiting_to_start = false;
 	_exporting = false;
 	_gain_automation_buffer = 0;
 	_pan_automation_buffer = 0;
 	_npan_buffers = 0;
 	pending_abort = false;
+	pending_clear_substate = false;
 	destructive_index = 0;
 	current_trans = 0;
 	first_file_data_format_reset = true;
 	first_file_header_format_reset = true;
 	butler_thread = (pthread_t) 0;
-	midi_thread = (pthread_t) 0;
 
 	AudioDiskstream::allocate_working_buffers();
 	
@@ -211,8 +219,6 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 	   waveforms for clicks.
 	*/
 	
-	click_data = 0;
-	click_emphasis_data = 0;
 	click_length = 0;
 	click_emphasis_length = 0;
 	_clicking = false;
@@ -239,7 +245,7 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 
 	/* slave stuff */
 
-	average_slave_delta = 1800;
+	average_slave_delta = 1800; // !!! why 1800 ????
 	have_first_delta_accumulator = false;
 	delta_accumulator_cnt = 0;
 	slave_state = Stopped;
@@ -342,9 +348,7 @@ Session::second_stage_init (bool new_session)
 	deliver_mmc (MIDI::MachineControl::cmdMmcReset, 0);
 	deliver_mmc (MIDI::MachineControl::cmdLocate, 0);
 
-	BootMessage (_("Reset Control Protocols"));
-
-	ControlProtocolManager::instance().set_session (*this);
+	/* initial program change will be delivered later; see ::config_changed() */
 
 	if (new_session) {
 		_end_location_is_free = true;
@@ -411,20 +415,9 @@ Session::setup_raid_path (string path)
 		sp.path = path;
 		sp.blocks = 0;
 		session_dirs.push_back (sp);
-
-		string fspath;
-
 		/* sounds dir */
 
-		fspath += sp.path;
-		if (fspath[fspath.length()-1] != '/') {
-			fspath += '/';
-		}
-
-		fspath += sound_dir (false);
-		
-		AudioFileSource::set_search_path (fspath);
-
+		AudioFileSource::set_search_path (Glib::build_filename(sp.path, sound_dir (false)));
 		return;
 	}
 
@@ -438,11 +431,7 @@ Session::setup_raid_path (string path)
 
 		/* add sounds to file search path */
 
-		fspath += sp.path;
-		if (fspath[fspath.length()-1] != '/') {
-			fspath += '/';
-		}
-		fspath += sound_dir (false);
+		fspath += Glib::build_filename(sp.path, sound_dir (false));
 		fspath += ':';
 
 		remaining = remaining.substr (colon+1);
@@ -454,11 +443,7 @@ Session::setup_raid_path (string path)
 		sp.path = remaining;
 
 		fspath += ':';
-		fspath += sp.path;
-		if (fspath[fspath.length()-1] != '/') {
-			fspath += '/';
-		}
-		fspath += sound_dir (false);
+		fspath += Glib::build_filename(sp.path, sound_dir (false));
 		fspath += ':';
 
 		session_dirs.push_back (sp);
@@ -674,18 +659,19 @@ Session::remove_state (string snapshot_name)
 }
 
 int
-Session::save_state (string snapshot_name, bool pending)
+Session::save_state (string snapshot_name, bool pending, bool switch_to_snapshot)
 {
 	XMLTree tree;
 	string xml_path;
 	string bak_path;
 
-	if (_state_of_the_state & CannotSave) {
+	if (!_writable || (_state_of_the_state & CannotSave)) {
 		return 1;
 	}
 
 	if (!_engine.connected ()) {
-		error << _("Ardour's audio engine is not connected and state saving would lose all I/O connections. Session not saved")
+		error << string_compose (_("The %1 audio engine is not connected and state saving would lose all I/O connections. Session not saved"),
+					 PROGRAM_NAME)
 		      << endmsg;
 		return 1;
 	}
@@ -694,6 +680,8 @@ Session::save_state (string snapshot_name, bool pending)
 
 	if (snapshot_name.empty()) {
 		snapshot_name = _current_snapshot_name;
+	} else if (switch_to_snapshot) {
+		_current_snapshot_name = snapshot_name;
 	}
 
 	if (!pending) {
@@ -814,6 +802,15 @@ Session::load_state (string snapshot_name)
 
 	set_dirty();
 
+	/* writable() really reflects the whole folder, but if for any
+	   reason the session state file can't be written to, still
+	   make us unwritable.
+	*/
+
+	if (::access (xmlpath.c_str(), W_OK) != 0) {
+		_writable = false;
+	}
+
 	if (!state_tree->read (xmlpath)) {
 		error << string_compose(_("Could not understand ardour file %1"), xmlpath) << endmsg;
 		delete state_tree;
@@ -824,7 +821,7 @@ Session::load_state (string snapshot_name)
 	XMLNode& root (*state_tree->root());
 	
 	if (root.name() != X_("Session")) {
-		error << string_compose (_("Session file %1 is not an Ardour session"), xmlpath) << endmsg;
+		error << string_compose (_("Session file %1 is not a session"), xmlpath) << endmsg;
 		delete state_tree;
 		state_tree = 0;
 		return -1;
@@ -855,8 +852,8 @@ Session::load_state (string snapshot_name)
 		/* don't make another copy if it already exists */
 
 		if (!Glib::file_test (backup_path, Glib::FILE_TEST_EXISTS)) {
-			info << string_compose (_("Copying old session file %1 to %2\nUse %2 with Ardour versions before 2.0 from now on"),
-						xmlpath, backup_path) 
+			info << string_compose (_("Copying old session file %1 to %2\nUse %2 with %3 versions before 2.0 from now on"),
+						xmlpath, backup_path, PROGRAM_NAME) 
 			     << endmsg;
 			
 			copy_file (xmlpath, backup_path);
@@ -877,9 +874,15 @@ Session::load_options (const XMLNode& node)
 
 	Config->set_variables (node, ConfigVariableBase::Session);
 
+	/* now reset MIDI ports because the session can have its own 
+	   MIDI configuration.
+	*/
+
+	setup_midi ();
+
 	if ((child = find_named_node (node, "end-marker-is-free")) != 0) {
 		if ((prop = child->property ("val")) != 0) {
-			_end_location_is_free = (prop->value() == "yes");
+			_end_location_is_free = string_is_affirmative (prop->value());
 		}
 	}
 
@@ -1176,10 +1179,10 @@ Session::set_state (const XMLNode& node)
 	if ((prop = node.property (X_("sample-rate"))) != 0) {
 
 		_nominal_frame_rate = atoi (prop->value());
-
+		
 		if (_nominal_frame_rate != _current_frame_rate) {
 			if (AskAboutSampleRateMismatch (_nominal_frame_rate, _current_frame_rate)) {
-				return -1;
+				throw SRMismatchRejected();
 			}
 		}
 	}
@@ -1206,10 +1209,10 @@ Session::set_state (const XMLNode& node)
 
 	/* Object loading order:
 
-	MIDI
 	Path
 	extra
 	Options/Config
+	MIDI  <= relies on data from Options/Config
 	Locations
 	Sources
 	AudioRegions
@@ -1222,9 +1225,6 @@ Session::set_state (const XMLNode& node)
 	ControlProtocols
 	*/
 
-	if (use_config_midi_ports ()) {
-	}
-
 	if ((child = find_named_node (node, "extra")) != 0) {
 		_extra_xml = new XMLNode (*child);
 	}
@@ -1237,6 +1237,9 @@ Session::set_state (const XMLNode& node)
 		error << _("Session: XML state has no options section") << endmsg;
 	}
 
+	if (use_config_midi_ports ()) {
+	}
+
 	if ((child = find_named_node (node, "Locations")) == 0) {
 		error << _("Session: XML state has no locations section") << endmsg;
 		goto out;
@@ -1377,6 +1380,13 @@ Session::load_routes (const XMLNode& node)
 	set_dirty();
 
 	for (niter = nlist.begin(); niter != nlist.end(); ++niter) {
+		XMLProperty* prop = (*niter)->property ("default-type");
+		
+		if (prop && prop->value() == "unknown" ) {
+			std::cout << "ignoring route with type unknown. (video-track)" << std::endl;
+			// Note: this may mess up remote_control IDs or more..
+			continue;
+		}
 
 		boost::shared_ptr<Route> route (XMLRouteFactory (**niter));
 
@@ -1643,7 +1653,7 @@ Session::XMLSourceFactory (const XMLNode& node)
 	}
 
 	catch (failed_constructor& err) {
-		error << _("Found a sound file that cannot be used by Ardour. Talk to the progammers.") << endmsg;
+		error << string_compose (_("Found a sound file that cannot be used by %1. Talk to the progammers."), PROGRAM_NAME) << endmsg;
 		return boost::shared_ptr<Source>();
 	}
 }
@@ -1672,9 +1682,7 @@ Session::save_template (string template_name)
 
 	tree.set_root (&get_template());
 
-	xml_path = dir;
-	xml_path += template_name;
-	xml_path += _template_suffix;
+	xml_path = Glib::build_filename(dir, template_name + _template_suffix);
 
 	ifstream in(xml_path.c_str());
 	
@@ -1696,8 +1704,8 @@ Session::save_template (string template_name)
 int
 Session::rename_template (string old_name, string new_name) 
 {
-	string old_path = template_dir() + old_name + _template_suffix;
-	string new_path = template_dir() + new_name + _template_suffix;
+	string old_path = Glib::build_filename(template_dir(), old_name + _template_suffix);
+	string new_path = Glib::build_filename(template_dir(), new_name + _template_suffix);
 
 	return rename (old_path.c_str(), new_path.c_str());
 }
@@ -1705,9 +1713,7 @@ Session::rename_template (string old_name, string new_name)
 int
 Session::delete_template (string name) 
 {
-	string template_path = template_dir();
-	template_path += name;
-	template_path += _template_suffix;
+	string template_path = Glib::build_filename(template_dir(), name + _template_suffix);
 
 	return remove (template_path.c_str());
 }
@@ -1751,27 +1757,21 @@ Session::ensure_sound_dir (string path, string& result)
 	
 	/* Ensure that the sounds directory exists */
 	
-	result = path;
-	result += '/';
-	result += sound_dir_name;
+	result = Glib::build_filename(path, sound_dir_name);
 	
 	if (g_mkdir_with_parents (result.c_str(), 0775)) {
 		error << string_compose(_("cannot create sounds directory \"%1\"; ignored"), result) << endmsg;
 		return -1;
 	}
 
-	dead = path;
-	dead += '/';
-	dead += dead_sound_dir_name;
+	dead = Glib::build_filename(path, dead_sound_dir_name);
 	
 	if (g_mkdir_with_parents (dead.c_str(), 0775)) {
 		error << string_compose(_("cannot create dead sounds directory \"%1\"; ignored"), dead) << endmsg;
 		return -1;
 	}
 
-	peak = path;
-	peak += '/';
-	peak += peak_dir_name;
+	peak = Glib::build_filename(path, peak_dir_name);
 	
 	if (g_mkdir_with_parents (peak.c_str(), 0775)) {
 		error << string_compose(_("cannot create peak file directory \"%1\"; ignored"), peak) << endmsg;
@@ -1780,7 +1780,7 @@ Session::ensure_sound_dir (string path, string& result)
 	
 	/* callers expect this to be terminated ... */
 			
-	result += '/';
+	result += G_DIR_SEPARATOR;
 	return 0;
 }	
 
@@ -2013,6 +2013,7 @@ Session::sound_dir (bool with_path) const
 {
 	string res;
 	string full;
+	vector<string> parts;
 
 	if (with_path) {
 		res = _path;
@@ -2020,12 +2021,11 @@ Session::sound_dir (bool with_path) const
 		full = _path;
 	}
 
-	res += interchange_dir_name;
-	res += '/';
-	res += legalize_for_path (_name);
-	res += '/';
-	res += sound_dir_name;
+	parts.push_back(interchange_dir_name);
+	parts.push_back(legalize_for_path (_name));
+	parts.push_back(sound_dir_name);
 
+	res += Glib::build_filename(parts);
 	if (with_path) {
 		full = res;
 	} else {
@@ -2044,7 +2044,7 @@ Session::sound_dir (bool with_path) const
 	string old_withpath;
 
 	old_nopath += old_sound_dir_name;
-	old_nopath += '/';
+	old_nopath += G_DIR_SEPARATOR;
 	
 	old_withpath = _path;
 	old_withpath += old_sound_dir_name;
@@ -2064,44 +2064,37 @@ Session::sound_dir (bool with_path) const
 string
 Session::peak_dir () const
 {
-	string res = _path;
-	res += peak_dir_name;
-	res += '/';
-	return res;
+	return Glib::build_filename (_path, peak_dir_name);
 }
 	
 string
 Session::automation_dir () const
 {
-	string res = _path;
-	res += "automation/";
-	return res;
+	return Glib::build_filename (_path, "automation");
 }
 
 string
 Session::analysis_dir () const
 {
-	string res = _path;
-	res += "analysis/";
-	return res;
+	return Glib::build_filename (_path, "analysis");
 }
 
 string
 Session::template_dir ()
 {
-	string path = get_user_ardour_path();
-	path += "templates/";
+	return Glib::build_filename (get_user_ardour_path(), "templates");
+}
 
-	return path;
+string
+Session::route_template_dir ()
+{
+	return Glib::build_filename (get_user_ardour_path(), "route_templates");
 }
 
 string
 Session::export_dir () const
 {
-	string res = _path;
-	res += export_dir_name;
-	res += '/';
-	return res;
+	return Glib::build_filename (_path, export_dir_name);
 }
 
 string
@@ -2128,7 +2121,7 @@ Session::suffixed_search_path (string suffix, bool data)
 	for (vector<string>::iterator i = split_path.begin(); i != split_path.end(); ++i) {
 		path += *i;
 		path += suffix;
-		path += '/';
+		path += G_DIR_SEPARATOR;
 		
 		if (distance (i, split_path.end()) != 1) {
 			path += ':';
@@ -2144,6 +2137,13 @@ Session::template_path ()
 	return suffixed_search_path (X_("templates"), true);
 }
 
+
+string
+Session::route_template_path ()
+{
+	return suffixed_search_path (X_("route_templates"), true);
+}
+
 string
 Session::control_protocol_path ()
 {
@@ -2231,7 +2231,7 @@ remove_end(string* state)
 	string statename(*state);
 	
 	string::size_type start,end;
-	if ((start = statename.find_last_of ('/')) != string::npos) {
+	if ((start = statename.find_last_of (G_DIR_SEPARATOR)) != string::npos) {
 		statename = statename.substr (start+1);
 	}
 		
@@ -2503,7 +2503,7 @@ Session::get_template_list (list<string> &template_names)
 		string fullpath = *(*i);
 		int start, end;
 
-		start = fullpath.find_last_of ('/') + 1;
+		start = fullpath.find_last_of (G_DIR_SEPARATOR) + 1;
 		if ((end = fullpath.find_last_of ('.')) <0) {
 			end = fullpath.length();
 		}
@@ -2512,11 +2512,47 @@ Session::get_template_list (list<string> &template_names)
 	}
 }
 
+void
+Session::get_route_templates (vector<RouteTemplateInfo>& template_names)
+{
+	vector<string *> *templates;
+	PathScanner scanner;
+	string path;
+
+	path = route_template_path ();
+	
+	templates = scanner (path, template_filter, 0, false, true);
+	
+	if (!templates) {
+	  return;
+	}
+
+	for (vector<string*>::iterator i = templates->begin(); i != templates->end(); ++i) {
+		string fullpath = *(*i);
+
+		XMLTree tree;
+
+		if (!tree.read (fullpath.c_str())) {
+		  continue;
+		}
+
+		XMLNode* root = tree.root();
+		
+		RouteTemplateInfo rti;
+
+		rti.name = IO::name_from_state (*root->children().front());
+		rti.path = fullpath;
+
+		template_names.push_back (rti);
+	}
+
+	delete templates;
+}
+
 int
 Session::read_favorite_dirs (FavoriteDirs & favs)
 {
-	string path = get_user_ardour_path();
-	path += "/favorite_dirs";
+	Glib::ustring path = Glib::build_filename (get_user_ardour_path(), "favorite_dirs");
 
 	ifstream fav (path.c_str());
 
@@ -2550,8 +2586,7 @@ Session::read_favorite_dirs (FavoriteDirs & favs)
 int
 Session::write_favorite_dirs (FavoriteDirs & favs)
 {
-	string path = get_user_ardour_path();
-	path += "/favorite_dirs";
+	Glib::ustring path = Glib::build_filename (get_user_ardour_path(), "favorite_dirs");
 
 	ofstream fav (path.c_str());
 
@@ -2607,17 +2642,21 @@ Session::find_all_sources (string path, set<string>& result)
 			continue;
 		}
 
-		if (prop->value()[0] == '/') {
+		if (Glib::path_is_absolute (prop->value())) {
 			/* external file, ignore */
 			continue;
 		}
 
-		string path = _path; /* /-terminated */
-		path += sound_dir_name;
-		path += '/';
-		path += prop->value();
+		/* now we have to actually find the file */
 
-		result.insert (path);
+		bool is_new;
+		uint16_t chan;
+		Glib::ustring path;
+		std::string name;
+		
+		if (AudioFileSource::find (prop->value(), true, false, is_new, chan, path, name)) {
+			result.insert (path);
+		}
 	}
 
 	return 0;
@@ -2635,7 +2674,7 @@ Session::find_all_sources_across_snapshots (set<string>& result, bool exclude_th
 
 	ripped = _path;
 
-	if (ripped[ripped.length()-1] == '/') {
+	if (ripped[ripped.length()-1] == G_DIR_SEPARATOR) {
 		ripped = ripped.substr (0, ripped.length() - 1);
 	}
 
@@ -2850,18 +2889,16 @@ Session::cleanup_sources (Session::cleanup_report& rep)
 			newpath = Glib::path_get_dirname (newpath); // "session-dir"
 		}
 
-		newpath += '/';
-		newpath += dead_sound_dir_name;
+                newpath = Glib::build_filename (newpath, dead_sound_dir_name);
 
 		if (g_mkdir_with_parents (newpath.c_str(), 0755) < 0) {
 			error << string_compose(_("Session: cannot create session peakfile folder \"%1\" (%2)"), newpath, strerror (errno)) << endmsg;
 			return -1;
 		}
-
-		newpath += '/';
-		newpath += Glib::path_get_basename ((*x));
+                
+		newpath = Glib::build_filename (newpath, Glib::path_get_basename ((*x)));
 		
-		if (access (newpath.c_str(), F_OK) == 0) {
+		if (Glib::file_test (newpath, Glib::FILE_TEST_EXISTS)) {
 			
 			/* the new path already exists, try versioning */
 			
@@ -2892,7 +2929,7 @@ Session::cleanup_sources (Session::cleanup_report& rep)
 		}
 
 		if (::rename ((*x).c_str(), newpath.c_str()) != 0) {
-			error << string_compose (_("cannot rename audio file source from %1 to %2 (%3)"),
+			error << string_compose (_("cannot 4 rename audio file source from %1 to %2 (%3)"),
 					  (*x), newpath, strerror (errno))
 			      << endmsg;
 			goto out;
@@ -2966,9 +3003,7 @@ Session::cleanup_trash_sources (Session::cleanup_report& rep)
 
 			string fullpath;
 
-			fullpath = dead_sound_dir;
-			fullpath += '/';
-			fullpath += dentry->d_name;
+			fullpath = Glib::build_filename (dead_sound_dir, dentry->d_name);
 
 			if (stat (fullpath.c_str(), &statbuf)) {
 				continue;
@@ -3075,7 +3110,9 @@ Session::controllable_by_id (const PBD::ID& id)
 void 
 Session::add_instant_xml (XMLNode& node, const std::string& dir)
 {
-	Stateful::add_instant_xml (node, dir);
+	if (_writable) {
+		Stateful::add_instant_xml (node, dir);
+	}
 	Config->add_instant_xml (node, get_user_ardour_path());
 }
 
@@ -3086,6 +3123,10 @@ Session::save_history (string snapshot_name)
     string xml_path;
     string bak_path;
 
+    if (!_writable) {
+	    return 0;
+    }
+
     if (snapshot_name.empty()) {
 	snapshot_name = _current_snapshot_name;
     }
@@ -3140,7 +3181,7 @@ Session::restore_history (string snapshot_name)
 
     /* read xml */
     xmlpath = _path + snapshot_name + ".history";
-    cerr << string_compose(_("Loading history from '%1'."), xmlpath) << endmsg;
+    info << string_compose(_("Loading history from '%1'."), xmlpath) << endmsg;
 
     if (!Glib::file_test (xmlpath, Glib::FILE_TEST_EXISTS)) {
 	    return 1;
@@ -3389,7 +3430,18 @@ Session::config_changed (const char* parameter_name)
 	} else if (PARAM_IS ("history-depth")) {
 		set_history_depth (Config->get_history_depth());
 	} else if (PARAM_IS ("sync-all-route-ordering")) {
-		sync_order_keys ();
+		sync_order_keys ("session"); 
+	} else if (PARAM_IS ("initial-program-change")) {
+
+		if (_mmc_port && Config->get_initial_program_change() >= 0) {
+			MIDI::byte* buf = new MIDI::byte[2];
+			
+			buf[0] = MIDI::program; // channel zero by default
+			buf[1] = (Config->get_initial_program_change() & 0x7f);
+			deliver_midi (_mmc_port, buf, 2);
+		}
+	} else if (PARAM_IS ("solo-mute-override")) {
+		catch_up_on_solo_mute_override ();
 	}
 
 	set_dirty ();
