@@ -25,7 +25,10 @@
 #include <sigc++/bind.h>
 
 #include <glibmm/thread.h>
+
 #include <pbd/xml++.h>
+#include <pbd/localeguard.h>
+
 #include <ardour/tempo.h>
 #include <ardour/utils.h>
 
@@ -107,7 +110,7 @@ TempoSection::TempoSection (const XMLNode& node)
 		throw failed_constructor();
 	}
 
-	set_movable (prop->value() == "yes");
+	set_movable (string_is_affirmative (prop->value()));
 }
 
 XMLNode&
@@ -183,7 +186,7 @@ MeterSection::MeterSection (const XMLNode& node)
 		throw failed_constructor();
 	}
 
-	set_movable (prop->value() == "yes");
+	set_movable (string_is_affirmative (prop->value()));
 }
 
 XMLNode&
@@ -528,6 +531,21 @@ TempoMap::change_initial_tempo (double beats_per_minute, double note_type)
 	}
 }
 
+void
+TempoMap::insert_time (nframes_t where, nframes_t amount)
+{
+	for (Metrics::iterator i = metrics->begin(); i != metrics->end(); ++i) {
+		if ((*i)->frame() >= where) {
+			(*i)->set_frame ((*i)->frame() + amount);
+		}
+	}
+
+	timestamp_metrics (false);
+	
+	StateChanged (Change (0));
+}
+
+
 void
 TempoMap::change_existing_tempo_at (nframes_t where, double beats_per_minute, double note_type)
 {
@@ -1141,16 +1159,16 @@ TempoMap::round_to_type (nframes_t frame, int dir, BBTPointType type)
 	case Bar:
 		if (dir < 0) {
 			/* relax */
-
 		} else if (dir > 0) {
 			if (bbt.beats > 0) {
 				bbt.bars++;
+			} else if (metric.frame() < frame) {
+				bbt.bars++;
 			}
 		} else {
 			if (bbt.beats > metric.meter().beats_per_bar()/2) {
 				bbt.bars++;
 			}
-
 		}
 		bbt.beats = 1;
 		bbt.ticks = 0;
@@ -1162,6 +1180,8 @@ TempoMap::round_to_type (nframes_t frame, int dir, BBTPointType type)
 		} else if (dir > 0) {
 			if (bbt.ticks > 0) {
 				bbt.beats++;
+			} else if (metric.frame() < frame) {
+				bbt.beats++;
 			}
 		} else {
 			if (bbt.ticks >= (Meter::ticks_per_beat/2)) {
@@ -1176,13 +1196,12 @@ TempoMap::round_to_type (nframes_t frame, int dir, BBTPointType type)
 		break;
 	
 	}
-
+	
 	/* 
-	   cerr << "for " << frame << " round to " << bbt << " using "
-	   << metric.start()
-	   << endl;
+	cerr << "for " << frame << " round to " << bbt << " using "
+	     << metric.start()
+	     << endl;
 	*/
-
 	return metric.frame() + count_frames_between (metric.start(), bbt);
 }
 
