@@ -15,7 +15,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: gtk_ui.cc 3139 2008-03-08 16:23:26Z drobilla $
+    $Id: gtk_ui.cc 6496 2010-01-15 17:50:03Z paul $
 */
 
 #include <cmath>
@@ -33,6 +33,8 @@
 #include <pbd/pthread_utils.h>
 #include <pbd/stacktrace.h>
 
+#include <gtkmm2ext/application.h>
+
 #include <gtkmm2ext/gtk_ui.h>
 #include <gtkmm2ext/textviewer.h>
 #include <gtkmm2ext/popup.h>
@@ -60,12 +62,13 @@ BaseUI::RequestType Gtkmm2ext::AddTimeout = BaseUI::new_request_type();
 
 #include <pbd/abstract_ui.cc>  /* instantiate the template */
 
-
 UI::UI (string namestr, int *argc, char ***argv) 
 	: AbstractUI<UIRequest> (namestr, true)
 {
 	theMain = new Main (argc, argv);
+#ifndef GTK_NEW_TOOLTIP_API
 	tips = new Tooltips;
+#endif
 
 	_active = false;
 	_auto_display_errors = true;
@@ -85,7 +88,7 @@ UI::UI (string namestr, int *argc, char ***argv)
 		       UI::signal_pipe_callback,
 		       this);
 
-	errors = new TextViewer (850,100);
+	errors = new TextViewer (800,300);
 	errors->text().set_editable (false); 
 	errors->text().set_name ("ErrorText");
 
@@ -97,16 +100,20 @@ UI::UI (string namestr, int *argc, char ***argv)
 
 	errors->dismiss_button().set_name ("ErrorLogCloseButton");
 	errors->signal_delete_event().connect (bind (sigc::ptr_fun (just_hide_it), (Window *) errors));
+	errors->set_type_hint (Gdk::WINDOW_TYPE_HINT_UTILITY);
 
 	register_thread (pthread_self(), X_("GUI"));
 
+	/* instantiate the Application singleton */
+
+	Application::instance();
 }
 
 UI::~UI ()
 {
+	delete Application::instance ();
 }
 
-
 bool
 UI::caller_is_ui_thread ()
 {
@@ -373,7 +380,16 @@ UI::do_request (UIRequest* req)
 
 	} else if (req->type == SetTip) {
 
-		/* XXX need to figure out how this works */
+#ifdef GTK_NEW_TOOLTIP_API
+		/* even if the installed GTK is up to date,
+		   at present (November 2008) our included
+		   version of gtkmm is not. so use the GTK
+		   API that we've verified has the right function.
+		*/
+		gtk_widget_set_tooltip_text (req->widget->gobj(), req->msg);
+#else
+		tips->set_tip (*req->widget, req->msg, "");
+#endif
 
 	} else {
 
@@ -483,7 +499,7 @@ UI::process_error_message (Transmitter::Channel chn, const char *str)
 		
 		display_message (prefix, prefix_len, ptag, mtag, str);
 		
-		if (_auto_display_errors) {
+		if (_auto_display_errors && chn != Transmitter::Info) {
 			show_error_log ();
 		}
 	}
@@ -558,7 +574,7 @@ UI::handle_fatal (const char *message)
 }
 
 void
-UI::popup_error (const char *text)
+UI::popup_error (const std::string& text)
 {
 	if (!caller_is_ui_thread()) {
 		error << "non-UI threads can't use UI::popup_error" 
@@ -573,30 +589,12 @@ UI::popup_error (const char *text)
 }
 
 #ifdef GTKOSX
-extern "C" {
-	int gdk_quartz_in_carbon_menu_event_handler ();
-}
+extern int gdk_quartz_in_menu_event_handler ();
 #endif
 
 void
 UI::flush_pending ()
 {
-#ifdef GTKOSX
-	/* as of february 11th 2008, gtk/osx has a problem in that mac menu events
-	   are handled using Carbon with an "internal" event handling system that 
-	   doesn't pass things back to the glib/gtk main loop. this makes
-	   gtk_main_iteration() block if we call it while in a menu event handler 
-	   because glib gets confused and thinks there are two threads running
-	   g_main_poll_func(). 
-
-	   this hack (relies on code in gtk2_ardour/sync-menu.c) works
-	   around that.
-	*/
-
-	if (gdk_quartz_in_carbon_menu_event_handler()) {
-		return;
-	}
-#endif
 	if (!caller_is_ui_thread()) {
 		error << "non-UI threads cannot call UI::flush_pending()"
 		      << endmsg;
@@ -613,7 +611,6 @@ UI::flush_pending ()
 bool
 UI::just_hide_it (GdkEventAny *ev, Window *win)
 {
-	cerr << "++++ JUST HIDING " << win->get_window() << endl;
 	win->hide ();
 	return true;
 }
