@@ -14,7 +14,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: fifomidi.cc 4 2005-05-13 20:47:18Z taybin $
+    $Id: fifomidi.cc 2489 2007-09-27 04:20:31Z paul $
 */
 
 #include <fcntl.h>
@@ -26,19 +26,19 @@
 
 using namespace MIDI;
 
-FIFO_MidiPort::FIFO_MidiPort (PortRequest &req) 
-	: FD_MidiPort (req, ".", "midi")
+FIFO_MidiPort::FIFO_MidiPort (const XMLNode& node) 
+	: FD_MidiPort (node, ".", "midi")
 
 {
 }
 
 void
-FIFO_MidiPort::open (PortRequest &req)
+FIFO_MidiPort::open (const Port::Descriptor& desc)
 
 {
 	/* This is a placeholder for the fun-and-games I think we will
 	   need to do with FIFO's.
 	*/
 
-	_fd = ::open (req.devname, req.mode|O_NDELAY);
+	_fd = ::open (desc.device.c_str(), desc.mode|O_NDELAY);
 }
